package com.megabras.bluelogg;

import android.util.Log;

import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.Date;
import java.util.Locale;
import java.util.TimeZone;

@SuppressWarnings("unused")
public class Command {
	private static final String TAG = "Command";
	private final int type;
	private String cmd;
	private String desc;
	private final int responseSize;
	public static String SF 				   = "";
	public static String EF 				   = "";
	private static final String IP             = "01";
	private static final String READ           = "03";
	private static final String WRITE          = "10"; //06 -> Single register | -> 10 Multiples registers	
	private static final String TEST_NUMBER    = "00810001";       //0081-ADDRESS | 0001-REGISTERS
//	private static final String STOP           = "00820001020002"; //0082-ADDRESS | 0001-REGISTERS | 02-BYTES | 0002->COMAND
//	private static final String START          = "00820001020001"; //0082-ADDRESS | 0001-REGISTERS | 02-BYTES | 0001->COMAND
//	private static final String PRINT          = "00820001020004"; //0082-ADDRESS | 0001-REGISTERS | 02-BYTES | 0004->COMAND
	private static final String MEMORY_MAP     = "00000001";
	private static final String STOP           = "00820002"; //0082-ADDRESS | 0001-REGISTERS | 02-BYTES | 0002->COMAND
	private static final String START          = "00820001"; //0082-ADDRESS | 0001-REGISTERS | 02-BYTES | 0001->COMAND
	private static final String PRINT          = "00820004"; //0082-ADDRESS | 0001-REGISTERS | 02-BYTES | 0004->COMAND
	private static final String MODE_RC        = "00800001020001"; //0080-ADDRESS | 0001-REGISTERS | 02-BYTES | 0001->COMAND
	private static final String MODE_R         = "00800001020000"; //0080-ADDRESS | 0001-REGISTERS | 02-BYTES | 0000->COMAND
	private static final String PRINT_STATUS   = "03460001";       //0346-ADDRESS | 0001-REGISTERS
	private static final String SHORT_STATUS   = "03460002";       //0346-ADDRESS | 0002-REGISTERS (STATUS + BATTERY)
	private static final String FULL_STATUS    = "03400008";       //0340-ADDRESS | 0007-REGISTERS
	private static final String BATTERY_STATUS = "03470001";       //0347-ADDRESS | 0001-REGISTERS
	private static final String TEST_MODE      = "00800001";       //0080-ADDRESS | 0001-REGISTERS
	private static final String FREE_MEMORY    = "02000001";       //0200-ADDRESS | 0001-REGISTERS
	private static final String MEMORY_POINTER = "02000001";       //0200-ADDRESS | 0001-REGISTERS
	private static String GET_DATE             = "000C0002";       //000C-ADDRESS | 0002-REGISTERS
	private static String SET_DATE             = "000C000204";     //000C-ADDRESS | 0002-REGISTERS | 04-BYTES
	private static String MODEL                = "00020005";       //0002-ADDRESS | 0005-REGISTERS
	private static String SERIAL               = "00100004";       //0010-ADDRESS | 0004-REGISTERS
	private static final String MEASUREMENTS   = "8000";           //8000-ADDRESS (Where measurements ares stored) 
	

	public Command(int type, String cmd, String desc){
		this.type = type;
		this.cmd  = cmd;
		this.desc = desc;
		responseSize = calculateResponseSize(cmd);
	}

	public static void setMemoryMap(int memoryMap)
	{
		if ( memoryMap >= 4 )
		{
			GET_DATE = "00180002";
			SET_DATE = "0018000204";
			MODEL    = "00030005";
			SERIAL   = "00120004";
		}
	}

	private int calculateResponseSize(String msg){
		int size = -1;
		msg = msg.replace(":","");
		msg = msg.replace("\r\n","");

		if( msg.startsWith("0110") || msg.startsWith("0106")  )
			size = 14;
		else if(msg.startsWith("0103")){
			String sub = msg.substring(8,12);
			size = MUtils.hexToInt(sub);
			size = 6 + (size*4) + 2;
		}
		return size;
	}

	private static boolean isFrameEnabled = false;
	public static void enableFrame(){
		isFrameEnabled = !isFrameEnabled;
		if(isFrameEnabled){
			SF = ":";
			EF = "\r\n";
		}
		else{
			SF = "";
			EF = "";
		}
	}

	public static void enableFrame(boolean enable){
		if(enable){
			SF = ":";
			EF = "\r\n";
		}
		else{
			SF = "";
			EF = "";
		}
	}

	public int getType()
	{
		return type;
	}
	
	public String getCommand(){
		return cmd;
	}

	public String getDescription(){
		return desc;
	}

	public int getResponseSize(){
		return responseSize;
	}
	
	public boolean checkResponse(String response){
		try{
			if(MUtils.checkSum(response)){				
				String expected = "";
				String type = response.substring(2, 4);
				if(type.equals(WRITE)){
					expected = cmd.substring(0,12);
				} else if (type.equals(READ)){
					expected = cmd.substring(0,4) + cmd.substring(10,12);					
				}
				return response.startsWith(expected);				
			} else {
				return false;
			}
		} catch (Exception e){
			return false;
		}
	}

	public static Command MEMORY_MAP(){
		return new Command(-1, MUtils.setCheckSum(IP + READ + MEMORY_MAP), "(R) MEMORY MAP");
	}

	public static Command SHORT_STATUS(){
		return new Command(1, MUtils.setCheckSum(IP + READ + SHORT_STATUS), "(R) PING (SHORT STATUS)");
	}

	public static Command START(){
		return new Command(2, MUtils.setCheckSum(IP + "06" + START), "(W) START");
	}

	public static Command TEST_NUMBER(){
		return new Command(3, MUtils.setCheckSum(IP + READ + TEST_NUMBER), "(R) TEST NUMBER");
	}
	
	public static Command FULL_STATUS(){
		return new Command(4, MUtils.setCheckSum(IP + READ + FULL_STATUS), "(R) FULL STATUS");
	}

	public static Command MODE_RC(){
		return new Command(5, MUtils.setCheckSum(IP + WRITE + MODE_RC), "(W) COMPENSATION MODE");
	}
	
	public static Command MODE_R(){
		return new Command(6, MUtils.setCheckSum(IP + WRITE + MODE_R), "(W) NORMAL MODE");
	}
	
	public static Command PRINT(){
		return new Command(7, MUtils.setCheckSum(IP + "06" + PRINT), "(W) PRINT");
	}	
	
	public static Command BATTERY_STATUS(){
		return new Command(8, MUtils.setCheckSum(IP + READ + BATTERY_STATUS), "(R) BATTERY STATUS");
	}	

	public static Command GET_DATE(){
		return new Command(9, MUtils.setCheckSum(IP + READ + GET_DATE), "(R) DATE/TIME");
	}
	
	public static Command SET_DATE(){
		final long tabletCurrentTime = getCurrentTime();
		String time = Long.toHexString(tabletCurrentTime);
		return new Command(10, MUtils.setCheckSum(IP + WRITE + SET_DATE + time), "(W) DATE/TIME");
	}

	public static Command TEST_MODE(){
		return new Command(11, MUtils.setCheckSum(IP + READ + TEST_MODE), "(R) TEST MODE");
	}

	public static Command MEMORY_POINTER(){
		return new Command(13, MUtils.setCheckSum(IP + READ + MEMORY_POINTER), "(R) MEMORY POINTER");
	}

	public static Command LAST_RESULT(String p){
		/**
		 * Short.valueOf("FF",16).byteValue(); // -1 (byte)
		 * Integer.valueOf("FFFF",16).shortValue(); // -1 (short)
		 * Long.valueOf("FFFFFFFF",16).intValue(); // -1 (int) new
		 * BigInteger("FFFFFFFFFFFFFFFF",16).longValue(); // -1 (long)
		 *
		 * */
		int pointer = Integer.valueOf(p, 16).shortValue();
		int address = Integer.valueOf(MEASUREMENTS, 16).shortValue();
		int result = (address + ((pointer - 2) * 4));

		String add = String.format("%04X", result);
		if (add.length() > 4) {
			add = add.substring(add.length() - 4, add.length());
		}
		String command = add.toUpperCase(Locale.getDefault()) + "0008";
		return new Command(14, MUtils.setCheckSum(IP + READ + command), "(R) LAST RESULT");
	}

	public static Command GET_MEM_PACK(int pointer){
		/**
		 * Short.valueOf("FF",16).byteValue(); // -1 (byte)
		 * Integer.valueOf("FFFF",16).shortValue(); // -1 (short)
		 * Long.valueOf("FFFFFFFF",16).intValue(); // -1 (int) new
		 * BigInteger("FFFFFFFFFFFFFFFF",16).longValue(); // -1 (long)
		 *
		 * */
		pointer = pointer * 4; //ponteiro indica quantidade de registros

		int address = Integer.valueOf(MEASUREMENTS, 16).shortValue();
		int result = (address + pointer);

		String add = String.format("%04X", result & 0xFFFF);

		int registrySize = 4; //Um registro é composto por dois HEX_BYTES (0000), neste caso estamos querendo ler 8 HEX_BYTES
		int numOfRegistries = 1;
		String NUM_OF_REGISTRIES = String.format("%04X", (registrySize * numOfRegistries) & 0xFFFF );


		String command = add + NUM_OF_REGISTRIES;
		return new Command(15, MUtils.setCheckSum(IP + READ + command), "(R) GET_MEM_PACK");
	}

	public static Command PRINT_STATUS(){
		return new Command(16, MUtils.setCheckSum(IP + READ + PRINT_STATUS), "(R) PRINT STATUS");
	}
	
	public static Command MODEL(){
		return new Command(17, MUtils.setCheckSum(IP + READ + MODEL), "(R) MODEL " + SF);
	}
	
	public static Command SERIAL(){
		return new Command(18, MUtils.setCheckSum(IP + READ + SERIAL), "(R) SERIAL");
	}

	public static Command FREE_MEMORY(){
		return new Command(919, MUtils.setCheckSum(IP + READ + FREE_MEMORY), "(R) FREE MEMORY");
	}

	public static Command ERASE_MEMORY(){
		return new Command(20, MUtils.setCheckSum("011002000001020000"), "(W) ERASE MEMORY");
	}

	public static Command STOP(){
		return new Command(21, MUtils.setCheckSum(IP + WRITE + STOP), "(W) STOP");
	}

	private static long getCurrentTime(){
		long currentTime;
		if(TimeZone.getDefault().inDaylightTime(new Date())){
			currentTime = ((new Date().getTime() + TimeZone.getDefault().getRawOffset() + TimeZone.getDefault().getDSTSavings()) / 1000L);
		} else {
			currentTime = ((new Date().getTime() + TimeZone.getDefault().getRawOffset()) / 1000L);
		}
		return currentTime;
	}

	/**
	 * Clean up the HEX message receveid from the device. Removes the device IP, function code, (bytes quantity|address) and CRC.
	 *
	 * @param response HEX message received from equipment
	 */
	public static String cleanResponse(String response){
		response = response.replace(Command.SF,"");
		response = response.replace(Command.EF,"");
		//WRITE RESPONSE:  01 10 0082 0001 6C
		//READ  RESPONSE:  01 03 02 0066 94
		try{
			response = response.substring(2, response.length()-2); // Removes the IP and CRC
			int type = MUtils.hexToInt( response.substring(0,2) );

			if(type==3){ //Response of a read request
				response = response.substring(4); // Removes the Function code and Byte size
			} else { //Response of write request
				response = response.substring(6); // Removes the Function code and Memory address
			}
		} catch (Exception e){
			response = "";
		}
		return response;
	}
	
	public static class MD10KVx{
		private static final String IP              = "01";
		private static final String READ            = "03";
		private static final String WRITE           = "10"; //06 -> Single register | -> 10 Multiples registers
		private static final String ADDRESS_CONTROL = "2002000102"; //2002-ADDRESS | 0001-REGISTERS | 02-BYTES
		private static final String ADDRESS_TIMEOUT = "2007000102"; //2002-ADDRESS | 0001-REGISTERS | 02-BYTES
		private static final String ADDRESS_STATUS  = "0350"; //0350-ADDRESS  
		private static final String ADDRESS_SETUP   = "2003000306"; //2003-ADDRESS | 0003-REGISTERS | 06-BYTES
		private static final String SHORT_STATUS    = "0002"; //0002-REGISTERS
		//private static final String MIDDLE_STATUS   = "000A"; //000A-REGISTERS
		private static final String FULL_STATUS     = "0012"; //0012-REGISTERS
		//private static final String CAPACITANCE     = "035A0001"; //0001-REGISTERS
		private static final String PRINTER         = "0004"; //COMAND PRINTER ON/OFF
		private static final String FILTER          = "0008"; //COMAND FILTER ON/OFF
		private static final String STOP            = "0002"; //COMAND STOP TEST
		private static final String START           = "0001"; //COMAND START TEST
		private static final String BUZZER          = "0020"; //COMAND BUZZER
		private static final String TEST_NUMBER     = "20010001"; //TEST NUMBER ADDRESS		
		private static final String[] OPMODE        = new String[]{ "- - -" , "SVT1" , "SVT2" , "00:30" , "01:00" , "03:00" , "10:00" , "30:00" };
		private static final String[] LIM           = new String[]{ "- - -" , "10 M\u03A9" , "100 M\u03A9" , "1 G\u03A9" , "10 G\u03A9" };
		private static int SEL_VOLTAGE              = 0;
		private static int SEL_OPMODE               = 0;
		private static int SEL_LIM                  = 0;
		private static int SEL_FILTER               = 0;
		
		public static String OPMODE(int option){
			return OPMODE[option];
		}
		
		public static String LIM(int option){
			return LIM[option];
		}
		
		public static void SEL_VOLTAGE(int voltage){
			SEL_VOLTAGE = voltage;
		}
		
		public static int SEL_VOLTAGE(){
			return SEL_VOLTAGE;
		}
		
		public static void SEL_OPMODE(int opmode){
			SEL_OPMODE = opmode;
		}
		
		public static int SEL_OPMODE(){
			return SEL_OPMODE;
		}
		
		public static void SEL_LIM(int lim){
			SEL_LIM = lim;
		}
		
		public static int SEL_LIM(){
			return SEL_LIM;
		}
		
		public static void SEL_FILTER(int filter){
			SEL_FILTER = filter;
		}
		
		public static int SEL_FILTER(){
			return SEL_FILTER;
		}
		
		public static Command PRINTER(){
			return new Command(101, MUtils.setCheckSum(IP + WRITE + ADDRESS_CONTROL + PRINTER), "(W) PRINTER ON/OFF");
		}
		
		public static Command STOP(){
			return new Command(102, MUtils.setCheckSum(IP + WRITE + ADDRESS_CONTROL + STOP), "(W) STOP TEST");
		}
		
		public static Command START(){
			return new Command(103, MUtils.setCheckSum(IP + WRITE + ADDRESS_CONTROL + START), "(W) START TEST");
		}
		
		public static Command SHORT_STATUS(){
			return new Command(104, MUtils.setCheckSum(IP + READ + ADDRESS_STATUS + SHORT_STATUS), "(R) SHORT STATUS");
		}

		public static Command FULL_STATUS(){
			return new Command(106, MUtils.setCheckSum(IP + READ + ADDRESS_STATUS + FULL_STATUS), "(R) FULL STATUS");
		}
		
		public static Command TEST_NUMBER(){
			return new Command(107, MUtils.setCheckSum(IP + READ + TEST_NUMBER), "(R) TEST NUMBER");
		}
		
		public static Command TIMEOUT(int milliseconds){
			milliseconds = milliseconds / 2;
			String time = String.format("%04X", milliseconds);			
			return new Command(108, MUtils.setCheckSum(IP + WRITE + ADDRESS_TIMEOUT + time), "(W) SET EQUIPMENT TIMEOUT ("+milliseconds+")");
		}
		
		public static Command READ_SETUP(){
			return new Command(109, MUtils.setCheckSum(IP + READ + ADDRESS_SETUP), "(R) READ SETUP");
		}
		
		public static Command WRITE_SETUP(int voltage, int mode, int lim){
			String PARAMS = String.format("%04X", voltage) + String.format("%04X", mode) + String.format("%04X", lim);			
			return new Command(110, MUtils.setCheckSum(IP + WRITE + ADDRESS_SETUP + PARAMS), "(W) WRITE SETUP");
			//return new Command(9, MUtils.setCheckSum(IP + READ + ADDRESS_SETUP), "(R) READ SETUP");
		}
		
		public static Command BUZZER(){
			return new Command(111, MUtils.setCheckSum(IP + WRITE + ADDRESS_CONTROL + BUZZER), "(W) BUZZER");
		}
		
		public static Command BUZZER(int milliseconds){
			milliseconds = milliseconds / 100;			
			String time = String.format("%04X", milliseconds);
			return new Command(112, MUtils.setCheckSum(IP + WRITE + "2006000102" + time), "(W) SET BUZZER TIME");
		}
		
		public static Command SET_DATE(){
			final long tabletCurrentTime = getCurrentTime();
			String time = Long.toHexString(tabletCurrentTime);		
			return new Command(113, MUtils.setCheckSum(IP + WRITE + SET_DATE + time), "(W) DATE/TIME");
		}
		
		public static Command FILTER(){
			return new Command(114, MUtils.setCheckSum(IP + WRITE + ADDRESS_CONTROL + FILTER), "(W) FILTER ON/OFF");
		}
		
		public static Command CAPACITANCE(){
			return new Command(115, MUtils.setCheckSum(IP + READ + ADDRESS_STATUS + FULL_STATUS), "(R) CAPACITANCE");
		}
		
		public static Command RESET_EXTERNAL(){
			return new Command(116, MUtils.setCheckSum(IP + WRITE + "0350" + "0001020000"), "(R) RESET");
		}
	}

	public static class EM4058{
		private static final String IP              = "01";
		private static final String READ            = "03";
		private static final String WRITE           = "10"; //06 -> Single register | -> 10 Multiples registers
		private static final String ADDRESS_CONFIG  = "200000050A"; //2000-ADDRESS | 0001-REGISTERS | 02-BYTES
		private static final String ADDRESS_MODE    = "200000050A"; //2000-ADDRESS | 0001-REGISTERS | 02-BYTES
		private static final String ADDRESS_CONTROL = "2002000102"; //2002-ADDRESS | 0001-REGISTERS | 02-BYTES
		private static final String ADDRESS_TIMEOUT = "2007000102"; //2002-ADDRESS | 0001-REGISTERS | 02-BYTES
		private static final String ADDRESS_STATUS  = "0343"; //0350-ADDRESS
		private static final String ADDRESS_FULL_STATUS
				                                    = "0340"; //0350-ADDRESS
		private static final String ADDRESS_SETUP   = "2003000306"; //2003-ADDRESS | 0003-REGISTERS | 06-BYTES
		private static final String SHORT_STATUS    = "0003"; //0002-REGISTERS
		private static final String FULL_STATUS     = "0010"; //000D-REGISTERS
		private static final String PRINTER         = "0004"; //COMAND PRINTER ON/OFF
		private static final String FILTER          = "0008"; //COMAND FILTER ON/OFF
		private static final String STOP            = "0002"; //COMAND STOP TEST
		private static final String START           = "0001"; //COMAND START TEST
		private static final String BUZZER          = "0020"; //COMAND BUZZER
		private static final String TEST_NUMBER     = "20010001"; //TEST NUMBER ADDRESS
		public static final String MODE_EARTH_RESISTANCE
													= "0000";
		public static final String MODE_EARTH_RESISTANCE_SCAN
													= "0001";
		public static final String MODE_SOIL_RESISTIVITY
													= "0002";
		private static final String ADDRESS_FREQUENCY
													= "2003000102";
		private static final String[] Frequency     = new String[]{ "270 Hz" , "570 Hz" , "870 Hz" , "1170 Hz" , "1470 Hz", "SCAN" };

		public static String formatResistanceResolution(float value){
			DecimalFormatSymbols separator = new DecimalFormatSymbols(Locale.getDefault());
			separator.setDecimalSeparator('.');
			DecimalFormat twoDigits = new DecimalFormat("0.00", separator);
			DecimalFormat oneDigits = new DecimalFormat("#.0", separator);
			DecimalFormat zeroDigits = new DecimalFormat("#");
			twoDigits.setRoundingMode(RoundingMode.DOWN);
			oneDigits.setRoundingMode(RoundingMode.DOWN);
			zeroDigits.setRoundingMode(RoundingMode.DOWN);

			String result;
			if(value>=20000){
				result = "> 20k\u03a9";
			}
			else if(value>2000){
				result = twoDigits.format(value/1000) + " k\u03a9";
			}
			else if(value>200){
				result = zeroDigits.format(value) + " \u03a9";
			}
			else if(value>20){
				result = oneDigits.format(value) + " \u03a9";
			}
			else{
				result = twoDigits.format(value) + " \u03a9";
			}
			return result;
		}

		public static String formatResistivityResolution(float value){
			DecimalFormatSymbols separator = new DecimalFormatSymbols(Locale.getDefault());
			separator.setDecimalSeparator('.');
			DecimalFormat twoDigits = new DecimalFormat("0.00", separator);
			DecimalFormat oneDigits = new DecimalFormat("#.0", separator);
			DecimalFormat zeroDigits = new DecimalFormat("#");
			twoDigits.setRoundingMode(RoundingMode.DOWN);
			oneDigits.setRoundingMode(RoundingMode.DOWN);
			zeroDigits.setRoundingMode(RoundingMode.DOWN);

			String result;
			if(value>=50000){
				result = "> 50k\u03a9.m";
			}
			else if(value>2000){
				result = twoDigits.format(value/1000) + " k\u03a9.m";
			}
			else if(value>200){
				result = zeroDigits.format(value) + " \u03a9.m";
			}
			else if(value>20){
				result = oneDigits.format(value) + " \u03a9.m";
			}
			else{
				result = twoDigits.format(value) + " \u03a9.m";
			}
			return result;
		}

		public static String[] getFrequencyList(){
			return Frequency;
		}

		public static String getFrequencyValue(int index){
			if(index>=Frequency.length){
				return "--- Hz";
			}
			return Frequency[index];
		}

		public static Command PRINTER(){
			return new Command(201, MUtils.setCheckSum(IP + WRITE + ADDRESS_CONTROL + PRINTER), "(W) PRINTER ON/OFF");
		}

		public static Command STOP(){
			return new Command(202, MUtils.setCheckSum(IP + WRITE + ADDRESS_CONTROL + STOP), "(W) STOP TEST");
		}

		public static Command START()
		{
			//return new Command( 203, MUtils.setCheckSum( IP + WRITE + ADDRESS_CONTROL + START ), "(W) START TEST" );
			return new Command( 203, MUtils.setCheckSum( IP + "062002" + START ), "(W) START TEST" );
		}

		public static Command SHORT_STATUS(){
			return new Command(204, MUtils.setCheckSum(IP + READ + ADDRESS_STATUS + SHORT_STATUS), "(R) SHORT STATUS");
		}

		public static Command GET_OPERATION_MODE(){
			return new Command(205, MUtils.setCheckSum(IP + READ + ADDRESS_CONFIG), "(R) OPERATION MODE");
		}

		public static Command FULL_STATUS(){
			return new Command(206, MUtils.setCheckSum(IP + READ + ADDRESS_FULL_STATUS + FULL_STATUS), "(R) FULL STATUS");
		}

		public static Command TEST_NUMBER(){
			return new Command(207, MUtils.setCheckSum(IP + READ + TEST_NUMBER), "(R) TEST NUMBER");
		}

		public static Command TIMEOUT(int milliseconds){
			milliseconds = milliseconds / 2;
			String time = String.format("%04X", milliseconds);
			return new Command(208, MUtils.setCheckSum(IP + WRITE + ADDRESS_TIMEOUT + time), "(W) SET EQUIPMENT TIMEOUT ("+milliseconds+")");
		}

		public static Command READ_SETUP(){
			return new Command(209, MUtils.setCheckSum(IP + READ + ADDRESS_SETUP), "(R) READ SETUP");
		}

		public static Command WRITE_MODE(String MODE){
			return new Command(210, MUtils.setCheckSum(IP + WRITE + ADDRESS_MODE + MODE), "(W) WRITE SETUP");
		}

		public static Command WRITE_MODE(String MODE, int selectedFrequency){
			String TEST_NUMBER    = "0000";
			String REMOTE_CONTROL = "0000";
			String ROD_DISTANCE   = "0000";
			String FREQUENCY      = String.format("%04X", selectedFrequency);

			//return new Command(210, MUtils.setCheckSum(IP + WRITE + ADDRESS_MODE + MODE + TEST_NUMBER + REMOTE_CONTROL + FREQUENCY + ROD_DISTANCE), "(W) WRITE SETUP");
			return new Command(210, MUtils.setCheckSum(IP + WRITE + "2000000408" + MODE + TEST_NUMBER + REMOTE_CONTROL + FREQUENCY), "(W) WRITE SETUP");
		}

		public static Command WRITE_MODE(String MODE, String distance){
			String TEST_NUMBER    = "0000";
			String REMOTE_CONTROL = "0000";
			String FREQUENCY      = "0000";
			String ROD_DISTANCE;

			if(distance.contains("m")){
				int value = Integer.parseInt(distance.replace(" m",""));
				ROD_DISTANCE = String.format("%04X", value);
			}
			else{
				ROD_DISTANCE = "0000";
			}

			return new Command(210, MUtils.setCheckSum(IP + WRITE + ADDRESS_MODE + MODE + TEST_NUMBER + REMOTE_CONTROL + FREQUENCY + ROD_DISTANCE), "(W) WRITE SETUP");
		}

//		public static Command WRITE_SETUP(int voltage, int mode, int lim){
//			String PARAMS = String.format("%04X", voltage) + String.format("%04X", mode) + String.format("%04X", lim);
//			return new Command(210, MUtils.setCheckSum(IP + WRITE + ADDRESS_SETUP + PARAMS), "(W) WRITE SETUP");
//			//return new Command(9, MUtils.setCheckSum(IP + READ + ADDRESS_SETUP), "(R) READ SETUP");
//		}

		public static Command BUZZER(){
			return new Command(211, MUtils.setCheckSum(IP + WRITE + ADDRESS_CONTROL + BUZZER), "(W) BUZZER");
		}

		public static Command BUZZER(int milliseconds){
			milliseconds = milliseconds / 100;
			String time = String.format("%04X", milliseconds);
			return new Command(212, MUtils.setCheckSum(IP + WRITE + "2006000102" + time), "(W) SET BUZZER TIME");
		}

		public static Command SET_DATE(){
			final long tabletCurrentTime = getCurrentTime();
			String time = Long.toHexString(tabletCurrentTime);
			return new Command(213, MUtils.setCheckSum(IP + WRITE + SET_DATE + time), "(W) DATE/TIME");
		}

		public static Command FILTER(){
			return new Command(214, MUtils.setCheckSum(IP + WRITE + ADDRESS_CONTROL + FILTER), "(W) FILTER ON/OFF");
		}

		public static Command CAPACITANCE(){
			return new Command(215, MUtils.setCheckSum(IP + READ + ADDRESS_STATUS + FULL_STATUS), "(R) CAPACITANCE");
		}

		public static Command RESET_EXTERNAL(){
			return new Command(216, MUtils.setCheckSum(IP + WRITE + "0343" + "0001020000"), "(R) RESET");
		}

		public static Command FREQUENCY(){
			return new Command(217, MUtils.setCheckSum(IP + READ + ADDRESS_FREQUENCY), "(R) FREQUENCY");
		}
	}


}
