package com.megabras.bluelogg.DT70KV;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Color;
import android.graphics.Paint;
import android.os.Bundle;
import android.util.DisplayMetrics;
import android.util.TypedValue;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.ListView;

import com.megabras.bluelogg.R;
import com.megabras.bluelogg.extended.Units;
import com.megabras.modbus.DT70KV.Measurement;

import org.achartengine.ChartFactory;
import org.achartengine.GraphicalView;
import org.achartengine.model.XYMultipleSeriesDataset;
import org.achartengine.model.XYSeries;
import org.achartengine.renderer.XYMultipleSeriesRenderer;
import org.achartengine.renderer.XYSeriesRenderer;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;


public class Charts extends Fragment
{

    public static final int
            VOLTAGE_CHART    = 0,
            CURRENT_CHART    = 1,
            TABLE            = 3;

    protected int
            CHART_TYPE       = -1;

    private XYSeries
            mXYSeries;

    private GraphicalView
            mChart;

    protected double
            minYdefault,
            maxYdefault;

    private long
            POWER            = 0,
            CHART_CORRECTION = 0;

    private XYMultipleSeriesRenderer
            mRenderer;

    private LinearLayout
            layout;

    private ListView
            list;

    private TableBaseAdapter
            adapter;

    protected String
            chartTitle = "";

    private Context
            context;

    protected com.megabras.modbus.DT70KV.FormatData
            formatData;

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState)
    {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.set(Calendar.MILLISECOND, 0);
        calendar.set(Calendar.SECOND     , 0);
        calendar.set(Calendar.MINUTE     , 0);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        CHART_CORRECTION = calendar.getTimeInMillis();
        context          = inflater.getContext();
        return inflater.inflate(R.layout.md10kvx_fgr_online, container, false);
    }

    @Override
    public void onStart()
    {
        super.onStart();
        createChart();
    }

    public void setChartType(int type, String chartTitle)
    {
        CHART_TYPE      = type;
        this.chartTitle = chartTitle;
        formatData      = new com.megabras.modbus.DT70KV.FormatData();
    }

    public int getChartType()
    {
        return CHART_TYPE;
    }

    private void createChart()
    {
        POWER = 0;

        switch (CHART_TYPE)
        {
            case Charts.VOLTAGE_CHART:
                Chart(chartTitle,"V");
                break;
            case Charts.CURRENT_CHART:
                Chart(chartTitle,"A");
                break;
            case Charts.TABLE:
                Table();
                break;
        }
    }

    public Bitmap getChartBitmap()
    {
        if (mRenderer==null || mChart==null)
            return null;

        int gridColor   = mRenderer.getGridColor   (0);
        int labelColor  = mRenderer.getLabelsColor ();
        int yLabelColor = mRenderer.getYLabelsColor(0);
        int xLabelColor = mRenderer.getXLabelsColor();
        int axesColor   = mRenderer.getAxesColor   ();

        mRenderer.setGridColor   (Color.argb(150, 0, 0, 0));
        mRenderer.setLabelsColor (Color.BLACK);
        mRenderer.setYLabelsColor(0, Color.BLACK);
        mRenderer.setXLabelsColor(Color.BLACK);
        mRenderer.setAxesColor   (Color.BLACK);

        mChart.setDrawingCacheEnabled(true);
        Bitmap bitmap = mChart.getDrawingCache();

        mRenderer.setGridColor   (gridColor);
        mRenderer.setLabelsColor (labelColor);
        mRenderer.setYLabelsColor(0, yLabelColor);
        mRenderer.setXLabelsColor(xLabelColor);
        mRenderer.setAxesColor   (axesColor);
        return bitmap;
    }

    public void Table()
    {
        if( this.getView()==null )
            return;

        layout = this.getView().findViewById(R.id.chart);

        if( layout.getChildCount()>1 )
            return;

        list = new ListView( layout.getContext() );
        list.setDividerHeight(0);
        layout.addView(list);

        adapter = new TableBaseAdapter( layout.getContext() );
        list.setAdapter( adapter );
        list.setVisibility(View.VISIBLE);
    }

    public void setGloveClass(String gloveClass)
    {
        if ( adapter != null )
            adapter.setGloveClass(gloveClass);
    }

    public void setGloveSerial(String gloveSerial)
    {
        if ( adapter != null )
        {
            adapter.setGloveSerial(gloveSerial);
            if ( adapter.getCount() == 0 )
                adapter.add( context.getString( R.string.MMSS), context.getString( R.string.voltage ).toUpperCase(), context.getString( R.string.current ).toUpperCase() );
            adapter.repaint();
            showTable(true);
        }
    }

    public void setTestNumber(int testNumber)
    {
        if (adapter!=null)
        {
            adapter.setTestNumber(testNumber);
            if ( adapter.getCount() == 0 )
                adapter.add( context.getString( R.string.MMSS), context.getString( R.string.voltage ).toUpperCase(), context.getString( R.string.current ).toUpperCase() );
            adapter.repaint();
        }
        showTable(true);
    }

    public void setDate(String date)
    {
        if (adapter!=null)
        {
            adapter.setDate(date);
            adapter.repaint();
        }
    }

    public void showTable(boolean show)
    {
        if ( list!=null )
        {
            if(show)
                list.setVisibility(View.VISIBLE);
            else
                list.setVisibility(View.GONE);
        }
    }

    public void Chart(String chartTitle, String yTitle)
    {
        if(this.getView()==null)
            return;

        layout = this.getView().findViewById(R.id.chart);

        if(layout.getChildCount()>0)
            return;

        XYMultipleSeriesDataset mDataset = new XYMultipleSeriesDataset();
        mRenderer = new XYMultipleSeriesRenderer();
        XYSeriesRenderer mXYSeriesRenderer;
        mXYSeries = new XYSeries("Chart");
        mDataset.addSeries(mXYSeries);
        mXYSeriesRenderer = new XYSeriesRenderer();
        mXYSeriesRenderer.setColor(Color.RED);
        mXYSeriesRenderer.setFillPoints(true);
        mXYSeriesRenderer.setLineWidth(2);

        minYdefault = mRenderer.getYAxisMin();
        maxYdefault = mRenderer.getYAxisMax();

        DisplayMetrics metrics = context.getResources().getDisplayMetrics();

        mRenderer.setLabelsTextSize(
                TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, context.getResources().getDimension(R.dimen.chart_label), metrics)
        );

        mRenderer.setChartTitleTextSize(
                TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, context.getResources().getDimension(R.dimen.chart_title), metrics)
        );

        mRenderer.setAxisTitleTextSize(
                TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, context.getResources().getDimension(R.dimen.chart_title), metrics)
        );

        mRenderer.addSeriesRenderer(mXYSeriesRenderer);
        mRenderer.setAntialiasing(false);
        mRenderer.setShowLegend(false);
        mRenderer.setYTitle(yTitle);
        mRenderer.setYLabelsAlign(Paint.Align.RIGHT);
        mRenderer.setXTitle("h:mm:ss");
        mRenderer.setChartTitle(chartTitle);
        mRenderer.setApplyBackgroundColor(true);
        mRenderer.setBackgroundColor(Color.TRANSPARENT);
        mRenderer.setMarginsColor(Color.argb(0x00, 0x01, 0x01, 0x01));
        mRenderer.setAxesColor(Color.TRANSPARENT);
        mRenderer.setShowGrid(true);
        mRenderer.setGridColor(Color.argb(60, 255, 255, 255));
        mRenderer.setYLabels(6);
        mRenderer.setXLabels(8);
        mRenderer.setXAxisMin(CHART_CORRECTION);
        mRenderer.setMargins(new int[] { 25, 65, 0, 10 }); //top - left - bottom - right
        mRenderer.setPanEnabled(false, false);
        mRenderer.setZoomEnabled(false, false);

        if (this.getActivity()!=null)
        {
            mChart = ChartFactory.getTimeChartView(
                    this.getActivity().getApplicationContext(),
                    mDataset,
                    mRenderer,
                    "H:mm:ss");

            layout.addView(mChart);
        }
    }

    private double resolution(double value, String unit)
    {
        double r;
        if( POWER == 0 && mRenderer != null )
        {
            if ( value >= Units.TERA)
            {
                r = value / Units.TERA;
                mRenderer.setYTitle("T"+unit);
                POWER = Units.TERA;
            }
            else if(value>=Units.GIGA)
            {
                r = value / Units.GIGA;
                mRenderer.setYTitle("G"+unit);
                POWER=Units.GIGA;
            }
            else if(value>=Units.MEGA){
                r = value / Units.MEGA;
                mRenderer.setYTitle("M"+unit);
                POWER=Units.MEGA;
            }
            else if(value>Units.KILO){
                r = value / Units.KILO;
                mRenderer.setYTitle("k"+unit);
                POWER=Units.KILO;
            }
            else if (value >= 1) {
                r = value;
                mRenderer.setYTitle(unit);
                POWER=1;
            }
            else if(value * Units.MILI>=1){
                r = (value * Units.MILI);
                mRenderer.setYTitle("m"+unit);
                POWER = Units.MILI;
            }
            else if(value * Units.MICRO>=1){
                r = (value * Units.MICRO);
                mRenderer.setYTitle("\u00B5"+unit);
                POWER = Units.MICRO;
            }
            else if (value * Units.NANO>=1) {
                r = (value * Units.NANO);
                mRenderer.setYTitle("n"+unit);
                POWER = Units.NANO;
            }
            else if (value * Units.PICO>=1) {
                r = (value * Units.PICO);
                mRenderer.setYTitle("p"+unit);
                POWER = Units.PICO;
            }
            else {
                r = 0;
            }
        }
        else {
            if(value >= 1){
                r = (value / POWER);
            } else {
                r = (value * POWER);
            }
        }
        return r;
    }

    private void updateScaleY()
    {
        double diff     = mXYSeries.getMaxY() - mXYSeries.getMinY();
        double perc     = (diff * 100) / mXYSeries.getMaxY();
        perc            = 100 - perc;
        perc            = perc / 100;
        double toAddMax = (mXYSeries.getMaxY() * perc) / 2;
        double toAddMin = (mXYSeries.getMaxY() * perc) / 2;
        double lowY     =  mXYSeries.getMinY() - toAddMin;
        double upY      =  mXYSeries.getMaxY() + toAddMax;

        if(lowY < 0)
            lowY = 0;

        if ( lowY != Double.POSITIVE_INFINITY && !Double.isNaN(lowY) )
        {
            mRenderer.setYAxisMin(lowY);
        }

        double marginY = mXYSeries.getMaxY() * 0.1;

        mRenderer.setYAxisMax( mXYSeries.getMaxY() + marginY );
    }

    public void setResult(int result)
    {
        adapter.setResult(result);
    }

    public void update(Measurement mea, boolean repaint)
    {
        long   millis = mea.getElapsedTotal() * 1000;
        double volts  = mea.getVolt();
        double amps   = mea.getAmpRT();

        if( mXYSeries != null && mXYSeries.getItemCount() > 0 )
        {
            int index = mXYSeries.getItemCount() - 1;
            if ( mXYSeries.getX(index) == millis + CHART_CORRECTION)
            {
                mXYSeries.remove(index);
            }
        }

        if( mChart != null && mChart.getVisibility() != View.VISIBLE && mRenderer != null )
        {
            mChart.setVisibility(View.VISIBLE);
            mRenderer.setAxesColor(Color.WHITE);
        }

        switch (CHART_TYPE)
        {
            case Charts.VOLTAGE_CHART:
                if ( mXYSeries != null )
                {
                    if ( volts < 10 ) volts = 0d;
                    mXYSeries.add(millis + CHART_CORRECTION, resolution(volts, "V"));
                    updateScaleY();
                    if ( mChart != null && repaint) mChart.repaint();
                }
                break;
            case Charts.CURRENT_CHART:
                if ( mXYSeries != null )
                {
                    if ( amps < 1E-6 ) amps = 0d;
                    mXYSeries.add(millis + CHART_CORRECTION, resolution(amps, "A"));
                    updateScaleY();
                    if ( mChart != null && repaint) mChart.repaint();
                }
                break;
            case Charts.TABLE:
                String sMillis  = formatData.elapsed ( mea.getElapsedTotal() );
                String sVolts   = formatData.volts   ( volts                 );
                String sCurrent = formatData.amps    ( amps                  );

                if ( adapter.getCount() == 0 )
                    adapter.add( context.getString( R.string.MMSS), context.getString( R.string.voltage ).toUpperCase(), context.getString( R.string.current ).toUpperCase() );

                if ( adapter.getCount() > 0 )
                {
                    int index = adapter.getCount()-1;
                    if ( adapter.getItem(index).getTime().equals( sMillis ) )
                        adapter.remove(index);
                }

                adapter.add
                (
                        sMillis,
                        sVolts,
                        sCurrent
                );

                if ( repaint ) adapter.repaint();
                break;
        }
    }

    public void repaint()
    {
        switch (CHART_TYPE)
        {
            case Charts.VOLTAGE_CHART:
                mChart.repaint();
                break;
            case Charts.CURRENT_CHART:
                mChart.repaint();
                break;
            case Charts.TABLE:
                adapter.repaint();
                break;
        }
    }

    public void setAllValues(ArrayList<Measurement> measurements)
    {
        if ( measurements.size() <= 0 )
            return;

        switch (CHART_TYPE)
        {
            case Charts.VOLTAGE_CHART:
            case Charts.CURRENT_CHART:
                mXYSeries.clear();
                for ( Measurement m : measurements )
                {
                    update(m, false);
                }
                mChart.repaint();
                break;
            case Charts.TABLE:
                adapter.clear();
                for ( Measurement m : measurements )
                {
                    update(m, false);
                }
                adapter.repaint();
                break;
        }
    }

    public void reset()
    {
        try
        {
            if ( layout == null || adapter == null )
                createChart();

            if( CHART_TYPE == Charts.TABLE )
            {
                adapter.clear();
                adapter.repaint();
                layout.removeAllViews();
                layout.addView(list);
                showTable(false);
            }
            else
            {
                mXYSeries.clear();
                mChart.repaint();
                mChart.setVisibility(View.GONE);
                POWER = 0;
                mRenderer.setYAxisMin(minYdefault);
                mRenderer.setYAxisMax(maxYdefault);
            }
        }
        catch (Exception e)
        {
            android.util.Log.e("ERROR", e.getMessage()+"");
        }
    }
}
