package com.megabras.bluelogg.MDXXKVR;

import android.annotation.SuppressLint;
import android.app.DatePickerDialog;
import android.app.Dialog;
import android.app.TimePickerDialog;
import android.graphics.Color;
import android.graphics.drawable.Drawable;
import android.os.Handler;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.DatePicker;
import android.widget.FrameLayout;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.Spinner;
import android.widget.Switch;
import android.widget.TextView;
import android.widget.TimePicker;

import com.megabras.DeviceInfo;
import com.megabras.bluelogg.extended.CommonUI;
import com.megabras.bluelogg.extended.FileDetails;
import com.megabras.bluelogg.MUtils;
import com.megabras.bluelogg.Main;
import com.megabras.bluelogg.R;
import com.megabras.bluelogg.bluetooth.BluetoothDev;
import com.megabras.bluelogg.extended.Audio;
import com.megabras.bluelogg.extended.Dialogs;
import com.megabras.bluelogg.extended.DialogsInterface;
import com.megabras.bluelogg.extended.GPS;
import com.megabras.bluelogg.extended.GoogleMaps;
import com.megabras.bluelogg.extended.LogCat;
import com.megabras.bluelogg.extended.MapData;
import com.megabras.bluelogg.extended.Picture;
import com.megabras.bluelogg.extended.SlidingTabLayout;
import com.megabras.modbus.MDXXKVR.Device;
import com.megabras.modbus.MDXXKVR.Device.Changed;
import com.megabras.modbus.common.DielectricDischarge;
import com.megabras.modbus.MDXXKVR.Measure;
import com.megabras.modbus.MDXXKVR.ParamManual;
import com.megabras.modbus.MDXXKVR.ParamRamp;
import com.megabras.modbus.MDXXKVR.ParamSVT;
import com.megabras.modbus.MDXXKVR.ParamTimer;
import com.megabras.modbus.MDXXKVR.TestData;
import com.megabras.modbus.Message;
import com.megabras.modbus.Modbus;
import com.megabras.modbus.NumberParser;

import java.io.File;
import java.lang.ref.WeakReference;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import androidx.annotation.NonNull;
import androidx.core.content.res.ResourcesCompat;
import androidx.fragment.app.FragmentActivity;
import androidx.viewpager.widget.ViewPager;

public class Controller extends com.megabras.bluelogg.Controller implements GPS.GpsInterface, Picture.PictureInterface, com.megabras.CommInterface, Observer, DialogsInterface
{

    /***********************************************************************************************
     * REGION Fields
     ***********************************************************************************************/

    @SuppressWarnings({"FieldCanBeLocal", "unused"})
    private final String
            TAG = "MDXXKVR.Controller";

    private FragmentActivity
            fragmentActivity;

    private FrameLayout
            OVERLAY;

    private CommonUI
            mCommonUI;

    private Picture
            picture;

    private GPS
            mGPS;

    private GoogleMaps
            map;

    private final Database
            database;

    private TestData
            openedTestData;

    private Audio
            audio;

    private ReportPDF
            report;

    private Device
            device;

    private String
            BOX_MESSAGE = "";

    private int
            BOX_COLOR = 0;

    private boolean
            BOX_BUSY = false;

    private ViewPager
            VIEW_PAGER;

    private PagerAdapter
            pagerAdapter;

    private View
            VIEW_SETUP,
            VIEW_GENERAL,
            VIEW_FILES;

    private MainFragment
            ui_main_screen;

    private ChartFragment
            ui_chart_ohms,
            ui_chart_volts,
            ui_chart_amps,
            ui_chart_idd,
            ui_table;

    private final Dialogs
            dlgs;

    private final int
            DLG_CONF_VOLTS = 0;

    private LinearLayout
            ui_lyt_ramp_lim,
            ui_lyt_ramp,
            ui_lyt_svt_lim,
            ui_lyt_svt,
            ui_lyt_timer_lim,
            ui_lyt_timer,
            ui_lyt_manual_lim,
            ui_lyt_manual;

    private ListView
            ui_liw_files;

    private MenuItem
            ui_menu_item_options;

    private Button
            ui_btn_start,
            ui_btn_stop,
            ui_btn_voltage,
            ui_btn_file_close,
            ui_btn_file_delete,
            ui_btn_file_open,
            ui_btn_file_export;

    private ImageButton
            ui_ibtn_printer,
            ui_ibtn_filter,
            ui_ibtn_test_setup,
            ui_ibtn_save,
            ui_ibtn_map,
            ui_ibtn_file_manager,
            ui_ibtn_picture,
            ui_ibtn_gps,
            ui_ibtn_voice;

    private TextView
            ui_general_txw_free_memory,
            ui_general_txw_date,
            ui_general_txw_time,
            ui_txw_ramp_time,
            ui_txw_ramp_lim,
            ui_txw_ramp_max_volts,
            ui_txw_svt_step_time,
            ui_txw_svt_step_volts,
            ui_txw_svt_lim,
            ui_txw_svt_max_volts,
            ui_txw_timer_dai_b,
            ui_txw_timer_dai_a,
            ui_txw_timer_pi_b,
            ui_txw_timer_pi_a,
            ui_txw_timer_time,
            ui_txw_timer_lim,
            ui_txw_timer_max_volts,
            ui_txw_manual_dai_b,
            ui_txw_manual_dai_a,
            ui_txw_manual_pi_b,
            ui_txw_manual_pi_a,
            ui_txw_manual_lim,
            ui_txw_manual_max_volts;

    private Spinner
            ui_general_spn_auto_connection,
            ui_general_spn_date_format,
            ui_general_spn_time_format,
            ui_general_spn_language,
            ui_spn_mode;

    private Switch
            ui_swt_manual_lim,
            ui_swt_timer_lim,
            ui_swt_svt_lim,
            ui_swt_ramp_lim,
            ui_swt_cap,
            ui_swt_dd;

    /***********************************************************************************************
     * REGION Constructor
     ***********************************************************************************************/

    public Controller( FragmentActivity fragmentActivity, CommonUI ui, String model, String serial )
    {
        this.fragmentActivity = fragmentActivity;
        dlgs         = new Dialogs( fragmentActivity, this );
        OVERLAY      = MUtils.getOverlayInstance( fragmentActivity, OVERLAY_ID );
        VIEW_GENERAL = View.inflate( fragmentActivity, R.layout.main_fgr_general_setup, null );
        VIEW_SETUP   = View.inflate( fragmentActivity, R.layout.mdxxkvr_fgr_test_setup, null );
        VIEW_FILES   = View.inflate( fragmentActivity, R.layout.mpkxx5r_files_manager , null );
        mCommonUI    = ui;
        picture      = Picture.getInstance();
        picture.initialize( fragmentActivity, this );
        mGPS         = new GPS( this, fragmentActivity );
        audio        = new Audio( fragmentActivity );
        map          = new GoogleMaps( fragmentActivity );
        database     = new Database( fragmentActivity, model );

        DeviceInfo deviceInfo = new DeviceInfo( model );
        deviceInfo.setSerial( serial );

        device       = new Device( deviceInfo,this );
        report       = new ReportPDF( fragmentActivity, database.getFolder(), database.getTemporaryFolder(), new mHandler( this ) );

        createPageFilesManager();
        createPageGeneralSetup();

        LogCat.LEVEL = LogCat.LEVEL_ALL;
    }

    private void createPageFilesManager()
    {
        ui_liw_files         = getLV( R.id.list_savedfiles, VIEW_FILES );
        ui_btn_file_close    = getBTN( R.id.bt_close      , VIEW_FILES );
        ui_btn_file_delete   = getBTN( R.id.bt_delete     , VIEW_FILES );
        ui_btn_file_open     = getBTN( R.id.bt_open       , VIEW_FILES );
        ui_btn_file_export   = getBTN( R.id.bt_export     , VIEW_FILES );
        Button ui_btn_cancel = getBTN( R.id.bt_cancel     , VIEW_FILES );

        ui_btn_cancel.setOnClickListener( v -> closeOverlay() );

        ui_liw_files.setAdapter( new TableFilesAdapter( fragmentActivity ) );
        ui_liw_files.setChoiceMode( ListView.CHOICE_MODE_SINGLE );

        ui_liw_files.setOnItemClickListener(  ( adapterView, view, i, l ) ->
        {
            ui_btn_file_delete.setEnabled( true );
            ui_btn_file_open.setEnabled( true );
            ui_btn_file_export.setEnabled( true );

            int selection = ui_liw_files.getCheckedItemPosition();
            TableFilesAdapter adapter = (TableFilesAdapter) ui_liw_files.getAdapter();
//            ui_btn_file_export.setEnabled( adapter.isOpended( selection ) );
            ui_btn_file_close.setEnabled( adapter.isOpended( selection ) );
            ui_btn_file_open.setEnabled( !adapter.isOpended( selection ) );
        } );

        ui_btn_file_open.setOnClickListener( v ->
        {
            int selection = ui_liw_files.getCheckedItemPosition();
            if ( selection > -1 )
            {
                TableFilesAdapter adapter = (TableFilesAdapter) ui_liw_files.getAdapter();
                openedTestData = database.getTestData( adapter.getItem( selection ).FILENAME );
                if ( openedTestData != null )
                {
                    ui_btn_file_export.setEnabled( true );
                    ui_ibtn_voice.setEnabled( true );
                    ui_ibtn_picture.setEnabled( true );
                    ui_ibtn_gps.setEnabled( true );
                    ui_ibtn_map.setEnabled( true );
                    ui_ibtn_save.setEnabled( false );

                    loadTestDataOnScreen( openedTestData );

                    if ( VIEW_PAGER != null && VIEW_PAGER.getCurrentItem() == 0 )
                    {
                        VIEW_PAGER.setCurrentItem( 1 );
                    }
                    closeOverlay();
                }
            }
        } );

        ui_btn_file_close.setOnClickListener( v ->
        {
            openedTestData = null;
            ui_chart_ohms.reset();
            ui_chart_volts.reset();
            ui_chart_amps.reset();
            ui_chart_idd.reset();
            ui_table.reset();
            ui_ibtn_voice.setEnabled( false );
            ui_ibtn_picture.setEnabled( false );
            ui_ibtn_gps.setEnabled( false );
            ui_ibtn_map.setEnabled( false );
            loadFilesManagerPage();
        } );

        ui_btn_file_export.setOnClickListener( v ->
        {
            if ( Main.permissionToMemoryAccepted )
            {
                export();
            }
            else
            {
                ((Main) fragmentActivity).requestPermission( Main.REQUEST_MEMORY_REPORT_PERMISSION );
            }
        } );

        ui_btn_file_delete.setOnClickListener( v ->
        {
            int selection = ui_liw_files.getCheckedItemPosition();
            if ( selection > -1 )
            {
                final Dialog dialog = new Dialog( fragmentActivity );
                dialog.requestWindowFeature( Window.FEATURE_NO_TITLE );
                dialog.setCanceledOnTouchOutside( false );
                dialog.setContentView( R.layout.dialog_yes_no );
                if ( dialog.getWindow() != null )
                {
                    dialog.getWindow().getAttributes().width = ViewPager.LayoutParams.MATCH_PARENT;
                }
                ((TextView) dialog.findViewById( R.id.dlg_title )).setText( R.string.warning );
                ((ImageView) dialog.findViewById( R.id.dlg_icon )).setImageResource( R.drawable.ic_warning );

                final TextView message = dialog.findViewById(R.id.message);
                message.setText(R.string.file_delete_all_confirm);
                dialog.findViewById(R.id.btn_no).setOnClickListener(v1 -> dialog.dismiss());
                dialog.findViewById(R.id.btn_yes).setOnClickListener(v12 ->
                {
                    TableFilesAdapter adapter = (TableFilesAdapter) ui_liw_files.getAdapter();
                    String tableName = adapter.getItem( selection ).FILENAME;
                    String sNum      = adapter.getItem( selection ).INDEX;
                    int iNum         = NumberParser.parseInt( sNum );
                    database.deleteTable( tableName );
                    if ( openedTestData != null && openedTestData.getTestNumber() == iNum )
                    {
                        openedTestData = null;
                        ui_chart_ohms.reset();
                        ui_chart_volts.reset();
                        ui_chart_amps.reset();
                        ui_chart_idd.reset();
                        ui_table.reset();
                    }
                    loadFilesManagerPage();
                    dialog.dismiss();
                });

                dialog.show();
            }
        } );
    }

    private void loadFilesManagerPage()
    {
        TableFilesAdapter adapter = (TableFilesAdapter) ui_liw_files.getAdapter();
        adapter.add( database.getSavedFilesList( device ) );

        ui_btn_file_open.setEnabled( false );

        if ( openedTestData != null )
        {
            int selection = adapter.setOpenedRecordId( openedTestData.getTestNumber() + "" );

            ui_liw_files.setItemChecked( selection, true );
            if ( selection > -1 )
            {
                ui_btn_file_delete.setEnabled( true );
                ui_btn_file_close.setEnabled( true );
                ui_btn_file_export.setEnabled( true );
            }
            else
            {
//                ui_btn_file_open.setEnabled( false );
                ui_btn_file_delete.setEnabled( false );
                ui_btn_file_close.setEnabled( false );
                ui_btn_file_export.setEnabled( false );
            }
        }
        else
        {
            adapter.setOpenedRecordId( "" );
            ui_liw_files.setItemChecked( -1, true );
            ui_btn_file_delete.setEnabled( false );
//            ui_btn_file_open.setEnabled( false );
            ui_btn_file_close.setEnabled( false );
            ui_btn_file_export.setEnabled( false );
        }
        adapter.repaint();
        ui_liw_files.invalidateViews();
    }

    private void loadTestDataOnScreen( TestData testData )
    {
        ui_chart_ohms.reset();
        ui_chart_volts.reset();
        ui_chart_amps.reset();
        ui_chart_idd.reset();
        ui_table.reset();

        if ( testData == null )
        {
            return;
        }

        //Load Saved Data
        ui_chart_ohms.setAllValues( testData.getMeasure() );
        ui_chart_volts.setAllValues( testData.getMeasure() );
        ui_chart_amps.setAllValues( testData.getMeasure() );
        ui_chart_idd.setAllIDD( testData.getDielectricDischarge() );
        ui_table.setAllValues( testData.getMeasure() );
        ui_table.getHeader().setTestNumber( testData.getTestNumber() );
        String[] modes = fragmentActivity.getResources().getStringArray( R.array.op_modes );
        if ( testData.getMode() < modes.length )
        {
            ui_table.getHeader().setMode( modes[testData.getMode()] );
        }
        ui_table.getHeader().setDate( testData.getDateAsString( device.getDateTimeFormatPattern() ) );
        ui_table.getHeader().setCAP( testData.getCapacitanceAsString() );
        ui_table.getHeader().setDDEnabled( testData.getDD() >= 0 );
        ui_table.getHeader().setDD( testData.getDDString() );
        ui_table.getHeader().setDaiPiEnabled( testData.getMode() == device.MODE_MANUAL || testData.getMode() == device.MODE_TIMER );
        ui_table.getHeader().setSVTEnabled( testData.getMode() == device.MODE_SVT );
        ui_table.getHeader().setSVT( testData.getSVTString() );
        if ( testData.getDAI() != null )
        {
            ui_table.getHeader().setTitleDAI( testData.getDAI().getTimeA() + "/" + testData.getDAI().getTimeB() );
            ui_table.getHeader().setDAI( testData.getDAI().getResultAsString() );
        }
        if ( testData.getPI() != null )
        {
            ui_table.getHeader().setTitlePI( testData.getPI().getTimeA() + "/" + testData.getPI().getTimeB() );
            ui_table.getHeader().setPI( testData.getPI().getResultAsString() );
        }
    }

    private void createPageGeneralSetup()
    {
        ui_general_spn_auto_connection = getSpinner( R.id.spinnerAutoConnection , R.array.enable_disable     , VIEW_GENERAL );
        ui_general_spn_date_format     = getSpinner( R.id.spinnerDateFormat     , R.array.date_format        , VIEW_GENERAL );
        ui_general_spn_time_format     = getSpinner( R.id.spinnerTimeFormat     , R.array.time_format        , VIEW_GENERAL );
        ui_general_spn_language        = getSpinner( R.id.spinnerLanguage       , R.array.language_options   , VIEW_GENERAL );

        ui_general_txw_free_memory     = getTV( R.id.device_memory   , VIEW_GENERAL );
        ui_general_txw_date            = getTV( R.id.device_date     , VIEW_GENERAL );
        ui_general_txw_time            = getTV( R.id.device_time     , VIEW_GENERAL );

        ImageButton ui_general_ibtn_erase_memory   = getIBTN( R.id.button_memory , VIEW_GENERAL );
        ImageButton ui_general_ibtn_date           = getIBTN( R.id.button_date   , VIEW_GENERAL );
        ImageButton ui_general_ibtn_time           = getIBTN( R.id.button_time   , VIEW_GENERAL );

        LinearLayout ui_lyt_gauge = getLL( R.id.line_gauge, VIEW_GENERAL );
        ui_lyt_gauge.setVisibility( View.GONE );

        ui_general_ibtn_erase_memory.setOnClickListener( v ->
        {
            final Dialog dialog = new Dialog( fragmentActivity );
            dialog.requestWindowFeature( Window.FEATURE_NO_TITLE );
            dialog.setCanceledOnTouchOutside( false );
            dialog.setContentView( R.layout.dialog_yes_no );
            if ( dialog.getWindow() != null )
            {
                dialog.getWindow().getAttributes().width = ViewPager.LayoutParams.MATCH_PARENT;
            }
            ((TextView) dialog.findViewById( R.id.dlg_title )).setText( R.string.erase_memory );
            ((ImageView) dialog.findViewById( R.id.dlg_icon )).setImageResource( R.drawable.ic_warning );

            final TextView message = dialog.findViewById( R.id.message );
            message.setText( R.string.erase_memory_confirm );

            final Button btnNo = dialog.findViewById( R.id.btn_no );
            btnNo.setOnClickListener( v1 ->  dialog.dismiss() );

            final Button btnYes = dialog.findViewById(R.id.btn_yes);
            btnYes.setOnClickListener( v12 ->
            {
                device.writeEraseMemory();
                dialog.dismiss();
            } );

            dialog.show();
        } );

        ui_general_ibtn_date.setOnClickListener( v -> pickDate() );
        ui_general_ibtn_time.setOnClickListener( v -> pickTime() );

        VIEW_GENERAL.findViewById( R.id.buttonSave ).setOnClickListener( v ->
        {
            switch ( ui_general_spn_auto_connection.getSelectedItemPosition() )
            {
                case 0:
                    device.setAutoConn( ui_general_spn_auto_connection.getSelectedItemPosition() == 0 );
                    break;
                case 1:
                    device.setAutoConn( false );
                    break;
            }

            // DATE
            switch ( ui_general_spn_date_format.getSelectedItemPosition() )
            {
                case 0:
                    device.setDateFormat( Modbus.DATE_DD_MM_YY );
                    break;
                case 1:
                    device.setDateFormat( Modbus.DATE_MM_DD_YY );
                    break;
                default:
                    device.setDateFormat( Modbus.DATE_YYYY_MM_DD );
                    break;
            }

            switch ( ui_general_spn_time_format.getSelectedItemPosition() )
            {
                case 0:
                    device.setTimeFormat( Modbus.TIME_24H );
                    break;
                default:
                    device.setTimeFormat( Modbus.TIME_12H );
                    break;
            }

            switch ( ui_general_spn_language.getSelectedItemPosition() )
            {
                case 1:
                    device.setLanguage( Modbus.LANG_SPANISH );
                    break;
                case 2:
                    device.setLanguage( Modbus.LANG_PORTUGUESE );
                    break;
                default:
                    device.setLanguage( Modbus.LANG_ENGLISH );
                    break;
            }

            device.writeSetup();
            device.writeDate();

            ((Main) fragmentActivity).updateAutoConn( device.getAutoConn() );
            ((Main) fragmentActivity).updateDateTimeFormat( device.getDateFormat(), device.getTimeFormat() );

            closeOverlay();
        } );

        Button buttonCancel = VIEW_GENERAL.findViewById( R.id.buttonCancel );
        buttonCancel.setOnClickListener( v -> closeOverlay() );
    }

    private void dlgTestSetup( Object params )
    {
        // Create dialog
        if ( ui_spn_mode == null && VIEW_SETUP != null )
        {
            ui_spn_mode = getSpinner( R.id.ui_spn_mode, R.array.op_modes, VIEW_SETUP );
            ui_lyt_manual = getLL( R.id.ui_lyt_manual, VIEW_SETUP );
            ui_lyt_timer = getLL( R.id.ui_lyt_timer, VIEW_SETUP );
            ui_lyt_svt = getLL( R.id.ui_lyt_svt, VIEW_SETUP );
            ui_lyt_ramp = getLL( R.id.ui_lyt_ramp, VIEW_SETUP );
            LinearLayout ui_lyt_dd = getLL( R.id.ui_lyt_dd, VIEW_SETUP );
            ui_lyt_dd.setVisibility( device.fwHasDD() || device.fwHasIDD() ? View.VISIBLE : View.GONE );

            // Param Manual
            ui_swt_manual_lim = getSWT( R.id.ui_swt_manual_lim, VIEW_SETUP );
            ui_lyt_manual_lim = getLL( R.id.ui_lyt_manual_lim, VIEW_SETUP );
            ui_txw_manual_max_volts = getTV( R.id.ui_txw_manual_max_volts, VIEW_SETUP );
            ui_txw_manual_lim = getTV( R.id.ui_txw_manual_lim, VIEW_SETUP );
            ui_txw_manual_pi_a = getTV( R.id.ui_txw_manual_pi_a, VIEW_SETUP );
            ui_txw_manual_pi_b = getTV( R.id.ui_txw_manual_pi_b, VIEW_SETUP );
            ui_txw_manual_dai_a = getTV( R.id.ui_txw_manual_dai_a, VIEW_SETUP );
            ui_txw_manual_dai_b = getTV( R.id.ui_txw_manual_dai_b, VIEW_SETUP );
            dlgs.btnNumberPicker( getIBTN( R.id.ui_ibtn_manual_max_volts, VIEW_SETUP ), ui_txw_manual_max_volts, device.VOLTS_MIN, device.VOLTS_MAX, device.VOLTS_STEP );
            ui_swt_manual_lim.setOnCheckedChangeListener( ( compoundButton, b ) -> ui_lyt_manual_lim.setVisibility( b ? View.VISIBLE : View.GONE ) );
            dlgs.btnOhmPicker( getIBTN( R.id.ui_ibtn_manual_lim, VIEW_SETUP ), ui_txw_manual_lim, device.LIM_MIN, device.LIM_MAX );
            dlgs.btnNumberPicker( getIBTN( R.id.ui_ibtn_manual_pi_a, VIEW_SETUP ), ui_txw_manual_pi_a, ui_txw_manual_pi_b, device.PI_TIME_A_MIN, device.PI_TIME_A_MAX, device.PI_TIME_A_STEP, 0.5f );// 0.5 is 50%, V / 2
            dlgs.btnNumberPicker( getIBTN( R.id.ui_ibtn_manual_pi_b, VIEW_SETUP ), ui_txw_manual_pi_b, device.PI_TIME_B_MIN, ui_txw_manual_pi_a, device.PI_TIME_B_STEP, 0.5f );// 0.5 is 50%, V / 2
            dlgs.btnNumberPicker( getIBTN( R.id.ui_ibtn_manual_dai_a, VIEW_SETUP ), ui_txw_manual_dai_a, ui_txw_manual_dai_b, device.DAI_TIME_A_MIN, device.DAI_TIME_A_MAX, device.DAI_TIME_A_STEP, 0.5f );// 0.5 is 50%, V / 2
            dlgs.btnNumberPicker( getIBTN( R.id.ui_ibtn_manual_dai_b, VIEW_SETUP ), ui_txw_manual_dai_b, device.DAI_TIME_B_MIN, ui_txw_manual_dai_a, device.DAI_TIME_B_STEP, 0.5f );// 0.5 is 50%, V / 2

            // Param Timer
            ui_swt_timer_lim = getSWT( R.id.ui_swt_timer_lim, VIEW_SETUP );
            ui_lyt_timer_lim = getLL( R.id.ui_lyt_timer_lim, VIEW_SETUP );
            ui_txw_timer_max_volts = getTV( R.id.ui_txw_timer_max_volts, VIEW_SETUP );
            ui_txw_timer_lim = getTV( R.id.ui_txw_timer_lim, VIEW_SETUP );
            ui_txw_timer_time = getTV( R.id.ui_txw_timer_time, VIEW_SETUP );
            ui_txw_timer_pi_a = getTV( R.id.ui_txw_timer_pi_a, VIEW_SETUP );
            ui_txw_timer_pi_b = getTV( R.id.ui_txw_timer_pi_b, VIEW_SETUP );
            ui_txw_timer_dai_a = getTV( R.id.ui_txw_timer_dai_a, VIEW_SETUP );
            ui_txw_timer_dai_b = getTV( R.id.ui_txw_timer_dai_b, VIEW_SETUP );
            dlgs.btnNumberPicker( getIBTN( R.id.ui_ibtn_timer_max_volts, VIEW_SETUP ), ui_txw_timer_max_volts, device.VOLTS_MIN, device.VOLTS_MAX, device.VOLTS_STEP );
            ui_swt_timer_lim.setOnCheckedChangeListener( ( compoundButton, b ) -> ui_lyt_timer_lim.setVisibility( b ? View.VISIBLE : View.GONE ) );
            dlgs.btnOhmPicker( getIBTN( R.id.ui_ibtn_timer_lim, VIEW_SETUP ), ui_txw_timer_lim, device.LIM_MIN, device.LIM_MAX );
            dlgs.btnTimerPicker( getIBTN( R.id.ui_ibtn_timer_time, VIEW_SETUP ), ui_txw_timer_time, device.TIMER_MIN, device.TIMER_MAX );
            dlgs.btnNumberPicker( getIBTN( R.id.ui_ibtn_timer_pi_a, VIEW_SETUP ), ui_txw_timer_pi_a, ui_txw_timer_pi_b, device.PI_TIME_A_MIN, device.PI_TIME_A_MAX, device.PI_TIME_A_STEP, 0.5f );// 0.5 is 50%, V / 2
            dlgs.btnNumberPicker( getIBTN( R.id.ui_ibtn_timer_pi_b, VIEW_SETUP ), ui_txw_timer_pi_b, device.PI_TIME_B_MIN, ui_txw_timer_pi_a, device.PI_TIME_B_STEP, 0.5f );// 0.5 is 50%, V / 2
            dlgs.btnNumberPicker( getIBTN( R.id.ui_ibtn_timer_dai_a, VIEW_SETUP ), ui_txw_timer_dai_a, ui_txw_timer_dai_b, device.DAI_TIME_A_MIN, device.DAI_TIME_A_MAX, device.DAI_TIME_A_STEP, 0.5f );// 0.5 is 50%, V / 2
            dlgs.btnNumberPicker( getIBTN( R.id.ui_ibtn_timer_dai_b, VIEW_SETUP ), ui_txw_timer_dai_b, device.DAI_TIME_B_MIN, ui_txw_timer_dai_a, device.DAI_TIME_B_STEP, 0.5f );// 0.5 is 50%, V / 2

            // Param SVT
            ui_swt_svt_lim = getSWT( R.id.ui_swt_svt_lim, VIEW_SETUP );
            ui_lyt_svt_lim = getLL( R.id.ui_lyt_svt_lim, VIEW_SETUP );
            ui_txw_svt_max_volts = getTV( R.id.ui_txw_svt_max_volts, VIEW_SETUP );
            ui_txw_svt_lim = getTV( R.id.ui_txw_svt_lim, VIEW_SETUP );
            ui_txw_svt_step_volts = getTV( R.id.ui_txw_svt_step_volts, VIEW_SETUP );
            ui_txw_svt_step_time = getTV( R.id.ui_txw_svt_step_time, VIEW_SETUP );
            dlgs.btnNumberPicker( getIBTN( R.id.ui_ibtn_svt_max_volts, VIEW_SETUP ), ui_txw_svt_max_volts, device.SVT_MIN, device.SVT_MAX, device.SVT_STEP_VOLTS );
            ui_swt_svt_lim.setOnCheckedChangeListener( ( compoundButton, b ) -> ui_lyt_svt_lim.setVisibility( b ? View.VISIBLE : View.GONE ) );
            dlgs.btnOhmPicker( getIBTN( R.id.ui_ibtn_svt_lim, VIEW_SETUP ), ui_txw_svt_lim, device.LIM_MIN, device.LIM_MAX );
            dlgs.btnNumberPicker( getIBTN( R.id.ui_ibtn_svt_step_time, VIEW_SETUP ), ui_txw_svt_step_time, device.SVT_STEP_TIME_MIN, device.SVT_STEP_TIME_MAX, device.SVT_STEP_TIME );
            dlgs.btnNumberPicker( getIBTN( R.id.ui_ibtn_svt_step_volts, VIEW_SETUP ), ui_txw_svt_step_volts, device.SVT_STEP_VOLTS_MIN, device.SVT_STEP_VOLTS_MAX, device.SVT_STEP_VOLTS );

            // Param Ramp
            ui_swt_ramp_lim = getSWT( R.id.ui_swt_ramp_lim, VIEW_SETUP );
            ui_lyt_ramp_lim = getLL( R.id.ui_lyt_ramp_lim, VIEW_SETUP );
            ui_txw_ramp_max_volts = getTV( R.id.ui_txw_ramp_max_volts, VIEW_SETUP );
            ui_txw_ramp_lim = getTV( R.id.ui_txw_ramp_lim, VIEW_SETUP );
            ui_txw_ramp_time = getTV( R.id.ui_txw_ramp_time, VIEW_SETUP );
            dlgs.btnNumberPicker( getIBTN( R.id.ui_ibtn_ramp_max_volts, VIEW_SETUP ), ui_txw_ramp_max_volts, device.RAMP_MIN, device.RAMP_MAX, device.VOLTS_STEP );
            ui_swt_ramp_lim.setOnCheckedChangeListener( ( compoundButton, b ) -> ui_lyt_ramp_lim.setVisibility( b ? View.VISIBLE : View.GONE ) );
            dlgs.btnOhmPicker( getIBTN( R.id.ui_ibtn_ramp_lim, VIEW_SETUP ), ui_txw_ramp_lim, device.LIM_MIN, device.LIM_MAX );
            dlgs.btnTimerPicker( getIBTN( R.id.ui_ibtn_ramp_time, VIEW_SETUP ), ui_txw_ramp_time, device.RAMP_TIME_MIN, device.RAMP_TIME_MAX );

            ui_swt_cap = getSWT( R.id.ui_swt_cap, VIEW_SETUP );
            ui_swt_dd = getSWT( R.id.ui_swt_dd, VIEW_SETUP );



            ui_spn_mode.setOnItemSelectedListener( new AdapterView.OnItemSelectedListener()
            {
                @Override
                public void onItemSelected( AdapterView<?> parentView, View selectedItemView, int position, long id )
                {
                    ui_lyt_manual.setVisibility( position == device.MODE_MANUAL ? View.VISIBLE : View.GONE );
                    ui_lyt_timer.setVisibility( position == device.MODE_TIMER ? View.VISIBLE : View.GONE );
                    ui_lyt_svt.setVisibility( position == device.MODE_SVT ? View.VISIBLE : View.GONE );
                    ui_lyt_ramp.setVisibility( position == device.MODE_RAMP ? View.VISIBLE : View.GONE );
                }

                @Override
                public void onNothingSelected( AdapterView<?> parentView )
                {
                }
            } );
            ui_swt_cap.setOnCheckedChangeListener( ( compoundButton, b ) ->
            {
                if ( !b )
                {
                    ui_swt_dd.setChecked( false );
                }
            } );
            VIEW_SETUP.findViewById( R.id.ui_btn_cancel ).setOnClickListener( v ->
            {
                // Restore data fields
                dlgTestSetup( device );
                dlgTestSetup( device.getParamManual() );
                dlgTestSetup( device.getParamTimer() );
                dlgTestSetup( device.getParamSVT() );
                dlgTestSetup( device.getParamRamp() );
                closeOverlay();
            } );
            VIEW_SETUP.findViewById( R.id.ui_btn_save ).setOnClickListener( v ->
            {
                // Save
                device.setMode( ui_spn_mode.getSelectedItemPosition() );
                device.setCapEnabled( ui_swt_cap.isChecked() );
                device.setDDEnabled( ui_swt_dd.isChecked() );
                device.writeSetup();

                if ( device.getMode() == device.MODE_MANUAL )
                {
                    device.getParamManual().setMaxVolts( NumberParser.parseInt( ui_txw_manual_max_volts.getText().toString() ) );
                    long lim = ui_swt_manual_lim.isChecked() ? NumberParser.parseLongStringUnity( ui_txw_manual_lim.getText().toString() ) : 0;
                    device.getParamManual().setLim( lim );
                    device.getParamManual().getPI().setTimeA( NumberParser.parseInt( ui_txw_manual_pi_a.getText().toString() ) );
                    device.getParamManual().getPI().setTimeB( NumberParser.parseInt( ui_txw_manual_pi_b.getText().toString() ) );
                    device.getParamManual().getDAI().setTimeA( NumberParser.parseInt( ui_txw_manual_dai_a.getText().toString() ) );
                    device.getParamManual().getDAI().setTimeB( NumberParser.parseInt( ui_txw_manual_dai_b.getText().toString() ) );
                    device.writeParamManual();
                    dlgTestSetup( device.getParamTimer() );
                    dlgTestSetup( device.getParamSVT() );
                    dlgTestSetup( device.getParamRamp() );
                }
                else if ( device.getMode() == device.MODE_TIMER )
                {
                    device.getParamTimer().setMaxVolts( NumberParser.parseInt( ui_txw_timer_max_volts.getText().toString() ) );
                    long lim = ui_swt_timer_lim.isChecked() ? NumberParser.parseLongStringUnity( ui_txw_timer_lim.getText().toString() ) : 0;
                    device.getParamTimer().setLim( lim );
                    device.getParamTimer().setTime( NumberParser.parseTimeMMSS( ui_txw_timer_time.getText().toString() ) );
                    device.getParamTimer().getPI().setTimeA( NumberParser.parseInt( ui_txw_timer_pi_a.getText().toString() ) );
                    device.getParamTimer().getPI().setTimeB( NumberParser.parseInt( ui_txw_timer_pi_b.getText().toString() ) );
                    device.getParamTimer().getDAI().setTimeA( NumberParser.parseInt( ui_txw_timer_dai_a.getText().toString() ) );
                    device.getParamTimer().getDAI().setTimeB( NumberParser.parseInt( ui_txw_timer_dai_b.getText().toString() ) );
                    device.writeParamTimer();
                    dlgTestSetup( device.getParamManual() );
                    dlgTestSetup( device.getParamSVT() );
                    dlgTestSetup( device.getParamRamp() );
                }
                else if ( device.getMode() == device.MODE_SVT )
                {
                    device.getParamSVT().setFinalVolts( NumberParser.parseInt( ui_txw_svt_max_volts.getText().toString() ) );
                    long lim = ui_swt_svt_lim.isChecked() ? NumberParser.parseLongStringUnity( ui_txw_svt_lim.getText().toString() ) : 0;
                    device.getParamSVT().setLim( lim );
                    device.getParamSVT().setStepTime( NumberParser.parseInt( ui_txw_svt_step_time.getText().toString() ) );
                    device.getParamSVT().setStepVolts( NumberParser.parseInt( ui_txw_svt_step_volts.getText().toString() ) );
                    device.writeParamSVT();
                    dlgTestSetup( device.getParamManual() );
                    dlgTestSetup( device.getParamTimer() );
                    dlgTestSetup( device.getParamRamp() );
                }
                else if ( device.getMode() == device.MODE_RAMP )
                {
                    device.getParamRamp().setFinalVolts( NumberParser.parseInt( ui_txw_ramp_max_volts.getText().toString() ) );
                    long lim = ui_swt_ramp_lim.isChecked() ? NumberParser.parseLongStringUnity( ui_txw_ramp_lim.getText().toString() ) : 0;
                    device.getParamRamp().setLim( lim );
                    device.getParamRamp().setTime( NumberParser.parseTimeMMSS( ui_txw_ramp_time.getText().toString() ) );
                    device.writeParamRamp();
                    dlgTestSetup( device.getParamManual() );
                    dlgTestSetup( device.getParamTimer() );
                    dlgTestSetup( device.getParamSVT() );
                }

                closeOverlay();
            } );
        }
        // Populate dialog fields
        if ( params instanceof Device )
        {
            Device d = (Device) params;
            if ( d.getMode() < ui_spn_mode.getAdapter().getCount() )
            {
                ui_spn_mode.setSelection( d.getMode() );
            }
            ui_swt_cap.setChecked( d.getCapEnabled() );
            ui_swt_dd.setChecked( d.getDDEnabled() );
        }
        else if ( params instanceof ParamManual )
        {
            ParamManual p = (ParamManual) params;
            ui_txw_manual_max_volts.setText( p.getMaxVoltsAsString() );
            ui_txw_manual_lim.setText( p.getLimAsString() );
            ui_swt_manual_lim.setChecked( p.getLim() > 0 );
            ui_txw_manual_pi_a.setText( p.getPI().getTimeAString() );
            ui_txw_manual_pi_b.setText( p.getPI().getTimeBString() );
            ui_txw_manual_dai_a.setText( p.getDAI().getTimeAString() );
            ui_txw_manual_dai_b.setText( p.getDAI().getTimeBString() );
        }
        else if ( params instanceof ParamTimer )
        {
            ParamTimer p = (ParamTimer) params;
            ui_txw_timer_max_volts.setText( p.getMaxVoltsAsString() );
            ui_txw_timer_lim.setText( p.getLimAsString() );
            ui_swt_timer_lim.setChecked( p.getLim() > 0 );
            ui_txw_timer_time.setText( p.getTimeAsString() );
            ui_txw_timer_pi_a.setText( p.getPI().getTimeAString() );
            ui_txw_timer_pi_b.setText( p.getPI().getTimeBString() );
            ui_txw_timer_dai_a.setText( p.getDAI().getTimeAString() );
            ui_txw_timer_dai_b.setText( p.getDAI().getTimeBString() );
        }
        else if ( params instanceof ParamSVT )
        {
            ParamSVT p = (ParamSVT) params;
            ui_txw_svt_max_volts.setText( p.getFinalVoltsAsString() );
            ui_txw_svt_lim.setText( p.getLimAsString() );
            ui_swt_svt_lim.setChecked( p.getLim() > 0 );
            ui_txw_svt_step_time.setText( p.getStepTimeAsString());
            ui_txw_svt_step_volts.setText( p.getStepVoltsAsString());
        }
        else if ( params instanceof ParamRamp )
        {
            ParamRamp p = (ParamRamp) params;
            ui_txw_ramp_max_volts.setText( p.getFinalVoltsAsString() );
            ui_txw_ramp_lim.setText( p.getLimAsString() );
            ui_swt_ramp_lim.setChecked( p.getLim() > 0 );
            ui_txw_ramp_time.setText( p.getTimeAsString());
        }
    }

    private void updateUiRealTime( Measure m )
    {
        if ( m == null )
        {

            ui_main_screen.ui_lb_realtime_test_number.setText( "- - -" );
            ui_main_screen.ui_lb_realtime_resistance.setText( "- - -" );
            ui_main_screen.ui_lb_realtime_elapsed.setText( "- - -" );
            ui_main_screen.ui_lb_realtime_voltage.setText( "- - -" );
            ui_main_screen.ui_lb_realtime_current.setText( "- - -" );
            ui_main_screen.ui_lb_pass.setVisibility( View.INVISIBLE );
            ui_main_screen.ui_lb_fail.setVisibility( View.GONE );
            return;
        }

        if ( device.getMode() >= 0 )
        {
            if ( device.getStatus( device.ST_WAIT ) )
            {
                ui_main_screen.ui_lb_realtime_resistance.setText( fragmentActivity.getString( R.string.wait ) );
                ui_main_screen.ui_lb_realtime_elapsed.setText( m.getElapsedAsString() );
                ui_main_screen.ui_lb_realtime_voltage.setText( m.getVoltsAsString() );
                ui_main_screen.ui_lb_realtime_current.setText( "- - -" );
            }
            else if ( device.getStatus( device.ST_OVERRANGE ) )
            {
                ui_main_screen.ui_lb_realtime_resistance.setText( fragmentActivity.getString( R.string.overrange ) );
                ui_main_screen.ui_lb_realtime_elapsed.setText( m.getElapsedAsString() );
                ui_main_screen.ui_lb_realtime_voltage.setText( m.getVoltsAsString() );
                ui_main_screen.ui_lb_realtime_current.setText( m.getAmpsAsString() );
            }
            else
            {
                ui_main_screen.ui_lb_realtime_resistance.setText( m.getOhmsAsString() );
                ui_main_screen.ui_lb_realtime_elapsed.setText( m.getElapsedAsString() );
                ui_main_screen.ui_lb_realtime_voltage.setText( m.getVoltsAsString() );
                ui_main_screen.ui_lb_realtime_current.setText( m.getAmpsAsString() );
            }

            double lim = device.getLimitResistance();
            if ( lim > 0 && !device.getStatus( device.ST_WAIT ) && !device.getStatus( device.ST_OVERRANGE ) )
            {
                if ( lim > m.getOhms() )
                {
                    ui_main_screen.ui_lb_pass.setVisibility( View.INVISIBLE );
                    ui_main_screen.ui_lb_fail.setVisibility( View.VISIBLE );
                }
                else
                {
                    ui_main_screen.ui_lb_pass.setVisibility( View.VISIBLE );
                    ui_main_screen.ui_lb_fail.setVisibility( View.GONE );
                }
            }
            else
            {
                ui_main_screen.ui_lb_pass.setVisibility( View.INVISIBLE );
                ui_main_screen.ui_lb_fail.setVisibility( View.GONE );
            }
        }
    }

    private void updateChartsAndTable( Object o )
    {
        if ( o instanceof Measure )
        {
            Measure m = (Measure)o;
            if ( m.getElapsed() >= 10 && m.getElapsed() % 5 == 0 )
            {
                ui_chart_ohms.update( m );
                ui_chart_amps.update( m );
                ui_chart_volts.update( m );
                ui_table.update( m );
            }
        }
        else if ( o instanceof DielectricDischarge )
        {
            DielectricDischarge dd = (DielectricDischarge)o;
            ui_chart_idd.update( dd );
        }
        else
        {
            ui_chart_ohms.reset();
            ui_chart_amps.reset();
            ui_chart_volts.reset();
            ui_chart_idd.reset();
            ui_table.reset();
        }
    }

    private TimePickerDialog.OnTimeSetListener timeListener = new TimePickerDialog.OnTimeSetListener()
    {
        @Override
        public void onTimeSet( TimePicker view, int hourOfDay, int minute )
        {
            if ( VIEW_GENERAL != null )
            {
                Calendar c = Calendar.getInstance();
                c.setTime( device.getDate() );
                c.set( Calendar.HOUR_OF_DAY, hourOfDay );
                c.set( Calendar.MINUTE, minute );
                c.set( Calendar.SECOND, 0 );
                device.setDate( c.getTime() );
                ui_general_txw_time.setText( device.getTimeAsString() );
            }
        }
    };

    private void pickTime()
    {
        Calendar calendar = new GregorianCalendar();
        calendar.setTime( device.getDate() );
        int     hour   = calendar.get( Calendar.HOUR_OF_DAY );
        int     minute = calendar.get( Calendar.MINUTE );
        boolean is24   = device.getTimeFormat() == Modbus.TIME_24H;
        new TimePickerDialog( fragmentActivity, timeListener, hour, minute, is24 ).show();
    }

    private DatePickerDialog.OnDateSetListener dateListener = new DatePickerDialog.OnDateSetListener()
    {
        public void onDateSet( DatePicker arg0, int year, int month, int day )
        {
            if ( VIEW_GENERAL != null )
            {
                String parts[] = ui_general_txw_time.getText().toString().split( ":" );
                int    hour    = Integer.parseInt( parts[0] );
                int    minutes = Integer.parseInt( parts[1] );

                Calendar c = Calendar.getInstance();
                c.set( year, month, day, hour, minutes );
                device.setDate( c.getTime() );
                ui_general_txw_date.setText( device.getDateAsString() );
            }
        }
    };

    private void pickDate()
    {
        Calendar calendar = new GregorianCalendar();
        calendar.setTime( device.getDate() );
        int year  = calendar.get( Calendar.YEAR );
        int month = calendar.get( Calendar.MONTH );
        int day   = calendar.get( Calendar.DAY_OF_MONTH );
        new DatePickerDialog( fragmentActivity, dateListener, year, month, day ).show();
    }

    private void showOverlay( View view )
    {
        if ( OVERLAY != null && view != null )
        {
            OVERLAY.removeAllViews();
            OVERLAY.addView( view );
            OVERLAY.setVisibility( View.VISIBLE );
            ui_menu_item_options.setEnabled( false );
        }
    }

    private void closeOverlay()
    {
        if ( OVERLAY != null )
        {
            OVERLAY.removeAllViews();
            OVERLAY.setVisibility( View.GONE );
            ui_menu_item_options.setEnabled( true );
        }
    }

    private void userMessage( String message, int color, boolean busy )
    {
        if ( !BOX_MESSAGE.equals( message ) || BOX_COLOR != color || BOX_BUSY != busy )
        {
            BOX_MESSAGE = message;
            BOX_COLOR = color;
            BOX_BUSY = busy;
            ((Main) fragmentActivity).message( message, color, busy );
        }
    }

    private LinearLayout getLL( int layoutID, View view )
    {
        return (LinearLayout) view.findViewById( layoutID );
    }

    @SuppressWarnings("SameParameterValue")
    private ListView getLV( int layoutID, View view )
    {
        return (ListView) view.findViewById( layoutID );
    }

    private TextView getTV( int layoutID, View view )
    {
        return (TextView) view.findViewById( layoutID );
    }

    private ImageButton getIBTN( int layoutID, View view )
    {
        return (ImageButton) view.findViewById( layoutID );
    }

    private Switch getSWT( int layoutID, View view )
    {
        return (Switch) view.findViewById( layoutID );
    }

    private Button getBTN( int layoutID, View view )
    {
        return (Button) view.findViewById( layoutID );
    }

    private Spinner getSpinner( int spinnerID, int arrayID, View inflatedView) {
        ArrayAdapter<CharSequence> filter =
                ArrayAdapter.createFromResource(fragmentActivity, arrayID, android.R.layout.simple_spinner_item);
        filter.setDropDownViewResource(R.layout.simple_spinner_dropdown_item);

        final Spinner spinner = inflatedView.findViewById(spinnerID);
        spinner.setAdapter(filter);
        return spinner;
    }

    /***********************************************************************************************
     * Controller Interface
     ***********************************************************************************************/

    @Override
    public void createSwipe()
    {
        pagerAdapter = new PagerAdapter( fragmentActivity.getSupportFragmentManager(), fragmentActivity.getBaseContext() );

        VIEW_PAGER = fragmentActivity.findViewById( R.id.pager );

        if ( VIEW_PAGER == null )
        {
            return;
        }

        VIEW_PAGER.setAdapter( pagerAdapter );
        VIEW_PAGER.setOffscreenPageLimit( pagerAdapter.getCount() );

        ui_main_screen = pagerAdapter.getMainFragment();
        ui_chart_ohms  = pagerAdapter.chartResistance();
        ui_chart_volts = pagerAdapter.chartVoltage();
        ui_chart_amps  = pagerAdapter.chartCurrent();
        ui_chart_idd   = pagerAdapter.chartIDD();
        ui_table       = pagerAdapter.table();

        //Create customs Tabs
        SlidingTabLayout mSlidingTabLayout = fragmentActivity.findViewById( R.id.sliding_tabs );
        mSlidingTabLayout.setDistributeEvenly( true );
        mSlidingTabLayout.setCustomTabView( R.layout.custom_tab, 0 );
        mSlidingTabLayout.setBackground( fragmentActivity.getResources().getDrawable( R.drawable.chamfer_black_top ) );
        mSlidingTabLayout.setCustomTabColorizer( new SlidingTabLayout.TabColorizer()
        {
            public int getIndicatorColor( int position )
            {
                return Color.GRAY;
            }

            public int getDividerColor( int position )
            {
                return Color.WHITE;
            }
        } );
        mSlidingTabLayout.setViewPager( VIEW_PAGER );

        pagerAdapter.setViewPagerRef( VIEW_PAGER );
        pagerAdapter.setSlidingTabLayoutRef( mSlidingTabLayout );
    }

    @Override
    public void createControlButtons( LinearLayout container )
    {
        final View commandButtons = View.inflate( fragmentActivity, R.layout.md15kvr_control_buttons, null );

        commandButtons.setLayoutParams( new LinearLayout.LayoutParams( LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT ) );
        container.removeAllViews();
        container.addView( commandButtons );

        ui_btn_start = commandButtons.findViewById( R.id.bt_start );
        ui_btn_stop = commandButtons.findViewById( R.id.bt_stop );
        ui_ibtn_printer = commandButtons.findViewById( R.id.bt_printer );
        ui_ibtn_filter = commandButtons.findViewById( R.id.bt_filter );
        ui_btn_voltage = commandButtons.findViewById( R.id.bt_voltage );
        ui_ibtn_test_setup = commandButtons.findViewById( R.id.bt_setup );

        ui_btn_start.setEnabled( false );
        ui_ibtn_printer.setEnabled( false );
        ui_ibtn_filter.setEnabled( false );
        ui_ibtn_test_setup.setEnabled( false );
        ui_btn_voltage.setEnabled( false );

        View.OnClickListener onClickStart = view ->
        {
            device.writeStart();
            ui_btn_start.setVisibility( View.GONE );
            ui_btn_stop.setVisibility( View.VISIBLE );
        };

        ui_btn_start.setOnClickListener( v -> dlgs.startTest( onClickStart ) );
        //ui_btn_start.setOnClickListener( v -> device.writeStart() );

        ui_btn_stop.setOnClickListener( v -> device.writeStop() );
        ui_btn_voltage.setOnClickListener( v -> dlgs.pickNumber( DLG_CONF_VOLTS, device.getConfVolts(), "V", device.VOLTS_MIN, device.getActualMaxVolts(), device.VOLTS_STEP ) );
        ui_ibtn_printer.setOnClickListener( v ->
        {
            ui_ibtn_printer.setEnabled( false );
            device.writePrinter();
        } );
        ui_ibtn_filter.setOnClickListener( v ->
        {
            ui_ibtn_filter.setEnabled( false );
            device.writeFilter();
        } );
        ui_ibtn_test_setup.setOnClickListener( v -> showOverlay( VIEW_SETUP ) );
    }

    @Override
    public void setToolBarId( int toolBarId )
    {
        LinearLayout ui_lyt_toolbar = fragmentActivity.findViewById( toolBarId );
        ui_lyt_toolbar.findViewById( R.id.bt_details ).setVisibility( View.GONE );
        ui_lyt_toolbar.findViewById( R.id.bt_export ).setVisibility( View.GONE );

        ui_ibtn_save = ui_lyt_toolbar.findViewById( R.id.bt_save );
        ui_ibtn_map = ui_lyt_toolbar.findViewById( R.id.btn_map );
        ui_ibtn_file_manager = ui_lyt_toolbar.findViewById( R.id.bt_manager );
        ui_ibtn_picture = ui_lyt_toolbar.findViewById( R.id.bt_picture );
        ui_ibtn_gps = ui_lyt_toolbar.findViewById( R.id.bt_gps );
        ui_ibtn_voice = ui_lyt_toolbar.findViewById( R.id.bt_record );

        ui_ibtn_save.setEnabled( false );
        ui_ibtn_save.setOnClickListener( v ->
        {
            database.save( device.getTestData() );
            openedTestData = database.getTestData( device.getTestData() ); // Why not simple openedTestData = device.getTestData()? Because it will be a pointer to the same Obj. In this manner, We get another TestData Obj
            ui_ibtn_save.setEnabled( false );
            ui_ibtn_voice.setEnabled( true );
            ui_ibtn_picture.setEnabled( true );
            ui_ibtn_gps.setEnabled( true );
            ui_ibtn_map.setEnabled( true );
        } );

        ui_ibtn_map.setEnabled( false );
        ui_ibtn_map.setOnClickListener( v ->
        {
            if ( Main.permissionToMemoryAccepted )
            {
                map();
            }
            else
            {
                ((Main) fragmentActivity).requestPermission( Main.REQUEST_MEMORY_MAP_PERMISSION );
            }
        } );

        ui_ibtn_file_manager.setOnClickListener( v ->
        {
            List<FileDetails> fileList = database.getSavedFilesList( device );
            if ( fileList != null )
            {
                loadFilesManagerPage();
                showOverlay( VIEW_FILES );
            }
        } );

        ui_ibtn_picture.setEnabled( false );
        ui_ibtn_picture.findViewById( R.id.bt_picture ).setOnClickListener( v ->
        {
            if ( Main.permissionToPictureAccepted )
            {
                takePicture();
            }
            else
            {
                ((Main) fragmentActivity).requestPermission( Main.REQUEST_PICTURE_PERMISSION );
            }
        } );

        ui_ibtn_gps.setEnabled( false );
        ui_ibtn_gps.setOnClickListener( v ->
        {
            if ( Main.permissionToGPSAccepted )
            {
                getGPS();
            }
            else
            {
                ((Main) fragmentActivity).requestPermission( Main.REQUEST_GPS_PERMISSION );
            }
        } );

        ui_ibtn_voice.setEnabled( false );
        ui_ibtn_voice.setOnClickListener( v ->
        {
            if ( Main.permissionToRecordAccepted )
            {
                recordAudio();
            }
            else
            {
                ((Main) fragmentActivity).requestPermission( Main.REQUEST_RECORD_AUDIO_PERMISSION );
            }
        } );
    }

    @Override
    public void stateChanged( int state )
    {
        switch ( state )
        {
            case BluetoothDev.STATE_CONNECTED:
                device.syncDevice();
                ui_btn_start.setVisibility( View.VISIBLE );
                ui_btn_stop.setVisibility( View.GONE );
                ui_btn_start.setEnabled( false );
                ui_ibtn_printer.setEnabled( false );
                ui_ibtn_filter.setEnabled( false );
                ui_ibtn_test_setup.setEnabled( false );
                ui_btn_voltage.setEnabled( false );
                ui_menu_item_options.setEnabled( false );
                break;
            case BluetoothDev.STATE_CONNECTING:
                break;
            case BluetoothDev.STATE_NOT_RESPONDING:
                device.stateNotResponding();
                break;
            case BluetoothDev.STATE_FAIL:
                break;
            case BluetoothDev.STATE_LOST:
            case BluetoothDev.STATE_CLOSED:
                device.stateDisconnected();
                ui_btn_start.setVisibility( View.VISIBLE );
                ui_btn_stop.setVisibility( View.GONE );
                ui_btn_start.setEnabled( false );
                ui_ibtn_printer.setEnabled( false );
                ui_ibtn_filter.setEnabled( false );
                ui_ibtn_test_setup.setEnabled( false );
                ui_btn_voltage.setEnabled( false );
                ui_ibtn_save.setEnabled( false );
                ui_menu_item_options.setEnabled( false );
//                ui_ibtn_file_manager.setEnabled( true );
//                ui_ibtn_voice.setEnabled( true );
//                ui_ibtn_picture.setEnabled( true );
//                ui_ibtn_gps.setEnabled( true );
//                ui_ibtn_map.setEnabled( true );
                closeOverlay();
                break;
        }
    }

    @Override
    public void processResponse( int id, String response )
    {
        try
        {
            device.messageReceived( id, response );
        }
        catch ( Exception ignored )
        {
        }
    }

    @Override
    public void sendMessage( Message message )
    {
        ((Main) fragmentActivity).addMessage( message );
    }

    @Override
    public Message statusMessage()
    {
        return device.getStatusMessage();
    }

    @Override
    public void readGeneralConfig()
    {
        device.syncDeviceBasic();
    }

    @Override
    public void setAutoConn( boolean enable )
    {
        device.setAutoConn( enable );
    }

    @Override
    public void setDateTimeFormat( int dateFormat, int timeFormat )
    {
        device.setDateFormat( dateFormat );
        device.setTimeFormat( timeFormat );
    }

    @Override
    public void setMenuItemOptions( Menu menu, int optionsID )
    {
        if ( ui_menu_item_options == null )
        {
            ui_menu_item_options = menu.findItem( optionsID );
        }
    }

    @Override
    public boolean getAutoConn()
    {
        return device.getAutoConn();
    }

    @Override
    public void onClose()
    {
        if ( mGPS != null )
        {
            mGPS.cancel();
        }
        if ( picture != null )
        {
            picture.close();
        }
    }

    @Override
    public void permissionCallback( int permission )
    {
        switch ( permission )
        {
            case Main.REQUEST_RECORD_AUDIO_PERMISSION:
                recordAudio();
                break;
            case Main.REQUEST_PICTURE_PERMISSION:
                takePicture();
                break;
            case Main.REQUEST_GPS_PERMISSION:
                getGPS();
                break;
            case Main.REQUEST_MEMORY_REPORT_PERMISSION:
                export();
                break;
            case Main.REQUEST_MEMORY_MAP_PERMISSION:
                map();
                break;
        }
    }

    /***********************************************************************************************
     * REGION Report
     ***********************************************************************************************/

    private void export()
    {
        if ( database == null )
        {
            return;
        }

        int selection = ui_liw_files.getCheckedItemPosition();
        if ( selection > -1 )
        {
            TableFilesAdapter adapter = (TableFilesAdapter) ui_liw_files.getAdapter();
            TestData exportTestData = database.getTestData( adapter.getItem( selection ).FILENAME );
            if ( exportTestData != null )
            {
                dlgs.showProgress( true );

                loadTestDataOnScreen( exportTestData );

                new Thread( () ->
                {
                    File[] chartImageArray;

                    if ( exportTestData.getDielectricDischarge().size() > 0 )
                    {
                        chartImageArray = new File[4];
                        chartImageArray[3] = ui_chart_idd.saveChartImage( database.getTemporaryFolder() );
                    }
                    else
                    {
                        chartImageArray = new File[3];
                    }

                    chartImageArray[0] = ui_chart_ohms.saveChartImage( database.getTemporaryFolder() );
                    chartImageArray[1] = ui_chart_volts.saveChartImage( database.getTemporaryFolder() );
                    chartImageArray[2] = ui_chart_amps.saveChartImage( database.getTemporaryFolder() );

                    report.generate
                            (
                                    exportTestData,
                                    device,
                                    database.getPictureFile( exportTestData.getDeviceInfo().getSerial(), exportTestData.getDate().getTime() ),
                                    database.getMapSnapshotFile( exportTestData.getDeviceInfo().getSerial(), exportTestData.getDate().getTime() ),
                                    chartImageArray
                            );
                } ).start();
                closeOverlay();
            }
        }
    }

    /***********************************************************************************************
     * REGION Handler
     ***********************************************************************************************/

    public static class mHandler extends Handler
    {
        private final WeakReference<Controller>
                mClass;

        static final int
                REPORT_UPDATE   = 0,
                REPORT_FINISHED = 1,
                REPORT_ERROR    = 2;

        public mHandler( Controller c )
        {
            mClass = new WeakReference<>( c );
        }

        @Override
        public void handleMessage( @NonNull android.os.Message msg )
        {
            final Controller c = mClass.get();

            switch( msg.what )
            {
                case mHandler.REPORT_UPDATE:
                    c.dlgs.updateProgressBar( (String) msg.obj );
                    break;
                case mHandler.REPORT_FINISHED:
                    c.dlgs.showProgress( false );
                    c.loadTestDataOnScreen( c.openedTestData );
                    break;
                case mHandler.REPORT_ERROR:
                    c.dlgs.showProgress( false );
                    c.loadTestDataOnScreen( c.openedTestData );
                    break;
            }
        }
    }

    /***********************************************************************************************
     * GPS Interface
     ***********************************************************************************************/

    @Override
    public void GpsData( int which, String[] data )
    {
        switch ( which )
        {
            case GPS.RESULT:
                if ( data != null && openedTestData != null )
                {
                    openedTestData.setGpsLongitude( Double.parseDouble(data[0]) );
                    openedTestData.setGpsLatitude ( Double.parseDouble(data[1]) );
                    openedTestData.setGpsAltitude ( Double.parseDouble(data[2]) );
                    openedTestData.setGpsAcuraccy ( Double.parseDouble(data[3]) );
                    database.saveOrUpdate( openedTestData );
                }
                mCommonUI.changeButtonImage( ui_ibtn_gps, R.drawable.ic_gps_states, true );
                break;
            case GPS.WAIT:
                mCommonUI.changeButtonImage( ui_ibtn_gps, R.drawable.ic_wait, false );
                break;
            case GPS.CANCELLED:
                mCommonUI.changeButtonImage( ui_ibtn_gps, R.drawable.ic_gps_states, true );
                break;
        }
    }

    private void map()
    {
        if ( openedTestData != null && openedTestData.getGpsAcuraccy() > 0 )
        {
            MapData mapData = new MapData(
                    openedTestData.getTestNumber() + "",
                    openedTestData.getGpsLongitude(),
                    openedTestData.getGpsLatitude(),
                    openedTestData.getGpsAltitudeAsString(),
                    openedTestData.getGpsAccuracyAsString()
            );
            map.showMap( database.getMapSnapshotFile( openedTestData.getDeviceInfo().getSerial(), openedTestData.getDate().getTime() ), mapData );
        }
    }

    private void getGPS()
    {
        if ( openedTestData == null )
        {
            return;
        }

        if ( openedTestData.getGpsAcuraccy() == 0 )
        {
            mGPS.getDataGPS();
        }
        else
        {
            String[] gps = new String[]{
                    openedTestData.getGpsLongitudeAsString(),
                    openedTestData.getGpsLatitudeAsString(),
                    openedTestData.getGpsAltitudeAsString(),
                    openedTestData.getGpsAccuracyAsString()
            };
            mGPS.dialogConfirmReplace( gps );
        }
    }

    /***********************************************************************************************
     * REGION Audio
     ***********************************************************************************************/

    private void recordAudio()
    {
        if ( openedTestData != null )
        {
            database.createFolder( fragmentActivity, openedTestData.getDeviceInfo().getModel() );
            File location = database.getAudioFile( openedTestData.getDeviceInfo().getSerial(), openedTestData.getDate().getTime() );
            audio.setFile( location );
            audio.showDialog( openedTestData.getTestNumber() + "" );
        }
    }

    /***********************************************************************************************
     * Picture Interface
     ***********************************************************************************************/

    @Override
    public void pictureStatus( int status )
    {
        fragmentActivity.runOnUiThread( () ->
        {
            switch ( status )
            {
                case Picture.WAIT:
                    mCommonUI.changeButtonImage( R.id.bt_picture, R.drawable.ic_wait, false );
                    break;
                case Picture.FINISHED:
                    mCommonUI.changeButtonImage( R.id.bt_picture, R.drawable.ic_picture_states, true );
                    break;
            }
        } );
    }

    private void takePicture()
    {
        if ( openedTestData != null )
        {
            database.createFolder( fragmentActivity, openedTestData.getDeviceInfo().getModel() );
            File file = database.getPictureFile( openedTestData.getDeviceInfo().getSerial(), openedTestData.getDate().getTime() );
            picture.showDialog( file );
        }
    }

    /***********************************************************************************************
     * CommInterface
     ***********************************************************************************************/

    @Override
    public void addMessage( Message message )
    {
        ((Main) fragmentActivity).addMessage( message );
    }

    /***********************************************************************************************
     * Dialogs Interface
     ***********************************************************************************************/

    public void dialogsResponse( int whichDialog, Object arg )
    {
        switch ( whichDialog )
        {
            case DLG_CONF_VOLTS:
            {
                if ( arg instanceof Integer )
                {
                    int val = (int) arg;
                    if ( val != device.getConfVolts() )
                    {
                        device.setConfVolts( val );
                        device.writeConfVolts();
                    }
                }
                break;
            }
        }
    }

    /***********************************************************************************************
     * Observer
     ***********************************************************************************************/

    @SuppressLint("SetTextI18n")
    @Override
    public void update( Observable o, Object arg )
    {
        if ( o instanceof Device && arg instanceof Device.Changed )
        {
            Device.Changed whatChanged = (Device.Changed) arg;
            Log.e( TAG,  whatChanged.toString() );

            switch ( whatChanged )
            {
                case DEVICE_BASIC:
                {
                    userMessage( fragmentActivity.getString( R.string.ready ), CommonUI.COLOR_GREEN, false );

                    ui_general_spn_auto_connection.setSelection( device.getAutoConn() ? 0 : 1 );

                    switch ( device.getDateFormat() )
                    {
                        case Modbus.DATE_DD_MM_YY:
                            ui_general_spn_date_format.setSelection( 0 );
                            break;
                        case Modbus.DATE_MM_DD_YY:
                            ui_general_spn_date_format.setSelection( 1 );
                            break;
                        default:
                            ui_general_spn_date_format.setSelection( 2 );
                            break;
                    }

                    switch ( device.getTimeFormat() )
                    {
                        case Modbus.TIME_24H:
                            ui_general_spn_time_format.setSelection( 0 );
                            break;
                        default:
                            ui_general_spn_time_format.setSelection( 1 );
                            break;
                    }

                    switch ( device.getLanguage() )
                    {
                        case Modbus.LANG_SPANISH:
                            ui_general_spn_language.setSelection( 1 );
                            break;
                        case Modbus.LANG_PORTUGUESE:
                            ui_general_spn_language.setSelection( 2 );
                            break;
                        default:
                            ui_general_spn_language.setSelection( 0 );
                            break;
                    }

                    ui_general_txw_date.setText( device.getDateAsString() );
                    ui_general_txw_time.setText( device.getTimeAsString() );
                    showOverlay( VIEW_GENERAL );
                    break;
                }
                case FREE_MEMORY:
                {
                    ui_general_txw_free_memory.setText( device.getFreeMemory() );
                }
                case VOLTAGE:
                {
                    ui_main_screen.ui_lb_voltage.setText( device.getConfVoltsAsString() );
                }
                case SETUP:
                {
                    String[] modes = fragmentActivity.getResources().getStringArray( R.array.op_modes );
                    if ( device.getMode() < modes.length )
                    {
                        ui_main_screen.ui_lb_mode.setText( modes[device.getMode()] );
                    }
                    ui_main_screen.ui_lb_result_dd.setVisibility( device.fwHasDD() ? View.VISIBLE : View.GONE );
                    pagerAdapter.setIDDEnabled( device.fwHasIDD() );
                    dlgTestSetup( device );
                    break;
                }
                case PARAM_MANUAL:
                {
                    if ( device.getMode() == device.MODE_MANUAL )
                    {
                        ui_main_screen.ui_lb_voltage.setText( device.getConfVoltsAsString() );
                        ui_main_screen.ui_lb_max_voltage.setText( device.getParamManual().getMaxVoltsAsString() );
                        ui_main_screen.ui_lb_timer.setText( "- - -" );
                        ui_main_screen.ui_lb_lim.setText( device.getParamManual().getLim() > 0 ? device.getParamManual().getLimAsString() : "- - -" );
                        ui_main_screen.ui_lb_svt.setText( "- - -" );
                    }
                    dlgTestSetup( device.getParamManual() );
                    break;
                }
                case PARAM_TIMER:
                {
                    if ( device.getMode() == device.MODE_TIMER )
                    {
                        ui_main_screen.ui_lb_voltage.setText( device.getConfVoltsAsString() );
                        ui_main_screen.ui_lb_max_voltage.setText( device.getParamTimer().getMaxVoltsAsString() );
                        ui_main_screen.ui_lb_timer.setText( device.getParamTimer().getTimeAsString() );
                        ui_main_screen.ui_lb_lim.setText( device.getParamTimer().getLim() > 0 ? device.getParamTimer().getLimAsString() : "- - -" );
                        ui_main_screen.ui_lb_svt.setText( "- - -" );
                    }
                    dlgTestSetup( device.getParamTimer() );
                    break;
                }
                case PARAM_SVT:
                {
                    if ( device.getMode() == device.MODE_SVT )
                    {
                        ui_main_screen.ui_lb_voltage.setText( "- - -" );
                        ui_main_screen.ui_lb_max_voltage.setText( device.getParamSVT().getFinalVoltsAsString() );
                        ui_main_screen.ui_lb_timer.setText( "- - -" );
                        ui_main_screen.ui_lb_lim.setText( device.getParamSVT().getLim() > 0 ? device.getParamSVT().getLimAsString() : "- - -" );
                        ui_main_screen.ui_lb_svt.setText( device.getParamSVT().getStepVoltsAsString() + " @ " + device.getParamSVT().getStepTimeAsString() );
                    }
                    dlgTestSetup( device.getParamSVT() );
                    break;
                }
                case PARAM_RAMP:
                {
                    if ( device.getMode() == device.MODE_RAMP )
                    {
                        ui_main_screen.ui_lb_voltage.setText( "- - -" );
                        ui_main_screen.ui_lb_max_voltage.setText( device.getParamRamp().getFinalVoltsAsString() );
                        ui_main_screen.ui_lb_timer.setText( device.getParamRamp().getTimeAsString() );
                        ui_main_screen.ui_lb_lim.setText( device.getParamRamp().getLim() > 0 ? device.getParamRamp().getLimAsString() : "- - -" );
                        ui_main_screen.ui_lb_svt.setText( "- - -" );
                    }
                    dlgTestSetup( device.getParamRamp() );
                    break;
                }
                case BATTERY_LEVEL:
                {
                    mCommonUI.batteryIcon( R.id.ic_battery, device.getBatteryLevel() );
                    break;
                }
                case PRINTER:
                {
                    int drawableId = device.getPrinterEnabled() ? R.drawable.ic_printer_on_states : R.drawable.ic_printer_off_states;
                    Drawable img = ResourcesCompat.getDrawable( fragmentActivity.getResources(), drawableId, null );
                    if ( img != null )
                    {
                        ui_ibtn_printer.setImageDrawable( img );
                    }
                    break;
                }
                case FILTER:
                {
                    int drawableId = device.getFilterEnabled() ? R.drawable.ic_filter_on_states : R.drawable.ic_filter_off_states;
                    Drawable img = ResourcesCompat.getDrawable( fragmentActivity.getResources(), drawableId, null );
                    if ( img != null )
                    {
                        ui_ibtn_filter.setImageDrawable( img );
                    }
                    break;
                }
                case NOT_RESPONDING:
                {
                    userMessage( fragmentActivity.getString( R.string.not_responding, device.info().getModel() ), CommonUI.COLOR_YELLOW, true );
                    break;
                }
                case BUSY:
                {
                    userMessage( fragmentActivity.getString( R.string.wait ), CommonUI.COLOR_YELLOW, true );
                    ui_btn_start.setEnabled( false );
                    ui_ibtn_test_setup.setEnabled( false );
                    ui_ibtn_printer.setEnabled( false );
                    ui_ibtn_filter.setEnabled( false );
                    ui_btn_voltage.setEnabled( false );
                    ui_menu_item_options.setEnabled( false );
                    break;
                }
                case AUTO_PILOT:
                case STANDY_BY:
                case TEST_FINISHED:
                case TEST_CANCELLED:
                case WAITING_CAP:
                case WAITING_DD:
                {
                    ui_btn_start.setVisibility( View.VISIBLE );
                    ui_btn_start.setEnabled( !device.isAutoPilot() );
                    ui_btn_stop.setVisibility( View.GONE );
                    ui_ibtn_printer.setEnabled( !device.isAutoPilot() );
                    ui_ibtn_filter.setEnabled( !device.isAutoPilot() );
                    ui_ibtn_test_setup.setEnabled( !device.isAutoPilot() );
                    ui_btn_voltage.setEnabled( ( device.getMode() == device.MODE_MANUAL || device.getMode() == device.MODE_TIMER ) && !device.isAutoPilot() );
                    ui_menu_item_options.setEnabled( true );
                    ui_ibtn_file_manager.setEnabled( true );
                    userMessage( fragmentActivity.getString( R.string.ready ), CommonUI.COLOR_GREEN, false );
                    if ( whatChanged == Changed.TEST_CANCELLED )
                    {
                        dlgs.finishedTest( false, "", fragmentActivity.getString( R.string.test_canceled ) );
                    }
                    else if ( whatChanged == Changed.WAITING_CAP )
                    {
                        dlgs.finishedTest( true, "", fragmentActivity.getString( R.string.md_calculating_capacitance ) );
                    }
                    else if ( whatChanged == Changed.WAITING_DD )
                    {
                        dlgs.finishedTest( true, "", fragmentActivity.getString( R.string.calculating_dd ) );
                    }
                    else if ( whatChanged == Changed.TEST_FINISHED )
                    {
                        dlgs.finishedTest( false, "", fragmentActivity.getString( R.string.md_test_finished ) );
                        if ( device.getTestData() != null && device.getTestData().getMeasure().size() > 0 )
                        {
                            ui_ibtn_save.setEnabled( true );
                        }
                    }
                    else if ( whatChanged == Changed.AUTO_PILOT )
                    {
                        if( device.isAutoPilot() )
                        {
                            dlgs.autoPilot();
                        }
                    }

                    break;
                }
                case TEST_STARTING:
                {
                    //ui_ibtn_save.setEnabled( false );
                    //ui_btn_device_setup.setEnabled( false );
                    ui_btn_start.setVisibility( View.GONE );
                    ui_btn_stop.setVisibility( View.VISIBLE );
                    ui_ibtn_test_setup.setEnabled( false );
                    ui_ibtn_printer.setEnabled( false );
                    ui_ibtn_filter.setEnabled( false );
                    ui_btn_voltage.setEnabled( ( device.getMode() == device.MODE_MANUAL || device.getMode() == device.MODE_TIMER ) && !device.isAutoPilot() );
                    ui_menu_item_options.setEnabled( false );
                    userMessage( fragmentActivity.getString( R.string.md_warning_test_start ), CommonUI.COLOR_YELLOW, true );
                    break;
                }
                case TEST_RUNNING:
                {
                    ui_ibtn_save.setEnabled( false );
                    ui_ibtn_file_manager.setEnabled( false );
                    ui_ibtn_voice.setEnabled( false );
                    ui_ibtn_picture.setEnabled( false );
                    ui_ibtn_gps.setEnabled( false );
                    ui_ibtn_map.setEnabled( false );
                    ui_btn_start.setVisibility( View.GONE );
                    ui_btn_stop.setVisibility( View.VISIBLE );
                    ui_ibtn_test_setup.setEnabled( false );
                    ui_ibtn_printer.setEnabled( !device.isAutoPilot() );
                    ui_ibtn_filter.setEnabled( !device.isAutoPilot() );
                    ui_btn_voltage.setEnabled( ( device.getMode() == device.MODE_MANUAL || device.getMode() == device.MODE_TIMER ) && !device.isAutoPilot() );
                    ui_menu_item_options.setEnabled( false );

                    openedTestData = null;

                    userMessage( fragmentActivity.getString( R.string.md_test_running ), CommonUI.COLOR_RED, true );
                    break;
                }
                case TEST_NUMBER:
                {
                    updateUiRealTime( null );
                    updateChartsAndTable( null );
                    ui_main_screen.ui_lb_result_capacitance.setText( "- - - " );
                    ui_main_screen.ui_lb_result_dd.setText( "- - - " );
                    ui_main_screen.ui_lb_result_dai.setText( "- - - " );
                    ui_main_screen.ui_lb_result_pi.setText( "- - - " );
                    ui_main_screen.ui_lb_result_svt.setText( "- - - " );
                    ui_main_screen.ui_lb_realtime_test_number.setText( device.getTestData().getTestNumber() + "" );
                    ui_table.getHeader().setTestNumber( device.getTestData().getTestNumber() );
                    String[] modes = fragmentActivity.getResources().getStringArray( R.array.op_modes );
                    if ( device.getMode() < modes.length )
                    {
                        ui_table.getHeader().setMode( modes[device.getMode()] );
                    }
                    ui_table.getHeader().setDate( device.getTestData().getDateAsString( device.getDateTimeFormatPattern() ) );
                    ui_table.getHeader().setDDEnabled( device.fwHasDD() );
                    ui_table.getHeader().setDaiPiEnabled( device.getMode() == device.MODE_MANUAL || device.getMode() == device.MODE_TIMER );
                    ui_table.getHeader().setSVTEnabled( device.getMode() == device.MODE_SVT );
                    if ( device.getConfDAI() != null )
                    {
                        ui_table.getHeader().setTitleDAI( device.getConfDAI().getTimeA() + "/" + device.getConfDAI().getTimeB() );
                    }
                    if ( device.getConfPI() != null )
                    {
                        ui_table.getHeader().setTitlePI( device.getConfPI().getTimeA() + "/" + device.getConfPI().getTimeB() );
                    }
                    break;
                }
                case RT_MEASURE:
                {
                    Measure m = device.getRtMeasurement();
                    updateUiRealTime( m );
                    updateChartsAndTable( m );
                    break;
                }
                case RT_IDD:
                {
                    DielectricDischarge idd = device.getRtIDD();
                    dlgs.finishedTest( true, "IDD= " + idd.getAmpsAsString() );
                    updateChartsAndTable( idd );
                    break;
                }
                case DAI_READY:
                {
                    ui_main_screen.ui_lb_result_dai.setText( device.getTestData().getDAI().getResultAsString() );
                    ui_table.getHeader().setDAI( device.getTestData().getDAI().getResultAsString() );
                    break;
                }
                case PI_READY:
                {
                    ui_main_screen.ui_lb_result_pi.setText( device.getTestData().getPI().getResultAsString() );
                    ui_table.getHeader().setPI( device.getTestData().getPI().getResultAsString() );
                    break;
                }
                case SVT_READY:
                {
                    ui_main_screen.ui_lb_result_svt.setText( device.getTestData().getSVTString() );
                    ui_table.getHeader().setSVT( device.getTestData().getSVTString() );
                    break;
                }
                case CAP_READY:
                {
                    ui_main_screen.ui_lb_result_capacitance.setText( device.getTestData().getCapacitanceAsString() );
                    ui_table.getHeader().setCAP( device.getTestData().getCapacitanceAsString() );
                    break;
                }
                case DD_READY:
                {
                    ui_main_screen.ui_lb_result_dd.setText( device.getTestData().getDDString() );
                    ui_table.getHeader().setDD( device.getTestData().getDDString() );
                    break;
                }
            }
        }
    }
}

    