package com.megabras.bluelogg.MPKXX5R;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.TextView;

import com.megabras.bluelogg.R;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

public class TableBaseAdapter extends BaseAdapter
{
    private LayoutInflater
            inflater;

    private List<TableValues>
            tableValues;

    @SuppressWarnings("FieldCanBeLocal")
    private int
            layoutID   = R.layout.mpkxx5r_table_item,
            headerID   = R.layout.mpkxx5r_table_header;

    private List<TableHeaderValues>
            headerValues;

    public TableBaseAdapter( Context context )
    {
        tableValues  = new ArrayList<>();
        headerValues = new ArrayList<>();
        inflater     = LayoutInflater.from( context );
    }

    static class ViewHolder
    {
        TextView
                ui_txw_elapsed,
                ui_txw_ohm,
                ui_txw_amp,
                ui_txw_loop;
    }

//    public void updateLastHeader(int testNumber, String date, String temperature, String metal)
//    {
//        if( headerValues.size() > 0 )
//        {
//            headerValues.get( headerValues.size() - 1 ).testNumber = testNumber;
//            headerValues.get( headerValues.size() - 1 ).date = date;
//            headerValues.get( headerValues.size() - 1 ).temperatureSet = temperature;
//            headerValues.get( headerValues.size() - 1 ).temperatureSet = metal;
//            //notifyDataSetChanged();
//        }
//    }

    public void setTestNumber( int testNumber )
    {
        if ( headerValues.size() == 0 || headerValues.get( headerValues.size() - 1 ).testNumber != testNumber )
        {
            TableHeaderValues values = new TableHeaderValues( tableValues.size(), testNumber );
            headerValues.add( values );
        }
    }

    public int getTestNumber()
    {
        return headerValues.get( headerValues.size() - 1 ).testNumber;
    }

    void setHeaderInfo( String date, String mode, String timer, String bsg )
    {
        int last = headerValues.size() - 1;
        if ( last > -1 )
        {
            headerValues.get( last ).date  = date;
            headerValues.get( last ).mode  = mode;
            headerValues.get( last ).timer = timer;
            headerValues.get( last ).bsg   = bsg;
            notifyDataSetChanged();
        }
    }

    public long getItemId( int position )
    {
        return 0;
    }

    public int getCount()
    {
        return tableValues.size();
    }

    public TableValues getItem( int position )
    {
        return tableValues.get( position );
    }

    void addAll( List<TableValues> tableValues )
    {
        this.tableValues = tableValues;
        notifyDataSetChanged();
    }

    public void add( String elapsed, String ohm, String amp, String loop )
    {
        tableValues.add( new TableValues( elapsed, ohm, amp, loop ) );
    }

    public void clear()
    {
        headerValues.clear();
        tableValues.clear();
        notifyDataSetChanged();
    }

    public void repaint()
    {
        notifyDataSetChanged();
    }

    private TextView getTV( int id, View v )
    {
        return ((TextView) v.findViewById( id ));
    }

    private TableHeaderValues getHeaderValues( int position )
    {
        for ( TableHeaderValues val : headerValues )
        {
            if ( val.index == position )
                return val;
        }
        return null;
    }

    @SuppressWarnings("RedundantCast")
    public View getView( int position, View vi, ViewGroup parent)
    {
        TableHeaderValues val = getHeaderValues( position );
        if( val != null )
        {
            vi = inflater.inflate( headerID, parent, false );
            getTV( R.id.ui_txw_number, vi ).setText( String.format( Locale.US, "%d", val.testNumber ) );
            getTV( R.id.ui_txw_date  , vi ).setText( val.date );
            getTV( R.id.ui_txw_mode  , vi ).setText( val.mode );
            getTV( R.id.ui_txw_timer , vi ).setText( val.timer );
            getTV( R.id.ui_txw_bsg   , vi ).setText( val.bsg );

            getTV( R.id.ui_txw_elapsed, vi ).setText( tableValues.get(position).time );
            getTV( R.id.ui_txw_ohm    , vi ).setText( tableValues.get(position).resistance );
            getTV( R.id.ui_txw_amp    , vi ).setText( tableValues.get(position).current );
            getTV( R.id.ui_txw_loop   , vi ).setText( tableValues.get(position).loop );
        }
        else
        {
            if (vi == null || (int)vi.getId() != layoutID)
            {
                ViewHolder h = new ViewHolder();
                vi = inflater.inflate( layoutID, parent, false );
                h.ui_txw_elapsed = getTV( R.id.tv_elapsed, vi );
                h.ui_txw_ohm     = getTV( R.id.tv_resistance, vi );
                h.ui_txw_amp     = getTV( R.id.tv_current, vi );
                h.ui_txw_loop    = getTV( R.id.tv_loop, vi );
                vi.setTag( h );
            }
            ViewHolder h = (ViewHolder) vi.getTag();
            h.ui_txw_elapsed.setText( tableValues.get(position).time );
            h.ui_txw_ohm.setText    ( tableValues.get(position).resistance );
            h.ui_txw_amp.setText    ( tableValues.get(position).current );
            h.ui_txw_loop.setText   ( tableValues.get(position).loop );
        }

        return vi;
    }

}

