package com.megabras.bluelogg;

import android.Manifest;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.os.CountDownTimer;
import android.os.Handler;
import android.os.Message;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.TextView;

import com.megabras.DeviceList;
import com.megabras.bluelogg.bluetooth.BluetoothBLE;
import com.megabras.bluelogg.bluetooth.BluetoothClassic;
import com.megabras.bluelogg.bluetooth.BluetoothDev;
import com.megabras.bluelogg.bluetooth.EnableBluetooth;
import com.megabras.bluelogg.extended.CommonUI;
import com.megabras.bluelogg.extended.LogCat;
import com.megabras.bluelogg.extended.MDialog;
import com.megabras.modbus.Modbus;

import java.lang.ref.WeakReference;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

public class Main extends AppCompatActivity
{
    @SuppressWarnings("unused")
    private final String
            TAG     = "Main";

    private String
            mac     = "",
            model   = "",
            serial  = "";

    private SharedPreferences
            sharedPref;

    private CommonUI
            mCommonUI;

    private BluetoothDev
            bluetooth = null;

    private mHandler
            handler;

    private Menu
            menu;

    private Controller
            controller;

    private int
            stateLevel = 0,
            BT_TIMEOUT = 600,
            BT_CYCLE   = 50;

    public static int
            equipmentId;

    public CountDown
            countDown;

    private LogCat
            log = new LogCat();

    // Requesting permission to RECORD_AUDIO
    public static boolean
            permissionToRecordAccepted  = false,
            permissionToPictureAccepted = false,
            permissionToGPSAccepted     = false,
            permissionToMemoryAccepted  = false;

    public static String []
            permissionsMemory   = {Manifest.permission.READ_EXTERNAL_STORAGE, Manifest.permission.WRITE_EXTERNAL_STORAGE},
            permissionsAudio    = {Manifest.permission.READ_EXTERNAL_STORAGE, Manifest.permission.WRITE_EXTERNAL_STORAGE, Manifest.permission.RECORD_AUDIO},
            permissionsPicture  = {Manifest.permission.READ_EXTERNAL_STORAGE, Manifest.permission.WRITE_EXTERNAL_STORAGE, Manifest.permission.CAMERA},
            permissionsGPS      = {Manifest.permission.ACCESS_COARSE_LOCATION, Manifest.permission.ACCESS_FINE_LOCATION};

    public static final int
            REQUEST_RECORD_AUDIO_PERMISSION  = 200,
            REQUEST_PICTURE_PERMISSION       = 201,
            REQUEST_GPS_PERMISSION           = 202,
            REQUEST_MEMORY_PERMISSION        = 203,
            REQUEST_MEMORY_REPORT_PERMISSION = 204,
            REQUEST_MEMORY_MAP_PERMISSION    = 205;

    private void checkPermissions()
    {
        if (ContextCompat.checkSelfPermission(this, Manifest.permission.READ_EXTERNAL_STORAGE)==PackageManager.PERMISSION_GRANTED &&
            ContextCompat.checkSelfPermission(this, Manifest.permission.WRITE_EXTERNAL_STORAGE)==PackageManager.PERMISSION_GRANTED)
        {
            permissionToMemoryAccepted = true;
        }
        if (ContextCompat.checkSelfPermission(this, Manifest.permission.READ_EXTERNAL_STORAGE)==PackageManager.PERMISSION_GRANTED &&
            ContextCompat.checkSelfPermission(this, Manifest.permission.WRITE_EXTERNAL_STORAGE)==PackageManager.PERMISSION_GRANTED &&
            ContextCompat.checkSelfPermission(this, Manifest.permission.CAMERA)==PackageManager.PERMISSION_GRANTED)
        {
            permissionToPictureAccepted = true;
        }
        if (ContextCompat.checkSelfPermission(this, Manifest.permission.READ_EXTERNAL_STORAGE)==PackageManager.PERMISSION_GRANTED &&
            ContextCompat.checkSelfPermission(this, Manifest.permission.WRITE_EXTERNAL_STORAGE)==PackageManager.PERMISSION_GRANTED &&
            ContextCompat.checkSelfPermission(this, Manifest.permission.RECORD_AUDIO)==PackageManager.PERMISSION_GRANTED)
        {
            permissionToRecordAccepted = true;
        }
        if (ContextCompat.checkSelfPermission(this, Manifest.permission.ACCESS_COARSE_LOCATION)==PackageManager.PERMISSION_GRANTED &&
            ContextCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION)==PackageManager.PERMISSION_GRANTED)
        {
            permissionToGPSAccepted = true;
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults)
    {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        switch (requestCode){
            case REQUEST_RECORD_AUDIO_PERMISSION:
                permissionToRecordAccepted  = (grantResults[0] == PackageManager.PERMISSION_GRANTED && grantResults[1] == PackageManager.PERMISSION_GRANTED && grantResults[2] == PackageManager.PERMISSION_GRANTED);
                if (permissionToRecordAccepted)
                    controller.permissionCallback(requestCode);
                break;
            case REQUEST_PICTURE_PERMISSION:
                permissionToPictureAccepted  = (grantResults[0] == PackageManager.PERMISSION_GRANTED && grantResults[1] == PackageManager.PERMISSION_GRANTED && grantResults[2] == PackageManager.PERMISSION_GRANTED);
                if (permissionToPictureAccepted)
                    controller.permissionCallback(requestCode);
                break;
            case REQUEST_GPS_PERMISSION:
                permissionToGPSAccepted = (grantResults[0] == PackageManager.PERMISSION_GRANTED && grantResults[1] == PackageManager.PERMISSION_GRANTED);
                if (permissionToGPSAccepted)
                    controller.permissionCallback(requestCode);
                break;
            case REQUEST_MEMORY_PERMISSION:
                permissionToMemoryAccepted = (grantResults[0] == PackageManager.PERMISSION_GRANTED && grantResults[1] == PackageManager.PERMISSION_GRANTED);
                if (permissionToMemoryAccepted)
                    controller.permissionCallback(requestCode);
                break;
            case REQUEST_MEMORY_REPORT_PERMISSION:
                permissionToMemoryAccepted = (grantResults[0] == PackageManager.PERMISSION_GRANTED && grantResults[1] == PackageManager.PERMISSION_GRANTED);
                if (permissionToMemoryAccepted)
                    controller.permissionCallback(requestCode);
                break;
            case REQUEST_MEMORY_MAP_PERMISSION:
                permissionToMemoryAccepted = (grantResults[0] == PackageManager.PERMISSION_GRANTED && grantResults[1] == PackageManager.PERMISSION_GRANTED);
                if (permissionToMemoryAccepted)
                    controller.permissionCallback(requestCode);
                break;
        }
    }

    public void requestPermission(int permission)
    {
        String[] list = null;
        switch (permission)
        {
            case REQUEST_RECORD_AUDIO_PERMISSION:
                list = permissionsAudio;
                break;
            case REQUEST_PICTURE_PERMISSION:
                list = permissionsPicture;
                break;
            case REQUEST_GPS_PERMISSION:
                list = permissionsGPS;
                break;
            case REQUEST_MEMORY_PERMISSION:
                list = permissionsMemory;
                break;
            case REQUEST_MEMORY_REPORT_PERMISSION:
                list = permissionsMemory;
                break;
            case REQUEST_MEMORY_MAP_PERMISSION:
                list = permissionsMemory;
                break;
        }
        if (list!=null)
            ActivityCompat.requestPermissions(this, list, permission);
    }

//    @Override
//    public void onWindowFocusChanged(boolean hasFocus) {
//        super.onWindowFocusChanged(hasFocus);
//        if (hasFocus) {
//            hideSystemUI();
//        }
//    }
//
//    private void hideSystemUI() {
//        // Enables regular immersive mode.
//        // For "lean back" mode, remove SYSTEM_UI_FLAG_IMMERSIVE.
//        // Or for "sticky immersive," replace it with SYSTEM_UI_FLAG_IMMERSIVE_STICKY
//        View decorView = getWindow().getDecorView();
//        decorView.setSystemUiVisibility(
//                View.SYSTEM_UI_FLAG_LAYOUT_STABLE
//                        | View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
//                        | View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
//                        | View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
//                        | View.SYSTEM_UI_FLAG_FULLSCREEN
//                        | View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY);
//
//        //NOT IN HERE
////        decorView.setOnSystemUiVisibilityChangeListener
////                (new View.OnSystemUiVisibilityChangeListener() {
////                    @Override
////                    public void onSystemUiVisibilityChange(int visibility) {
////                        // Note that system bars will only be "visible" if none of the
////                        // LOW_PROFILE, HIDE_NAVIGATION, or FULLSCREEN flags are set.
////                        if ((visibility & View.SYSTEM_UI_FLAG_FULLSCREEN) == 0) {
////                            //  The system bars are visible. Make any desired
////                            // adjustments to your UI, such as showing the action bar or
////                            // other navigational controls.
////                        } else {
////                            //  The system bars are NOT visible. Make any desired
////                            // adjustments to your UI, such as hiding the action bar or
////                            // other navigational controls.
////                        }
////                    }
////                });
//
//    }


    @Override
    public void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main_view);

        checkPermissions();

        mCommonUI = new CommonUI(this);
        mCommonUI.keepTheScreenOn(true);
        mCommonUI.turnOffLED(R.id.status_led);

        //Retrieve equipment data
        Bundle extras = getIntent().getExtras();
        if (extras != null) {
            mac         = MUtils.checkNull(extras.getString("MAC"));
            serial      = MUtils.checkNull(extras.getString("SERIAL"));
            model       = MUtils.checkNull(extras.getString("MODEL"));
            equipmentId = extras.getInt("ID");

            String equipment = model + " " + serial;
            ((TextView) findViewById(R.id.lb_model)).setText(equipment);
            try
            {
                String versionName = "v"+getPackageManager().getPackageInfo(getPackageName(), 0).versionName;
                ((TextView) findViewById(R.id.lb_version)).setText(versionName);
                if (getSupportActionBar()!=null)
                {
                    getSupportActionBar().setDisplayShowHomeEnabled(true);
                    getSupportActionBar().setIcon(R.drawable.brand_small);
                    getSupportActionBar().setDisplayShowTitleEnabled(false);
                    getSupportActionBar().setElevation(0);
                }
            }
            catch (Exception e)
            {
                findViewById(R.id.lb_version).setVisibility(View.GONE);
            }
        }

        handler   = new mHandler(this);
        countDown = new CountDown(20000,250, handler);
        //Retrieve shared preferences
        sharedPref = this.getSharedPreferences("BLUELOGG", Context.MODE_PRIVATE);
        //editor = sharedPref.edit();

        // EQUIPMENT SELECTOR
        switch ( equipmentId )
        {
            case 70: // DT70KV
                this.findViewById(R.id.ic_battery).setVisibility(View.GONE);
                controller = new com.megabras.bluelogg.DT70KV.Controller(this, mCommonUI, model, serial);
                controller.createControlButtons(findViewById(R.id.controls_buttons));
                controller.createSwipe();
                controller.setToolBarId(R.id.tool_bar);
                BT_CYCLE = 300;
                break;
            case 60000: // HP60KV
                this.findViewById(R.id.ic_battery).setVisibility(View.GONE);
                controller = new com.megabras.bluelogg.hp60kv.Controller(this, mCommonUI, model, serial);
                controller.createControlButtons(findViewById(R.id.controls_buttons));
                controller.createSwipe();
                controller.setToolBarId(R.id.tool_bar);
                BT_CYCLE   = 300; //BT_TIMEOUT = 1200;
                break;
            case 80: // HP80KV
                this.findViewById(R.id.ic_battery).setVisibility(View.GONE);
                controller = new com.megabras.bluelogg.hp80kv.Controller(this, mCommonUI, model, serial);
                controller.createControlButtons(findViewById(R.id.controls_buttons));
                controller.createSwipe();
                controller.setToolBarId(R.id.tool_bar);
                BT_CYCLE   = 300;
                break;
            case DeviceList.ID_HP30KV: // HP30KV
                this.findViewById(R.id.ic_battery).setVisibility(View.GONE);
                controller = new com.megabras.bluelogg.HP30KV.Controller(this, mCommonUI, model, serial);
                controller.createControlButtons(findViewById(R.id.controls_buttons));
                controller.createSwipe();
                controller.setToolBarId(R.id.tool_bar);
                BT_CYCLE = 300;
                break;
            case 256: //MPK256 or MPK257
            case 257:
                controller = new com.megabras.bluelogg.mpk257.Controller(this, mCommonUI, model, serial);
                controller.createControlButtons(findViewById(R.id.controls_buttons));
                controller.createSwipe();
                controller.setToolBarId(R.id.tool_bar);
                ((com.megabras.bluelogg.mpk257.Controller)controller).setTemperatureUnity( sharedPref.getInt("TEMP_UNITY", 0) );
                break;
            case 105: //MPK105R or MPK215R
            case 215:
                this.findViewById( R.id.ic_battery ).setVisibility( View.GONE );
                controller = new com.megabras.bluelogg.MPKXX5R.Controller( this, mCommonUI, model, serial );
                controller.createControlButtons( findViewById( R.id.controls_buttons ) );
                controller.createSwipe();
                controller.setToolBarId( R.id.tool_bar );
                break;
//            case 5000: // MD5KVR
//                controller = new com.megabras.bluelogg.MD5KVR.Controller(this, mCommonUI, model, serial);
//                controller.createControlButtons(findViewById(R.id.controls_buttons));
//                controller.createSwipe();
//                controller.setToolBarId(R.id.tool_bar);
//                BT_TIMEOUT = 300;
//                break;
//            case 10000: // MD10KVR
//                controller = new com.megabras.bluelogg.md10kvr.Controller(this, mCommonUI, model, serial);
//                controller.createControlButtons(findViewById(R.id.controls_buttons));
//                controller.createSwipe();
//                controller.setToolBarId(R.id.tool_bar);
//                BT_TIMEOUT = 300;
//                break;
//            case DeviceList.ID_MD15KVR: // MD15KVR
//                controller = new com.megabras.bluelogg.md15kvr.Controller( this, mCommonUI, model, serial );
//                controller.createControlButtons( findViewById( R.id.controls_buttons ) );
//                controller.createSwipe();
//                controller.setToolBarId( R.id.tool_bar );
//                BT_TIMEOUT = 300;
//                break;
            case DeviceList.ID_MD5KVR:
            case DeviceList.ID_MD10KVR:
            case DeviceList.ID_MD15KVR:
                controller = new com.megabras.bluelogg.MDXXKVR.Controller( this, mCommonUI, model, serial );
                controller.createControlButtons( findViewById( R.id.controls_buttons ) );
                controller.createSwipe();
                controller.setToolBarId( R.id.tool_bar );
                BT_TIMEOUT = 300;
                break;
            case 25: // TM25R
                controller = new com.megabras.bluelogg.tm25r.Controller(this, mCommonUI, model, serial);
                controller.createControlButtons(findViewById(R.id.controls_buttons));
                controller.createSwipe();
                controller.setToolBarId(R.id.tool_bar);
                BT_TIMEOUT = 1000;
                break;
        }

        controller.setAutoConn( sharedPref.getBoolean("autoConn", true) );
        controller.setDateTimeFormat(
                sharedPref.getInt("DATE_FORMAT", Modbus.DATE_YYYY_MM_DD),
                sharedPref.getInt("TIME_FORMAT", Modbus.TIME_24H)
        );

        message("",CommonUI.COLOR_BLACK, false);
    }

    @Override
    protected void onDestroy()
    {
        //Log.e(TAG, "onDestroy #########################################################################");
        this.closeApp();
        super.onDestroy();
    }

    @Override
    public void onBackPressed()
    {
        if(bluetooth!=null && bluetooth.getState()== BluetoothClassic.STATE_CONNECTING)
            return;
        MDialog.ExitApplication(this).show();
    }

    public void closeApp() {
        try {
            closeConnection();
//			if (shutter != null){
//				shutter.release();
//			}
            controller.onClose();
            this.finish();
        } catch (Exception e) {
            MUtils.error(this, "closeApp", e);
        }
    }

    private void openConnection( String mac, Handler handler, Controller controller )
    {
        BluetoothAdapter adaptador = BluetoothAdapter.getDefaultAdapter();
        if ( adaptador == null )
        {
            return;
        }
        adaptador.cancelDiscovery();
        BluetoothDevice device = adaptador.getRemoteDevice( mac );

        if ( bluetooth == null || bluetooth.getState() != BluetoothClassic.STATE_CONNECTED )
        {
            if ( device.getType() == BluetoothDevice.DEVICE_TYPE_LE )
            {
                bluetooth = new BluetoothBLE( this, device, handler, controller.statusMessage() );
            }
            else
            {
                bluetooth = new BluetoothClassic( this, device, handler, controller.statusMessage() );
            }
            bluetooth.connect( false );
        }

//        if ( bluetooth == null )
//        {
//            bluetooth = new BluetoothClassic( device, handler, controller.statusMessage() );
//            bluetooth.connect( false );
//        }
//        else
//        {
//            if ( bluetooth.getState() != BluetoothClassic.STATE_CONNECTED )
//            {
//                bluetooth = new BluetoothClassic( device, handler, controller.statusMessage() );
//                bluetooth.connect( false );
//            }
//        }
    }

    public void closeConnection()
    {
        try
        {
            if ( bluetooth != null )
            {
                bluetooth.stop();
            }
        }
        catch ( Exception e )
        {
            MUtils.error( this, "Disconnect", e );
        }
    }

    private static class mHandler extends Handler
    {
        private final WeakReference<Main>
                mActivity;

        public mHandler( Main activity )
        {
            mActivity = new WeakReference<>( activity );
        }

        @Override
        public void handleMessage( @NonNull Message msg )
        {
            final Main atv = mActivity.get();

            switch ( msg.what )
            {
                case BluetoothDev.MESSAGE_STATE_CHANGE:
                    int state = msg.arg1;

                    switch ( state )
                    {
                        case BluetoothClassic.STATE_CONNECTING:
                            atv.stateLevel = 0;
                            atv.menu.findItem(R.id.connect).setEnabled(false);
                            atv.message(atv.getString(R.string.connecting), CommonUI.COLOR_YELLOW, true);
                            break;
                        case BluetoothClassic.STATE_CONNECTED:
                            atv.stateLevel = 1;
                            atv.updateMenu(true);
                            atv.message(atv.getString(R.string.open_port), CommonUI.COLOR_YELLOW, true);
                            atv.bluetooth.setTimeout(atv.BT_TIMEOUT);
                            atv.bluetooth.setCycleTime(atv.BT_CYCLE);
                            break;
                        case BluetoothClassic.STATE_NOT_RESPONDING:
                            atv.mCommonUI.statusBlinkLEDnotRx(R.id.status_led);
                            if ( atv.bluetooth.getPrevState() == BluetoothClassic.STATE_NOT_RESPONDING )
                            {
                                atv.message( atv.getString( R.string.not_responding, atv.model ), CommonUI.COLOR_YELLOW, true );
                            }
                            break;
                        case BluetoothClassic.STATE_FAIL:
                            atv.updateMenu(false);
                            atv.message(atv.getString(R.string.check_powered_on, atv.model), CommonUI.COLOR_BLACK, false);
                            break;
                        case BluetoothClassic.STATE_LOST:
                            atv.updateMenu(false);
                            switch ( atv.stateLevel )
                            {
                                case 1:
                                    if(!atv.countDown.isFinished())
                                        atv.countDown.start();
                                    else
                                        atv.message(atv.getString(R.string.not_ready_yet, atv.model), CommonUI.COLOR_BLACK, false);
                                    break;
                                case 2:
                                    atv.message(atv.getString(R.string.connection_lost), CommonUI.COLOR_BLACK, false);
                                    break;
                            }
                            break;
                        case BluetoothClassic.STATE_CLOSED:
                            atv.updateMenu(false);
                            atv.message(atv.getString(R.string.disconnected), CommonUI.COLOR_BLACK, false);
                            break;
                    }
                    atv.controller.stateChanged( state );
                    break;
                case CountDown.STATE_TICK:
                    atv.message(atv.getString(R.string.not_read, atv.model, msg.obj), CommonUI.COLOR_YELLOW, true);
                    break;
                case CountDown.STATE_FINISH:
                    atv.openConnection(atv.mac, atv.handler, atv.controller);
                    break;
                case BluetoothDev.MESSAGE_READ:
                    atv.log.logcatRx(msg);
                    atv.stateLevel = 2;
                    atv.controller.processResponse(msg.arg2, (String) msg.obj);
                    atv.mCommonUI.statusBlinkLEDrx(R.id.status_led);
                    break;
                case BluetoothDev.MESSAGE_WRITE:
                    atv.log.logcatTx(msg);
                    atv.mCommonUI.statusBlinkLED(R.id.status_led);
                    break;
            }
        }
    }

    public void addMessage( com.megabras.modbus.Message message )
    {
        if ( bluetooth != null )
        {
            bluetooth.addMessage( message );
        }
    }

    public void addPriorityMessage( com.megabras.modbus.Message message )
    {
        if ( bluetooth != null )
        {
            bluetooth.priorityMessage( message );
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu)
    {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.main_menu, menu);
        this.menu = menu;
        menu.findItem(R.id.changedevice).setVisible(true);
        menu.findItem(R.id.disconnect).setVisible(false);
        menu.findItem(R.id.options).setEnabled(false);
        //Code execute here has the insurance that UI elements are inflated
        if (controller.getAutoConn())
        {
            openConnection(mac, handler, controller);
		}
        controller.setMenuItemOptions(menu, R.id.options);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected( @NonNull MenuItem item )
    {
        switch ( item.getItemId() )
        {
            case R.id.connect:
                countDown.reset();
                openConnection( mac, handler, controller );
                return true;
            case R.id.changedevice:
                selectAnotherDevice();
                return true;
            case R.id.disconnect:
                closeConnection();
                return true;
            case R.id.options:
                if ( bluetooth != null )
                {
                    controller.readGeneralConfig();
                }
                return true;
            default:
                return super.onOptionsItemSelected( item );
        }
    }

    public void selectAnotherDevice(){
        Intent intent = new Intent(this, EnableBluetooth.class);
        startActivity(intent);
        this.closeApp();
    }

    public void message(String message, int color, boolean progressBar)
    {
        mCommonUI.messageBox(message, color, progressBar);
    }

    public void updateBattery(int level)
    {
        mCommonUI.batteryIcon(R.id.ic_battery, level);
    }

    public void setFirmwareVersion(String firmware)
    {
        TextView ui_firmware = findViewById(R.id.lb_firmware);
        if (ui_firmware!=null)
            ui_firmware.setText(firmware);
    }

    public void updateAutoConn(boolean enabled){
        sharedPref.edit().putBoolean("autoConn", enabled).apply();
    }

    public void updateDateTimeFormat(int dateFormat, int timeFormat){
        sharedPref.edit().putInt("DATE_FORMAT", dateFormat).apply();
        sharedPref.edit().putInt("TIME_FORMAT", timeFormat).apply();
    }

    public void updateTemperatureUnity(int unity){
        sharedPref.edit().putInt("TEMP_UNITY", unity).apply();
    }

    protected void updateMenu(boolean connected){
        menu.findItem(R.id.connect).setVisible(!connected);
        menu.findItem(R.id.connect).setEnabled(!connected);
        menu.findItem(R.id.disconnect).setVisible(connected);
        menu.findItem(R.id.options).setVisible(connected);
        menu.findItem(R.id.options).setEnabled(connected);
        menu.findItem(R.id.changedevice).setVisible(!connected);
        if(connected) {
            mCommonUI.changeAlphaLED(R.id.status_led, CommonUI.LED_GREEN, true);
        } else {
            mCommonUI.turnOffLED(R.id.status_led);
            mCommonUI.batteryIcon(R.id.ic_battery, 0);
        }
    }

     // ############################################
    // CALLBACKs
    // ############################################

    public class CountDown extends CountDownTimer {
        @SuppressWarnings("unused")
        private final String
                TAG             = "CountDown";

        static final int
                STATE_TICK      = -1,
                STATE_FINISH    = -2;

//        private long
//                time;

        private boolean
                finished = false;

        private Handler
                handler;

        CountDown(final long time, long tick, final Handler handler) {
            super(time, tick);
            //this.time = time;
            this.handler = handler;
        }

        public void onTick(long millisUntilFinished) {
            this.handler.obtainMessage(STATE_TICK, -1, -1, (millisUntilFinished/1000)+"").sendToTarget();
            finished = false;
        }

        public void onFinish() {
            this.handler.obtainMessage(STATE_FINISH, -1, 1).sendToTarget();
            finished = true;
        }

        public boolean isFinished(){
            return finished;
        }

        public void reset(){
            finished = false;
        }
    }
}
