package com.megabras.bluelogg.bluetooth;

import android.app.Activity;
import android.bluetooth.BluetoothDevice;
import android.content.Intent;

import androidx.annotation.NonNull;

public class MBluetoothDevice implements Comparable<MBluetoothDevice>{


	private final Activity
			activity;

	private final String
			name,
			serial,
			targetClass;

	private final BluetoothDevice
			device;

	private boolean
			near = false;

	private final int
			id;

	private int
			type = -1;

	MBluetoothDevice( Activity activity, BluetoothDevice device, String targetClass, int id )
	{
		this.activity    = activity;
		this.device      = device;
        int i            = device.getName().lastIndexOf("-");
        this.name        = device.getName().substring(0, i);
        this.serial      = device.getName().substring(i+1);
		this.targetClass = targetClass;
		this.id          = id;
	}

	MBluetoothDevice( Activity activity, BluetoothDevice device, String model, String serial, String targetClass, int id, int type )
	{
		this.activity    = activity;
		this.device      = device;
		this.name        = model.trim();
		this.serial      = serial.trim();
		this.targetClass = targetClass;
		this.id          = id;
		this.type        = type;
	}

	void setIsNear( boolean near )
	{
		this.near = near;
	}

	boolean isNear()
	{
		return near;
	}

	public int getType()
	{
		if ( type > -1 )
		{
			return type;
		}
		else
		{
			return device.getType();
		}
	}

	public String getName()
	{
		return name;
	}

	public String getSerial()
	{
		return serial;
	}

	public String getAddress()
	{
		return device.getAddress();
	}

	public BluetoothDevice getDevice()
	{
		return device;
	}

	public void startActivity( boolean closeParent )
	{
		try
		{
			String PACKAGE_NAME = activity.getApplicationContext().getPackageName();

			Intent intent = new Intent( activity, Class.forName( PACKAGE_NAME + "." + targetClass ) );
			intent.putExtra("MAC"   , device.getAddress());
			intent.putExtra("SERIAL", serial);
			intent.putExtra("MODEL" , name);
			intent.putExtra("ID"    , id);
			activity.startActivity(intent);

			if ( closeParent )
			{
				activity.finish();
			}
		}
		catch ( Exception e )
		{
			e.printStackTrace();
		}
	}

	@Override
	public boolean equals(Object o) {
		try {
			if (o == this) return true;
			if (o == null) return false;
			if (!(o instanceof MBluetoothDevice)) return false;

			MBluetoothDevice dev = (MBluetoothDevice) o;
			return getDevice().getAddress().equals(dev.getDevice().getAddress()) &&
					name.equals(dev.name) &&
					serial.equals(dev.serial);
		}
		catch (Exception e){
			return false;
		}
	}

	@Override
	public int hashCode() {
		int result = 17;
		result     = 31 * result + name.hashCode();
		result     = 31 * result + serial.hashCode();
		result     = 31 * result + device.getAddress().hashCode();
		return result;
	}

	@NonNull
	@Override
	public String toString(){
		return "| NAME: " + name + " | SN: " + serial + " | MAC: " + device.getAddress() + " | EQUIPMENT: ";
	}

	@Override
	public int compareTo(@NonNull MBluetoothDevice o) {
		return this.name.compareToIgnoreCase(o.name);
	}
}
