package com.megabras.bluelogg.em4058;

import android.app.Activity;
import android.graphics.Typeface;
import android.graphics.pdf.PdfDocument;
import android.os.Handler;
import android.util.Log;

import com.megabras.bluelogg.MUtils;
import com.megabras.bluelogg.R;
import com.megabras.bluelogg.extended.Paper;
import com.megabras.bluelogg.extended.ReportProvider;
//import com.megabras.bluelogg.tm25r.Controller;
//import com.megabras.bluelogg.tm25r.Database;
//import com.megabras.modbus.TM25R.Objects;
//import com.megabras.modbus.TM25R.TestData;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.Locale;

@SuppressWarnings("WeakerAccess")
public class ReportPDF extends ReportProvider
{

    @SuppressWarnings("FieldCanBeLocal")
    private final String
            TAG = "ReportPDF";

//    private TestData
//            testData;
//
//    private ArrayList<Objects>
//            objList;
//
//    private Database
//            database;

    public ReportPDF(Activity activity, Handler handler)
    {
        super(activity);
        this.handler = handler;
    }

    @SuppressWarnings("ResultOfMethodCallIgnored")
    public void generate(EM4058test test, String selSite)
    {
        //handler.obtainMessage(Controller.mHandler.REPORT_UPDATE, 1, -1, activity.getString(R.string.report_creating_file)).sendToTarget();
        handler.obtainMessage(0, 1, -1, activity.getString(R.string.report_creating_file)).sendToTarget();
        //handler.obtainMessage(Controller.mHandler.REPORT_UPDATE, 1, -1, activity.getString(R.string.report_creating_file)).sendToTarget();

        PdfDocument document = new PdfDocument();
        PdfDocument.PageInfo pageInfo;

        if ( Locale.getDefault().getLanguage().equals( Locale.ENGLISH.getLanguage() ) )
        {
            pageInfo = new PdfDocument.PageInfo.Builder(Paper.Letter.getWidth(), Paper.Letter.getHeight(), 1).create();
        }
        else
        {
            pageInfo = new PdfDocument.PageInfo.Builder(Paper.A4.getWidth(), Paper.A4.getHeight(), 1).create();
        }

        CURSOR_Y     = 0;
        CURSOR_X     = 0;
        PAPER_WIDTH  = pageInfo.getPageWidth()  - (MARGIN * 2);
        PAPER_HEIGHT = pageInfo.getPageHeight() - (MARGIN * 2);

        PdfDocument.Page page = document.startPage( pageInfo );
        canvas = page.getCanvas();

        moveCursor(MARGIN, MARGIN);

        // BORDER
        top   (0);
        left  (0);
        right (PAPER_WIDTH);
        bottom(PAPER_HEIGHT);
        this.drawSquare();

        handler.obtainMessage(0, 1, -1, activity.getString(R.string.report_general_info)).sendToTarget();

        // TITLE
        top   (0);
        left  (0);
        right (PAPER_WIDTH);
        bottom(20);
        this.drawTitle( activity.getString(R.string.ground_tester_report).toUpperCase() );

        drawLineCompanyTestSite(test, selSite);

        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm", Locale.US);
        ArrayList<String> titles = new ArrayList<>();
        ArrayList<String> values = new ArrayList<>();

        // A
        titles.add( activity.getString( R.string.date      ).toUpperCase() );
        titles.add( activity.getString( R.string.equipment ).toUpperCase() );

        values.add( dateFormat.format(new Date()) );
        values.add( test.getEquipmentModel() );

        left  (0);
        top   (0);
        right (PAPER_WIDTH);
        bottom(20);
        this.drawLineItem(titles, values, ALIGN_LEFT);

        // B
        titles.clear();
        titles.add( activity.getString( R.string.operator  ).toUpperCase() );

        values.clear();

        String[] operator = test.getOperatorFromFile( selSite );
        values.add( operator[0] + " / " + operator[1] + " / " + operator[2] );

        left  (0);
        top   (0);
        right (PAPER_WIDTH);
        bottom(20);
        this.drawLineItem(titles, values, ALIGN_LEFT );

        // SNAPSHOT
        File snapshot = test.getSnapshotFile( selSite );
        if ( snapshot != null && snapshot.exists() )
        {
            left(0);
            top(0);
            right(PAPER_WIDTH);
            bottom(300);

            this.drawLineItem(activity.getString(R.string.map).toUpperCase(), snapshot);
        }

        // COMMENTS
        left  (0);
        top   (0);
        right (PAPER_WIDTH);
        bottom(20);
        drawMultilineText( activity.getString( R.string.comments).toUpperCase(), test.getComments( selSite ), false );

        drawGeneratedBy();

        String[] objects = test.getObjectsFromFile( selSite );

        drawPageNumber(1, objects.length+1, ALIGN_LEFT);


        document.finishPage(page);

        int curPage = 2;
        for ( String obj : objects )
        {
            PdfDocument.Page p = document.startPage( pageInfo );
            canvas = p.getCanvas();

            resetCursor();

            moveCursor( MARGIN, MARGIN );

            saveCursor();

            // BORDER
            top   (0);
            left  (0);
            right (PAPER_WIDTH);
            bottom(PAPER_HEIGHT);
            this.drawSquare();

            // TITLE
            top   (0);
            left  (0);
            right (PAPER_WIDTH);
            bottom(20);
            this.drawTitle( activity.getString(R.string.ground_tester_report).toUpperCase() );

            top   (0);
            left  (0);
            right (PAPER_WIDTH);
            bottom(20);
            drawLineItem( activity.getString(R.string.object_name).toUpperCase(), obj.toUpperCase(), ALIGN_LEFT );

            // GPS
            String[] gps = test.getGpsDataFromFile( obj, selSite );

            titles.clear();
            titles.add( activity.getString( R.string.latitude  ).toUpperCase() );
            titles.add( activity.getString( R.string.longitude ).toUpperCase() );
            titles.add( activity.getString( R.string.altitude  ).toUpperCase() );
            titles.add( activity.getString( R.string.accuracy  ).toUpperCase() );

            values.clear();
            values.add( gps[1] );
            values.add( gps[0] );
            values.add( gps[2] );
            values.add( gps[3] );

            left  (0);
            top   (0);
            right (PAPER_WIDTH);
            bottom(20);
            this.drawLineItem(titles, values, ALIGN_LEFT);

            // BLANK LINE
            moveCursor(0, 20);

            // MEASUREMENTS

            int[] widths = new int[]
            {
                25,
                50,
                115,
                40,
                40,
                45,
                35,
                35,
                35,
                35,
                35,
            };

            values.clear();
            values.add("#");
            values.add( activity.getString(R.string.date).toUpperCase()      );
            values.add( activity.getString(R.string.type).toUpperCase()      );
            values.add( activity.getString(R.string.distance).toUpperCase()  );
            values.add( activity.getString(R.string.frequency).toUpperCase() );
            values.add( activity.getString(R.string.result).toUpperCase()    );
            values.add("270 Hz");
            values.add("570 Hz");
            values.add("870 Hz");
            values.add("1170 Hz");
            values.add("1470 Hz");

            left  (0);
            top   (0);
            right (PAPER_WIDTH);
            bottom(10);
            this.drawRow(Typeface.create("sans-serif-condensed", Typeface.NORMAL), widths, values);

            ArrayList<EM4058test.EM4058values> list = test.getEM4058TestResultsFromFile( selSite, obj );

            for ( EM4058test.EM4058values ev : list )
            {
                values.clear();

                String[] data = ev.getAll();
                values.addAll( Arrays.asList(data) );

                left  (0);
                top   (0);
                right (PAPER_WIDTH);
                bottom(10);
                this.drawRow(Typeface.create("sans-serif-condensed", Typeface.NORMAL), widths, values);
            }

            // PICTURE
            File picture = test.getPictureFileFromFile( selSite, obj );

            if ( picture != null && picture.exists() )
            {
                restoreCursor();
                saveCursor();

                int imgHeight = 300;
                moveCursor(0, PAPER_HEIGHT - imgHeight);

                left(0);
                top(0);
                right(PAPER_WIDTH);
                bottom(imgHeight);

                this.drawLineItem(activity.getString(R.string.picture).toUpperCase(), picture);

                restoreCursor();
            }

            drawPageNumber(curPage++, objects.length+1, ALIGN_LEFT);

            document.finishPage( p );
        }

        handler.obtainMessage(0, 1, -1, activity.getString(R.string.saving_file)).sendToTarget();

        String fileName = getFileName( test.getRootFolder(), selSite );
        File tmpFile    = new File( fileName );
        if( tmpFile.exists() )
            tmpFile.delete();

        try
        {
            document.writeTo(new FileOutputStream(tmpFile));
            // close the document
            document.close();

            handler.obtainMessage(0, 1, -1, activity.getString(R.string.finished)).sendToTarget();

            sendFile(tmpFile);

        }
        catch (IOException e)
        {
            Log.e(TAG, "error "+e.toString());
            handler.obtainMessage(0, 1, -1, activity.getString(R.string.finished)).sendToTarget();
        }
    }

    private String getFileName(File folder, String site)
    {
        site      = MUtils.removeSpecialChars( site );
        File file = new File(folder, site + ".pdf");
        return file.toString();
    }
}

