package com.megabras.bluelogg.extended;

import android.app.Activity;
import android.app.Dialog;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.media.MediaPlayer;
import android.media.MediaRecorder;
import android.os.SystemClock;
import android.util.Log;
import android.view.View;
import android.view.Window;
import android.widget.Button;
import android.widget.Chronometer;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.megabras.bluelogg.R;

import java.io.File;
import java.io.IOException;

public class Audio
{
    private static final String
            TAG = "Audio";

    private Activity
            activity;

    private Audio
            audio;

    private Dialog
            dialog;

    private File
            file;

    private MediaRecorder
            mRecorder = null;

    private MediaPlayer
            mPlayer = null;

    private AudioCloseDialogInterface
            parentClose;

    public Audio(Activity activity)
    {
        this.activity = activity;
        audio = this;
    }

    public void setAudioCloseDialogInterface(AudioCloseDialogInterface parent){
        parentClose = parent;
    }

    public void setFile(File file)
    {
        this.file = file;
    }

    public void showDialog(String name)
    {
        if (dialog == null)
            dialog = createAudioDialog();

        Chronometer chrono  = dialog.findViewById(R.id.chrono);
        TextView duration   = dialog.findViewById(R.id.txt_duration);
        ImageButton btnPlay = dialog.findViewById(R.id.btn_play);
        TextView objectName = dialog.findViewById(R.id.txt_object_name);

        objectName.setText(name);

        if(file.exists())
        {
            int seconds = audio.getAudioLenght() / 1000;
            String time = seconds + " s";
            duration.setText(time);
            Util.enableButton(btnPlay, true);
        }
        else
        {
            duration.setText("");
            chrono.setBase(SystemClock.elapsedRealtime());
            Util.enableButton(btnPlay, false);
        }
        dialog.show();
    }

    private Dialog createAudioDialog()
    {
        final Dialog dialog = new Dialog( activity );
        dialog.requestWindowFeature( Window.FEATURE_NO_TITLE );
        dialog.setCanceledOnTouchOutside( false );
        dialog.setContentView( R.layout.dialog_audio );
        if ( dialog.getWindow() != null )
        {
            dialog.getWindow().setBackgroundDrawable( new ColorDrawable( Color.TRANSPARENT ) );
        }

        ((TextView) dialog.findViewById( R.id.dlg_title )).setText( R.string.voice_annotation );
        ((ImageView) dialog.findViewById( R.id.dlg_icon )).setImageResource( R.drawable.ic_audio );

        final Chronometer chrono = dialog.findViewById( R.id.chrono );

        dialog.findViewById( R.id.line_text_number ).setVisibility( View.GONE );

        View.OnClickListener onClose = view ->
        {
            dialog.dismiss();
            if ( parentClose != null )
            {
                parentClose.audioDialogClosed( audio.file );
            }
        };

        //dialog.findViewById( R.id.btn_close ).setOnClickListener( onClose );
        dialog.findViewById( R.id.btn_close_dlg ).setOnClickListener( onClose );

        dialog.findViewById( R.id.btn_record ).setOnClickListener( v ->
        {
            chrono.setVisibility( View.VISIBLE );
            audio.startRecording();
        } );

        dialog.findViewById( R.id.btn_stop_recording ).setOnClickListener(v -> audio.stopRecording());

        dialog.findViewById( R.id.btn_play ).setOnClickListener(v -> audio.startPlaying());

        dialog.findViewById( R.id.btn_stop ).setOnClickListener(v -> audio.stopPlaying());

        dialog.setOnShowListener( dialog1 ->
        {
            String time = "00/60s";
            chrono.setText( time );
        } );

        return dialog;
    }

    private void startRecording()
    {
        mRecorder = new MediaRecorder();
        mRecorder.setAudioSource(MediaRecorder.AudioSource.MIC);
        mRecorder.setOutputFormat(MediaRecorder.OutputFormat.THREE_GPP);
        mRecorder.setOutputFile(file.getAbsolutePath());
        mRecorder.setMaxDuration(60000);

        mRecorder.setOnInfoListener((mr, what, extra) -> stopRecording());
        mRecorder.setAudioEncoder(MediaRecorder.AudioEncoder.AMR_NB);

        try
        {
            mRecorder.prepare();
            recording(true);
            mRecorder.start();
        }
        catch (IOException e)
        {
            recording(false);
            Log.e(TAG, "prepare() failed");
        }
    }

    private void stopRecording()
    {
        if(mRecorder!=null)
        {
            mRecorder.stop();
            mRecorder.release();
            mRecorder = null;
            recording(false);
        }
    }

    private void startPlaying()
    {
        mPlayer = new MediaPlayer();
        try
        {
            mPlayer.setDataSource(file.getAbsolutePath());
            mPlayer.prepare();
            mPlayer.setOnCompletionListener(mp -> playing(false));
            playing(true);
            mPlayer.start();
        }
        catch (IOException e)
        {
            Log.e(TAG, "prepare() failed");
        }
    }

    private int getAudioLenght()
    {
        try
        {
            mPlayer = new MediaPlayer();
            int duration;
            mPlayer.setDataSource(file.getAbsolutePath());
            mPlayer.prepare();
            duration = mPlayer.getDuration();
            mPlayer.release();
            mPlayer = null;
            return duration;
        }
        catch (IOException e)
        {
            Log.e(TAG, "prepare() failed");
            return 0;
        }
    }

    private void stopPlaying()
    {
        if(mPlayer!=null)
        {
            mPlayer.release();
            mPlayer = null;
            playing(false);
        }
    }

    private void recording(boolean isRecording)
    {
        TextView     duration  = dialog.findViewById(R.id.txt_duration);
        Chronometer  chrono    = dialog.findViewById(R.id.chrono);
        ImageButton  btnClose  = dialog.findViewById(R.id.btn_close_dlg);
        ImageButton	 btnRecord = dialog.findViewById(R.id.btn_record);
        ImageButton	 btnStop   = dialog.findViewById(R.id.btn_stop_recording);
        ImageButton	 btnPlay   = dialog.findViewById(R.id.btn_play);

        if (isRecording)
        {
            Util.enableButton(btnClose , false);
            Util.enableButton(btnPlay  , false);
            Util.enableButton(btnRecord, false, false);
            Util.enableButton(btnStop  , true , true);
            chrono.setBase(SystemClock.elapsedRealtime());
            chrono.setOnChronometerTickListener(onChronoTick());
            chrono.start();
        }
        else
        {
            Util.enableButton(btnClose , true);
            Util.enableButton(btnPlay  , true);
            Util.enableButton(btnRecord, true, true);
            Util.enableButton(btnStop  , false, false);
            chrono.stop();
            String time = "00/60s";
            chrono.setText(time);
            int seconds = audio.getAudioLenght() / 1000;
            time = seconds + " s";
            duration.setText(time);
        }
    }

    private void playing(boolean isPlaying)
    {
        Chronometer  chrono    = dialog.findViewById(R.id.chrono);
        ImageButton  btnClose  = dialog.findViewById(R.id.btn_close_dlg);
        ImageButton	 btnRecord = dialog.findViewById(R.id.btn_record);
        ImageButton	 btnPlay   = dialog.findViewById(R.id.btn_play);
        ImageButton	 btnStop   = dialog.findViewById(R.id.btn_stop);

        if (isPlaying)
        {
            Util.enableButton(btnClose , false);
            Util.enableButton(btnRecord, false);
            Util.enableButton(btnPlay  , false, false);
            Util.enableButton(btnStop  , true , true);
            chrono.setBase(SystemClock.elapsedRealtime());
            chrono.setOnChronometerTickListener(onChronoTick());
            chrono.start();
        }
        else
        {
            Util.enableButton(btnClose , true);
            Util.enableButton(btnRecord, true);
            Util.enableButton(btnPlay  , true , true);
            Util.enableButton(btnStop  , false, false);
            chrono.stop();
            String time = "00/60s";
            chrono.setText(time);
        }
    }

    private Chronometer.OnChronometerTickListener onChronoTick()
    {
        return chronometer -> {
            CharSequence mm = chronometer.getText().subSequence(0, 2);
            CharSequence ss = chronometer.getText().subSequence(3, 5);
            String time;
            if (mm.equals("01") && ss.equals("00"))
            {
                time = "60/60s";
            } else
            {
                time = ss + "/60s";
            }
            chronometer.setText(time);
        };
    }
}
