package com.megabras.bluelogg.extended;

import android.app.Activity;
import android.graphics.drawable.LayerDrawable;
import android.view.View;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.FrameLayout;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.TextView;

import com.megabras.bluelogg.R;

@SuppressWarnings("WeakerAccess")
public class CommonUI
{
    private Activity
            activity;

    public static final int
            COLOR_BLACK  = R.drawable.chamfer_small_black,
            COLOR_YELLOW = R.drawable.chamfer_small_yellow,
            COLOR_GREEN  = R.drawable.chamfer_small_green,
            COLOR_RED    = R.drawable.chamfer_small_red,
            LED_GREEN    = 1;
//            LED_YELLOW   = 2,
//            LED_RED      = 3;

    public CommonUI(Activity activity)
    {
        this.activity = activity;
    }

    public void batteryIcon(int view, int charge)
    {
        ImageView batteryIcon = activity.findViewById(view);

        if (batteryIcon==null)
            return;

        if (charge > 99)
            batteryIcon.setImageResource(R.drawable.bat_100);
        else if (charge > 80)
            batteryIcon.setImageResource(R.drawable.bat_90);
        else if (charge > 70)
            batteryIcon.setImageResource(R.drawable.bat_80);
        else if (charge > 60)
            batteryIcon.setImageResource(R.drawable.bat_70);
        else if (charge > 50)
            batteryIcon.setImageResource(R.drawable.bat_60);
        else if (charge > 40)
            batteryIcon.setImageResource(R.drawable.bat_50);
        else if (charge > 30)
            batteryIcon.setImageResource(R.drawable.bat_40);
        else if (charge > 20)
            batteryIcon.setImageResource(R.drawable.bat_30);
        else if (charge > 10)
            batteryIcon.setImageResource(R.drawable.bat_20);
        else if (charge > 5)
            batteryIcon.setImageResource(R.drawable.bat_10);
        else
            batteryIcon.setImageResource(R.drawable.bat_0);
    }

    public void enableButton(Button btn, boolean enable)
    {
        if (btn!=null)
            btn.setEnabled(enable);
    }

    public void enableButtonBg(Button btn, boolean enable)
    {
        if (btn!=null)
            btn.setEnabled(enable);
    }

    public void enableButton(ImageButton btn, boolean enable)
    {
        if (btn!=null)
            btn.setEnabled(enable);
    }

    public void enableButtonImage(int buttonId, boolean enable)
    {
        ImageButton btn = activity.findViewById(buttonId);
        if (btn!=null)
            btn.setEnabled(enable);
    }

    public void enableButtonImage(ImageButton btn, boolean enable)
    {
        if (btn!=null)
            btn.setEnabled(enable);
    }

    public void changeButtonImage(int buttonId, int image, boolean enable)
    {
        ImageButton btn = activity.findViewById(buttonId);
        if (btn==null)
            return;
        btn.setImageResource(image);
        enableButtonImage(buttonId, enable);
    }

    public void changeButtonImage(ImageButton button, int image, boolean enable)
    {
        if (button==null)
            return;
        button.setImageResource(image);
        enableButtonImage(button, enable);
    }

    public void enableButton(ImageButton btn, boolean enable, boolean hide)
    {
        if (btn==null)
            return;
        int visibility = hide ? View.VISIBLE : View.GONE;
        btn.setVisibility(visibility);
        btn.setEnabled(enable);
    }

    public void messageBox(String msg, int color, boolean progressBar)
    {
        FrameLayout box  = activity.findViewById(R.id.ct_user_message);
        TextView lbMsg   = activity.findViewById(R.id.lb_user_message);
        ProgressBar pBar = activity.findViewById(R.id.pBar);

        if (box==null || lbMsg==null || pBar==null)
            return;

        box.setBackground( activity.getResources().getDrawable(color) );
        lbMsg.setText(msg);

        if(progressBar)
            pBar.setVisibility(View.VISIBLE);
        else
            pBar.setVisibility(View.GONE);
    }

    public void keepTheScreenOn(boolean keep)
    {
        if (activity==null || activity.getWindow()==null)
            return;
        if (keep)
            activity.getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        else
            activity.getWindow().clearFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
    }

    public void changeAlphaLED(int ledID, int led, boolean show)
    {
        ImageView img = activity.findViewById(ledID);
        if (img==null)
            return;
        LayerDrawable layer = (LayerDrawable) img.getDrawable();
        if (layer==null)
            return;
        if (show)
            layer.getDrawable(led).setAlpha(255);
        else
            layer.getDrawable(led).setAlpha(0);
    }

    public void turnOffLED(int ledID)
    {
        changeAlphaLED(ledID, 1, false);
        changeAlphaLED(ledID, 2, false);
        changeAlphaLED(ledID, 3, false);
    }

    public void statusBlinkLED(final int ledID)
    {
        changeAlphaLED(ledID, 3, true);
    }

    public void statusBlinkLEDrx(final int ledID)
    {
        changeAlphaLED(ledID, 3, false);
        changeAlphaLED(ledID, 2, true);
    }

    public void statusBlinkLEDnotRx(final int ledID)
    {
        changeAlphaLED(ledID, 3, false);
    }
}
