package com.megabras.bluelogg.extended;

import android.graphics.Rect;
import android.os.CountDownTimer;
import android.os.Handler;
import android.view.MotionEvent;
import android.widget.Button;

public class CountDown extends CountDownTimer {
    @SuppressWarnings("unused")
    private final String
            TAG = "CountDown";

    public static final int       STATE_TICK   = 0;
    public static final int       STATE_CANCEL = 1;
    public static final int       STATE_FINISH = 2;
    private             long      time;
    private             boolean   finished     = false;
    private             Button    button;
    private             Handler   handler;
    private             CountDown countDown;
    private             Rect      rect;

    public CountDown(final long time, long tick, final Button button, final Handler handler) {
        super(time, tick);
        this.time = time;
        this.button = button;
        this.handler = handler;
        countDown = this;

        this.button.setOnTouchListener( ( v, event ) ->
        {
            Button btn = (Button) v;
            switch ( event.getAction() )
            {
                case MotionEvent.ACTION_DOWN:
                    rect = new Rect( v.getLeft(), v.getTop(), v.getRight(), v.getBottom() );
                    countDown.start();
                    btn.setPressed( true );
                    break;
                case MotionEvent.ACTION_UP:
                    onCancel( btn );
                    break;
                case MotionEvent.ACTION_MOVE:
                    if ( !rect.contains( v.getLeft() + (int) event.getX(), v.getTop() + (int) event.getY() ) )
                    {
                        onCancel( btn );
                    }
                    break;
            }
            return true;
        } );
    }

    public void onTick(long millisUntilFinished) {
        if (button.isPressed()) {
            handler.obtainMessage(STATE_TICK, -1, -1, (millisUntilFinished) + "ms").sendToTarget();
        }
        finished = false;
    }

    public void onFinish() {
        if (button.isPressed()) {
            handler.obtainMessage(STATE_FINISH, -1, 1).sendToTarget();
            finished = true;
        }
    }

    private void onCancel(Button btn){
        btn.performClick();
        countDown.cancel();
        btn.setPressed(false);
        if(finished)
            return;
        handler.obtainMessage(STATE_CANCEL, -1, -1,((time / 1000) + "s")).sendToTarget();
    }

//    public boolean isFinished() {
//        return finished;
//    }
}
