package com.megabras.bluelogg.extended;

import android.Manifest;
import android.annotation.TargetApi;
import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.graphics.ImageFormat;
import android.graphics.SurfaceTexture;
import android.graphics.drawable.ColorDrawable;
import android.hardware.camera2.CameraAccessException;
import android.hardware.camera2.CameraCaptureSession;
import android.hardware.camera2.CameraCharacteristics;
import android.hardware.camera2.CameraDevice;
import android.hardware.camera2.CameraManager;
import android.hardware.camera2.CameraMetadata;
import android.hardware.camera2.CaptureRequest;
import android.hardware.camera2.TotalCaptureResult;
import android.hardware.camera2.params.StreamConfigurationMap;
import android.media.Image;
import android.media.ImageReader;
import android.os.Handler;
import android.os.HandlerThread;
import android.util.Log;
import android.util.Size;
import android.util.SparseIntArray;
import android.view.Surface;
import android.view.TextureView;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.TextView;

import com.megabras.bluelogg.R;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AlertDialog;
import androidx.core.app.ActivityCompat;

//import android.support.annotation.NonNull;
//import android.support.v4.app.ActivityCompat;
//import android.support.v4.content.FileProvider;

@SuppressWarnings("WeakerAccess")
@TargetApi(21)
public class PictureNewApi extends Picture {

    private final String
            TAG = "Picture";

    private android.media.MediaPlayer
            shutter;

    private Activity
            activity;

    private static final SparseIntArray
            ORIENTATIONS = new SparseIntArray();

    static {
        ORIENTATIONS.append(Surface.ROTATION_0, 90);
        ORIENTATIONS.append(Surface.ROTATION_90, 0);
        ORIENTATIONS.append(Surface.ROTATION_180, 270);
        ORIENTATIONS.append(Surface.ROTATION_270, 180);
    }

    protected CameraDevice
            cameraDevice;

    protected CameraCaptureSession
            cameraCaptureSessions;

    protected CaptureRequest.Builder
            captureRequestBuilder;

    private Size
            imageDimension;

    private ImageReader
            imageReader;

    private File
            file;

//    private Handler
//            mBackgroundHandler;
//
//    private HandlerThread
//            mBackgroundThread;

    private TextureView
            textureView;

    private PictureInterface
            parent;

//    private Semaphore
//            mCameraOpenCloseLock = new Semaphore(1);



    public void initialize(Activity activity, PictureInterface parent){
        this.activity = activity;
        this.parent   = parent;
        try {
            android.content.res.AssetFileDescriptor afd = activity.getAssets().openFd("shutter.mp3");
            shutter = new android.media.MediaPlayer();
            shutter.setDataSource(afd.getFileDescriptor(), afd.getStartOffset(), afd.getLength());
            shutter.prepare();
        }
        catch (IOException e) {
            errorLog(e);
        }
    }

    TextureView.SurfaceTextureListener textureListener = new TextureView.SurfaceTextureListener() {
        @Override
        public void onSurfaceTextureAvailable(SurfaceTexture surface, int width, int height) {
            openCamera();
        }

        @Override public void onSurfaceTextureSizeChanged(SurfaceTexture surface, int width, int height) {}
        @Override public boolean onSurfaceTextureDestroyed(SurfaceTexture surface) {
            return false;
        }
        @Override public void onSurfaceTextureUpdated(SurfaceTexture surface) {}
    };

    private final CameraDevice.StateCallback stateCallback = new CameraDevice.StateCallback() {
        @Override
        public void onOpened(@NonNull CameraDevice camera) {
            cameraDevice = camera;
            createCameraPreview();
        }

        @Override
        public void onDisconnected(@NonNull CameraDevice camera) {
            cameraDevice.close();
        }

        @Override
        public void onError(@NonNull CameraDevice camera, int error) {
            if (cameraDevice != null) {
                cameraDevice.close();
            }
            cameraDevice = null;
            Log.e(TAG, "onError: " + error);
        }
    };

//    protected void startBackgroundThread() {
//        mBackgroundThread = new HandlerThread("Camera Background");
//        mBackgroundThread.start();
//        mBackgroundHandler = new Handler(mBackgroundThread.getLooper());
//    }
//
//    protected void stopBackgroundThread() {
//        mBackgroundThread.quitSafely();
//        try {
//            mBackgroundThread.join();
//            mBackgroundThread = null;
//            mBackgroundHandler = null;
//        } catch (InterruptedException e) {
//            e.printStackTrace();
//        }
//    }

    protected void createCameraPreview() {
        try {
            SurfaceTexture texture = textureView.getSurfaceTexture();
            assert texture != null;
            texture.setDefaultBufferSize(imageDimension.getWidth(), imageDimension.getHeight());
            Surface surface = new Surface(texture);

            HandlerThread mBackgroundThread = new HandlerThread("Camera Background");
            mBackgroundThread.start();
            final Handler mBackgroundHandler = new Handler(mBackgroundThread.getLooper());

            captureRequestBuilder = cameraDevice.createCaptureRequest(CameraDevice.TEMPLATE_PREVIEW);
            captureRequestBuilder.addTarget(surface);
            cameraDevice.createCaptureSession(Collections.singletonList(surface), new CameraCaptureSession.StateCallback() {
                @Override
                public void onConfigured(@NonNull CameraCaptureSession cameraCaptureSession) {
                    if (null == cameraDevice) {
                        return;
                    }
                    cameraCaptureSessions = cameraCaptureSession;
                    captureRequestBuilder.set(CaptureRequest.CONTROL_MODE, CameraMetadata.CONTROL_MODE_AUTO);
                    try {
                        cameraCaptureSessions.setRepeatingRequest(captureRequestBuilder.build(), null, mBackgroundHandler);
                    } catch (CameraAccessException e) {
                        errorLog(e);
                    }
                }

                @Override
                public void onConfigureFailed(@NonNull CameraCaptureSession cameraCaptureSession) {}
            }, mBackgroundHandler);
        } catch (CameraAccessException e) {
            e.printStackTrace();
        }
    }

    private void openCamera()
    {
        CameraManager manager = (CameraManager) activity.getSystemService( Context.CAMERA_SERVICE );
        try
        {
            if ( manager != null )
            {
                String cameraId = manager.getCameraIdList()[0];
                CameraCharacteristics characteristics = manager.getCameraCharacteristics( cameraId );
                StreamConfigurationMap map = characteristics.get( CameraCharacteristics.SCALER_STREAM_CONFIGURATION_MAP );
                assert map != null;
                imageDimension = map.getOutputSizes( SurfaceTexture.class )[0];
                if ( ActivityCompat.checkSelfPermission( activity, Manifest.permission.CAMERA ) != PackageManager.PERMISSION_GRANTED )
                {
                    return;
                }
                manager.openCamera( cameraId, stateCallback, null );
            }
            else
            {
                new AlertDialog.Builder( activity )
                        .setMessage(R.string.internal_error)
                        .setPositiveButton(R.string.ok, ( dialog, id ) ->{} )
                        .create()
                        .show();
            }
        }
        catch ( CameraAccessException e )
        {
            new AlertDialog.Builder( activity )
                    .setMessage(R.string.internal_error)
                    .setPositiveButton(R.string.ok, ( dialog, id ) ->{} )
                    .create()
                    .show();
        }
    }


    public void showDialog(final File picFile) {
//        new Thread(new Runnable() {
//            public void run() {
//                Looper.prepare();

                parent.pictureStatus(Picture.WAIT);
                if (picFile.exists())
                    previewPicture("TEST", picFile, null);
                else
                    takePicture(picFile, null);

//                Looper.loop();
//            }
//        }).start();
    }

    public void showDialog(final File picFile, final DialogInterface.OnDismissListener onDismissListener) {
        if (picFile.exists())
            previewPicture("TEST", picFile, onDismissListener);
        else
            takePicture(picFile, onDismissListener);
    }

    public void takePicture( final File files, final DialogInterface.OnDismissListener onDismissListener )
    {
        parent.pictureStatus( Picture.WAIT );
        file = files;
        final Dialog dialog = new Dialog( activity, R.style.MyThemeCompat );
        dialog.requestWindowFeature( Window.FEATURE_NO_TITLE );
        dialog.setCanceledOnTouchOutside( false );
        dialog.setContentView( R.layout.dialog_picture_new_api );
        if ( dialog.getWindow() != null )
        {
            dialog.getWindow().getAttributes().width = WindowManager.LayoutParams.MATCH_PARENT;
            dialog.getWindow().getAttributes().height = WindowManager.LayoutParams.MATCH_PARENT;
        }

        dialog.setOnShowListener( dialog1 -> parent.pictureStatus( Picture.FINISHED ) );

        textureView = dialog.findViewById( R.id.texture );

        if ( textureView == null )
        {
            return;
        }

        textureView.setSurfaceTextureListener( textureListener );

        final Button captureButton = dialog.findViewById( R.id.button_capture );
        final Button btnPreview = dialog.findViewById( R.id.btn_preview );
        final Button btnClose = dialog.findViewById( R.id.btn_close );

        captureButton.setOnClickListener( v ->
        {
            savePicture();
            shutter.start();
            captureButton.setVisibility( View.GONE );
            btnPreview.setVisibility( View.VISIBLE );
        } );

        btnPreview.setOnClickListener( v ->
        {
            closeCamera();
            takePicture( file, onDismissListener );
            dialog.dismiss();
        } );

        btnClose.setOnClickListener( v ->
        {
            closeCamera();
            if ( onDismissListener != null )
                dialog.setOnDismissListener( onDismissListener );

            dialog.dismiss();
        } );

        btnPreview.setVisibility( View.GONE );

        dialog.show();
    }

    protected void savePicture() {
        if(null == cameraDevice) {
            Log.e(TAG, "cameraDevice is null");
            return;
        }
        //CameraManager manager = (CameraManager) activity.getSystemService(activity.getApplicationContext().CAMERA_SERVICE);
        //CameraManager manager = (CameraManager) activity.getSystemService(Context.CAMERA_SERVICE);
        try {
            //CameraCharacteristics characteristics = manager.getCameraCharacteristics(cameraDevice.getId());
//            Size[] jpegSizes = null;
//            if (characteristics != null) {
//                jpegSizes = characteristics
//                            .get(CameraCharacteristics.SCALER_STREAM_CONFIGURATION_MAP)
//                            .getOutputSizes(ImageFormat.JPEG);
//            }

            HandlerThread mBackgroundThread = new HandlerThread("Camera Background");
            mBackgroundThread.start();
            final Handler mBackgroundHandler = new Handler(mBackgroundThread.getLooper());

            int width  = 640;
            int height = 480;

//            int width  = textureView.getWidth();
//            int height = textureView.getHeight();

            //Log.e(TAG, textureView.getWidth() + " <> " + textureView.getHeight());

            ImageReader reader = ImageReader.newInstance(width, height, ImageFormat.JPEG, 1);
            List<Surface> outputSurfaces = new ArrayList<>(2);
            outputSurfaces.add(reader.getSurface());
            outputSurfaces.add(new Surface(textureView.getSurfaceTexture()));

            final CaptureRequest.Builder captureBuilder =
                    cameraDevice.createCaptureRequest(CameraDevice.TEMPLATE_STILL_CAPTURE);
            captureBuilder.addTarget(reader.getSurface());
            captureBuilder.set(CaptureRequest.CONTROL_MODE, CameraMetadata.CONTROL_MODE_AUTO);
            // Orientation
            int rotation = activity.getWindowManager().getDefaultDisplay().getRotation();
            captureBuilder.set(CaptureRequest.JPEG_ORIENTATION, ORIENTATIONS.get(rotation));

            ImageReader.OnImageAvailableListener readerListener =
                    new ImageReader.OnImageAvailableListener() {
                @SuppressWarnings("TryFinallyCanBeTryWithResources")
                @Override
                public void onImageAvailable( ImageReader reader )
                {
                    Image image = null;
                    try
                    {
                        image = reader.acquireLatestImage();
                        ByteBuffer buffer = image.getPlanes()[0].getBuffer();
                        byte[] bytes = new byte[buffer.capacity()];
                        buffer.get( bytes );
                        save( bytes );
                    }
                    catch ( Exception e )
                    {
                        errorLog( e );
                    }
                    finally
                    {
                        if ( image != null )
                        {
                            image.close();
                        }
                    }
                }
                private void save(byte[] bytes) throws IOException
                {
                    OutputStream output = null;
                    //noinspection TryFinallyCanBeTryWithResources
                    try
                    {
                        output = new FileOutputStream( file );
                        Bitmap realImage = BitmapFactory.decodeByteArray( bytes, 0, bytes.length );

                        if ( realImage.getWidth() > realImage.getHeight() )
                        {
                            realImage = rotate( realImage, 90 );
                        }

                        realImage.compress( Bitmap.CompressFormat.JPEG, 100, output );
                        output.write( bytes );
                    }
                    finally
                    {
                        if ( null != output )
                        {
                            output.close();
                        }
                    }
                }
            };
            reader.setOnImageAvailableListener(readerListener, mBackgroundHandler);
            final CameraCaptureSession.CaptureCallback captureListener = new CameraCaptureSession.CaptureCallback() {
                @Override
                public void onCaptureCompleted(@NonNull CameraCaptureSession session, @NonNull CaptureRequest request, @NonNull TotalCaptureResult result) {
                    super.onCaptureCompleted(session, request, result);
                }
            };
            cameraDevice.createCaptureSession(outputSurfaces, new CameraCaptureSession.StateCallback() {
                @Override
                public void onConfigured(@NonNull CameraCaptureSession session) {
                    try {
                        session.capture(captureBuilder.build(), captureListener, mBackgroundHandler);
                    } catch (CameraAccessException e) {
                        e.printStackTrace();
                    }
                }
                @Override
                public void onConfigureFailed(@NonNull CameraCaptureSession session) {}
            }, mBackgroundHandler);
        } catch (CameraAccessException e) {
            e.printStackTrace();
        }
    }

    @SuppressWarnings("ResultOfMethodCallIgnored")
    @Override
    public void previewPicture( final String obj, final File files, final DialogInterface.OnDismissListener onDismissListener )
    {
        parent.pictureStatus( Picture.WAIT );
        file = files;
        final Dialog dialog = new Dialog( activity, R.style.MyThemeCompat );
        //final Dialog dialog = new Dialog( activity );
        dialog.requestWindowFeature( Window.FEATURE_NO_TITLE );
        dialog.setCanceledOnTouchOutside( false );
        dialog.setContentView( R.layout.dialog_picture_preview );
        if ( dialog.getWindow() != null )
        {
            dialog.getWindow().getAttributes().width = WindowManager.LayoutParams.MATCH_PARENT;
            dialog.getWindow().getAttributes().height = WindowManager.LayoutParams.MATCH_PARENT;
        }

        dialog.setOnShowListener( dialog1 -> parent.pictureStatus( Picture.FINISHED ) );

        File imgFile;
        Button btnNewPic = dialog.findViewById( R.id.btn_new_picture );

        imgFile = file;
        btnNewPic.setOnClickListener( v ->
        {
            if ( file.exists() )
            {
                file.delete();
            }
            takePicture( file, onDismissListener );
            dialog.dismiss();
        } );

        ImageView img = dialog.findViewById( R.id.img_preview );
        Bitmap myBitmap = BitmapFactory.decodeFile( imgFile.getAbsolutePath() );
        img.setImageBitmap( myBitmap );

        dialog.findViewById( R.id.btn_close ).setOnClickListener( v ->
        {
            if ( onDismissListener != null )
            {
                dialog.setOnDismissListener( onDismissListener );
            }
            dialog.dismiss();
        } );

        dialog.show();
    }

//    protected void updatePreview() {
//        if(null == cameraDevice) {
//            Log.e(TAG, "updatePreview error, return");
//        }
//        captureRequestBuilder.set(CaptureRequest.CONTROL_MODE, CameraMetadata.CONTROL_MODE_AUTO);
//        try {
//            cameraCaptureSessions.setRepeatingRequest(captureRequestBuilder.build(), null, mBackgroundHandler);
//        } catch (CameraAccessException e) {
//            errorLog(e);
//        }
//    }

    private void closeCamera()
    {
//        if (null != cameraCaptureSessions) {
//            cameraCaptureSessions.close();
//            cameraCaptureSessions = null;
//        }
        if ( null != cameraDevice )
        {
            cameraDevice.close();
            cameraDevice = null;
        }
        if ( null != imageReader )
        {
            imageReader.close();
            imageReader = null;
        }
    }

//    public void close() {
//        try {
//            closeCamera();
//            if (shutter != null){
//                shutter.release();
//            }
//        } catch (Exception e) {
//            Log.e("close", e.getMessage());
//        }
//    }

//    protected void onResume() {
//        startBackgroundThread();
//        if (textureView.isAvailable()) {
//            openCamera();
//        } else {
//            textureView.setSurfaceTextureListener(textureListener);
//        }
//    }
//
//    protected void onPause() {
//        //closeCamera();
//        stopBackgroundThread();
//    }

    private void errorLog( Exception e )
    {
        Log.e( TAG, e.toString() );
        for ( StackTraceElement ste : e.getStackTrace() )
            Log.e( TAG, ste.toString() );
    }
}
