package com.megabras.bluelogg.hp60kv;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.os.Environment;
import android.util.Log;

import com.megabras.bluelogg.extended.FileDetails;
import com.megabras.bluelogg.R;
import com.megabras.modbus.HP60KV.Device;
import com.megabras.modbus.HP60KV.Measurement;
import com.megabras.modbus.HP60KV.TestDataAndroid;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

public final class Database extends SQLiteOpenHelper
{
    @SuppressWarnings("FieldCanBeLocal")
    private String
            TAG          = "Database";

    @SuppressWarnings("unused")
    private static final String
            TEXT_TYPE    = " TEXT",
            INT_TYPE     = " INTEGER",
            FLOAT_TYPE   = " FLOAT",
            DOUBLE_TYPE  = " DOUBLE",
            DATE_TYPE    = " DATETIME",
            COMMA_SEP    = ",";

    private String
            root         = Environment.getExternalStorageDirectory().toString();

    private File
            dir,
            tempDir,
            rootDir;

    protected final com.megabras.modbus.HP60KV.FormatData
            formatData;

    public Database(Context context, String model, String dbName)
    {
        super(context, dbName, null, Entry.DB_VERSION);
        createFolder(context, model);
        formatData = new com.megabras.modbus.HP60KV.FormatData();
    }

    public void save(TestDataAndroid testData)
    {
        String TABLE_NAME = generateTableName(testData);
        SQLiteDatabase db = this.getWritableDatabase();

        Cursor c  = db.rawQuery("SELECT * FROM sqlite_master WHERE type='table' AND name='" + TABLE_NAME + "';", null);
        int count = c.getCount();
        c.close();
        // TABLE ALREADY EXITS
        if( count>0 )
            return;

        String FIELD_ENTRIES =
                Entry.DESCRIPTION   + TEXT_TYPE + COMMA_SEP +
                Entry.COL_A         + TEXT_TYPE + COMMA_SEP +
                Entry.COL_B         + TEXT_TYPE + COMMA_SEP +
                Entry.COL_C         + TEXT_TYPE + COMMA_SEP +
                Entry.COL_D         + TEXT_TYPE + COMMA_SEP +
                Entry.COL_E         + TEXT_TYPE;

        String SQL_CREATE_TABLE =
                "CREATE TABLE " + TABLE_NAME + " (" + FIELD_ENTRIES + " )";
        db.execSQL(SQL_CREATE_TABLE);

        db.beginTransaction();

        ContentValues values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.MODEL );
        values.put( Entry.COL_A      , testData.getModel() );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.SERIAL );
        values.put( Entry.COL_A      , testData.getSerial() );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.MEMORY_MAP );
        values.put( Entry.COL_A      , testData.getMemoryMap() );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.HARDWARE );
        values.put( Entry.COL_A      , testData.getHardware() );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.FIRMWARE );
        values.put( Entry.COL_A      , testData.getFirmware() );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.TEST_NUMBER );
        values.put( Entry.COL_A      , testData.getTestNumber() );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.TEST_NUMBER );
        values.put( Entry.COL_A      , testData.getTestNumber() );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.DATE );
        values.put( Entry.COL_A      , testData.getDate().getTime()+"" );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        Device device = testData.getDevice();

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.OP_MODE );
        values.put( Entry.COL_A      , device.getOperationMode() );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.GPS_ACCURACY );
        values.put( Entry.COL_A      , testData.getGpsAcuraccy()+"" );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.GPS_ALTITUDE );
        values.put( Entry.COL_A      , testData.getGpsAltitude()+"" );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.GPS_LATITUDE );
        values.put( Entry.COL_A      , testData.getGpsLatitude()+"" );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.GPS_LONGITUDE );
        values.put( Entry.COL_A      , testData.getGpsLongitude()+"" );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.PI );
        values.put( Entry.COL_A      , testData.getPI() );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.SVT );
        values.put( Entry.COL_A      , testData.getSVT() );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        values = new ContentValues();
        values.put( Entry.DESCRIPTION, Entry.RESULT );
        values.put( Entry.COL_A      , testData.getTestResult() );
        db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

        for (Measurement m : testData.getMeasurement())
        {
            values = new ContentValues();
            values.put( Entry.DESCRIPTION, Entry.MEASURE );
            values.put( Entry.COL_A      , m.getElapsed()    );
            values.put( Entry.COL_B      , m.getResistance() );
            values.put( Entry.COL_C      , m.getVoltage()    );
            values.put( Entry.COL_D      , m.getCurrent()    );
            values.put( Entry.COL_E      , m.getStats()      );
            db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);
        }

        db.setTransactionSuccessful();
        db.endTransaction();
    }

    void updateGpsData(TestDataAndroid testData)
    {
        try
        {
            String TABLE_NAME = generateTableName(testData);
            SQLiteDatabase db = this.getWritableDatabase();
            db.beginTransaction();

            ContentValues values;

            values = new ContentValues();
            values.put(Entry.DESCRIPTION, Entry.GPS_LATITUDE );
            values.put(Entry.COL_A, testData.getGpsLatitude());
            db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

            values = new ContentValues();
            values.put(Entry.DESCRIPTION, Entry.GPS_LONGITUDE );
            values.put(Entry.COL_A, testData.getGpsLongitude());
            db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

            values = new ContentValues();
            values.put(Entry.DESCRIPTION, Entry.GPS_ALTITUDE );
            values.put(Entry.COL_A, testData.getGpsAltitude());
            db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

            values = new ContentValues();
            values.put(Entry.DESCRIPTION, Entry.GPS_ACCURACY );
            values.put(Entry.COL_A, testData.getGpsAcuraccy());
            db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_REPLACE);

            db.setTransactionSuccessful();
            db.endTransaction();
        }
        catch (Exception e)
        {
            Log.e(TAG, e.toString());
        }
    }

    List<FileDetails> getSavedFilesList(String serial, int dateFormat, int timeFormat)
    {
        try
        {
            List<FileDetails>
                    filesList = new ArrayList<>();
            SQLiteDatabase db = this.getReadableDatabase();
            Cursor c          = db.rawQuery("SELECT name FROM sqlite_master WHERE type='table' AND name LIKE '%"+serial+"%'", null);

            if (c.moveToFirst())
            {
                while ( !c.isAfterLast() )
                {
                    String tableName = c.getString(0);
                    if ( !tableName.contains("metadata") )
                    {
                        FileDetails file = new FileDetails();
                        String[] temp           = tableName.split("_");
                        file.FILENAME           = tableName;
                        file.MODEL              = temp[0];
                        file.SERIAL             = temp[1];
                        file.INDEX              = temp[2];
                        String seconds          = temp[3];
                        long time = Long.parseLong(seconds);
                        Date date = new Date(time);
                        file.DATE               = formatData.getDateFormatted(date, dateFormat, timeFormat);
                        filesList.add(file);
                    }
                    c.moveToNext();
                }
            }
            c.close();
            return filesList;
        }
        catch (Exception e)
        {
            return null;
        }
    }

    TestDataAndroid getResult(String TABLE_NAME)
    {
        try
        {
            TestDataAndroid testData = new TestDataAndroid();
            Device    device         = new Device();

            SQLiteDatabase db = this.getReadableDatabase();
            Cursor c = db.rawQuery("SELECT * FROM " + TABLE_NAME, null);

            if (c.moveToFirst())
            {
                while ( !c.isAfterLast() )
                {
                    String colA = c.getString(0);
                    String colB = c.getString(1);
                    String colC = c.getString(2);
                    String colD = c.getString(3);
                    String colE = c.getString(4);
                    String colF = c.getString(5);

                    switch (colA) {
                        case Entry.MODEL:
                            testData.setModel(colB);
                            break;
                        case Entry.SERIAL:
                            testData.setSerial(colB);
                            break;
                        case Entry.MEMORY_MAP:
                            if (colB!=null)
                                testData.setMemoryMap(Integer.parseInt(colB));
                            break;
                        case Entry.HARDWARE:
                            if (colB!=null)
                                testData.setHardware(Integer.parseInt(colB));
                            Log.e(TAG, "HARDWARE: " + testData.getHardware());
                            break;
                        case Entry.FIRMWARE:
                            if (colB!=null)
                                testData.setFirmware(Integer.parseInt(colB));
                            Log.e(TAG, "FIRMWARE: " + testData.getFirmware());
                            break;
                        case Entry.TEST_NUMBER:
                            testData.setTestNumber((short) Integer.parseInt(colB));
                            break;
                        case Entry.DATE:
                            Date datef = new Date();
                            datef.setTime(Long.parseLong(colB));
                            testData.setDate(datef);
                            break;
                        case Entry.OP_MODE:
                            testData.setMode(Integer.parseInt(colB));
                            device.setOperationMode(testData.getMode());
                            break;
                        case Entry.RESULT:
                            testData.setTestResult(Integer.parseInt(colB));
                            break;
                        case Entry.GPS_LATITUDE:
                            testData.setGpsLatitude(Double.parseDouble(colB));
                            break;
                        case Entry.GPS_LONGITUDE:
                            testData.setGpsLongitude(Double.parseDouble(colB));
                            break;
                        case Entry.GPS_ALTITUDE:
                            testData.setGpsAltitude(Double.parseDouble(colB));
                            break;
                        case Entry.GPS_ACCURACY:
                            testData.setGpsAcuraccy(Double.parseDouble(colB));
                            break;
                        case Entry.PI:
                            testData.setPI(Double.parseDouble(colB));
                            break;
                        case Entry.MEASURE:
                            Measurement m = new Measurement();
                            m.setElapsed(Integer.parseInt(colB));
                            m.setResistance(Double.parseDouble(colC));
                            m.setVoltage(Integer.parseInt(colD));
                            m.setCurrent(Double.parseDouble(colE));
                            m.setStats(Integer.parseInt(colF));
                            testData.addMeasument(m);
                            break;
                    }
                    c.moveToNext();
                }
            }

            testData.setDevice(device);

            c.close();
            return testData;
        } catch (Exception e) {
            Log.e("TEST", e.toString());
            return null;
        }
    }

    void deleteTable(String name)
    {
        SQLiteDatabase db = this.getWritableDatabase();
        String SQL_DELETE_TABLE =
                "DROP TABLE IF EXISTS " + name;
        db.execSQL(SQL_DELETE_TABLE);

        String[] parts = name.split("_");

        deleteFiles(parts[1], parts[3]);
    }

    //##########################################################################
    //# Folders
    //##########################################################################

    private boolean isExternalStorageAvaiable()
    {
        String state = Environment.getExternalStorageState();
        return Environment.MEDIA_MOUNTED.equals(state);
    }

    private String generateTableName(TestDataAndroid testData)
    {
        String name = "m" + testData.getModel().replaceAll("-","");
        return name + "_" + testData.getSerial() + "_" + testData.getTestNumber() + "_" + testData.getDate().getTime();
    }

    @SuppressWarnings("ResultOfMethodCallIgnored")
    public boolean createFolder(Context context, String model)
    {
        if ( !this.isExternalStorageAvaiable() )
            return false;

        rootDir     = new File(root + "/" + context.getResources().getString(R.string.company));
        if ( !rootDir.exists() )
            rootDir.mkdir();

        dir     = new File(root + "/" + context.getResources().getString(R.string.company)+ "/" + model);
        tempDir = new File(dir + "/temp");

        if ( !dir.exists() )
            dir.mkdir();

        if ( !tempDir.exists() )
            tempDir.mkdir();

        return dir.exists();
    }

    @SuppressWarnings("ResultOfMethodCallIgnored")
    public File getFolder()
    {
        if ( !rootDir.exists() )
            rootDir.mkdir();

        if ( !dir.exists() )
            dir.mkdir();

        return dir;
    }

    @SuppressWarnings("ResultOfMethodCallIgnored")
    File getTemporaryFolder()
    {
        if ( !rootDir.exists() )
            rootDir.mkdir();

        if ( !dir.exists() )
            dir.mkdir();

        if ( !tempDir.exists() )
            tempDir.mkdir();

        return tempDir;
    }

    public File getPictureFile(String serial, String timeStamp)
    {
        String img = serial + "_" + timeStamp + ".jpg";
        return new File(dir, img);
    }

    public File getPictureFile(String serial, String testNumber, String timeStamp)
    {
        String img = serial + "_" + testNumber + "_" + timeStamp + ".jpg";
        return new File(dir, img);
    }

    File getAudioFile(String serial, String timeStamp)
    {
        String audioSuffix = "_annotation.3gp";
        String audio = serial + "_" + timeStamp + audioSuffix;
        return new File(dir, audio);
    }

    File getMapSnapshotFile(String serial, String timeStamp)
    {
        String mapSuffix = "_map_snapshot.jpg";
        String map = serial + "_" + timeStamp + mapSuffix;
        return new File(dir, map);
    }

    @SuppressWarnings("ResultOfMethodCallIgnored")
    private void deleteFiles(String serial, String timeStamp)
    {
        if (dir.listFiles()==null)
            return;
        for ( File child : Objects.requireNonNull( dir.listFiles() ) )
        {
            if ( child.getName().contains( serial + "_" + timeStamp ) )
            {
                child.delete();
            }
        }
    }

    @Override public void onCreate(SQLiteDatabase db) {}
    @Override public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) { }

    public static abstract class Entry
    {
        static final int
                DB_VERSION = 2;

        static final String
                //DB_NAME         = Controller.model + "_" + Controller.serial + "_REGISTERS.db",
                DESCRIPTION     = "DESCRIPTION",
                COL_A           = "COL_A",
                COL_B           = "COL_B",
                COL_C           = "COL_C",
                COL_D           = "COL_D",
                COL_E           = "COL_E";

        static final String
                MODEL           = "MODEL",
                SERIAL          = "SERIAL",
                MEMORY_MAP      = "MEMORY_MAP",
                FIRMWARE        = "FIRMWARE",
                HARDWARE        = "HARDWARE",
                OP_MODE         = "OP_MODE",
                DATE            = "DATE",
                TEST_NUMBER     = "TEST_NUMBER",
                PI              = "PI",
                SVT             = "SVT",
                MEASURE         = "MEASURE",
                RESULT          = "RESULT",
                GPS_LATITUDE    = "GPS_LATITUDE",
                GPS_LONGITUDE   = "GPS_LONGITUDE",
                GPS_ALTITUDE    = "GPS_ALTITUDE",
                GPS_ACCURACY    = "GPS_ACCURACY";
    }

}

