package com.megabras.bluelogg.hp60kv;

import android.content.Context;
import android.content.res.Configuration;
import android.graphics.drawable.Drawable;
import android.text.Spannable;
import android.text.SpannableString;
import android.text.style.ImageSpan;
import android.view.View;

import com.megabras.bluelogg.R;
import com.megabras.modbus.HP60KV.Device;
import com.megabras.modbus.HP60KV.Measurement;
import com.megabras.modbus.HP60KV.TestDataAndroid;

import java.util.ArrayList;
import java.util.List;

import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentStatePagerAdapter;


public class PagerAdapter extends FragmentStatePagerAdapter
{
    @SuppressWarnings("unused")
    private final String
            TAG = "PagerAdapter";

    private Context
            context;

    private MainScreen
            mainScreen;

    private Charts
            voltageChart,
            currentChart,
            ohmChart,
            table;

    private final List<Fragment> mFragmentList = new ArrayList<>();

    private final List<CharSequence> mFragmentTitleList = new ArrayList<>();

    protected final com.megabras.modbus.HP60KV.FormatData
            formatData = new com.megabras.modbus.HP60KV.FormatData();

    public PagerAdapter(FragmentManager fm, Context context)
    {
        super(fm);
        this.context = context;
        mainScreen   = new MainScreen();
        currentChart = new Charts();
        currentChart.setChartType( Charts.CURRENT_CHART, context.getString(R.string.md_current) );
        voltageChart = new Charts();
        voltageChart.setChartType( Charts.VOLTAGE_CHART, context.getString(R.string.md_voltage) );
        ohmChart     = new Charts();
        ohmChart.setChartType( Charts.RESISTANCE_CHART, context.getString(R.string.md_resistance) );
        table        = new Charts();
        table.setChartType( Charts.TABLE, "" );

        mFragmentList.add(mainScreen);
        mFragmentList.add(voltageChart);
        mFragmentList.add(currentChart);
        //mFragmentList.add(ohmChart);
        mFragmentList.add(table);


        mFragmentTitleList.add(convertTitle(0));
        mFragmentTitleList.add(convertTitle(1));
        mFragmentTitleList.add(convertTitle(2));
        //mFragmentTitleList.add(convertTitle(3));
        mFragmentTitleList.add(convertTitle(4));

    }

    @Override
    public Fragment getItem(int i)
    {
        return mFragmentList.get(i);
//        switch (i)
//        {
//            case 0:
//                return mainScreen;
//            case 1:
//                return voltageChart;
//            case 2:
//                return currentChart;
//            case 3:
//                return ohmChart;
//            case 4:
//                return table;
//        }
//        return null;
    }

    @Override
    public int getCount()
    {
        return mFragmentList.size();
        //return numPages;
    }

    @Override
    public CharSequence getPageTitle(int position)
    {
        return mFragmentTitleList.get(position);
//        int[] images;
//        if(screenSizeIsLarge(context))
//            images = imageResId;
//        else
//            images = imageResIdSmall;
//
//        Drawable image      = context.getResources().getDrawable(images[position]);
//        image.setBounds(0, 0, image.getIntrinsicWidth(), image.getIntrinsicHeight());
//        SpannableString sb  = new SpannableString(" ");
//        ImageSpan imageSpan = new ImageSpan(image, ImageSpan.ALIGN_BOTTOM);
//        sb.setSpan(imageSpan, 0, 1, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE);
//        return sb;
    }

    private CharSequence convertTitle(int position)
    {
        int[] images;
        if(screenSizeIsLarge(context))
            images = imageResId;
        else
            images = imageResIdSmall;

        Drawable image      = context.getResources().getDrawable(images[position]);
        image.setBounds(0, 0, image.getIntrinsicWidth(), image.getIntrinsicHeight());
        SpannableString sb  = new SpannableString(" ");
        ImageSpan imageSpan = new ImageSpan(image, ImageSpan.ALIGN_BOTTOM);
        sb.setSpan(imageSpan, 0, 1, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE);
        return sb;
    }

    private int[] imageResId =
    {
        R.drawable.ic_analog,
        R.drawable.ic_chart_voltage,
        R.drawable.ic_chart_current,
        R.drawable.ic_chart_resistance,
        R.drawable.ic_table
    };

    private int[] imageResIdSmall =
    {
        R.drawable.ic_analog_20,
        R.drawable.ic_chart_voltage_20,
        R.drawable.ic_chart_current_20,
        R.drawable.ic_chart_resistance_20,
        R.drawable.ic_table_20
    };

//    void removeFragment(int position)
//    {
//
//        mFragmentList.remove(position);
//        mFragmentTitleList.remove(position);
//        notifyDataSetChanged();
//        //mFragmentTitleList.remove(position);
//    }

    void setTestNumber(int number)
    {
        for (Fragment f : mFragmentList)
        {
            if (f instanceof Charts)
            {
                Charts c = (Charts)f;
                if (c.getChartType()==Charts.TABLE)
                {
                    c.setTestNumber(number);
                }
            }
            else if (f instanceof MainScreen)
            {
                MainScreen m = (MainScreen)f;
                m.setTestNumber(number);
            }
        }
    }

    void setTestDate(String dateTime)
    {
        for (Fragment f : mFragmentList)
        {
            if (f instanceof Charts)
            {
                Charts c = (Charts)f;
                if (c.getChartType()==Charts.TABLE)
                {
                    c.setDate(dateTime);
                }
            }
        }
    }

    void setTestMode(String mode)
    {
        for (Fragment f : mFragmentList)
        {
            if (f instanceof Charts)
            {
                Charts c = (Charts)f;
                if (c.getChartType()==Charts.TABLE)
                {
                    c.setMode(mode);
                }
            }
        }
    }

    void waitonDisplay()
    {
        mainScreen.waitonDisplay();
    }

    void setTestResult(int result)
    {
        table.setTestResult(result);
    }


    void fillChartsAndTable(TestDataAndroid testData, int firmware, boolean connected)
    {
        for (Fragment f : mFragmentList)
        {
            if (f instanceof Charts)
            {
                Charts c = (Charts)f;
                c.reset();
                if (c.getChartType()==Charts.TABLE)
                {
                    c.setAllValues(testData.getMeasurement(), firmware);
                    c.setTestNumber(testData.getTestNumber());
                    c.setMode(context.getResources().getStringArray(R.array.hp60kv_op_modes)[testData.getMode()]);
                    c.setTestResult(testData.getTestResult());
                    c.setDate(formatData.getDateFormatted( testData.getDate(), testData.getDevice().getDateFormat(), testData.getDevice().getTimeFormat()));
                }
                else
                {
                    c.setAllValues( testData.getMeasurement(), firmware );
                }
            }
            else if (f instanceof MainScreen)
            {
                MainScreen m = (MainScreen)f;
                m.reset();
                if ( connected && testData.getDevice().getOperationMode()>=0 )
                    m.setTestParameters( testData.getDevice(), firmware );
            }
        }
    }

//    void fillCharts(TestDataAndroid testData, int firmware)
//    {
//        for (Fragment f : mFragmentList)
//        {
//            if (f instanceof Charts)
//            {
//                Charts c = (Charts)f;
//                c.reset();
//                if (c.getChartType()!=Charts.TABLE)
//                {
//                    c.setAllValues( testData.getMeasurement(), firmware );
//                }
//            }
//        }
//    }

    void resetAll()
    {
        for (Fragment f : mFragmentList)
        {
            if (f instanceof Charts)
            {
                Charts c = (Charts)f;
                c.reset();
            }
            else if (f instanceof MainScreen)
            {
                MainScreen m = (MainScreen)f;
                m.reset();
            }
        }
    }

    void resetAll(boolean connected, Device device, int firmware)
    {
        for (Fragment f : mFragmentList)
        {
            if (f instanceof Charts)
            {
                Charts c = (Charts)f;
                c.reset();
            }
            else if (f instanceof MainScreen)
            {
                MainScreen m = (MainScreen)f;
                m.reset();
                if ( connected && device.getOperationMode()>=0 )
                    m.setTestParameters( device, firmware );
            }
        }
    }

    void resetMainScreen(boolean connected, Device device, int firmware)
    {
        for (Fragment f : mFragmentList)
        {
            if (f instanceof MainScreen)
            {
                MainScreen m = (MainScreen)f;
                m.reset();
                if ( connected && device.getOperationMode()>0 )
                    m.setTestParameters( device, firmware );
            }
        }
    }

    void setPI(double pi)
    {
        if (!Double.isNaN(pi))
        {
            String piString;
            if (pi<0)
                piString = " - - - ";
            else
                piString = formatData.formatIndexes(pi);
            mainScreen.setPI(piString);
            table.setPI(piString);
        }
    }

    void setPITable(double pi)
    {
        if (!Double.isNaN(pi))
        {
            String piString;
            if (pi<0)
                piString = " - - - ";
            else
                piString = formatData.formatIndexes(pi);
            table.setPI(piString);
        }
    }

    void setDeviceID( int id )
    {
        formatData.setDeviceId( id );
        mainScreen.setDeviceID( id );
        table.setDeviceID( id );
    }

    void hidePI()
    {
        mainScreen.hidePI();
    }

    void setTestParameters(Device device, int firmware)
    {
        mainScreen.setTestParameters( device, firmware );
    }

    void resetCharts()
    {
        for (Fragment f : mFragmentList)
        {
            if (f instanceof Charts)
            {
                Charts c = (Charts)f;
                c.reset();
            }
        }
    }

    void updateChartsAndTable(Measurement measurement, int firmware, boolean updateLast)
    {
        for (Fragment f : mFragmentList)
        {
            if (f instanceof Charts)
            {
                Charts c = (Charts)f;
                c.update(measurement, firmware, updateLast);
            }
        }
    }

    void showChartResistance(boolean show)
    {
        boolean contains = false;
        for (Fragment f : mFragmentList)
        {
            if (f instanceof Charts)
            {
                Charts c = (Charts)f;
                if (c.getChartType()==Charts.RESISTANCE_CHART)
                    contains=true;
            }
        }
        if (show)
        {
            mainScreen.setVisibility(R.id.lb_resistance, View.VISIBLE);
            if (!contains)
            {
                mFragmentList.add(3, ohmChart);
                mFragmentTitleList.add(3 ,convertTitle(3));
                //notifyDataSetChanged();
            }
        }
        else
        {
            mainScreen.setVisibility(R.id.lb_resistance, View.GONE);
            if (contains)
            {
                mFragmentList.remove(3);
                mFragmentTitleList.remove(3);
                notifyDataSetChanged();
            }
        }
    }

    void showResistanceFields(boolean show)
    {
        if (show)
        {
            mainScreen.setVisibility(R.id.lb_resistance, View.VISIBLE);
        }
        else
        {
            mainScreen.setVisibility(R.id.lb_resistance, View.GONE);
        }
        table.showResistance(show);
    }

    void updateMainScreen(Measurement measurement, Device device, int firmware)
    {
        mainScreen.update(measurement, device, firmware);
    }

    void updateStatus(boolean[] statusOne)
    {
        for (Fragment f : mFragmentList)
        {
            if (f instanceof MainScreen)
            {
                MainScreen m = (MainScreen)f;
                m.updateStatus(statusOne);
            }
        }
    }

    Charts chartCurrent()
    {
        return currentChart;
    }

    Charts chartVoltage()
    {
        return voltageChart;
    }

    Charts chartOhms()
    {
        return ohmChart;
    }

    private boolean screenSizeIsLarge(Context context)
    {
        return (context.getResources().getConfiguration().screenLayout &
                Configuration.SCREENLAYOUT_SIZE_MASK) == Configuration.SCREENLAYOUT_SIZE_LARGE;
    }






}



