package com.megabras.bluelogg.hp60kv;

import android.app.Activity;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Handler;
import android.util.Log;

import com.itextpdf.text.Document;
import com.itextpdf.text.PageSize;
import com.itextpdf.text.pdf.PdfWriter;
import com.itextpdf.tool.xml.XMLWorkerHelper;
import com.megabras.DeviceList;
import com.megabras.bluelogg.R;
import com.megabras.bluelogg.extended.ReportProvider;
import com.megabras.modbus.HP60KV.Device;
import com.megabras.modbus.HP60KV.Measurement;
import com.megabras.modbus.HP60KV.TestDataAndroid;
import com.megabras.modbus.MUtils;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StringReader;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Locale;

@SuppressWarnings("FieldCanBeLocal")
public class Report extends ReportProvider
{
    private final String
            TAG = "Report";

    private String
            ohmSymbol = "<span class=\"style-symbol\">W</span>";

    protected final com.megabras.modbus.HP60KV.FormatData
            formatData;

    public Report(Activity activity, File dir, File tempDir, Handler handler)
    {
        super(activity, dir, tempDir, handler);
        formatData = new com.megabras.modbus.HP60KV.FormatData();
    }

    @SuppressWarnings("ResultOfMethodCallIgnored")
    void saveDocReport(TestDataAndroid testData, File picture, File map, File voltageImage, File currentImage, File ohmsImage)
    {
        try
        {
            formatData.setDeviceId( DeviceList.getIdByModel( testData.getModel() ) );

            handler.obtainMessage(com.megabras.bluelogg.md15kvr.Controller.mHandler.REPORT_UPDATE, 1, -1, activity.getString(R.string.report_creating_file)).sendToTarget();
            Device device  = testData.getDevice();

            String docFile =
                    testData.getModel()  + "_" +
                    testData.getSerial() + "_" +
                    testData.getDate().getTime() + "_report.pdf";

            File tmpFile        = new File(dir + "/"+ docFile);
            Document document   = new Document(PageSize.A4, 10, 10, 10, 8);
            PdfWriter pdfWriter = PdfWriter.getInstance(document, new FileOutputStream(tmpFile));
            document.open();
            document.addTitle(activity.getString(R.string.dielectric_withstand_test));
            InputStream fin     = activity.getAssets().open("HP60KV-template.html");
            BufferedReader r    = new BufferedReader(new InputStreamReader(fin));
            StringBuilder total = new StringBuilder();
            String line;
            String  altitude    = "",
                    accuracy    = "",
                    latitude    = "",
                    longitud    = "";

            if ( testData.getGpsAcuraccy()>0 )
            {
                altitude = testData.getGpsAltitude() + " m";
                accuracy = "\u00B1" + testData.getGpsAcuraccy() + " m";
                latitude = testData.getGpsLatitude() + "";
                longitud = testData.getGpsLongitude() + "";
            }

            String hex          = com.megabras.modbus.MUtils.intToHex(testData.getTestResult(), 4);
            boolean[] statusOne = com.megabras.modbus.MUtils.hexToBitArray(hex);
            String reason       = " - - - ";
            String result;

            if ( statusOne[0] )
                reason = activity.getString(R.string.spark_detection).toUpperCase();
            else if (statusOne[1])
                reason = activity.getString(R.string.overcurrent).toUpperCase();


            if ( statusOne[2] && statusOne[3] )
                result = activity.getString(R.string.pass);
            else if ( !statusOne[2] && statusOne[3] )
                result = activity.getString(R.string.fail);
            else if ( statusOne[2] ) // else if ( statusOne[2] && !statusOne[3] )
                result = activity.getString(R.string.aborted);
            else
                result = " - - - ";


            while ( (line=r.readLine())!=null ) {
                total.append(line);
            }

            handler.obtainMessage(com.megabras.bluelogg.md15kvr.Controller.mHandler.REPORT_UPDATE, 1, -1, activity.getString(R.string.report_general_info)).sendToTarget();

            XMLWorkerHelper worker = XMLWorkerHelper.getInstance();
            String htmlToPDF       = total.toString();

            htmlToPDF = htmlToPDF.replace("{TITLE}"             , activity.getString(R.string.dielectric_withstand_test).toUpperCase());
            htmlToPDF = htmlToPDF.replace("{LB_MODEL}"          , activity.getString(R.string.md_model).toUpperCase(Locale.getDefault()));
            htmlToPDF = htmlToPDF.replace("{MODEL}"             , testData.getModel());
            htmlToPDF = htmlToPDF.replace("{LB_SERIAL}"         , activity.getString(R.string.md_serial).toUpperCase(Locale.getDefault()));
            htmlToPDF = htmlToPDF.replace("{SERIAL}"            , testData.getSerial());

            Log.e(TAG, "isEquipmentVersionDefinied: " + testData.isEquipmentVersionDefinied() );
            Log.e(TAG, "getEquipmentVersion       : " + testData.getEquipmentVersion()        );

            if ( testData.isEquipmentVersionDefinied() )
            {
                htmlToPDF = htmlToPDF.replace("{LB_FIRMWARE}"  , "FIRMWARE");
                htmlToPDF = htmlToPDF.replace("{LB_PI_DOTS_A}" , ":");
                htmlToPDF = htmlToPDF.replace("{FIRMWARE}"     , testData.getEquipmentVersion());
            }
            else
            {
                htmlToPDF = htmlToPDF.replace("{LB_FIRMWARE}"  , "");
                htmlToPDF = htmlToPDF.replace("{LB_PI_DOTS_A}" , "");
                htmlToPDF = htmlToPDF.replace("{FIRMWARE}"     , "");
            }
            htmlToPDF = htmlToPDF.replace("{LB_DATE}"           , activity.getString(R.string.md_test_date).toUpperCase(Locale.getDefault()));
            htmlToPDF = htmlToPDF.replace("{DATE}"              , formatData.getDateFormatted(testData.getDate(), device.getDateFormat(), device.getTimeFormat()));
            htmlToPDF = htmlToPDF.replace("{LB_TEST_MODE}"      , activity.getString(R.string.md_mode).toUpperCase(Locale.getDefault()));
            htmlToPDF = htmlToPDF.replace("{TEST MODE}"         , activity.getResources().getStringArray(R.array.hp60kv_op_modes)[ testData.getMode() ].toUpperCase(Locale.getDefault()) );
            htmlToPDF = htmlToPDF.replace("{LB_TEST_NUM}"       , activity.getString(R.string.md_test_number).toUpperCase(Locale.getDefault()));
            htmlToPDF = htmlToPDF.replace("{TEST NUMBER}"       , testData.getTestNumber()+"");
            htmlToPDF = htmlToPDF.replace("{LB_LATITUDE}"       , activity.getString(R.string.md_latitude));
            htmlToPDF = htmlToPDF.replace("{LATITUDE}"          , latitude);
            htmlToPDF = htmlToPDF.replace("{LB_LONGITUDE}"      , activity.getString(R.string.md_longitude));
            htmlToPDF = htmlToPDF.replace("{LONGITUDE}"         , longitud);
            htmlToPDF = htmlToPDF.replace("{LB_ALTITUDE}"       , activity.getString(R.string.md_altitude));
            htmlToPDF = htmlToPDF.replace("{ALTITUDE}"          , altitude);
            htmlToPDF = htmlToPDF.replace("{LB_GPS_ACCURACY}"   , activity.getString(R.string.md_gps_accuracy));
            htmlToPDF = htmlToPDF.replace("{GPS_ACCURACY}"      , accuracy);
            htmlToPDF = htmlToPDF.replace("{LB_RESULT}"         , activity.getString(R.string.result).toUpperCase());
            htmlToPDF = htmlToPDF.replace("{RESULT}"            , result);
            htmlToPDF = htmlToPDF.replace("{LB_REASON}"         , activity.getString(R.string.reason).toUpperCase());
            htmlToPDF = htmlToPDF.replace("{REASON}"            , reason);
            if ( testData.getFirmware() >= Controller.FIRMWARE_OHMS && testData.getFirmware() < Controller.FIRMWARE_HIDE_OHMS )
            {
                htmlToPDF = htmlToPDF.replace("{LB_PI}"     , activity.getString(R.string.PI));
                htmlToPDF = htmlToPDF.replace("{LB_PI_DOTS}", ":");
                htmlToPDF = htmlToPDF.replace("{PI}"        , formatData.formatIndexes(testData.getPI()));
            }
            else
            {
                htmlToPDF = htmlToPDF.replace("{LB_PI}"     , "");
                htmlToPDF = htmlToPDF.replace("{LB_PI_DOTS}", "");
                htmlToPDF = htmlToPDF.replace("{PI}"        , "");
            }
            //htmlToPDF = htmlToPDF.replace("{SVT}"               , FormatData.formatIndexes(testData.getSVT()));
            htmlToPDF = htmlToPDF.replace("{LB_MEASURED_VALUES}", activity.getString(R.string.md_measured_values));

            ArrayList<Measurement> results = testData.getMeasurement();

            double
                maxCurrent     = 0.0,
                averageCurrent = 0.0,
                maxOhms        = 0.0,
                averageOhms    = 0.0;

            int
                maxVoltage     = 0,
                averageVoltage = 0,
                ohmSize        = 0;

            for ( int i=0; i<results.size(); i++ )
            {
                    boolean[] stats = new boolean[16];
                    boolean[] st = MUtils.intToBitArray(results.get(i).getStats());
                    System.arraycopy(st, 0, stats, 0, st.length);

                    if (maxCurrent < results.get(i).getCurrent())
                    {
                        maxCurrent = results.get(i).getCurrent();
                    }

                    if (maxVoltage < results.get(i).getVoltage())
                    {
                        maxVoltage = results.get(i).getVoltage();
                    }

                    if (maxOhms < results.get(i).getResistance())
                    {
                        if (!stats[Device.ST_TWO_MEASURE_WAIT]
                            && !stats[Device.ST_TWO_MEASURE_OVERRANGE]
                            && !stats[Device.ST_TWO_MEASURE_INVALID]
                            && results.get(i).getResistance() <= device.DEVICE_OHMS_MAX)
                        {
                            maxOhms = results.get(i).getResistance();
                        }
                    }

//                    if ( results.get(i).getCurrent() > (Controller.DEVICE_MIN_CURRENT*Controller.DEVICE_CURRENT_FACTOR))
//                    {
                        averageCurrent += results.get(i).getCurrent();
                        //ampSize++;
                    //}
                    averageVoltage += results.get(i).getVoltage();

                    if (!stats[Device.ST_TWO_MEASURE_WAIT]
                        && !stats[Device.ST_TWO_MEASURE_OVERRANGE]
                        && !stats[Device.ST_TWO_MEASURE_INVALID]
                        && results.get(i).getResistance() <= device.DEVICE_OHMS_MAX)
                    {
                        averageOhms += results.get(i).getResistance();
                        ohmSize++;
                    }
                //}
            }

            averageCurrent = averageCurrent / results.size();
            averageVoltage = averageVoltage / results.size();
            averageOhms    = averageOhms / ohmSize;


            htmlToPDF = htmlToPDF.replace("{LB_VALUE}"       , activity.getResources().getString(R.string.value));
            htmlToPDF = htmlToPDF.replace("{LB_TB_VOLTAGE}"  , activity.getResources().getString(R.string.voltage));
            htmlToPDF = htmlToPDF.replace("{LB_TB_CURRENT}"  , activity.getResources().getString(R.string.current));
            if ( testData.getFirmware() >= Controller.FIRMWARE_OHMS && testData.getFirmware() < Controller.FIRMWARE_HIDE_OHMS)
                htmlToPDF = htmlToPDF.replace("{LB_TB_OHMS}"     , activity.getResources().getString(R.string.resistance));
            else
                htmlToPDF = htmlToPDF.replace("{LB_TB_OHMS}"     , "");
            htmlToPDF = htmlToPDF.replace("{LB_LAST}"        , activity.getResources().getString(R.string.last));
            htmlToPDF = htmlToPDF.replace("{LB_MAX}"         , activity.getResources().getString(R.string.maximum));
            htmlToPDF = htmlToPDF.replace("{LB_AVERAGE}"     , activity.getResources().getString(R.string.average));
            htmlToPDF = htmlToPDF.replace("{LAST_VOLTAGE}"   , formatData.formatVoltage(results.get(results.size()-1).getVoltage()));
            htmlToPDF = htmlToPDF.replace("{LAST_CURRENT}"   , formatData.formatCurrentRT(results.get(results.size()-1).getCurrent(), testData.getFirmware()).replace("<", "&lt;"));
            if ( testData.getFirmware() >= Controller.FIRMWARE_OHMS && testData.getFirmware() < Controller.FIRMWARE_HIDE_OHMS )
                htmlToPDF = htmlToPDF.replace("{LAST_OHMS}"      , formatData.formatResistance(results.get(results.size()-1).getResistance(), results.get(results.size()-1).getStats()).replace("<", "&lt;").replace("\u03A9", ohmSymbol));
            else
                htmlToPDF = htmlToPDF.replace("{LAST_OHMS}"      , "");
            htmlToPDF = htmlToPDF.replace("{MAX_VOLTAGE}"    , formatData.formatVoltage(maxVoltage));
            htmlToPDF = htmlToPDF.replace("{MAX_CURRENT}"    , formatData.formatCurrentRT(maxCurrent, testData.getFirmware()).replace("<", "&lt;"));
            if ( testData.getFirmware() >= Controller.FIRMWARE_OHMS && testData.getFirmware() < Controller.FIRMWARE_HIDE_OHMS )
                htmlToPDF = htmlToPDF.replace("{MAX_OHMS}"       , formatData.formatResistance(maxOhms, 0).replace("<", "&lt;").replace("\u03A9", ohmSymbol));
            else
                htmlToPDF = htmlToPDF.replace("{MAX_OHMS}"       , "");
            htmlToPDF = htmlToPDF.replace("{AVERAGE_VOLTAGE}", formatData.formatVoltage(averageVoltage));
            htmlToPDF = htmlToPDF.replace("{AVERAGE_CURRENT}", formatData.formatCurrentRT(averageCurrent, testData.getFirmware()).replace("<", "&lt;"));
            if ( testData.getFirmware() >= Controller.FIRMWARE_OHMS && testData.getFirmware() < Controller.FIRMWARE_HIDE_OHMS )
                htmlToPDF = htmlToPDF.replace("{AVERAGE_OHMS}"   , formatData.formatResistance(averageOhms, 0).replace("<", "&lt;").replace("\u03A9", ohmSymbol));
            else
                htmlToPDF = htmlToPDF.replace("{AVERAGE_OHMS}"   , "");


            handler.obtainMessage(com.megabras.bluelogg.md15kvr.Controller.mHandler.REPORT_UPDATE, 1, -1, activity.getString(R.string.report_loading_images)).sendToTarget();
            // IMAGES
            if ( picture.exists() )
                htmlToPDF = htmlToPDF.replace("{PICTURE}"   , picture.getAbsolutePath());
            if ( map.exists() )
                htmlToPDF = htmlToPDF.replace("{PICMAP}"    , map.getAbsolutePath());
            htmlToPDF = htmlToPDF.replace("{VOLTAGE CHART}"     , voltageImage.getAbsolutePath());
            htmlToPDF = htmlToPDF.replace("{CURRENT CHART}"     , currentImage.getAbsolutePath());

            if ( testData.getFirmware() >= Controller.FIRMWARE_OHMS && testData.getFirmware() < Controller.FIRMWARE_HIDE_OHMS && ohmsImage != null)
                htmlToPDF = htmlToPDF.replace("{RESISTANCE CHART}"  , ohmsImage.getAbsolutePath());

            handler.obtainMessage(com.megabras.bluelogg.md15kvr.Controller.mHandler.REPORT_UPDATE, 1, -1, activity.getString(R.string.report_tables)).sendToTarget();

            // MEASUREMENTS
            StringBuilder table = new StringBuilder();
            table.append("<table>");
            table.append("<tr>");
            table.append("<td class=\"style-cell-values-25\">").append(activity.getString(R.string.MMSS)).append("</td>");
            table.append("<td class=\"style-cell-values-25\">").append(activity.getString(R.string.V)).append("</td>");
            table.append("<td class=\"style-cell-values-25\">").append(activity.getString(R.string.A)).append("</td>");
            if ( testData.getFirmware() >= Controller.FIRMWARE_OHMS && testData.getFirmware() < Controller.FIRMWARE_HIDE_OHMS )
            {
                table.append("<td class=\"style-cell-values-25\">").append(ohmSymbol).append("</td>");
            }
            table.append("</tr>");

            boolean lastRow = false;
            int sample      = 1;
            int maxRows     = 85;

            if ( results.size() > maxRows )
                sample = (results.size() / maxRows) + 1;

            DecimalFormat voltageFormat = new DecimalFormat("0.00 kV");
            voltageFormat.setRoundingMode(RoundingMode.DOWN);

            for ( int i = 0; i < results.size(); i += sample )
            {
                int baseSeconds = results.get(i).getElapsed();
                int voltage     = results.get(i).getVoltage();
                double current  = results.get(i).getCurrent();
                double ohms     = results.get(i).getResistance();

                String formattedMilliseconds = formatData.formatElapsedTime ( (short) baseSeconds );
                String formattedVoltage      = formatData.formatVoltage     ( voltage );
                String formattedCurrent      = formatData.formatCurrentRT   ( current, testData.getFirmware() );
                String formattedOhms         = formatData.formatResistance  ( ohms, results.get(i).getStats() ).replace("<", "&lt;").replace("\u03A9", ohmSymbol);

                table.append("<tr>");
                table.append("<td class=\"style-cell-values-25\">").append(formattedMilliseconds).append("</td>");
                table.append("<td class=\"style-cell-values-25\">").append(formattedVoltage).append("</td>");
                table.append("<td class=\"style-cell-values-25\">").append(formattedCurrent.replace("<", "&lt;")).append("</td>");
                if ( testData.getFirmware() >= Controller.FIRMWARE_OHMS && testData.getFirmware() < Controller.FIRMWARE_HIDE_OHMS )
                    table.append("<td class=\"style-cell-values-25\">").append(formattedOhms).append("</td>");
                table.append("</tr>");

                if (i == results.size() - 1)
                    lastRow = true;
            }

            if ( !lastRow )
            {
                int    baseSeconds = results.get(results.size()-1).getElapsed();
                int    voltage     = results.get(results.size()-1).getVoltage();
                double current     = results.get(results.size()-1).getCurrent();
                double ohms        = results.get(results.size()-1).getResistance();
                int    stats       = results.get(results.size()-1).getStats();

                String formattedMilliseconds = formatData.formatElapsedTime((short)baseSeconds);
                String formattedVoltage      = voltageFormat.format((double)voltage/1000);
                String formattedCurrent      = formatData.formatCurrentRT(current, testData.getFirmware());
                String formattedOhms         = formatData.formatResistance(ohms, stats).replace("<", "&lt;").replace("\u03A9", ohmSymbol);

                table.append("<tr>");
                table.append("<td class=\"style-cell-values-25\">").append(formattedMilliseconds).append("</td>");
                table.append("<td class=\"style-cell-values-25\">").append(formattedVoltage).append("</td>");
                table.append("<td class=\"style-cell-values-25\">").append(formattedCurrent.replace("<", "&lt;")).append("</td>");
                if ( testData.getFirmware() >= Controller.FIRMWARE_OHMS && testData.getFirmware() < Controller.FIRMWARE_HIDE_OHMS )
                    table.append("<td class=\"style-cell-values-25\">").append(formattedOhms).append("</td>");
                table.append("</tr>");
            }

            table.append("</table>");

            handler.obtainMessage(com.megabras.bluelogg.md15kvr.Controller.mHandler.REPORT_UPDATE, 1, -1, activity.getString(R.string.saving_file)).sendToTarget();

            Bitmap bm = BitmapFactory.decodeResource( activity.getResources(), R.drawable.brand_report);
            File file = new File(tempDir, "ic_brand.png");
            FileOutputStream outStream = new FileOutputStream(file);

            int   newHeight = 25;
            float scale     = (float)(bm.getHeight() / newHeight);
            int   newWidth  = (int)(bm.getWidth() / scale);

            bm.compress(Bitmap.CompressFormat.PNG, 100, outStream);
            outStream.flush();
            outStream.close();
            htmlToPDF = htmlToPDF.replace("{REPORT BRAND}", file.getAbsolutePath());
            htmlToPDF = htmlToPDF.replace("{LB_WIDTH}"    , newWidth+"");
            htmlToPDF = htmlToPDF.replace("{LB_HEIGHT}"   , newHeight+"");

            String generatedBy = activity.getString(R.string.md_report_generated) + "<br />" +
                    activity.getString(R.string.app_name) + "<br />" +
                    activity.getString(R.string.version) + ": " +
                    activity.getPackageManager().getPackageInfo(activity.getPackageName(), 0).versionName;
            htmlToPDF = htmlToPDF.replace("{LB_GENERATED}", generatedBy);
            htmlToPDF = htmlToPDF.replace("{TABLE}", table.toString());

            worker.parseXHtml(pdfWriter, document, new StringReader(htmlToPDF));
            document.close();

            if ( tempDir.exists() )
            {
                File[] children = tempDir.listFiles();
                if ( children != null )
                {
                    for ( File aChildren : children )
                    {
                        aChildren.delete();
                    }
                }
                tempDir.delete();
            }
            handler.obtainMessage(com.megabras.bluelogg.md15kvr.Controller.mHandler.REPORT_FINISHED, 1, -1, activity.getString(R.string.finished)).sendToTarget();
            sendFile( tmpFile );
        }
        catch(Exception e){
            for (StackTraceElement ste : e.getStackTrace())
                android.util.Log.e(TAG, ste.toString());
            handler.obtainMessage(Controller.mHandler.REPORT_ERROR, 1, -1, activity.getString(R.string.finished)).sendToTarget();
        }
    }
}
