package com.megabras.bluelogg.hp60kv;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.TextView;

import com.megabras.bluelogg.R;

import java.util.ArrayList;
import java.util.List;

public class TableBaseAdapter extends BaseAdapter
{
    private LayoutInflater
            inflater;

    private List<TableValues>
            tableValues;

    private boolean
            showResistance = true;

    private String
            testNumber   = " - - -",
            reason       = " - - -",
            mode         = " - - -",
            result       = " - - -",
            date         = " - - -",
            pi           = " - - -";

    public TableBaseAdapter(Context context)
    {
        tableValues  = new ArrayList<>();
        inflater     = LayoutInflater.from(context);
    }

    static class ViewHolder
    {
        TextView
                tv_test_number,
                tv_test_date,
                tv_mode,
                tv_reason,
                tv_result,
                tv_elapsed,
                tv_voltage,
                tv_current,
                tv_ohms,
                tv_title_col_d,
                ui_tv_pi_title,
                ui_tv_pi;
    }

    void showResistance(boolean show)
    {
        showResistance = show;
    }

    public int getCount()
    {
        return tableValues.size();
    }

    public TableValues getItem(int position)
    {
        return tableValues.get(position);
    }

    @Override
    public long getItemId(int position)
    {
        return 0;
    }

    public void add(String time, String voltage, String current, String ohms)
    {
        tableValues.add( new TableValues(time, voltage, current, ohms) );
    }

    public void remove(int position)
    {
        tableValues.remove(position);
    }

    public void clear()
    {
        if ( testNumber!=null )
            testNumber = " - - -";
        if ( reason!=null )
            reason = " - - -";
        if ( mode!=null )
            mode = " - - -";
        if ( result!=null )
            result = " - - -";
        if ( date!=null )
            date = " - - -";
        if ( pi!=null )
            pi = " - - -";
        tableValues.clear();
    }

    public void repaint()
    {
        notifyDataSetChanged();
    }

    private TextView getTV(int id, View v)
    {
        return ( (TextView)v.findViewById(id) );
    }

    public void setTestNumber(int testNumber)
    {
        this.testNumber = testNumber+"";
    }

    public void setMode(String mode)
    {
        this.mode = mode;
    }

    public void setPI(String pi)
    {
        this.pi = pi;
    }

    public void setDate(String date)
    {
        this.date = date;
    }

    public void setResult(int result)
    {
        String hex          = com.megabras.modbus.MUtils.intToHex(result, 4);
        boolean[] statusOne = com.megabras.modbus.MUtils.hexToBitArray(hex);

        if ( statusOne[0] )
            this.reason = inflater.getContext().getString(R.string.spark_detection).toUpperCase();
        else if (statusOne[1])
            this.reason = inflater.getContext().getString(R.string.overcurrent).toUpperCase();

        if ( statusOne[2] && statusOne[3] )
            this.result = inflater.getContext().getString(R.string.pass);
        else if ( !statusOne[2] && statusOne[3] )
            this.result = inflater.getContext().getString(R.string.fail);
        else if (statusOne[2])
            this.result = inflater.getContext().getString(R.string.aborted);
        else
            this.result = " - - - ";
        notifyDataSetChanged();
    }

    @SuppressWarnings("RedundantCast")
    public View getView(int position, View vi, ViewGroup parent)
    {
        int headerLayout = R.layout.hp60kv_table_header;
        int lineLayout   = (int)R.layout.table_item;
        if ( position==0 ) // First result
        {
            if ( vi == null || vi.getId()!=lineLayout)
            {
                ViewHolder h        = new ViewHolder();
                vi                  = inflater.inflate(headerLayout, parent, false);
                h.tv_test_number    = getTV(R.id.tv_number          , vi);
                h.tv_test_date      = getTV(R.id.tv_date            , vi);
                h.tv_mode           = getTV(R.id.tv_mode            , vi);
                h.tv_reason         = getTV(R.id.tv_reason          , vi);
                h.tv_result         = getTV(R.id.tv_test_result     , vi);
                h.tv_elapsed        = getTV(R.id.col_a              , vi);
                h.tv_voltage        = getTV(R.id.col_b              , vi);
                h.tv_current        = getTV(R.id.col_c              , vi);
                h.tv_ohms           = getTV(R.id.col_d              , vi);
                h.tv_title_col_d    = getTV(R.id.title_col_d        , vi);
                h.ui_tv_pi_title    = getTV(R.id.ui_tv_pi_title     , vi);
                h.ui_tv_pi          = getTV(R.id.ui_tv_pi           , vi);
                vi.setTag(h);
            }
            ViewHolder h = (ViewHolder) vi.getTag();
            h.tv_test_number    .setText( testNumber );
            h.tv_test_date      .setText( date );
            h.tv_mode           .setText( mode );
            h.tv_reason         .setText( reason );
            h.tv_result         .setText( result );
            h.ui_tv_pi          .setText( pi );
            h.tv_elapsed        .setText( tableValues.get(position).getTime()       );
            h.tv_voltage        .setText( tableValues.get(position).getVoltage()    );
            h.tv_current        .setText( tableValues.get(position).getCurrent()    );
            h.tv_ohms           .setText( tableValues.get(position).getOhms()       );
            if (showResistance)
            {
                h.tv_title_col_d .setVisibility(View.VISIBLE);
                h.tv_ohms        .setVisibility(View.VISIBLE);
                h.ui_tv_pi_title .setVisibility(View.VISIBLE);
                h.ui_tv_pi       .setVisibility(View.VISIBLE);
            }
            else
            {
                h.tv_title_col_d .setVisibility(View.GONE);
                h.tv_ohms        .setVisibility(View.GONE);
                h.ui_tv_pi_title .setVisibility(View.GONE);
                h.ui_tv_pi       .setVisibility(View.GONE);
            }
        }
        else // Second or higher result
        {
            if ( vi == null || (int)vi.getId()!= headerLayout)
            {
                ViewHolder h    = new ViewHolder();
                vi              = inflater.inflate(lineLayout, parent, false);
                h.tv_elapsed    = getTV(R.id.col_a, vi);
                h.tv_voltage    = getTV(R.id.col_b, vi);
                h.tv_current    = getTV(R.id.col_c, vi);
                h.tv_ohms       = getTV(R.id.col_d, vi);
                vi.setTag(h);
            }
            ViewHolder h = (ViewHolder) vi.getTag();
            h.tv_elapsed.setText( tableValues.get(position).getTime()       );
            h.tv_voltage.setText( tableValues.get(position).getVoltage()    );
            h.tv_current.setText( tableValues.get(position).getCurrent()    );
            h.tv_ohms   .setText( tableValues.get(position).getOhms()       );
            if (showResistance)
                h.tv_ohms.setVisibility(View.VISIBLE);
            else
                h.tv_ohms.setVisibility(View.GONE);

        }
        return vi;
    }
}

