package com.megabras.bluelogg.md15kvr;

import android.content.Context;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.megabras.bluelogg.R;
import com.megabras.modbus.MD15KVR.Device;
import com.megabras.modbus.MD15KVR.Measurement;
import com.megabras.modbus.MD15KVR.Protocol;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;

//import android.support.v4.app.Fragment;


@SuppressWarnings("WeakerAccess")
public class MainScreen extends Fragment
{
    @SuppressWarnings("unused")
    private final String
            TAG             = "MainScreen";

    public int
            layoutID        =  R.layout.md15kvr_fgr_main;

    protected Context
            context;

    protected View
            view;

//    private Device
//            device;

    private LinearLayout
            result_pass,
            result_fail;

    private TextView
            tvTestNumber,
            def_mode,
            def_voltage,
            def_max_voltage,
            def_limite,
            def_timer,
            def_svt,
            tvResistance,
            tvElapsed,
            tvVoltage,
            tvCurrent,
            tvResultDAI,
            tvResultPI,
            tvResultCAP,
            tvResultSVT,
            tvResultDD;

    private String
            testNumber = " - - - ";

    protected com.megabras.modbus.MD15KVR.FormatData
            formatData;

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        context = inflater.getContext();
        view = inflater.inflate(layoutID, container, false);
        view.setBackgroundColor(context.getResources().getColor(R.color.transparent));
        formatData = new com.megabras.modbus.MD15KVR.FormatData();
        return view;
    }

    @Override
    public void onStart() {
        super.onStart();
        if (this.getView()==null)
            return;
        result_pass     = this.getView().findViewById(R.id.result_pass);
        result_fail     = this.getView().findViewById(R.id.result_fail);
        tvTestNumber    = this.getView().findViewById(R.id.lb_test_number);
        def_mode        = this.getView().findViewById(R.id.lb_mode);
        def_voltage     = this.getView().findViewById(R.id.lb_defined_voltage);
        def_max_voltage = this.getView().findViewById(R.id.lb_defined_max_voltage);
        def_limite      = this.getView().findViewById(R.id.lb_lim);
        def_timer       = this.getView().findViewById(R.id.lb_timer);
        def_svt         = this.getView().findViewById(R.id.lb_svt);
        tvResistance    = this.getView().findViewById(R.id.lb_resistance);
        tvElapsed       = this.getView().findViewById(R.id.lb_time_elapsed);
        tvVoltage       = this.getView().findViewById(R.id.lb_voltage);
        tvCurrent       = this.getView().findViewById(R.id.lb_current);
        tvResultDAI     = this.getView().findViewById(R.id.lb_result_dai);
        tvResultPI      = this.getView().findViewById(R.id.lb_result_pi);
        tvResultCAP     = this.getView().findViewById(R.id.lb_result_capacitance);
        tvResultDD      = this.getView().findViewById(R.id.lb_result_dd);
        tvResultSVT     = this.getView().findViewById(R.id.lb_result_svt);
        result_pass.setVisibility(View.INVISIBLE);
        result_fail.setVisibility(View.GONE);
        tvResultDD.setVisibility(View.GONE);
    }

    public void setTestNumber(int number)
    {
        if(number<=0)
            testNumber = " - - - ";
        else
            testNumber = number +"";
        tvTestNumber.setText( testNumber );
    }

    public void resetTestParameters()
    {
        try
        {
            def_mode.setText(" - - - ");
            def_voltage.setText(" - - - ");
            def_max_voltage.setText(" - - - ");
            def_limite.setText(" - - - ");
            def_timer.setText(" - - - ");
            def_svt.setText(" - - - ");
        }
        catch (NullPointerException ignored){}
    }

    public void setTestParameters(Device device)
    {
        //this.reset();
        if ( Controller.firmware<Controller.FIRMWARE_DD)
            tvResultDD.setVisibility(View.GONE);
        else
            tvResultDD.setVisibility(View.VISIBLE);

        String[] modes = context.getResources().getStringArray(R.array.op_modes);

        if ( device.getOperationMode() < 0 || device.getOperationMode() > modes.length )
            return;

        //this.device = device;

        def_mode.setText( modes[device.getOperationMode()] );

        double lim = 0f;
        switch (device.getOperationMode())
        {
            case Protocol.MODE_MANUAL:
                def_voltage.setVisibility(View.VISIBLE);
                def_voltage.setText( formatData.formatVoltage( device.getTestVoltage() ) );
                def_max_voltage.setText( formatData.formatVoltage( device.getParamManual().getMAX_VOLTAGE() ) );
                lim = device.getParamManual().getLIMIT_RESISTANCE();
                def_timer.setVisibility( View.GONE );
                def_svt.setVisibility( View.GONE );
                break;
            case Protocol.MODE_TIMER:
                def_voltage.setVisibility(View.VISIBLE);
                def_voltage.setText( formatData.formatVoltage( device.getTestVoltage() ) );
                def_max_voltage.setText( formatData.formatVoltage( device.getParamTimer().getMAX_VOLTAGE() ) );
                lim = device.getParamTimer().getLIMIT_RESISTANCE();
                def_timer.setVisibility( View.VISIBLE );
                def_timer.setText( formatData.formatElapsedTime( (short)device.getParamTimer().getTEST_TIME() ) );
                def_svt.setVisibility( View.GONE );
                break;
            case Protocol.MODE_SVT:
                def_voltage.setVisibility(View.GONE);
                def_max_voltage.setText( formatData.formatVoltage( device.getParamSVT().getFINAL_VOLTAGE() ) );
                lim = device.getParamSVT().getLIMIT_RESISTANCE();
                def_timer.setVisibility( View.GONE );
                def_svt.setVisibility( View.VISIBLE );
                String svt = formatData.formatVoltage( device.getParamSVT().getSTEP_VOLTAGE() ) + " @ " + device.getParamSVT().getSTEP_TIME() + " s";
                def_svt.setText(svt);
                break;
            case Protocol.MODE_RAMP:
                def_voltage.setVisibility(View.GONE);
                def_max_voltage.setText( formatData.formatVoltage( device.getParamRamp().getFINAL_VOLTAGE() ) );
                lim = device.getParamRamp().getLIMIT_RESISTANCE();
                def_timer.setVisibility( View.VISIBLE );
                def_timer.setText( formatData.formatElapsedTime( (short)device.getParamRamp().getTEST_TIME() ) );
                def_svt.setVisibility( View.GONE );
                break;
        }
        setLIM( lim );
    }

    private void setLIM(double lim)
    {
        TextView tv = def_limite;
        if (lim<=0)
        {
            tv.setVisibility(View.GONE);
            result_pass.setVisibility(View.INVISIBLE);
            result_fail.setVisibility(View.GONE);
        }
        else
        {
            tv.setText(formatData.formatMaxResistance((float)lim));
            tv.setVisibility(View.VISIBLE);
        }
    }

    public void update(Measurement measurement, Device device)
    {
        if ( this.getView()==null )
            return;

//        this.device = device;

        tvTestNumber.setText( testNumber );

        if ( device.getOperationMode()>=0 )
        {
            if( device.getStatus(Device.ST_WAIT) )
            {
                tvResistance.setText( this.getView().getResources().getString(R.string.wait) );
                tvElapsed.setText   ( formatData.formatElapsedTime( (short)measurement.getElapsed() ) );
                tvVoltage.setText   ( formatData.formatVoltage(measurement.getVoltage()) );
                tvCurrent.setText   ( " - - - " );
            }
            else if( device.getStatus(Device.ST_OVERRANGE) )
            {
                //tvResistance.setText( this.getView().getResources().getString(R.string.overrange) );
                tvResistance.setText( formatData.formatResistance( measurement.getVoltage(), measurement.getResistance() ) );
                tvElapsed.setText   ( formatData.formatElapsedTime( (short)measurement.getElapsed() ) );
                tvVoltage.setText   ( formatData.formatVoltage(measurement.getVoltage()) );
                tvCurrent.setText   ( formatData.formatCurrent(measurement.getCurrent()) );
            }
            else
            {
                tvResistance.setText( formatData.formatResistance( measurement.getResistance() ) );
                tvElapsed.setText   ( formatData.formatElapsedTime( (short)measurement.getElapsed() ) );
                tvVoltage.setText   ( formatData.formatVoltage(measurement.getVoltage()) );
                tvCurrent.setText   ( formatData.formatCurrent(measurement.getCurrent()) );
            }
        }
        else
        {
            tvResistance.setText( formatData.formatResistance( measurement.getResistance() ) );
            tvElapsed.setText   ( formatData.formatElapsedTime( (short)measurement.getElapsed() ) );
            tvVoltage.setText   ( formatData.formatVoltage(measurement.getVoltage()) );
            tvCurrent.setText   ( formatData.formatCurrent(measurement.getCurrent()) );
        }

        if ( device.getOperationMode()>=0 )
        {
            double lim = 0f;
            switch ( device.getOperationMode() )
            {
                case Protocol.MODE_MANUAL:
                    lim = device.getParamManual().getLIMIT_RESISTANCE();
                    break;
                case Protocol.MODE_TIMER:
                    lim = device.getParamTimer().getLIMIT_RESISTANCE();
                    break;
                case Protocol.MODE_SVT:
                    lim = device.getParamSVT().getLIMIT_RESISTANCE();
                    break;
                case Protocol.MODE_RAMP:
                    lim = device.getParamRamp().getLIMIT_RESISTANCE();
                    break;
            }
            if(lim>0 && !device.getStatus(Device.ST_WAIT) && !device.getStatus(Device.ST_OVERRANGE))
            {
                if(lim>measurement.getResistance())
                {
                    result_pass.setVisibility(View.GONE);
                    result_fail.setVisibility(View.VISIBLE);
                }
                else
                {
                    result_pass.setVisibility(View.VISIBLE);
                    result_fail.setVisibility(View.GONE);
                }
            }
            else
            {
                result_pass.setVisibility(View.INVISIBLE);
                result_fail.setVisibility(View.GONE);
            }
        }
    }

    public boolean isResultApproved()
    {
        return result_pass.getVisibility()==View.VISIBLE;
    }

    public void setDAI(double dai)
    {
//        if ( device.getStatus(Device.ST_OVERRANGE) )
//            return;
        String t = formatData.formatIndexes(dai);
        tvResultDAI.setText(t);
    }

    public void setPI(double pi)
    {
//        if ( device.getStatus(Device.ST_OVERRANGE) )
//            return;
        String t = formatData.formatIndexes(pi);
        tvResultPI.setText(t);
    }

    public void setSVT(double svt)
    {
//        if ( device.getStatus(Device.ST_OVERRANGE) )
//            return;
        String t = formatData.formatIndexes(svt);
        tvResultSVT.setText(t);
    }

    public void setCapacitance(int cap)
    {
        //String t = cap + " nF";
        String t = formatData.formatCapacitance(cap);
        tvResultCAP.setText(t);
    }

    public void setDielectricDischarge(double dd)
    {
        if (tvResultDD.getVisibility()!=View.VISIBLE)
            tvResultDD.setVisibility(View.VISIBLE);
        String t = formatData.formatIndexes(dd);
        tvResultDD.setText(t);
    }

    public void reset()
    {
        try
        {
            tvTestNumber.setText(" - - - ");
            def_mode.setText(" - - - ");
            def_voltage.setText(" - - - ");
            def_max_voltage.setText(" - - - ");
            def_limite.setText(" - - - ");
            def_timer.setText(" - - - ");
            def_svt.setText(" - - - ");
            tvResistance.setText(" - - - ");
            tvElapsed.setText(" - - - ");
            tvVoltage.setText(" - - - ");
            tvCurrent.setText(" - - - ");
            tvResultDAI.setText(" - - - ");
            tvResultPI.setText(" - - - ");
            tvResultCAP.setText(" - - - ");
            tvResultSVT.setText(" - - - ");
            tvResultDD.setText(" - - - ");
            result_pass.setVisibility(View.INVISIBLE);
            result_fail.setVisibility(View.GONE);
        }
        catch (Exception ignored){}
    }

//    public void reset()
//    {
//        try
//        {
//            ui_txw_test_number.setText("999");
//            def_mode.setText("Manual");
//            def_voltage.setText("15.0 kV");
//            def_max_voltage.setText("15.0 kV");
//            def_limite.setText("300 G\u03A9");
//            def_timer.setText("95:00");
//            def_svt.setText("5000 V @ 300 s");
//            tvResistance.setText("300 G\u03A9");
//            tvElapsed.setText("10:00");
//            tvVoltage.setText("15.1 kV");
//            tvCurrent.setText("200.5 nA");
//            tvResultDAI.setText("99.99");
//            tvResultPI.setText("99.99");
//            tvResultCAP.setText("417 nF");
//            tvResultSVT.setText("99.99");
//            tvResultDD.setText("99.99");
//            result_pass.setVisibility(View.VISIBLE);
//            result_fail.setVisibility(View.VISIBLE);
//        }
//        catch (Exception ignored){}
//    }
}

