package com.megabras.bluelogg.md15kvr;

import android.content.Context;
import android.content.res.Configuration;
import android.graphics.drawable.Drawable;
import android.text.Spannable;
import android.text.SpannableString;
import android.text.style.ImageSpan;

import com.megabras.bluelogg.R;
import com.megabras.modbus.MD15KVR.Device;
import com.megabras.modbus.MD15KVR.FormatData;
import com.megabras.modbus.MD15KVR.Measurement;
import com.megabras.modbus.MD15KVR.Protocol;
import com.megabras.modbus.MD15KVR.TestData;

import java.util.ArrayList;
import java.util.List;

import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentStatePagerAdapter;


@SuppressWarnings("WeakerAccess")
public class PagerAdapter extends FragmentStatePagerAdapter
{
    @SuppressWarnings("unused")
    private final String
            TAG = "PagerAdapter";

    private Context
            context;

    protected MainScreen
            mainScreen;

    protected Charts
            resistanceChart,
            voltageChart,
            currentChart,
            iddChart,
            table;

    private FormatData
            formatData = new FormatData();

    private final List<Fragment>
            mFragmentList = new ArrayList<>();

    private final List<CharSequence>
            mFragmentTitleList = new ArrayList<>();

    public PagerAdapter(FragmentManager fm, Context context, int OVERRANGE)
    {
        super(fm);
        this.context = context;
        mainScreen = new MainScreen();
        resistanceChart = new Charts();
        resistanceChart.setChartType( Charts.RESISTANCE_CHART, context.getString(R.string.md_resistance) );
        resistanceChart.setOverrange(OVERRANGE);
        currentChart = new Charts( );
        currentChart.setChartType( Charts.CURRENT_CHART, context.getString(R.string.md_current) );
        currentChart.setOverrange(OVERRANGE);
        voltageChart = new Charts( );
        voltageChart.setChartType( Charts.VOLTAGE_CHART, context.getString(R.string.md_voltage) );
        voltageChart.setOverrange(OVERRANGE);
        iddChart = new Charts( );
        iddChart.setChartType( Charts.IDD_CHART, context.getString(R.string.dielectric_discharge).toUpperCase() );
        iddChart.setOverrange(OVERRANGE);
        table = new Charts();
        table.setChartType( Charts.TABLE, "" );
        table.setOverrange(OVERRANGE);

        mFragmentList.add(mainScreen);
        mFragmentList.add(resistanceChart);
        mFragmentList.add(voltageChart);
        mFragmentList.add(currentChart);
        mFragmentList.add(iddChart);
        mFragmentList.add(table);

        mFragmentTitleList.add(convertTitle(0));
        mFragmentTitleList.add(convertTitle(1));
        mFragmentTitleList.add(convertTitle(2));
        mFragmentTitleList.add(convertTitle(3));
        mFragmentTitleList.add(convertTitle(4));
        mFragmentTitleList.add(convertTitle(5));
    }

    @Override
    public Fragment getItem(int i)
    {
        return mFragmentList.get(i);
    }

    @Override
    public int getCount()
    {
        return mFragmentList.size();
    }

    public MainScreen getMainFragment()
    {
        return mainScreen;
    }

    public Charts chartResistance()
    {
        return resistanceChart;
    }

    public Charts chartCurrent()
    {
        return currentChart;
    }

    public Charts chartVoltage()
    {
        return voltageChart;
    }

    public Charts chartIDD()
    {
        return iddChart;
    }

    public Charts table()
    {
        return table;
    }

    private CharSequence convertTitle(int position)
    {
        int[] images;
        if(screenSizeIsLarge(context))
            images = imageResId;
        else
            images = imageResIdSmall;

        Drawable image      = context.getResources().getDrawable(images[position]);
        image.setBounds(0, 0, image.getIntrinsicWidth(), image.getIntrinsicHeight());
        SpannableString sb  = new SpannableString(" ");
        ImageSpan imageSpan = new ImageSpan(image, ImageSpan.ALIGN_BOTTOM);
        sb.setSpan(imageSpan, 0, 1, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE);
        return sb;
    }

    private int[] imageResId = {
            R.drawable.ic_analog,
            R.drawable.ic_chart_resistance,
            R.drawable.ic_chart_voltage,
            R.drawable.ic_chart_current,
            R.drawable.ic_chart_dd,
            R.drawable.ic_table
    };

    private int[] imageResIdSmall = {
            R.drawable.ic_analog_20,
            R.drawable.ic_chart_resistance_20,
            R.drawable.ic_chart_voltage_20,
            R.drawable.ic_chart_current_20,
            R.drawable.ic_chart_dd_20,
            R.drawable.ic_table_20
    };

    @Override
    public CharSequence getPageTitle(int position)
    {
        return mFragmentTitleList.get(position);
    }

    public void showChartIDD(boolean show)
    {
        int iddPos = 4;
        boolean contains = false;
        for (Fragment f : mFragmentList)
        {
            if (f instanceof Charts)
            {
                Charts c = (Charts)f;
                if (c.getChartType()==Charts.IDD_CHART)
                    contains=true;
            }
        }
        if (show)
        {
            if (!contains)
            {
                mFragmentList.add(iddPos, iddChart);
                mFragmentTitleList.add(iddPos ,convertTitle(iddPos));
            }
        }
        else
        {
            if (contains)
            {
                mFragmentList.remove(iddPos);
                mFragmentTitleList.remove(iddPos);
                notifyDataSetChanged();
            }
        }
    }

    public void updateChartsAndTables(Measurement measurement, Device device, boolean repaint, boolean updateLast)
    {
        if (mainScreen!=null && device!=null)
            mainScreen.update(measurement, device);
        if (resistanceChart!=null && device!=null)
            resistanceChart.update(measurement, device, repaint, updateLast);
        if (voltageChart!=null && device!=null)
            voltageChart.update(measurement, device, repaint, updateLast);
        if (currentChart!=null && device!=null)
            currentChart.update(measurement, device, repaint, updateLast);
        if (table!=null && device!=null)
            table.update(measurement, device, repaint, updateLast);
    }

    public void setAllValues(TestData testData)
    {
        if (testData==null)
            return;

        for (Fragment f : mFragmentList)
        {
            if (f instanceof Charts)
            {
                Charts c = (Charts)f;
                if (c.getChartType()==Charts.IDD_CHART)
                    c.setAllIDD(testData.getDielectricDischarge());
                else if (c.getChartType()==Charts.TABLE)
                {
                    String titleDAI = "DAI ";
                    String titlePI  = "PI ";

                    if ( testData.getMode()== Protocol.MODE_MANUAL || testData.getMode()==Protocol.MODE_TIMER )
                    {
                        titleDAI += "(" + testData.getDAI().getTimeA() + "/" + testData.getDAI().getTimeB() + ")";
                        titlePI  += "(" + testData.getPI().getTimeA() + "/" + testData.getPI().getTimeB() + ")";
                    }

                    c.setAllValues ( testData.getMeasurement() );
                    c.setTestNumber( testData.getTestNumber() );
                    c.setMode      ( testData.getMode() );
                    c.setDAI       ( formatData.formatIndexes(testData.getDAI().getResult()) );
                    c.setPI        ( formatData.formatIndexes(testData.getPI().getResult()) );
                    c.setSVT       ( formatData.formatIndexes(testData.getSVT()) );
                    c.setCAP       ( formatData.formatCapacitance( testData.getCapacitance() ));
                    c.setDD        ( formatData.formatIndexes( testData.getDD() ));
                    c.setDate      ( formatData.getDateFormatted( testData.getDate(), testData.getDevice().getDateFormat(), testData.getDevice().getTimeFormat()) );
                    c.setTitleDAI  ( titleDAI );
                    c.setTitlePI   ( titlePI  );
                }
                else
                    c.setAllValues(testData.getMeasurement());
            }
        }
    }

    public void resetAll(boolean connected, Device device)
    {
        for (Fragment f : mFragmentList)
        {
            if (f instanceof Charts)
            {
                Charts c = (Charts)f;
                c.reset();
            }
            else if (f instanceof MainScreen)
            {
                MainScreen m = (MainScreen)f;
                m.reset();
                if (device!=null)
                {
                    if (connected && device.getOperationMode() >= 0)
                        m.setTestParameters(device);
                }
            }
        }
    }

    private boolean screenSizeIsLarge(Context context)
    {
        return (context.getResources().getConfiguration().screenLayout &
                Configuration.SCREENLAYOUT_SIZE_MASK) == Configuration.SCREENLAYOUT_SIZE_LARGE;
    }

}



