package com.megabras.bluelogg.mpk257;

import android.content.Context;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

import com.megabras.bluelogg.Main;
import com.megabras.bluelogg.R;
import com.megabras.modbus.MPK257.FormatData;
import com.megabras.modbus.MPK257.Protocol;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import androidx.fragment.app.Fragment;

//import android.support.v4.app.Fragment;

public class ChartFragment extends Fragment
{
    @SuppressWarnings("unused")
    private final String
            TAG                 = "ChartFragment";

    private LinearLayout
            layout;

    private View
            objectView;

    private TextView
            tvObjectName,
            tvLongitude,
            tvLatitude,
            tvAltitude,
            tvAccuracy;

    private ListView
            list;

    private TableBaseAdapter
            adapter;

    private List<TableValues>
            tableValues = new ArrayList<>();

    private Context
            context;

    @SuppressWarnings("NullableProblems")
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState)
    {
        context = inflater.getContext();
        return inflater.inflate(R.layout.md10kvx_fgr_online, container, false);
    }

    @Override
    public void onStart()
    {
        super.onStart();
        table();
    }

    public void table(){
        if(this.getView()!=null){
            layout = this.getView().findViewById(R.id.chart);
        }
        else {
            return;
        }

        if(layout.getChildCount()>1){
            return;
        }
        objectView   = View.inflate(context, R.layout.mpk257_table_object, null);

        tvLongitude  = objectView.findViewById(R.id.tv_longitude);
        tvLatitude   = objectView.findViewById(R.id.tv_latitude);
        tvAltitude   = objectView.findViewById(R.id.tv_altitude);
        tvAccuracy   = objectView.findViewById(R.id.tv_accuracy);
        tvObjectName = objectView.findViewById(R.id.tv_object_name);

        tvObjectName.setText("");

        layout.addView(objectView);

        list = new ListView( layout.getContext() );
        list.setDividerHeight(0);
        layout.addView(list);

        adapter = new TableBaseAdapter( layout.getContext() );
        list.setAdapter( adapter );
    }

    void showTable( boolean show ){
        if(list!=null){
            if(show)
                list.setVisibility(View.VISIBLE);
            else
                list.setVisibility(View.GONE);
        }
        if(objectView!=null){
            if(show)
                objectView.setVisibility(View.VISIBLE);
            else
                objectView.setVisibility(View.GONE);
        }
    }

    void updateRows( com.megabras.modbus.MPK257.Measurement measure, com.megabras.modbus.MPK257.Device device, boolean repaint )
    {
        String
                time        = FormatData.formatElapsedTime( measure.getElapsedTime() ),
                current     = FormatData.formatCurrent    ( measure.getCurrentRange(), measure.getCurrent() ),
                resistanceC = FormatData.formatResistance ( measure.getCurrentRange(), measure.getResistanceCompensated() ),
                temperature = FormatData.formatTemperature( device.getTempUnity()    , measure.getTemperature() );

        String resistance;
        //if (Controller.model.equals(Controller.fullModel))
        if(Main.equipmentId==257)
            resistance = com.megabras.modbus.MPK257
                    .FormatData.formatResistance  ( measure.getCurrentRange(), measure.getResistance());
        else
            resistance = com.megabras.modbus.MPK256
                    .FormatData.formatResistance  ( measure.getCurrentRange(), measure.getResistance());

        if ( measure.getTemperature() <= 0 || measure.getTemperature() > Protocol.TEMP_MAX  )
        {
            resistanceC = "";
            temperature = "";
        }

        if ( measure.getResistance() > 0 && measure.getResistanceCompensated() <= 0 )
        {
            resistanceC = "";
            temperature = "";
        }

        if ( measure.getStatus()>0 )
        {
            resistance  = FormatData.getStringStatus(measure.getStatus(), Locale.getDefault().getLanguage());
            resistanceC = "";
        }

        adapter.add(time, resistance, resistanceC, current, temperature);

        if(repaint)
            adapter.repaint();
    }

    public void update(Database.ObjectData obj)
    {
        tvObjectName.setText(obj.getName());
        if(!obj.getLongitude().equals("")){
            String
                    longitude = context.getString(R.string.longitude) + " : "       + obj.getLongitude(),
                    latitude  = context.getString(R.string.latitude ) + " : "       + obj.getLatitude(),
                    altitude  = context.getString(R.string.altitude ) + " : "       + obj.getAltitude() + " m",
                    accuracy  = context.getString(R.string.accuracy ) + " : \u00B1" + obj.getGpsAccuracy() + " m";

            tvLongitude.setText(longitude);
            tvLatitude.setText(latitude);
            tvAltitude.setText(altitude);
            tvAccuracy.setText(accuracy);
        } else {
            tvLongitude.setText("");
            tvLatitude.setText("");
            tvAltitude.setText("");
            tvAccuracy.setText("");
        }
    }

    void setHeader( int testNumber, String date, String temperature, String metal )
    {
        adapter.setTestNumber(testNumber);
        adapter.setDate(date);
        adapter.setTemperatureSet(temperature);
        adapter.setMetal(metal);
        //adapter.repaint();
    }

    void updateLastHeader( int testNumber, String date, String temperature, String metal )
    {
        adapter.setTestNumber(testNumber);
        adapter.setDate(date);
        adapter.setTemperatureSet(temperature);
        adapter.setMetal(metal);
        //adapter.repaint();
    }

    public void repaint()
    {
        adapter.repaint();
    }

    public void setAllValues()
    {
        adapter.addAll( tableValues );
    }

    public void reset()
    {
        try
        {
            if (layout==null)
            {
                table();
            }
            tableValues.clear();
        }
        catch (Exception e)
        {
            android.util.Log.e("ERROR", e.getMessage()+"");
        }
    }
}
