package com.megabras.bluelogg.mpk257;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.os.Environment;
import android.provider.BaseColumns;
import android.util.Log;

import com.megabras.bluelogg.R;
import com.megabras.modbus.MPK257.Device;
import com.megabras.modbus.MPK257.Measurement;
import com.megabras.modbus.MPK257.TestData;

import java.io.File;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Date;

@SuppressWarnings("FieldCanBeLocal")
public final class Database extends SQLiteOpenHelper {

    //##########################################################################
    //# Fields
    //##########################################################################

    private String
            TAG          = "Database";

    private static final String
            TEXT_TYPE    = " TEXT",
            INT_TYPE     = " INTEGER",
            //FLOAT_TYPE   = " FLOAT",
            DOUBLE_TYPE  = " DOUBLE",
            DATE_TYPE    = " DATETIME",
            COMMA_SEP    = ",";

    private String
            //model        = "",
            root         = Environment.getExternalStorageDirectory().toString(),
            picPreffix   = "OBJID_",
            picSuffix    = ".jpg",
            audioSuffix  = "_annotation.3gp",
            reportSuffix = "_report.pdf",
            mapSuffix    = "_map_snapshot.jpg";

    private File
            dir,
            tempDir,
            rootDir;

    private DecimalFormat
            fileFormat   = new DecimalFormat("000000");

    //##########################################################################
    //# Constructor
    //##########################################################################

    public Database(Context context, String model)
    {
        super(context, Entry.DB_NAME, null, Entry.DB_VERSION);
        createTables();
        createFolder(context, model);
    }

    //##########################################################################
    //# Folders
    //##########################################################################

    private boolean isExternalStorageAvaiable()
    {
        String state = Environment.getExternalStorageState();
        return Environment.MEDIA_MOUNTED.equals(state);
    }

    @SuppressWarnings("ResultOfMethodCallIgnored")
    public boolean createFolder(Context context, String model)
    {
        if(!this.isExternalStorageAvaiable())
            return false;

        rootDir     = new File(root + "/" + context.getResources().getString(R.string.company));

        if(!rootDir.exists()){
            rootDir.mkdir();
        }

        dir     = new File(root + "/" + context.getResources().getString(R.string.company)+ "/" + model);
        tempDir = new File(dir + "/temp");

        if(!dir.exists()){
            dir.mkdir();
        }

        if(!tempDir.exists()){
            tempDir.mkdir();
        }

        return dir.exists();
    }

    @SuppressWarnings("ResultOfMethodCallIgnored")
    public File getFolder(){
        if(!rootDir.exists()){
            rootDir.mkdir();
        }
        if(!dir.exists()){
            dir.mkdir();
        }
        return dir;
    }

    @SuppressWarnings("ResultOfMethodCallIgnored")
    File getTemporaryFolder(){
        if(!rootDir.exists()){
            rootDir.mkdir();
        }
        if(!dir.exists()){
            dir.mkdir();
        }
        if(!tempDir.exists()){
            tempDir.mkdir();
        }
        return tempDir;
    }

    public File getPictureFile(int objectID){
        String img = picPreffix + fileFormat.format(objectID) + picSuffix;
        return new File(dir, img);
    }

    File getAudioFile(int objectID){
        String img = picPreffix + fileFormat.format(objectID) + audioSuffix;
        return new File(dir, img);
    }

    File getMapSnapshotFile(int objectID){
        String img = picPreffix + fileFormat.format(objectID) + mapSuffix;
        return new File(dir, img);
    }

    File getReportFile(int objectID){
        String img = picPreffix + fileFormat.format(objectID) + reportSuffix;
        return new File(dir, img);
    }

    @SuppressWarnings("ResultOfMethodCallIgnored")
    private void deleteFiles(int objectID){
        for ( File child : dir.listFiles() ){
            if( child.getName().contains( picPreffix + fileFormat.format(objectID) ) ){
                child.delete();
            }
        }
    }

    //##########################################################################
    //# Methods
    //##########################################################################

    @Override
    public void onCreate(SQLiteDatabase db) {}

    @Override
    public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion){
        Log.e(TAG, "onUpgrade");
        if (oldVersion < 2) {
            Log.e(TAG, "COLUNM ADDED");
            db.execSQL("ALTER TABLE " + Entry.TABLE_MEASUREMENTS + " ADD COLUMN " + Entry.MEAS_STATS + " " + INT_TYPE + ";");
        }
    }

    private void createTables(){
        SQLiteDatabase db = this.getWritableDatabase();

        //TABLE OBJECTS
        Cursor c = db.rawQuery("SELECT * FROM sqlite_master WHERE type='table' AND name='" + Entry.TABLE_OBJS + "';", null);
        if( c.getCount()==0 ){
            String FIELD_ENTRIES =
                    Entry.OBJ_ID        + INT_TYPE  + " PRIMARY KEY AUTOINCREMENT NOT NULL" + COMMA_SEP +
                    Entry.OBJ_NAME      + TEXT_TYPE + COMMA_SEP +
                    Entry.OBJ_MODEL     + TEXT_TYPE + COMMA_SEP +
                    Entry.OBJ_SERIAL    + TEXT_TYPE + COMMA_SEP +
                    Entry.OBJ_LONGITUDE + TEXT_TYPE + COMMA_SEP +
                    Entry.OBJ_LATITUDE  + TEXT_TYPE + COMMA_SEP +
                    Entry.OBJ_ALTITUDE  + TEXT_TYPE + COMMA_SEP +
                    Entry.OBJ_ACCURACY  + TEXT_TYPE;

            String SQL_CREATE_TABLE =
                    "CREATE TABLE " + Entry.TABLE_OBJS + " (" + FIELD_ENTRIES + " )";
            db.execSQL(SQL_CREATE_TABLE);
        }
        c.close();

        //TABLE TESTS INFO
        c = db.rawQuery("SELECT * FROM sqlite_master WHERE type='table' AND name='" + Entry.TABLE_TESTS_INFO + "';", null);
        if( c.getCount()==0 ){
            String FIELD_ENTRIES =
                    Entry.INFO_ID               + INT_TYPE  + " PRIMARY KEY AUTOINCREMENT NOT NULL" + COMMA_SEP +
                    Entry.OBJ_ID                + INT_TYPE  + COMMA_SEP +
                    Entry.INFO_DATE             + DATE_TYPE + COMMA_SEP +
                    Entry.INFO_NUMBER           + INT_TYPE  + COMMA_SEP +
                    Entry.INFO_TEMP_UNITY       + INT_TYPE  + COMMA_SEP +
                    Entry.INFO_REF_TEMP_CELCIUS + INT_TYPE  + COMMA_SEP +
                    Entry.INFO_METAL            + INT_TYPE;

            String SQL_CREATE_TABLE =
                    "CREATE TABLE " + Entry.TABLE_TESTS_INFO + " (" + FIELD_ENTRIES + " )";
            db.execSQL(SQL_CREATE_TABLE);
        }
        c.close();

        //TABLE MEASUREMENTS
        c = db.rawQuery("SELECT * FROM sqlite_master WHERE type='table' AND name='" + Entry.TABLE_MEASUREMENTS + "';", null);
        if( c.getCount()==0 ){
            String FIELD_ENTRIES =
                    Entry.MEAS_ID            + INT_TYPE    + " PRIMARY KEY AUTOINCREMENT NOT NULL" + COMMA_SEP +
                    Entry.INFO_ID            + INT_TYPE    + COMMA_SEP +
                    Entry.MEAS_ELAPSED       + INT_TYPE    + COMMA_SEP +
                    Entry.MEAS_RESISTANCE    + DOUBLE_TYPE + COMMA_SEP +
                    Entry.MEAS_RESIST_COMP   + DOUBLE_TYPE + COMMA_SEP +
                    Entry.MEAS_CURRENT       + DOUBLE_TYPE + COMMA_SEP +
                    Entry.MEAS_CURRENT_RANGE + INT_TYPE    + COMMA_SEP +
                    Entry.MEAS_TEMP_CELCIUS  + DOUBLE_TYPE + COMMA_SEP +
                    Entry.MEAS_STATS         + INT_TYPE;

            String SQL_CREATE_TABLE =
                    "CREATE TABLE " + Entry.TABLE_MEASUREMENTS + " (" + FIELD_ENTRIES + " )";
            db.execSQL(SQL_CREATE_TABLE);
        }
        c.close();
    }

    void addObject(String name, String model, String serial){
        try {
            SQLiteDatabase db = this.getWritableDatabase();
            db.beginTransaction();

            ContentValues values = new ContentValues();
            values.put(Entry.OBJ_NAME     , name);
            values.put(Entry.OBJ_MODEL    , model);
            values.put(Entry.OBJ_SERIAL   , serial);
            values.put(Entry.OBJ_LONGITUDE, "");
            values.put(Entry.OBJ_LATITUDE , "");
            values.put(Entry.OBJ_ALTITUDE , "");
            values.put(Entry.OBJ_ACCURACY , "");

            db.insertWithOnConflict(Entry.TABLE_OBJS, null, values, SQLiteDatabase.CONFLICT_REPLACE);
            db.setTransactionSuccessful();
            db.endTransaction();
        } catch (Exception e) {
            android.util.Log.e(TAG, e.getMessage());
        }
    }

    void updateObjectName(int id, String name){
        try {
            SQLiteDatabase db = this.getWritableDatabase();
            db.beginTransaction();

            ContentValues values = new ContentValues();
            values.put(Entry.OBJ_NAME, name);

            db.update(Entry.TABLE_OBJS, values, Entry.OBJ_ID+"="+id, null);
            //db.insertWithOnConflict(Entry.TABLE_OBJS, null, values, SQLiteDatabase.CONFLICT_REPLACE);
            db.setTransactionSuccessful();
            db.endTransaction();
        } catch (Exception e) {
            android.util.Log.e(TAG, e.getMessage());
        }
    }

    void updateObjectGPS(int id, String longitude, String latitude, String altitude, String accuracy){
        try {
            SQLiteDatabase db = this.getWritableDatabase();
            db.beginTransaction();

            ContentValues values = new ContentValues();
            values.put(Entry.OBJ_LONGITUDE, longitude);
            values.put(Entry.OBJ_LATITUDE , latitude);
            values.put(Entry.OBJ_ALTITUDE , altitude);
            values.put(Entry.OBJ_ACCURACY , accuracy);

            db.update(Entry.TABLE_OBJS, values, Entry.OBJ_ID+"="+id, null);
            //db.insertWithOnConflict(Entry.TABLE_OBJS, null, values, SQLiteDatabase.CONFLICT_REPLACE);
            db.setTransactionSuccessful();
            db.endTransaction();
        } catch (Exception e) {
            android.util.Log.e(TAG, e.getMessage());
        }
    }

    private int getLastObjectID(){
        int id = 0;
        SQLiteDatabase db = this.getReadableDatabase();
        Cursor c = db.rawQuery("SELECT seq FROM SQLITE_SEQUENCE WHERE name='" + Entry.TABLE_OBJS + "'", null);
        if(c.moveToFirst())
            id = c.getInt(0);
        c.close();

        return id;
    }

    ObjectData getObjectData(int objectID){
        try{
            SQLiteDatabase db = this.getReadableDatabase();
            Cursor c = db.rawQuery("SELECT * FROM " + Entry.TABLE_OBJS + " WHERE " + Entry.OBJ_ID + "=" + objectID, null);
            ObjectData obj = new ObjectData();

            if(c.moveToFirst()){
                obj.setId         ( c.getInt(0)    );
                obj.setName       ( c.getString(1) );
                obj.setModel      ( c.getString(2) );
                obj.setSerial     ( c.getString(3) );
                obj.setLongitude  ( c.getString(4) );
                obj.setLatitude   ( c.getString(5) );
                obj.setAltitude   ( c.getString(6) );
                obj.setGpsAccuracy( c.getString(7) );
            }

            c.close();
            return obj;
        } catch (Exception e){
            return null;
        }
    }

    public ArrayList<ObjectData> getObjectsList(){
        try{
            ArrayList<ObjectData> objs = new ArrayList<>();

            SQLiteDatabase db = this.getReadableDatabase();
            Cursor c = db.rawQuery("SELECT * FROM " + Entry.TABLE_OBJS + " ORDER BY " + Entry.OBJ_NAME + " ASC", null);

            if(c.moveToFirst()){
                while ( !c.isAfterLast() ) {
                    ObjectData obj = new ObjectData();
                    obj.setId         ( c.getInt(0)    );
                    obj.setName       ( c.getString(1) );
                    obj.setModel      ( c.getString(2) );
                    obj.setSerial     ( c.getString(3) );
                    obj.setLongitude  ( c.getString(4) );
                    obj.setLatitude   ( c.getString(5) );
                    obj.setAltitude   ( c.getString(6) );
                    obj.setGpsAccuracy( c.getString(7) );
                    objs.add(obj);
                    c.moveToNext();
                }
            }

            c.close();
            return objs;
        } catch (Exception e){
            return new ArrayList<>();
        }
    }

    void deleteObject(int id){
        try {
            SQLiteDatabase db = this.getWritableDatabase();
            db.delete(Entry.TABLE_OBJS, Entry.OBJ_ID + "=" + id, null);
            deleteFiles(id);
        } catch (Exception e) {
            android.util.Log.e(TAG, e.getMessage());
        }
    }

    int addTestData(int objectID, TestData testData){
        try {
            SQLiteDatabase db = this.getWritableDatabase();
            db.beginTransaction();

            ContentValues values = new ContentValues();
            values.put(Entry.OBJ_ID               , objectID);
            values.put(Entry.INFO_DATE            , testData.getDate().getTime());
            values.put(Entry.INFO_NUMBER          , testData.getTestNumber());
            values.put(Entry.INFO_TEMP_UNITY      , testData.getSetup().getTempUnity());
            values.put(Entry.INFO_REF_TEMP_CELCIUS, testData.getSetup().getTempRange());
            values.put(Entry.INFO_METAL           , testData.getSetup().getMetalType());

            db.insertWithOnConflict(Entry.TABLE_TESTS_INFO, null, values, SQLiteDatabase.CONFLICT_REPLACE);
            db.setTransactionSuccessful();
            db.endTransaction();

            int id = -1;
            Cursor c = db.rawQuery("SELECT seq FROM SQLITE_SEQUENCE WHERE name='" + Entry.TABLE_TESTS_INFO + "'", null);
            if(c.moveToFirst())
                id = c.getInt(0);
            c.close();

            return id;
        } catch (Exception e) {
            android.util.Log.e(TAG, e.getMessage());
            return -1;
        }
    }

    ArrayList<TestData> getTestData(int objectID)
    {
        try
        {
            SQLiteDatabase db = this.getReadableDatabase();
            Cursor c          = db.rawQuery("SELECT * FROM " + Entry.TABLE_OBJS, null);
            ObjectData obj    = new ObjectData();

            if ( c.moveToFirst() )
            {
                obj.setId( c.getInt( 0 ) );
                obj.setName( c.getString( 1 ) );
                obj.setModel( c.getString( 2 ) );
                obj.setSerial( c.getString( 3 ) );
            }
            c.close();

            ArrayList<TestData> objs = new ArrayList<>();

            c = db.rawQuery("SELECT * FROM " + Entry.TABLE_TESTS_INFO + " WHERE " + Entry.OBJ_ID + "=" + objectID, null);

            if ( c.moveToFirst() )
            {
                while ( !c.isAfterLast() )
                {
                    TestData test   = new TestData( obj.getModel(), obj.getSerial() );
                    Device   device = new Device();
                    test.setID( c.getInt( 0 ) );
                    test.setDate( new Date( c.getLong( 2 ) ) );
                    test.setTestNumber( c.getShort( 3 ) );
                    device.setTempUnity( c.getShort( 4 ) );
                    device.setTempRange( c.getShort( 5 ) );
                    device.setMetalType( c.getShort( 6 ) );
                    test.setSetup( device );

                    objs.add( test );
                    c.moveToNext();
                }
            }

            c.close();
            return objs;
        } catch (Exception e){
            return null;
        }
    }

    void addMeasurement(int testDataID, Measurement measurement){
        try {
            SQLiteDatabase db = this.getWritableDatabase();
            db.beginTransaction();

            ContentValues values = new ContentValues();
            values.put(Entry.INFO_ID           , testDataID);
            values.put(Entry.MEAS_ELAPSED      , measurement.getElapsedTime());
            values.put(Entry.MEAS_RESISTANCE   , measurement.getResistance());
            values.put(Entry.MEAS_RESIST_COMP  , measurement.getResistanceCompensated());
            values.put(Entry.MEAS_CURRENT      , measurement.getCurrent());
            values.put(Entry.MEAS_CURRENT_RANGE, measurement.getCurrentRange());
            values.put(Entry.MEAS_TEMP_CELCIUS , measurement.getTemperature());
            values.put(Entry.MEAS_STATS        , measurement.getStatus());

            db.insertWithOnConflict(Entry.TABLE_MEASUREMENTS, null, values, SQLiteDatabase.CONFLICT_REPLACE);
            db.setTransactionSuccessful();
            db.endTransaction();
        } catch (Exception e) {
            android.util.Log.e(TAG, e.getMessage());
        }
    }

    ArrayList<Measurement> getMeasurements(int testDataID){
        try{
            SQLiteDatabase db = this.getReadableDatabase();
            Cursor c = db.rawQuery("SELECT * FROM " + Entry.TABLE_MEASUREMENTS + " WHERE " + Entry.INFO_ID + "=" + testDataID, null);

            ArrayList<Measurement> meas = new ArrayList<>();

            if(c.moveToFirst()){
                while ( !c.isAfterLast() ) {
                    Measurement m = new Measurement();
                    m.setID                   ( c.getShort(1)  );
                    m.setElapsedTime          ( c.getShort(2)  );
                    m.setResistance           ( c.getDouble(3) );
                    m.setResistanceCompensated( c.getDouble(4) );
                    m.setCurrent              ( c.getDouble(5) );
                    m.setCurrentRange         ( c.getShort(6)  );
                    m.setTemperature          ( c.getFloat(7)  );
                    try
                    {
                        m.setStatus( c.getInt(8) );
                    }
                    catch ( Exception e )
                    {
                        m.setStatus( 0 );
                    }

                    meas.add(m);

                    c.moveToNext();
                }
            }

            c.close();
            return meas;
        } catch (Exception e){
            return null;
        }
    }

    ObjectData getNewObjectData(){
        ObjectData obj = new ObjectData();
        int id         = this.getLastObjectID();
        obj.setId(id+1);
        return obj;
    }

    /* Inner class that defines the table contents */
    public static abstract class Entry implements BaseColumns {
        // If you change the database schema, you must increment the database version.
        static final int
                DB_VERSION           = 2;
        static final String
                //DB_NAME              = "MPK257.db",
                DB_NAME              = Controller.model + "_" + Controller.serial + "_REGISTERS.db",
                //TABLES
                TABLE_OBJS           = "TABLE_OBJECTS",
                //TABLE_RELATIONAL     = "TABLE_RELATIONAL",
                TABLE_TESTS_INFO     = "TABLE_TESTS_INFO",
                TABLE_MEASUREMENTS   = "TABLE_MEASUREMENTS",
                //FIELDS
                OBJ_ID               = "OBJID",
                OBJ_NAME             = "NAME",
                OBJ_MODEL            = "MODEL",
                OBJ_SERIAL           = "SERIAL",
                OBJ_LONGITUDE        = "LONGITUDE",
                OBJ_LATITUDE         = "LATITUDE",
                OBJ_ALTITUDE         = "ALTITUDE",
                OBJ_ACCURACY         = "ACCURACY",
                INFO_ID              = "INFOID",
                INFO_DATE            = "DATE",
                INFO_NUMBER          = "NUMBER",
                INFO_TEMP_UNITY      = "TEMPERATURE_UNITY",
                INFO_REF_TEMP_CELCIUS= "TEMPERATURE_CELCIUS",
                INFO_METAL           = "METAL",
                MEAS_ID              = "MEASID",
                MEAS_ELAPSED         = "ELAPSED",
                MEAS_RESISTANCE      = "R",
                MEAS_RESIST_COMP     = "RC",
                MEAS_CURRENT         = "I",
                MEAS_CURRENT_RANGE   = "I_RANGE",
                MEAS_TEMP_CELCIUS    = "TEMPERATURE_CELCIUS",
                MEAS_STATS           = "STATS";
    }

    public class ObjectData{

        //##########################################################################
        //# Fields
        //##########################################################################

        private int
                id;

        private String
                name        = "",
                model       = "",
                serial      = "",
                latitude    = "",
                longitude   = "",
                altitude    = "",
                gpsAccuracy = "";

        //##########################################################################
        //# Setters
        //##########################################################################

        public void setId(int id) {
            this.id = id;
        }

        public void setName(String name) {
            this.name = name;
        }

        public void setModel(String model) {
            this.model = model;
        }

        public void setSerial(String serial) {
            this.serial = serial;
        }

        public void setLatitude(String latitude){
            this.latitude = latitude;
        }

        public void setLongitude(String longitude){
            this.longitude = longitude;
        }

        public void setAltitude(String altitude){
            this.altitude = altitude;
        }

        void setGpsAccuracy(String gpsAccuracy){
            this.gpsAccuracy = gpsAccuracy;
        }

        //##########################################################################
        //# Getters
        //##########################################################################

        public String getSerial() {
            return serial;
        }

        public String getModel() {
            return model;
        }

        public String getName() {
            return name;
        }

        public int getId() {
            return id;
        }

        public String getLatitude(){
            return latitude;
        }

        public String getLongitude(){
            return longitude;
        }

        public String getAltitude(){
            return altitude;
        }

        String getGpsAccuracy(){
            return gpsAccuracy;
        }
    }
}
