package com.megabras.bluelogg.tm25r;

import android.annotation.SuppressLint;
import android.app.Dialog;
import android.content.DialogInterface;
import android.graphics.Color;
import android.os.Handler;
import android.os.Message;
//import android.support.v4.app.FragmentActivity;
//import android.support.v4.content.res.ResourcesCompat;
//import android.support.v4.view.ViewPager;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.widget.Button;
import android.widget.ImageButton;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.megabras.bluelogg.bluetooth.BluetoothClassic;
import com.megabras.bluelogg.extended.CommonUI;
import com.megabras.bluelogg.Main;
import com.megabras.bluelogg.R;
import com.megabras.bluelogg.extended.SlidingTabLayout;
import com.megabras.bluelogg.extended.Audio;
import com.megabras.bluelogg.extended.AudioCloseDialogInterface;
import com.megabras.bluelogg.extended.BooleanObservable;
import com.megabras.bluelogg.extended.GPS;
import com.megabras.bluelogg.extended.GoogleMaps;
import com.megabras.bluelogg.extended.IntegerObservable;
import com.megabras.bluelogg.extended.MapData;
import com.megabras.bluelogg.extended.Picture;
import com.megabras.modbus.TM25R.Objects;
import com.megabras.modbus.TM25R.TestData;
import com.megabras.modbus.TM25R.Device;
import com.megabras.modbus.TM25R.Measurement;
import com.megabras.modbus.TM25R.Protocol;

import java.io.File;
import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import androidx.core.content.res.ResourcesCompat;
import androidx.fragment.app.FragmentActivity;
import androidx.viewpager.widget.ViewPager;

@SuppressWarnings("FieldCanBeLocal, unused")
@SuppressLint("SetTextI18n")
public class Controller extends com.megabras.bluelogg.Controller implements GPS.GpsInterface, Picture.PictureInterface, Observer, DialogsInterface, AudioCloseDialogInterface {

    /***********************************************************************************************
     * REGION Fields
     ***********************************************************************************************/

    private final String
            TAG = "tm25r.Controller";

    public static String
            model,
            serial;

    public static int
            firmware,
            minFirmareVersion = 1;

    private FragmentActivity
            activity;

    private Menu
            menu;

    private MenuItem
            menuOptions;

    private MainScreen
            mainScreen;

    private Table
            table;

    private ViewPager
            viewPager;

    private View
            pageTestSetup,
            pageGeneral;

    private int
            controlButtonsID = R.layout.tm25r_control_buttons,
            optionsID,
            selectedIndex    = -1;

    private LinearLayout
            toolBar;

    private Protocol
            modbus = new Protocol();

    private Device
            device = new Device();

    private Button
            btTest,
            btStop;

    private ImageButton
            btSave,
            btManage,
            btGPS,
            btPicture,
            btVoice,
            btMap,
            btDetails,
            btPrinter,
            btRc,
            btR;

    private boolean
            permissionCallBackShowDialog = false,
            initialConfig                = true,
            isPrinting                   = false,
            resetAfter                   = false;

    private CommonUI
            mCommonUI;

    private Picture
            picture;

    private GPS
            mGPS;

    private Audio
            audio;

    private GoogleMaps
            map;

//    private Report
//            report;

    private ReportPDF
            reportPDF;

    private mHandler
            handler;

    private Dialog
            dlgProgress;

    private Main
            parent;

    private TestData
            testData = new TestData();

    private Dialogs
            dlg;

    private Database
            database;

    private BooleanObservable
            isTestRunning    = new BooleanObservable(1, true),
            isConnected      = new BooleanObservable(2, true),
            isAutoPilot      = new BooleanObservable(5, false);

    private IntegerObservable
            messageID        = new IntegerObservable(1, 0);

    /***********************************************************************************************
     * REGION Constructor
     ***********************************************************************************************/

    @SuppressLint("CommitPrefEdits")
    public Controller(FragmentActivity fragmentActivity, CommonUI ui, String model, String serial){
        activity          = fragmentActivity;
        Protocol.setRevisionMap(0);//TEMP
        modbus.enableFrameDelimiters(false);
        firmware          = -1;
        database          = new Database(activity, model, serial);
        dlg               = new Dialogs(this, activity, database);
        parent            = (Main)activity;
        pageGeneral       = View.inflate(activity, R.layout.main_fgr_general_setup, null);
        pageTestSetup     = View.inflate(activity, R.layout.md15kvr_fgr_test_setup, null);
        Controller.model  = model;
        Controller.serial = serial.trim();
        handler           = new mHandler(this);
        mCommonUI         = ui;
        picture           = Picture.getInstance();
        picture.initialize(fragmentActivity, this);
        mGPS              = new GPS(this, fragmentActivity);
        audio             = new Audio(fragmentActivity);
        audio.setAudioCloseDialogInterface(this);
        map               = new GoogleMaps(fragmentActivity);
        isTestRunning.addObserver(this);
        isConnected.addObserver  (this);
        isAutoPilot.addObserver  (this);
        messageID.addObserver    (this);
        testData.addObserver     (this);
        testData.setSiteName     ("");

        mainScreen  = new MainScreen();
        table       = new Table();
        table.setChartType( Table.TABLE, "" );

        device.setDateFormat(Protocol.DATE_YYYY_MM_DD);
        device.setTimeFormat(Protocol.TIME_24H);

        reportPDF = new ReportPDF(activity, handler);
    }

    /***********************************************************************************************
     * REGION On Destroy
     ***********************************************************************************************/

    public void onClose(){
        if (mGPS != null) {
            mGPS.cancel();
        }
        if (picture != null) {
            picture.close();
        }
    }

    /***********************************************************************************************
     * REGION Controller - Implements
     ***********************************************************************************************/

    public void createSwipe() {
        PagerAdapter
                mPagerAdapter = new PagerAdapter(activity.getSupportFragmentManager(), activity.getBaseContext());
        mPagerAdapter.addFragment(mainScreen);
        mPagerAdapter.addFragment(table);

        viewPager = activity.findViewById(R.id.pager);

        if (viewPager == null)
            return;

        viewPager.setAdapter(mPagerAdapter);
        viewPager.setOffscreenPageLimit(mPagerAdapter.getCount());

//        mainScreen      = mPagerAdapter.getMainFragment();
//        table           = mPagerAdapter.table();

        //Create customs Tabs
        SlidingTabLayout mSlidingTabLayout =
                activity.findViewById(R.id.sliding_tabs);
        mSlidingTabLayout.setDistributeEvenly(true);
        mSlidingTabLayout.setCustomTabView(R.layout.custom_tab, 0);
        mSlidingTabLayout.setBackground(
                ResourcesCompat.getDrawable(activity.getResources(), R.drawable.chamfer_black_top, null)
        );
        mSlidingTabLayout.setCustomTabColorizer(new SlidingTabLayout.TabColorizer() {
            public int getIndicatorColor(int position) {
                return Color.GRAY;
            }

            public int getDividerColor(int position) {
                return Color.WHITE;
            }
        });
        mSlidingTabLayout.setViewPager(viewPager);
    }

    public void createControlButtons(LinearLayout container){
        final View commandButtons = View.inflate(activity, controlButtonsID, null);

        commandButtons.setLayoutParams(
                new LinearLayout.LayoutParams(
                        LinearLayout.LayoutParams.MATCH_PARENT,
                        (int)activity.getResources().getDimension(R.dimen.md15kvr_controls_height)));
        container.removeAllViews();
        container.addView(commandButtons);

        btTest    = commandButtons.findViewById(R.id.bt_test);
        btStop    = commandButtons.findViewById(R.id.bt_stop);
        btR       = commandButtons.findViewById(R.id.bt_r);
        btRc      = commandButtons.findViewById(R.id.bt_rc);
        btPrinter = commandButtons.findViewById(R.id.bt_printer);

        btR.setOnClickListener(v -> {
            sendMessage( modbus.write.mode(Protocol.MODE_NORMAL) );
            mCommonUI.enableButtonBg(btTest, false);
            mCommonUI.enableButton  (btR   , false);
            mCommonUI.enableButton  (btRc  , false);
            messageID.set( R.string.updating );
        });

        btRc.setOnClickListener(v -> {
            sendMessage( modbus.write.mode(Protocol.MODE_COMPENSATION) );
            mCommonUI.enableButtonBg(btTest, false);
            mCommonUI.enableButton  (btR   , false);
            mCommonUI.enableButton  (btRc  , false);
            messageID.set( R.string.updating );
        });

        btPrinter.setOnClickListener(v -> {
            isPrinting = true;
            messageID.set(R.string.printing);
            sendMessage( modbus.write.print() );
            mCommonUI.enableButton(btPrinter, false);
        });

        btTest.setOnClickListener(v -> {
            sendMessage( modbus.write.start() );
            messageID.set(R.string.md_warning_test_start);
            if (menuOptions !=null)
                menuOptions.setEnabled(false);
            mCommonUI.enableButtonBg(btTest, false);
            mCommonUI.enableButton(btR      ,false);
            mCommonUI.enableButton(btRc     ,false);
            mCommonUI.enableButton(btPrinter,false);
            enableToolBar(false);
        });

        btStop.setOnClickListener(v ->
            sendMessage( modbus.write.stop() )
        );

        isTestRunning.set(false);
        isConnected.set(false);
    }

    public void setToolBarId(int toolBarId){
        toolBar = activity.findViewById(toolBarId);
        toolBar.findViewById(R.id.bt_export).setVisibility(View.GONE);
        btMap     = toolBar.findViewById(R.id.btn_map);
        btSave    = toolBar.findViewById(R.id.bt_save);
        btManage  = toolBar.findViewById(R.id.bt_manager);
        btGPS     = toolBar.findViewById(R.id.bt_gps);
        btPicture = toolBar.findViewById(R.id.bt_picture);
        btVoice   = toolBar.findViewById(R.id.bt_record);
        btDetails = toolBar.findViewById(R.id.bt_details);

        mCommonUI.enableButton(btDetails, false);
        btVoice.setVisibility(View.GONE);

        btSave.setOnClickListener(v -> {
            if(Main.permissionToMemoryAccepted){
                save();
            }
            else {
                ((Main)activity).requestPermission(Main.REQUEST_MEMORY_PERMISSION);
            }
        });

        btManage.setOnClickListener(v -> {
            if(Main.permissionToMemoryAccepted){
                database.createFolder();
                dlg.showManager(0, testData);
            }
            else {
                permissionCallBackShowDialog = true;
                ((Main)activity).requestPermission(Main.REQUEST_MEMORY_PERMISSION);
            }
        });

        btDetails.setOnClickListener(v -> {
            if(!testData.getSiteName().equals(""))
                dlg.objectsList(testData);
        });

        btPicture.findViewById(R.id.bt_picture).setOnClickListener(v -> {
            if (Main.permissionToPictureAccepted)
                takePicture();
            else
                ((Main)activity).requestPermission(Main.REQUEST_PICTURE_PERMISSION);
        });

        btGPS.setOnClickListener(v -> {
            if (Main.permissionToGPSAccepted)
                getGPS();
            else
                ((Main)activity).requestPermission(Main.REQUEST_GPS_PERMISSION);
        });

        btMap.setOnClickListener(v -> {
            if(Main.permissionToMemoryAccepted)
                map();
            else
                ((Main)activity).requestPermission(Main.REQUEST_MEMORY_MAP_PERMISSION);
        });

        enableToolBar(false);
        mCommonUI.enableButton(btManage, true);
    }

    private void save(){
        if(testData!=null) {
            database.createFolder();
            if (testData.getSiteName().equals("") && testData.getObjectName().equals("obj")){
                if (database.getSites().size() > 0)
                    dlg.testSetup(false, testData);
                else
                    dlg.addTestSite(1, testData);
            }
            else if (!testData.getSiteName().equals("") && testData.getObjectName().equals("obj")){
                dlg.newObjectName();
            }
            else {
                database.save(testData);
                testData.setSaved(true);
                mCommonUI.enableButton(btSave, false);
                update(testData.getObject().getLastMeasurement(), true);
            }
        }
    }

    public void stateChanged(int state){
        switch (state){
            case BluetoothClassic.STATE_CONNECTED:
                isConnected.set(true);
                messageID.set( R.string.read_general_params  );
                sendMessage( modbus.read.memoryMap()         );
                sendMessage( modbus.read.firmware()          );
                sendMessage( modbus.read.date()              );
                sendMessage( modbus.read.mode()              );
                break;
            case BluetoothClassic.STATE_CONNECTING:
                break;
            case BluetoothClassic.STATE_NOT_RESPONDING:
                //isConnected.set(false);
                messageID.set(R.string.not_responding);
                break;
            case BluetoothClassic.STATE_FAIL:
                isConnected.set(false);
                break;
            case BluetoothClassic.STATE_LOST:
                isConnected.set(false);
                isTestRunning.set(false);
                mCommonUI.enableButton(btManage, true);
                break;
            case BluetoothClassic.STATE_CLOSED:
                isConnected.set(false);
                isTestRunning.set(false);
                mCommonUI.enableButton(btManage, true);
                break;
        }
    }

    public void sendMessage(com.megabras.modbus.Message message){
        ( (Main)activity ).addMessage(message);
    }

    public com.megabras.modbus.Message statusMessage(){
        return modbus.read.status();
    }

    public com.megabras.modbus.Message stopMessage(){
        return modbus.write.stop();
    }

    public void readGeneralConfig(){
        messageID.set(R.string.read_general_params);
        if(menuOptions ==null) {
            menuOptions = menu.findItem(optionsID);
        }
        menuOptions.setEnabled(false);
        sendMessage( modbus.read.date() );
        sendMessage( modbus.read.memoryPointer() );
    }

    public void setAutoConn(boolean enable){
        device.setAutoConnection(enable);
    }

    public void setDateTimeFormat(int dateFormat, int timeFormat){}

    public boolean getAutoConn(){
        return device.getAutoConnection();
    }

    public void permissionCallback(int permission)
    {
        switch (permission)
        {
            case Main.REQUEST_RECORD_AUDIO_PERMISSION:
                recordAudio();
                break;
            case Main.REQUEST_PICTURE_PERMISSION:
                takePicture();
                break;
            case Main.REQUEST_GPS_PERMISSION:
                getGPS();
                break;
            case Main.REQUEST_MEMORY_REPORT_PERMISSION:
                break;
            case Main.REQUEST_MEMORY_PERMISSION:
                database.createFolder();
                if(permissionCallBackShowDialog){
                    dlg.showManager(0, testData);
                    permissionCallBackShowDialog = false;
                }
                else {
                    save();
                }
                break;
            case Main.REQUEST_MEMORY_MAP_PERMISSION:
                map();
                break;
        }
    }

    public void sendPriorityMessage(com.megabras.modbus.Message message){
        ( (Main)activity ).addPriorityMessage(message);
    }

    @SuppressWarnings("SameParameterValue")
    private void userMessage(String message, int color, boolean busy)
    {
        ( (Main)activity ).message(message, color, busy);
    }

    private void userMessage(int stringID, int color, boolean busy){
        String message = activity.getString(stringID);
        ( (Main)activity )
                .message(message, color, busy);
    }

    /***********************************************************************************************
     * REGION Handler
     ***********************************************************************************************/

    private void export(int index){}

    public static class mHandler extends Handler
    {
        private final WeakReference<Controller> mClass;
        public static final int
                REPORT_UPDATE   = 0,
                REPORT_FINISHED = 1,
                REPORT_ERROR    = 2;

        public mHandler(Controller c)
        {
            mClass = new WeakReference<>(c);
        }

        @Override
        public void handleMessage(Message msg)
        {
            final Controller c = mClass.get();

            switch(msg.what)
            {
                case mHandler.REPORT_UPDATE:
                    c.update( (String) msg.obj );
                    break;
                case mHandler.REPORT_FINISHED:
                    c.dlgProgress.dismiss();
                    break;
                case mHandler.REPORT_ERROR:
                    c.dlgProgress.dismiss();
                    break;
            }
        }
    }

    /***********************************************************************************************
     * REGION AUDIO
     ***********************************************************************************************/

    //AudioCloseDialogInterface
    public void audioDialogClosed(File file){
        dlg.objectDetails(testData, file);
    }

    private void recordAudio()
    {
        audio.showDialog("- - -");
//        if ( testData!=null )
//        {
//            File location  = database.getAudioFile(testData.getSerial(), testData.getDate().getTime()+"");
//            audio.setFile( location );
//            audio.showDialog(testData.getTestNumber()+"");
//        }
    }

    /***********************************************************************************************
     * REGION Picture
     ***********************************************************************************************/

    public void takePicture() {
        if(database!=null && testData!=null && testData.getFolder()!=null && !testData.getObjectName().equals(""))
        {
            File file = database.getPictureFile(testData.getFolder(), testData.getObjectName() );
            picture.showDialog(file);
        }
    }

    public void pictureStatus(final int status)
    {
        activity.runOnUiThread(() ->
        {
            switch (status)
            {
                case Picture.WAIT:
                    mCommonUI.changeButtonImage(btPicture, R.drawable.ic_wait, false);
                    break;
                case Picture.FINISHED:
                    mCommonUI.changeButtonImage(btPicture, R.drawable.ic_picture_states, true);
                    break;
            }
        });
    }

    /***********************************************************************************************
     * REGION GPS
     ***********************************************************************************************/

    private void map(){
        if ( database!=null && testData!=null && testData.getFolder()!=null && !testData.getObject().getAccuracy().equals("")  ) {

            ArrayList<MapData> maps = new ArrayList<>();
            List<String> objList    = database.getObjectsList(testData.getFile());

            for (String objName : objList){
                String[] gps = database.getGpsData(testData.getFile(), objName);
                if (!gps[0].equals("")){
                    MapData mapData = new MapData(
                            objName,
                            Double.parseDouble(gps[0]),
                            Double.parseDouble(gps[1]),
                            gps[2],
                            gps[3]
                    );
                    maps.add(mapData);
                }
            }
//
//            MapData mapData = new MapData(
//                    testData.getObject().getName(),
//                    Double.parseDouble(testData.getObject().getLongitude()),
//                    Double.parseDouble(testData.getObject().getLatitude()),
//                    testData.getObject().getAltitude(),
//                    testData.getObject().getAccuracy()
//            );
            if (maps.size()>0)
                map.showMap(database.getMapSnapshotFile(testData.getFolder()), maps);
        }
    }

    private void getGPS()
    {
        if ( database!=null && testData!=null && testData.getFolder()!=null && !testData.getObjectName().equals("") ){
            if ( testData.getObject().getAccuracy().equals("") )
                mGPS.getDataGPS();
            else{
                String[] gps = new String[]{
                        testData.getObject().getLongitude(),
                        testData.getObject().getLatitude(),
                        testData.getObject().getAltitude(),
                        testData.getObject().getAccuracy()
                };
                mGPS.dialogConfirmReplace( gps );
            }
        }
    }

    public void GpsData(int which, String[] data) {
        switch (which)
        {
            case GPS.RESULT:
                if ( data!=null && testData!=null)
                {
                    testData.getObject().setLongitude( data[0] );
                    testData.getObject().setLatitude ( data[1] );
                    testData.getObject().setAltitude ( data[2] );
                    testData.getObject().setAccuracy ( data[3] );
                    database.addGpsData( testData.getFile(), testData.getObject() );
                }
                mCommonUI.changeButtonImage(btGPS, R.drawable.ic_gps_states, true);
                break;
            case GPS.WAIT:
                mCommonUI.changeButtonImage(btGPS, R.drawable.ic_wait, false);
                break;
            case GPS.CANCELLED:
                mCommonUI.changeButtonImage(btGPS, R.drawable.ic_gps_states, true);
                break;
        }
    }

    /***********************************************************************************************
     * REGION Dialogs interface
     ***********************************************************************************************/

    public void deletedFile(String name){
        if (testData!=null && testData.getFolder()!=null && testData.getFolder().getName().equals(name)){
            testData = null;
            testData = new TestData();
            testData.addObserver(this);
            testData.setSiteName("");
            testData.addObject("");
            mainScreen.reset();
            table.reset();
        }
    }

    public void loadedFile(String name){
        database.load(name, testData);
        //testData.addObserver(this);
        //testData.setSiteName(testData.getSiteName());
        mainScreen.setTestSite(testData.getSiteName());
        dlg.objectsList(testData);
        //table.setAllValues( testData.getSelectedObject().getMeasurements() );
    }

    public void closeFile(){
        testData = null;
        testData = new TestData();
        testData.addObserver(this);
        testData.setSiteName("");
        testData.addObject("");
        mainScreen.setTestSite("- - -");
        mainScreen.setObject("- - -");
        mainScreen.resetMeasurement();
        table.reset();
    }

    public void objectSelected(){
        mCommonUI.enableButton(btSave, false);
        mainScreen.setTestSite(testData.getSiteName());
        if (testData.getObject()!=null && testData.getObject().getMeasurements()!=null)
            table.setAllValues( testData.getObject().getMeasurements() );
        if (testData.getObject()!=null)
            mainScreen.setObject(testData.getObjectName());
    }

    public void savedFile(String siteName, String objectName, String operatorName){
        testData.setSiteName(siteName);
        testData.setOperatorName(operatorName);

        if (testData.getObject()!=null)
            testData.setObjectName(objectName);
        else {
            testData.addObject(objectName);
        }

        table.reset();

        for (Measurement m : testData.getObject().getMeasurements()){
            File audio = new File(database.getAudioFileName( testData.getObjectName(), m.getTestNumber() ));
            m.setAudio(audio);
            //table.update(m);
        }

        database.save(testData);
        mCommonUI.enableButton(btSave, false);
        mainScreen.setTestSite(testData.getSiteName());
        mainScreen.setObject(testData.getObjectName());
        update(testData.getObject().getLastMeasurement(), true);
    }

    public void playAudio(File file, String testNumber){
        if(file!=null) {
            audio.setFile(file);
            if(Main.permissionToRecordAccepted){
                audio.showDialog(testNumber);
            }
            else {
                ((Main)activity).requestPermission(Main.REQUEST_RECORD_AUDIO_PERMISSION);
            }
        }
    }

    public void newObjectName(String name){
        testData.setObjectName(name);
        mainScreen.setObject(name);
        save();
    }

    public void pictureShow(File file, final DialogInterface.OnDismissListener onDismissListener){
        picture.showDialog(file, onDismissListener);
    }

    public void export(String siteName){
        if(Main.permissionToMemoryAccepted)
        {
            Log.e(TAG, "REPORT: " + siteName);

            TestData reportData        = new TestData();
            ArrayList<Objects> objList = database.loadDataForReport(siteName, reportData);

            if(dlgProgress==null)
                creatDialogProgress();
            dlgProgress.show();

            new Thread(() ->
                    reportPDF.generate(database, reportData, objList)
            ).start();
        }
        else
        {
            ((Main)activity).requestPermission(Main.REQUEST_MEMORY_PERMISSION);
        }
    }

    public void updateDevice(Device device){
        messageID.set(R.string.updating);
        parent.updateAutoConn(device.getAutoConnection());
        sendMessage(modbus.write.date(device.getDate()));
    }

    public void eraseMemory(){
        messageID.set(R.string.updating);
        sendMessage( modbus.write.memoryErase() );
    }

    /***********************************************************************************************
     * REGION Report
     ***********************************************************************************************/

    private void creatDialogProgress(){
        dlgProgress = new Dialog(activity);
        dlgProgress.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dlgProgress.setCanceledOnTouchOutside(false);
        dlgProgress.setContentView(R.layout.progress_bar);
        ((TextView) dlgProgress.findViewById(R.id.dlg_title)).setText(R.string.please_wait);
        ((TextView) dlgProgress.findViewById(R.id.message)).setText(R.string.saving_data_in_pdf);
        dlgProgress.findViewById(R.id.dlg_icon).setVisibility(View.GONE);
    }

    public void update(String progress){
        ((TextView)dlgProgress.findViewById(R.id.status)).setText(progress);
    }

    public void setMenuItemOptions(Menu menu, int optionsID){
        this.menu      = menu;
        this.optionsID = optionsID;
        if( menuOptions==null) {
            menuOptions = menu.findItem(optionsID);
        }
        if ( mainScreen.getView()==null )
            return;

        mainScreen.reset();
    }

    private void update(Measurement measurement, boolean updateLast)
    {
        if( measurement != null )
        {
            mainScreen.update( device, measurement );
            if ( updateLast )
            {
                table.update( measurement );
            }
        }
    }

    /***********************************************************************************************
     * REGION CALLBACKs
     ***********************************************************************************************/

    private void enableCommandButtons(boolean enabled)
    {
        mCommonUI.enableButtonBg(btTest   , enabled);
        mCommonUI.enableButton  (btPrinter, enabled);
        mCommonUI.enableButton  (btR      , enabled);
        mCommonUI.enableButton  (btRc     , enabled);
    }

    private void enableToolBar(boolean enable)
    {
        if (toolBar==null)
            return;

        mCommonUI.enableButton(btManage , enable);

        if(testData==null || testData.getSiteName().equals(""))
            mCommonUI.enableButton(btDetails, false);
        else
            mCommonUI.enableButton(btDetails, enable);

        if(testData!=null && testData.getObject()!=null && testData.getObject().getMeasurements().size()>0)
            mCommonUI.enableButton(btSave   , enable);
        else
            mCommonUI.enableButton(btSave   , false);

        if(testData!=null && testData.getObject()==null || testData.getObjectName().equals("") || testData.getObjectName().equals("obj"))
            enable = false;
        mCommonUI.enableButton(btMap    , enable);
        mCommonUI.enableButton(btGPS    , enable);
        mCommonUI.enableButton(btPicture, enable);
        mCommonUI.enableButton(btVoice  , enable);
    }

    /***********************************************************************************************
     * REGION State machine
     ***********************************************************************************************/
    @Override
    public void update(Observable observable, Object arg1) {
        if (observable instanceof BooleanObservable) {
            BooleanObservable bol = (BooleanObservable) observable;
            if ( bol.ID == isTestRunning.ID )
            {
                if ( bol.get() )
                {
                    messageID.set( R.string.running );
                    sendMessage( modbus.read.testNumber() );
                    mainScreen.resetMeasurement();
                    //table.reset();
                    viewPager.setCurrentItem(0);
                    if (menuOptions !=null)
                        menuOptions.setEnabled(false);
                    btTest.setVisibility(View.GONE);
                    btStop.setVisibility(View.VISIBLE);
                    mCommonUI.enableButton(btR,false);
                    mCommonUI.enableButton(btRc,false);
                    enableToolBar(false);
                }
                else
                {
                    messageID.set( R.string.md_test_finished );
                    if (menuOptions !=null)
                        menuOptions.setEnabled(true);
                    btTest.setVisibility(View.VISIBLE);
                    mCommonUI.enableButtonBg(btTest, true);
                    btStop.setVisibility(View.GONE);
                    mCommonUI.enableButton(btR,true);
                    mCommonUI.enableButton(btRc,true);
                    enableToolBar(true);
                }
            }
            else if ( bol.ID == isConnected.ID )
            {
                enableCommandButtons(bol.get());
                messageID.set(R.string.open_port);
            }
        }
        else if (observable instanceof TestData)
        {
            int itemID = (int) arg1;

            switch (itemID){
                case TestData.ID_SELECTED_OBJECT:
                    if (testData.getObject()==null || testData.getObject()!=null && testData.getObjectName().equals("") || testData.getObject()!=null && testData.getObjectName().equals("obj") ) {
                        table.reset();
                        mainScreen.resetObject();
                        mCommonUI.enableButton(btPicture, false);
                        mCommonUI.enableButton(btGPS    , false);
                        mCommonUI.enableButton(btMap    , false);
                    }
                    else {
                        mCommonUI.enableButton(btPicture, true);
                        mCommonUI.enableButton(btGPS    , true);
                        mCommonUI.enableButton(btMap    , true);
                    }
                    break;
                case TestData.ID_TEST_NAME:
                    try {
                        if (!testData.getSiteName().equals(""))
                            mCommonUI.enableButton(btDetails, true);
                        else
                            mCommonUI.enableButton(btDetails, false);
                    }
                    catch (Exception ignored){}
                    break;
            }
        }
        else if (observable instanceof IntegerObservable)
        {
            IntegerObservable ob = (IntegerObservable) observable;
            if ( ob.ID == messageID.ID )
            {
                switch ( ob.get() )
                {
                    case R.string.read_general_params:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true );
                        break;
                    case R.string.buzzer_param_was_read:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true );
                        break;
                    case R.string.erase_memory:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true );
                        break;
                    case R.string.firmware_was_read:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true );
                        break;
                    case R.string.manual_parameters_was_read:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true );
                        break;
                    case R.string.memory_map_was_read:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true );
                        break;
                    case R.string.md_warning_test_start:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true );
                        break;
                    case R.string.running:
                        userMessage(ob.get(), CommonUI.COLOR_RED, true );
                        break;
                    case R.string.md_configurations_changed:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true );
                        break;
                    case R.string.operation_mode_was_read:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true );
                        break;
                    case R.string.open_port:
                        userMessage(ob.get(), CommonUI.COLOR_GREEN, false );
                        break;
                    case R.string.ready:
                        userMessage(ob.get(), CommonUI.COLOR_GREEN, false);
                        break;
                    case R.string.md_test_finished:
                        userMessage(ob.get(), CommonUI.COLOR_GREEN, false);
                        break;
                    case R.string.test_cancelled:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, false );
                        break;
                    case R.string.test_voltage_was_read:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true );
                        break;
                    case R.string.updating:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true );
                        break;
                    case R.string.check_pole_h:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, false );
                        mCommonUI.enableButton(btSave, false);
                        mainScreen.resetMeasurement();
                        break;
                    case R.string.not_responding:
                        userMessage(activity.getString(R.string.not_responding, model), CommonUI.COLOR_YELLOW, true );
                        break;
                    case R.string.printing:
                        userMessage(ob.get(), CommonUI.COLOR_YELLOW, true );
                        break;
                }
            }
        }
    }

    public void processResponse(int id, String response){
        switch (id)
        {
            case Protocol.ID_STATUS_READ:
                if (!isTestRunning.get()){
                    modbus.parse.status(device, response);
                    parent.updateBattery( device.getBattery() );

                    if(response.length()>16){
                        modbus.parse.statusFull(device, testData.getObject().getLastMeasurement(), response);
                    }

                    if (!isPrinting)
                        mCommonUI.enableButton(btPrinter, device.getStatus(Device.ST_READY_TO_PRINT));
                }
                else {
                    if(response.length()>16){
                        modbus.parse.statusFull(device, testData.getObject().getLastMeasurement(), response);
                        File audio = new File(database.getAudioFileName( testData.getObjectName(), testData.getObject().getLastMeasurement().getTestNumber() ));
                        testData.getObject().getLastMeasurement().setAudio(audio);
                        update(testData.getObject().getLastMeasurement(), false);
                    }
                    sendMessage(modbus.read.statusFull());
                }

                if(device.getStatus(Device.ST_CHECK_POLE))
                    messageID.set( R.string.check_pole_h );

                isTestRunning.set( device.getStatus(Device.ST_TEST_RUNNING) );
                break;
            case Protocol.ID_MEMORY_MAP_READ:
                int mapVersion = modbus.parse.memoryMap(response);
                Protocol.setRevisionMap(mapVersion);
                //modbus.setRevisionMap(mapVersion);
                messageID.set(R.string.memory_map_was_read);
                break;
            case Protocol.ID_FIRMWARE_READ:
                firmware = modbus.parse.firmware(response);
                messageID.set(R.string.firmware_was_read);
                break;
            case Protocol.ID_MODE_WRITE:
                if(response.equals(modbus.write.mode(Protocol.MODE_COMPENSATION).getMessage()))
                    device.setMode( Protocol.MODE_COMPENSATION );
                else
                    device.setMode( Protocol.MODE_NORMAL );
                mainScreen.setMode( device.getMode() );
                mCommonUI.enableButtonBg(btTest, true);
                mCommonUI.enableButton(btR   , true);
                mCommonUI.enableButton(btRc  , true);
                messageID.set( R.string.ready );
                break;
            case Protocol.ID_MODE_READ:
                device.setMode( modbus.parse.mode(response) );
                mainScreen.setMode( device.getMode() );
                messageID.set(R.string.ready);
                break;
            case Protocol.ID_DATE_READ:
                device.setDate( modbus.parse.date(response) );
                break;
            case Protocol.ID_DATE_WRITE:
                messageID.set(R.string.ready);
                dlg.options(device);
                break;
            case Protocol.ID_MEM_POINTER_WRITE:
                messageID.set(R.string.ready);
                device.setFreeMemory(100);
                dlg.options(device);
                break;
            case Protocol.ID_MEM_POINTER_READ:
                device.setFreeMemory(modbus.parse.memoryFree(response));
                messageID.set(R.string.ready);
                dlg.options(device);
                menuOptions.setEnabled(true);
                break;
            case Protocol.ID_TEST_NUMBER_READ:
                int testNumber = modbus.parse.testNumber(response);
                Measurement m  = new Measurement();
                m.setTestNumber(testNumber+"");
                if(testData.getObject()==null || testData.getObjectName().equals("")){
                    testData.addObject("obj");
                }
                testData.getObject().addMeasurement(m);
                testData.setSaved(false);
                break;
            case Protocol.ID_PRINT_WRITE:
                mCommonUI.enableButton(btPrinter, true);
                messageID.set(R.string.ready);
                isPrinting = false;
                break;
        }
    }
}
