package com.megabras.bluelogg.tm25r;

import android.app.Activity;
import android.content.Context;
import android.content.SharedPreferences;
import android.os.Environment;
import android.util.Log;

import com.megabras.bluelogg.MUtils;
import com.megabras.bluelogg.R;
import com.megabras.modbus.TM25R.Measurement;
import com.megabras.modbus.TM25R.Objects;
import com.megabras.modbus.TM25R.TestData;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.math.RoundingMode;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

@SuppressWarnings("ResultOfMethodCallIgnored")
public class Database {
    private final String
            TAG             = "tm25r.Database";

    private String
            model           = "",
            serial          = "",
            root            = Environment.getExternalStorageDirectory().toString(),
            fileCompany     = "company.xml",
            fileTestsites   = "testsites.xml";

    private Activity
            activity;

    private File
            dirlib,
            dirdata;

    public Database(Activity activity, String model, String serial){
        this.activity = activity;
        this.model    = model;
        this.serial   = serial;
    }

    public boolean createFolder(){
        if (!Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED))
            return false;

        File dir    = new File(root + "/" + activity.getResources().getString(R.string.company));
        dirlib      = new File(dir + "/lib");
        dirdata     = new File(dir + "/data");

        boolean
            dirExists     = false,
            dirlibExists  = false,
            dirdataExists = false;

        if(!dirlib.exists()){ // Check if already has old TM25R data
            if(!dir.exists())
                dirExists = dir.mkdir();
            dir     = new File(root + "/" + activity.getResources().getString(R.string.company) + "/" + model);
            dirlib  = new File(dir + "/lib");
            dirdata = new File(dir + "/data");
        }

        if(!dir.exists())
            dirExists = dir.mkdir();

        if(!dirlib.exists())
            dirlibExists = dirlib.mkdir();

        if(!dirdata.exists())
            dirdataExists = dirdata.mkdir();

        createCompanyFile();
        createSitesFile();

        return dirExists && dirlibExists && dirdataExists;
    }

    private void createCompanyFile(){
        try {
            File file = new File(dirlib, fileCompany);

            if (file.exists())
                return;

            Document doc = uGetDoc(null);
            Element root = doc.createElement("companydata");
            doc.appendChild(root);

            Element companyDetails = doc.createElement("companydetails");
            root.appendChild(companyDetails);

            companyDetails.appendChild(setNode("company","",doc));
            companyDetails.appendChild(setNode("address","",doc));
            companyDetails.appendChild(setNode("city","",doc));
            companyDetails.appendChild(setNode("state","",doc));
            companyDetails.appendChild(setNode("zipcode","",doc));
            companyDetails.appendChild(setNode("phone","",doc));
            companyDetails.appendChild(setNode("fax","",doc));

            Element operators = doc.createElement("operators");
            root.appendChild(operators);

            TransformerFactory factory = TransformerFactory.newInstance();
            Transformer transformer = factory.newTransformer();

            DOMSource domSource = new DOMSource(doc.getDocumentElement());
            OutputStream output = new ByteArrayOutputStream();
            StreamResult result = new StreamResult(output);

            transformer.transform(domSource, result);

            String string = output.toString();
            FileOutputStream outputStream;

            outputStream = new FileOutputStream(file);
            outputStream.write(string.getBytes());
            outputStream.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public boolean createSitesFile(){
        try {
            File file = new File (dirlib, fileTestsites);

            if(file.exists())
                return true;

            DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
            Document doc = documentBuilder.newDocument();

            Element root = doc.createElement("testsites");
            doc.appendChild(root);

            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            Transformer transformer = transformerFactory.newTransformer();
            DOMSource source = new DOMSource(doc);

            //StreamResult result = new StreamResult(file);
            StreamResult result = new StreamResult(new FileOutputStream(file));
            transformer.transform(source, result);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    public void save(TestData testData){
        try {
            if(testData.getFile()!=null && testData.getObject()!=null && objectExists(testData.getFile(), testData.getObjectName())) {
                addMeasurement(testData);
                return;
            }

            if (testData.getFolder()==null){
                if(testData.getSiteName().equals(""))
                    testData.setSiteName("testsite");

                String name             = MUtils.removeSpecialChars(testData.getSiteName());
                DateFormat dateFormat   = new SimpleDateFormat("yyyy-MM-dd_HHmm", Locale.getDefault());
                Date date               = new Date();
                String time             = dateFormat.format(date);
                String fileData         = name + "_" + time;
                File dirsite            = new File(dirdata + "/" + fileData);
                testData.setFolder(dirsite);
            }

            if (!testData.getFolder().exists())
                testData.getFolder().mkdir();

            if ( testData.getFile()==null ){
                File file = new File(testData.getFolder() + "/" + testData.getFolder().getName() + ".xml");
                testData.setFile(file);
            }

            Document doc                = uGetDoc(null);
            Element test                = doc.createElement("test");
            doc.appendChild(test);

            Element operator = doc.createElement("operator");
            test.appendChild(operator);
            operator.appendChild( setNode("opname", testData.getOperatorName(), doc) );
            operator.appendChild( setNode("extension", testData.getOperatorExtension(), doc) );
            operator.appendChild( setNode("email", testData.getOperatorEmail(), doc) );

            Element equipment = doc.createElement("equipment");
            test.appendChild(equipment);
            equipment.appendChild( setNode("model"		, model		, doc) );
            equipment.appendChild( setNode("serial"		, serial	, doc) );

            Element site = doc.createElement("testsite");
            test.appendChild(site);
            site.appendChild(	setNode( "sitename"	, testData.getSiteName()	, doc )	);
            site.appendChild(	setNode( "contact"	, testData.getSiteContact() , doc )	);
            site.appendChild(	setNode( "address"	, testData.getSiteAddress() , doc )	);
            site.appendChild(	setNode( "city"		, testData.getSiteCity()    , doc )	);
            site.appendChild(	setNode( "state"	, testData.getSiteState()   , doc )	);
            site.appendChild(	setNode( "zipcode"	, testData.getSiteZipCode() , doc )	);
            site.appendChild(	setNode( "phone"	, testData.getSitePhone()	, doc )	);
            site.appendChild(	setNode( "fax"		, testData.getSiteFax()		, doc )	);

            Element objects = doc.createElement("objects");
            test.appendChild(objects);

            Element comments = doc.createElement("comments");
            test.appendChild(comments);

            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            Transformer transformer = transformerFactory.newTransformer();
            DOMSource source = new DOMSource(doc);

            StreamResult result = new StreamResult(new FileOutputStream(testData.getFile()));
            transformer.transform(source, result);

            if (!objectExists(testData.getFile(), testData.getObjectName()))
                addTestObject(testData);
            addMeasurement(testData);
        } catch (Exception e) {
            Log.e(TAG, "ERROR: " + e.getMessage() );
        }
    }

    private Document uGetDoc(File file) throws ParserConfigurationException, SAXException, IOException{
        DocumentBuilderFactory
                documentBuilderFactory  = DocumentBuilderFactory.newInstance();
        DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
        if (file==null)
            return documentBuilder.newDocument();
        else
            return documentBuilder.parse(file);
    }

    public void addTestObject(TestData testData){
        try {
            File file = testData.getFile();
            Document doc    = uGetDoc(file);
            Node objects    = doc.getElementsByTagName("objects").item(0);
            Element obj     = doc.createElement("object");

            objects.appendChild(obj);
            obj.setAttribute("id", testData.getObjectName());
            obj.appendChild(	setNode( "name"	       , testData.getObjectName(), doc )	);
            obj.appendChild(	setNode( "longititude" , ""	            , doc )	);
            obj.appendChild(	setNode( "latitude"	   , ""	            , doc )	);
            obj.appendChild(	setNode( "altitude"	   , ""	            , doc )	);
            obj.appendChild(	setNode( "accuracy"	   , ""	            , doc )	);
            obj.appendChild(	setNode( "readings"	   , ""	            , doc )	);

            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            Transformer transformer = transformerFactory.newTransformer();
            DOMSource source = new DOMSource(doc);

            StreamResult result = new StreamResult(new FileOutputStream(file));
            transformer.transform(source, result);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public Objects getObject(TestData testData, String name){
        try {
            Document doc    = uGetDoc(testData.getFile());
            Element el      = doc.getElementById(name);
            NodeList nl     = el.getElementsByTagName("read");
            Objects o       = new Objects();
            o.setObjectName ( content(el, "name"       ) );
            o.setLongitude  ( content(el, "longititude") );
            o.setLatitude   ( content(el, "latitude"   ) );
            o.setAltitude   ( content(el, "altitude"   ) );
            o.setAccuracy   ( content(el, "accuracy"   ) );

            SimpleDateFormat dateFormatOLD = new SimpleDateFormat("dd/MM/yyyy HH:mm");
            SimpleDateFormat dateFormatNew = new SimpleDateFormat("yyyy-MM-dd HH:mm");

            for (int a=0; a<nl.getLength(); a++){
                if (nl.item(a).getNodeType() == Node.ELEMENT_NODE){
                    Element reading = (Element) nl.item(a);
                    Measurement m   = new Measurement();
                    m.setTestNumber             ( content(reading, "number") );

                    String dateString = content(reading, "date");
                    try{
                        if (dateString.matches("\\d{2}/\\d{2}/\\d{4}\\s\\d{2}:\\d{2}")){
                            Date d     = dateFormatOLD.parse( dateString );
                            dateString = dateFormatNew.format(d);
                        }
                    }catch (ParseException pe){}
                    m.setDate                   ( dateString );
                    m.setResistance             ( content(reading, "r") );
                    m.setResistanceCompensated  ( content(reading, "rc") );
                    m.setCapacitance            ( content(reading, "c") );
                    m.setInductance             ( content(reading, "l") );

                    File audio = new File(testData.getFolder(), getAudioFileName(name, m.getTestNumber()));
                    m.setAudio(audio);

                    o.addMeasurement(m);
                }
            }
            return o;
        }
        catch (Exception e){
            return null;
        }
    }

    public String getAudioFileName(String objectName, String testNumber){
        return "object_" + MUtils.removeSpecialChars(objectName) + "_test-" + testNumber + "_annotation.3gp";
    }

    public void addMeasurement(TestData testData){
        try{
            File file       = testData.getFile();
            Document doc    = uGetDoc(file);
            Node readings   = getObject(doc, testData.getObjectName());
            Element read    = doc.createElement("read");
            Measurement m   = testData.getObject().getLastMeasurement();

            if (m==null || readings==null)
                return;

            readings.appendChild(read);
            read.appendChild(	setNode( "number", m.getTestNumber()           , doc )	);
            read.appendChild(	setNode( "date"	 , m.getDate()  		       , doc )	);
            read.appendChild(	setNode( "r"	 , m.getResistance()           , doc )	);
            read.appendChild(	setNode( "rc"	 , m.getResistanceCompensated(), doc )	);
            read.appendChild(	setNode( "c"	 , m.getCapacitance()          , doc )	);
            read.appendChild(	setNode( "l"	 , m.getInductance()           , doc )	);

            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            Transformer transformer = transformerFactory.newTransformer();
            DOMSource source = new DOMSource(doc);

            StreamResult result = new StreamResult(new FileOutputStream(file));
            transformer.transform(source, result);
        } catch (Exception e){
            for (StackTraceElement ste : e.getStackTrace())
                Log.e(TAG, ste.toString());
        }
    }

    private Node getObject(Document doc, String objName){
        Node objects  = doc.getElementsByTagName("objects").item(0);
        NodeList list = objects.getChildNodes();
        for (int i=0; i<list.getLength(); i++) {
            Node node    = list.item(i);
            Node subnode = node.getFirstChild();
            if (objName.equals(subnode.getTextContent())) {
                return node.getLastChild();
            }
        }
        return null;
    }

    public boolean objectExists(File file, String objName){
        try{
            DocumentBuilderFactory
                    documentBuilderFactory  = DocumentBuilderFactory.newInstance();
            DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
            Document doc                    = documentBuilder.parse(file);
            Node obj                        = doc.getElementById(objName);
            return obj!=null;
        } catch (Exception e){
            e.printStackTrace();
            return false;
        }
    }

    public String[] getOperators(){
        try {
            DocumentBuilderFactory
                    documentBuilderFactory  = DocumentBuilderFactory.newInstance();
            DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
            Document doc                    = documentBuilder.parse(new File (dirlib, fileCompany));
            Node operators                  = doc.getElementsByTagName("operators").item(0);
            NodeList list                   = operators.getChildNodes();
            String[] op                     = new String[list.getLength()];

            for (int i = 0; i < list.getLength(); i++) {
                Node node = list.item(i);
                Node subnode = node.getFirstChild().getFirstChild();
                if(subnode.getNodeValue().equals(this.getDefaultOperator())){
                    op[i] = subnode.getNodeValue() + " " + activity.getResources().getString(R.string.defaults);
                } else {
                    op[i] = subnode.getNodeValue();
                }
            }
            return op;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public String[] getOperatorDetails(String name){
        try {
            Document doc    = uGetDoc(new File (dirlib, fileCompany));
            Node operators  = doc.getElementsByTagName("operators").item(0);
            NodeList list   = operators.getChildNodes();
            String[] op     = new String[]{"",""};

            for (int i=0; i<list.getLength(); i++) {
                Node node    = list.item(i);
                Node subnode = node.getFirstChild().getFirstChild();
                if (name.equals(subnode.getNodeValue())) {
                    Node operator   = doc.getElementsByTagName("operator").item(i);
                    NodeList oplist = operator.getChildNodes();
                    op[0]           = oplist.item(1).getTextContent();
                    op[1]           = oplist.item(2).getTextContent();
                    return op;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return new String[]{"","",""};
    }

    public boolean removeOperator(String name){
        try {
            Document doc    = uGetDoc(new File (dirlib, fileCompany));
            Node operators  = doc.getElementsByTagName("operators").item(0);
            NodeList list   = operators.getChildNodes();

            for (int i = 0; i < list.getLength(); i++) {
                Node node = list.item(i);
                Node subnode = node.getFirstChild().getFirstChild();
                if (name.equals(subnode.getNodeValue())) {
                    operators.removeChild(node);
                }
            }

            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            Transformer transformer = transformerFactory.newTransformer();
            DOMSource source = new DOMSource(doc);

            File file = new File(dirlib, fileCompany);

            StreamResult result = new StreamResult(new FileOutputStream(file));
            transformer.transform(source, result);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    @SuppressWarnings("SimplifiableIfStatement")
    public boolean updateOperator(String name, String extension, String email){
        if(!removeOperator(name)){
            return false;
        }
        return addOperator(name,extension,email);
    }

    public boolean addOperator(String name, String extension, String email){
        try {
            Document doc    = uGetDoc(new File (dirlib, fileCompany));
            Node operators  = doc.getElementsByTagName("operators").item(0);
            NodeList list   = operators.getChildNodes();

            for (int i = 0; i < list.getLength(); i++) {
                Node node = list.item(i);
                Node subnode = node.getFirstChild().getFirstChild();
                if (name.equals(subnode.getNodeValue())) {
                    return false;
                }
            }

            Element operator = doc.createElement("operator");
            operators.appendChild(operator);
            operator.appendChild(setNode("name",name,doc));
            operator.appendChild(setNode("extension",extension,doc));
            operator.appendChild(setNode("email",email,doc));

            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            Transformer transformer = transformerFactory.newTransformer();
            DOMSource source = new DOMSource(doc);

            File file = new File(dirlib, fileCompany);

            StreamResult result = new StreamResult(new FileOutputStream(file));
            transformer.transform(source, result);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }

    public String[] getCompany(){
        try {
            String[] data = new String[7];
            DocumentBuilderFactory
                    documentBuilderFactory  = DocumentBuilderFactory.newInstance();
            DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
            Document doc                    = documentBuilder.parse(new File (dirlib, fileCompany));
            data[0] = doc.getElementsByTagName("company").item(0).getTextContent();
            data[1] = doc.getElementsByTagName("address").item(0).getTextContent();
            data[2] = doc.getElementsByTagName("city").item(0).getTextContent();
            data[3] = doc.getElementsByTagName("state").item(0).getTextContent();
            data[4] = doc.getElementsByTagName("zipcode").item(0).getTextContent();
            data[5] = doc.getElementsByTagName("phone").item(0).getTextContent();
            data[6] = doc.getElementsByTagName("fax").item(0).getTextContent();
            return data;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public boolean operatorExists(String opName){
        String[] operators = getOperators();
        for (String op : operators) {
            if (op.equals(opName)){
                return true;
            }
        }
        return false;
    }

    public boolean updateCompany(String companyname, String address, String city, String state, String zipcode, String phone, String fax){
        try {
            DocumentBuilderFactory
                    documentBuilderFactory  = DocumentBuilderFactory.newInstance();
            DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
            Document doc                    = documentBuilder.parse(new File (dirlib, fileCompany));

            doc.getElementsByTagName("company").item(0).setTextContent(companyname);
            doc.getElementsByTagName("address").item(0).setTextContent(address);
            doc.getElementsByTagName("city").item(0).setTextContent(city);
            doc.getElementsByTagName("state").item(0).setTextContent(state);
            doc.getElementsByTagName("zipcode").item(0).setTextContent(zipcode);
            doc.getElementsByTagName("phone").item(0).setTextContent(phone);
            doc.getElementsByTagName("fax").item(0).setTextContent(fax);

            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            Transformer transformer = transformerFactory.newTransformer();
            DOMSource source = new DOMSource(doc);

            File file = new File(dirlib, fileCompany);

            //StreamResult result = new StreamResult(file);
            StreamResult result = new StreamResult(new FileOutputStream(file));
            transformer.transform(source, result);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }

    public TestData load(String name, TestData testData){
        try {
            //TestData testData = new TestData();

            File folder = new File(dirdata + "/" + name);
            File file   = new File(folder + "/" + name + ".xml");

            if (!folder.exists())
                return null;

            if (!file.exists())
                return null;

            testData.setFolder(folder);
            testData.setFile(file);

            DocumentBuilderFactory
                    documentBuilderFactory  = DocumentBuilderFactory.newInstance();
            DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
            Document doc                    = documentBuilder.parse(file);

            Element operator  = (Element) doc.getElementsByTagName("operator").item(0);
            Element equipment = (Element) doc.getElementsByTagName("equipment").item(0);
            Element testsite  = (Element) doc.getElementsByTagName("testsite").item(0);
            NodeList objects  = doc.getElementsByTagName("object");

            testData.setOperatorName        ( content(operator, "opname"    ) );
            testData.setOperatorExtension   ( content(operator, "extension" ) );
            testData.setOperatorEmail       ( content(operator, "email"     ) );

            testData.setModel       ( content(equipment, "model"  ) );
            testData.setSerial      ( content(equipment, "serial" ) );

            testData.setSiteName    ( content(testsite, "sitename") );
            testData.setSiteContact ( content(testsite, "contact" ) );
            testData.setSiteAddress ( content(testsite, "address" ) );
            testData.setSiteCity    ( content(testsite, "city"    ) );
            testData.setSiteState   ( content(testsite, "state"   ) );
            testData.setSiteZipCode ( content(testsite, "zipcode" ) );
            testData.setSitePhone   ( content(testsite, "phone"   ) );
            testData.setSiteFax     ( content(testsite, "fax"     ) );

            for (int i=0; i<objects.getLength(); i++){
                if (objects.item(i).getNodeType() == Node.ELEMENT_NODE) {
                    Element p = (Element) objects.item(i);
                    Objects o = new Objects();
                    o.setObjectName ( content(p, "name") );
                    o.setLongitude  ( content(p, "longititude") );
                    o.setLatitude   ( content(p, "latitude") );
                    o.setAltitude   ( content(p, "altitude") );
                    o.setAccuracy   ( content(p, "accuracy") );
                    NodeList r = p.getElementsByTagName("read");
                    for (int a=0; a<r.getLength(); a++){
                        if (r.item(a).getNodeType() == Node.ELEMENT_NODE){
                            Element reading = (Element) r.item(a);
                            Measurement m   = new Measurement();
                            m.setTestNumber             ( content(reading, "number") );
                            m.setDate                   ( content(reading, "date") );
                            m.setResistance             ( content(reading, "r") );
                            m.setResistanceCompensated  ( content(reading, "rc") );
                            m.setCapacitance            ( content(reading, "c") );
                            m.setInductance             ( content(reading, "l") );
                            o.addMeasurement(m);
                        }
                    }
                    testData.addObject(o);
                }
            }

            return testData;
        }
        catch (Exception e){
            Log.e(TAG, "Error: " + e.getMessage());
            for (StackTraceElement ste : e.getStackTrace())
                Log.e(TAG, ste.toString());
            return null;
        }
    }

    public ArrayList<Objects> loadDataForReport(String name, TestData testData){
        try {
            File folder = new File(dirdata + "/" + name);
            File file   = new File(folder + "/" + name + ".xml");

            if (!folder.exists())
                return null;

            if (!file.exists())
                return null;

            testData.setFolder(folder);
            testData.setFile(file);

            DocumentBuilderFactory
                    documentBuilderFactory  = DocumentBuilderFactory.newInstance();
            DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
            Document doc                    = documentBuilder.parse(file);

            Element operator  = (Element) doc.getElementsByTagName("operator").item(0);
            Element equipment = (Element) doc.getElementsByTagName("equipment").item(0);
            Element testsite  = (Element) doc.getElementsByTagName("testsite").item(0);
            NodeList objects  = doc.getElementsByTagName("object");

            testData.setOperatorName        ( content(operator, "opname"    ) );
            testData.setOperatorExtension   ( content(operator, "extension" ) );
            testData.setOperatorEmail       ( content(operator, "email"     ) );

            testData.setModel       ( content(equipment, "model"  ) );
            testData.setSerial      ( content(equipment, "serial" ) );

            testData.setSiteName    ( content(testsite, "sitename") );
            testData.setSiteContact ( content(testsite, "contact" ) );
            testData.setSiteAddress ( content(testsite, "address" ) );
            testData.setSiteCity    ( content(testsite, "city"    ) );
            testData.setSiteState   ( content(testsite, "state"   ) );
            testData.setSiteZipCode ( content(testsite, "zipcode" ) );
            testData.setSitePhone   ( content(testsite, "phone"   ) );
            testData.setSiteFax     ( content(testsite, "fax"     ) );

            ArrayList<Objects> objList = new ArrayList<>();

            for (int i=0; i<objects.getLength(); i++){
                if (objects.item(i).getNodeType() == Node.ELEMENT_NODE) {
                    Element p = (Element) objects.item(i);
                    Objects o = new Objects();
                    o.setObjectName ( content(p, "name") );
                    o.setLongitude  ( content(p, "longititude") );
                    o.setLatitude   ( content(p, "latitude") );
                    o.setAltitude   ( content(p, "altitude") );
                    o.setAccuracy   ( content(p, "accuracy") );
                    NodeList r = p.getElementsByTagName("read");
                    for (int a=0; a<r.getLength(); a++){
                        if (r.item(a).getNodeType() == Node.ELEMENT_NODE){
                            Element reading = (Element) r.item(a);
                            Measurement m   = new Measurement();
                            m.setTestNumber             ( content(reading, "number") );
                            m.setDate                   ( content(reading, "date") );
                            m.setResistance             ( content(reading, "r") );
                            m.setResistanceCompensated  ( content(reading, "rc") );
                            m.setCapacitance            ( content(reading, "c") );
                            m.setInductance             ( content(reading, "l") );
                            o.addMeasurement(m);
                        }
                    }
                    objList.add(o);
                }
            }

            return objList;
        }
        catch (Exception e){
            Log.e(TAG, "Error: " + e.getMessage());
            for (StackTraceElement ste : e.getStackTrace())
                Log.e(TAG, ste.toString());
            return null;
        }
    }

    private String content(Element e, String tagName){
        try {
            return e.getElementsByTagName(tagName).item(0).getTextContent();
        }
        catch (Exception ex){
            return "";
        }
    }

    public String[] getGpsData(File file, String objName){
        try{
            Document doc = uGetDoc(file);

            Node obj = doc.getElementById(objName);
            NodeList objChilds = obj.getChildNodes();

            String[] gps = new String[4];
            if(objChilds.getLength() > 2){
                gps[0] = objChilds.item(1).getTextContent();
                gps[1] = objChilds.item(2).getTextContent();
                gps[2] = objChilds.item(3).getTextContent();
                gps[3] = objChilds.item(4).getTextContent();
            } else {
                gps[0] = "";
                gps[1] = "";
                gps[2] = "";
                gps[3] = "";
            }

            return gps;
        } catch (Exception e){
            e.printStackTrace();
            return null;
        }
    }

    public List<String> getObjectsList(File file){
        List<String> objs = new ArrayList<>();
        try{
            DocumentBuilderFactory
                    documentBuilderFactory  = DocumentBuilderFactory.newInstance();
            DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
            Document doc                    = documentBuilder.parse(file);
            Node objects                    = doc.getElementsByTagName("objects").item(0);
            NodeList list                   = objects.getChildNodes();

            for (int i = 0; i < list.getLength(); i++) {
                Node node = list.item(i);
                objs.add(node.getFirstChild().getTextContent());
            }

            return objs;

        } catch (Exception e){
            e.printStackTrace();
            objs.add("");
            return objs;
        }
    }

    public boolean removeObject(File file, String object){
        try {
            DocumentBuilderFactory
                    documentBuilderFactory  = DocumentBuilderFactory.newInstance();
            DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
            Document doc                    = documentBuilder.parse(file);
            Node objects                    = doc.getElementsByTagName("objects").item(0);
            Node obj                        = doc.getElementById(object);
            objects.removeChild(obj);
            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            Transformer transformer = transformerFactory.newTransformer();
            DOMSource source = new DOMSource(doc);

            StreamResult result = new StreamResult(new FileOutputStream(file));
            transformer.transform(source, result);

            File img = getPictureFile(file, object); // Delete picture
            if (img.exists()) {
                img.delete();
            }

//            File snap = getMapSnapshotFile(file); // Delete map
//            if (snap.exists()) {
//                snap.delete();
//            }

            String objName = MUtils.removeSpecialChars( object ); // Delete audio files
            if ( file.isDirectory() ){
                for ( File child : file.listFiles() ){
                    String fileName = child.getName();
                    if( fileName.contains( "object_" + objName ) ){
                        File fileToDelete = new File( file, fileName);
                        if ( fileToDelete.exists() ){
                            fileToDelete.delete();
                        }
                    }
                }
            }

            return true;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }

    public boolean removeMeasurement(File file, String object, String testNumber){
        try {
            Document doc      = uGetDoc(file);
            Element  obj      = doc.getElementById(object);
            Element  readings = (Element) obj.getElementsByTagName("readings").item(0);
            NodeList reads    = obj.getElementsByTagName("read");

            for (int i=0; i<reads.getLength(); i++){
                Element n = (Element) reads.item(i);
                if (n.getNodeType() == Node.ELEMENT_NODE) {
                    if (testNumber.equals(n.getElementsByTagName("number").item(0).getTextContent())){
                        readings.removeChild(n);
                    }
                }
            }

            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            Transformer transformer = transformerFactory.newTransformer();
            DOMSource source = new DOMSource(doc);

            StreamResult result = new StreamResult(new FileOutputStream(file));
            transformer.transform(source, result);

            String folder = file.getAbsolutePath().replace(file.getName(), "");

            File audio = new File( folder + getAudioFileName(object, testNumber));
            if(audio.exists())
                audio.delete();

            return true;
        } catch (Exception e) {
            Log.e(TAG, e.getMessage());
            for (StackTraceElement ste : e.getStackTrace())
                Log.e(TAG, ste.toString());
        }
        return false;
    }

    public File getPictureFile(File dirsite, String name){
        name = name.trim();
        String img = MUtils.removeSpecialChars(name) + ".jpg";
        return new File(dirsite, img);
    }

    public File getMapSnapshotFile(File dirsite){
        String name = "snapshot";
        String img = MUtils.removeSpecialChars(name) + ".jpg";
        File file = new File(dirsite, img);
        return file;
    }

    public boolean updateSiteInFile(TestData testData){
        try {
            Document doc = uGetDoc(testData.getFile());
            doc.getElementsByTagName("contact").item(0).setTextContent(testData.getSiteContact());
            doc.getElementsByTagName("address").item(0).setTextContent(testData.getSiteAddress());
            doc.getElementsByTagName("city").item(0).setTextContent(testData.getSiteCity());
            doc.getElementsByTagName("state").item(0).setTextContent(testData.getSiteState());
            doc.getElementsByTagName("zipcode").item(0).setTextContent(testData.getSiteZipCode());
            doc.getElementsByTagName("phone").item(0).setTextContent(testData.getSitePhone());
            doc.getElementsByTagName("fax").item(0).setTextContent(testData.getSiteFax());
            doc.getElementsByTagName("opname").item(0).setTextContent(testData.getOperatorName());
            doc.getElementsByTagName("extension").item(0).setTextContent(testData.getOperatorExtension());
            doc.getElementsByTagName("email").item(0).setTextContent(testData.getOperatorEmail());

            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            Transformer transformer = transformerFactory.newTransformer();
            DOMSource source = new DOMSource(doc);

            StreamResult result = new StreamResult(new FileOutputStream(testData.getFile()));
            transformer.transform(source, result);
            return true;
        } catch (Exception e) {
            Log.e("Error", e.toString());
            e.printStackTrace();
        }
        return false;
    }

    public void addComments(File file, String msg){
        try {
            Document doc  = uGetDoc(file);
            Node comments = doc.getElementsByTagName("comments").item(0);

            comments.setTextContent(msg);

            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            Transformer transformer = transformerFactory.newTransformer();
            DOMSource source = new DOMSource(doc);

            StreamResult result = new StreamResult(new FileOutputStream(file));
            transformer.transform(source, result);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    public String getComments(File file){
        try {
//
//            File dirsites = new File(dirdata + "/" + selectedSite);
//            String fileDatas = selectedSite + ".xml";
//            File file = new File(dirsites, fileDatas);
            Document doc  = uGetDoc(file);
            Node comments = doc.getElementsByTagName("comments").item(0);
            return comments.getTextContent();
        } catch (Exception e) {
            e.printStackTrace();
            return "";
        }
    }

    public ArrayList<ListSites> getSavedFiles(){
        try{
            String[] list = dirdata.list();
            Arrays.sort(list);
            ArrayList<ListSites> sites = new ArrayList<>();
            for (String aList : list) {
                ListSites site = new ListSites(aList);
                sites.add(site);
            }
            return sites;
        }
        catch (Exception e){
            Log.e(TAG, e.getMessage());
            return null;
        }
    }

    public String[] getSiteDetails(String name){
        try {
            Document doc    = uGetDoc(new File (dirlib, fileTestsites));
            Node sites      = doc.getElementsByTagName("testsites").item(0);
            NodeList list   = sites.getChildNodes();
            String[] site   = new String[]{"","","","","","",""};

            for (int i = 0; i < list.getLength(); i++) {
                Node node = list.item(i);
                Node subnode = node.getFirstChild().getFirstChild();
                if (name.equals(subnode.getNodeValue())) {
                    Node operator = doc.getElementsByTagName("site").item(i);
                    NodeList sitelist = operator.getChildNodes();
                    site[0] = sitelist.item(1).getTextContent();
                    site[1] = sitelist.item(2).getTextContent();
                    site[2] = sitelist.item(3).getTextContent();
                    site[3] = sitelist.item(4).getTextContent();
                    site[4] = sitelist.item(5).getTextContent();
                    site[5] = sitelist.item(6).getTextContent();
                    site[6] = sitelist.item(7).getTextContent();
                    return site;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public boolean addSite(String sitename, String contact, String address, String city, String state, String zipcode, String phone, String fax){
        try {
            File file = new File (dirlib, fileTestsites);

            if(!file.exists()){
                return false;
            }

            DocumentBuilderFactory
                    documentBuilderFactory  = DocumentBuilderFactory.newInstance();
            DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
            Document doc                    = documentBuilder.parse(file);
            Node testsites                  = doc.getElementsByTagName("testsites").item(0);
            NodeList list                   = testsites.getChildNodes();

            for (int i = 0; i < list.getLength(); i++) {
                Node node = list.item(i);
                Node subnode = node.getFirstChild().getFirstChild();
                if (sitename.equals(subnode.getNodeValue())) {
                    return false;
                }
            }

            Element site = doc.createElement("site");
            testsites.appendChild(site);
            site.appendChild(setNode("sitename",sitename,doc));
            site.appendChild(setNode("contact",contact,doc));
            site.appendChild(setNode("address",address,doc));
            site.appendChild(setNode("city",city,doc));
            site.appendChild(setNode("state",state,doc));
            site.appendChild(setNode("zipcode",zipcode,doc));
            site.appendChild(setNode("phone",phone,doc));
            site.appendChild(setNode("fax",fax,doc));

            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            Transformer transformer = transformerFactory.newTransformer();
            DOMSource source = new DOMSource(doc);

            StreamResult result = new StreamResult(new FileOutputStream(file));
            transformer.transform(source, result);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    public boolean removeSite(String name){
        try {
            File file = new File(dirlib, fileTestsites);
            DocumentBuilderFactory
                    documentBuilderFactory  = DocumentBuilderFactory.newInstance();
            DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
            Document doc                    = documentBuilder.parse(file);
            Node testsites                  = doc.getElementsByTagName("testsites").item(0);
            NodeList list                   = testsites.getChildNodes();

            for (int i = 0; i < list.getLength(); i++) {
                Node node = list.item(i);
                Node subnode = node.getFirstChild().getFirstChild();
                if (name.equals(subnode.getNodeValue())) {
                    testsites.removeChild(node);
                }
            }

            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            Transformer transformer = transformerFactory.newTransformer();
            DOMSource source = new DOMSource(doc);

            StreamResult result = new StreamResult(new FileOutputStream(file));
            transformer.transform(source, result);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }

    public boolean testsiteExists(String siteName){
        List<String> sites = getSites();
        for (String s : sites) {
            if (s.equals(siteName)){
                return true;
            }
        }
        return false;
    }

    public String[] getOperatorsOrderByDefault(){
        try {
            Document doc    = uGetDoc(new File(dirlib, fileCompany));
            Node operators  = doc.getElementsByTagName("operators").item(0);
            NodeList list   = operators.getChildNodes();
            String[] op;
            if(!getDefaultOperator().equals("")){
                op    = new String[list.getLength()];
                op[0] = getDefaultOperator();
            }
            else {
                op    = new String[list.getLength()+1];
                op[0] = "";
            }

            for (int i = 0; i < list.getLength(); i++) {
                Node node    = list.item(i);
                Node subnode = node.getFirstChild().getFirstChild();
                if(!subnode.getNodeValue().equals(getDefaultOperator()))
                    op[i+1] = subnode.getNodeValue();
            }
            return op;
        } catch (Exception e) {
            Log.e("getOperatorsOrderByDef", e.toString());
            Log.e("getOperatorsOrderByDef", e.getMessage());
            return new String[]{""};
        }
    }

    public void setDefaultOperator(String operator){
        if(operator.equals(this.getDefaultOperator())){
            operator="";
        }
        SharedPreferences sharedPref = activity.getPreferences(Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = sharedPref.edit();
        editor.putString("default_op", operator);
        editor.apply();
    }


    public String getDefaultOperator(){
        SharedPreferences sharedPref = activity.getPreferences(Context.MODE_PRIVATE);
        return sharedPref.getString("default_op", "");
    }

    public void deleteDataFile(String selectedSite){
        File dirToDelete = new File(dirdata + "/" + selectedSite);
        try{
            if (dirToDelete.isDirectory()){
                for (File child : dirToDelete.listFiles()){
                    child.delete();
                }
            }
            dirToDelete.delete();
        } catch (Exception e){
            Log.e("deletDataFile", e.getMessage());
        }
    }

    public String[] getDataFileDetails(String selectedSite){
        try{
            File dirsites = new File(dirdata, selectedSite);
            File files = new File(dirsites, selectedSite + ".xml");
            int voices = 0;
            int pictures = 0;
            String operator = "";
            String objects = "";
            String tests = "";
            String[] details = new String[6];

            DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
            Document doc   =   documentBuilder.parse(files);
            Node op        =   doc.getElementsByTagName("opname").item(0);
            if(op != null){
                operator   =   op.getTextContent();
            }

            NodeList ob    =   doc.getElementsByTagName("object");
            if(ob != null){
                objects    =   ob.getLength() + "";
            }

            NodeList tt    =   doc.getElementsByTagName("read");
            if(tt != null){
                tests      =   tt.getLength() + "";
            }

            long size = 0;
            if (dirsites.isDirectory()){
                for (File child : dirsites.listFiles()){
                    if(child.getName().contains(".3gp")){
                        voices++;
                    }
                    if(child.getName().contains(".jpg") && !child.getName().contains("snapshot")){
                        pictures++;
                    }
                    size = size + child.length();
                }
                details[0] = sizeFormater(size);
                details[1] = String.valueOf(voices);
                details[2] = String.valueOf(pictures);
                details[3] = operator;
                details[4] = objects;
                details[5] = tests;
            }
            return details;
        } catch (Exception e){
            Log.e("getDataFileDetails", e.getMessage());
            return null;
        }
    }

    public List<String> getSites(){
        try {
            DocumentBuilderFactory
                    documentBuilderFactory  = DocumentBuilderFactory.newInstance();
            DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
            Document doc                    = documentBuilder.parse(new File (dirlib, fileTestsites));
            Node sites                      = doc.getElementsByTagName("testsites").item(0);
            NodeList list                   = sites.getChildNodes();

            List<String> site = new ArrayList<>();

            for (int i=0; i<list.getLength(); i++) {
                Node node    = list.item(i);
                Node subnode = node.getFirstChild().getFirstChild();
                site.add(subnode.getNodeValue());
            }

            return site;
        } catch (Exception e) {
            Log.e(TAG, e.getMessage());
        }
        return null;
    }

    public void addGpsData(File file, Objects objects){
        try {
            Document doc        = uGetDoc(file);
            Node obj            = doc.getElementById(objects.getObjectName());
            NodeList objChilds  = obj.getChildNodes();

            DecimalFormatSymbols separator = new DecimalFormatSymbols(Locale.getDefault());
            separator.setDecimalSeparator('.');
            DecimalFormat sixDigit = new DecimalFormat("#.000000", separator);
            sixDigit.setRoundingMode(RoundingMode.DOWN);

            Double d = Double.parseDouble( objects.getLongitude() );
            objChilds.item(1).setTextContent( sixDigit.format(d) );

            d = Double.parseDouble( objects.getLatitude() );
            objChilds.item(2).setTextContent( sixDigit.format(d) );

            if (objects.getAltitude().contains("m")){
                objChilds.item(3).setTextContent( objects.getAltitude() );
            } else {
                d = Double.parseDouble( objects.getAltitude() );
                objChilds.item(3).setTextContent( d.intValue() + " m" );
            }

            if (objects.getAccuracy().contains("m")){
                objChilds.item(4).setTextContent( objects.getAccuracy() );
            } else {
                objChilds.item(4).setTextContent( "\u00B1" + objects.getAccuracy() + " m");
            }

            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            Transformer transformer = transformerFactory.newTransformer();
            DOMSource source = new DOMSource(doc);

            StreamResult result = new StreamResult(new FileOutputStream(file));
            transformer.transform(source, result);
        } catch (Exception e){
            e.printStackTrace();
        }
    }



    private String sizeFormater(float bytes){
        DecimalFormat formater = new DecimalFormat("#.##");
        final long KILO = 1024;
        final long MEGA = KILO * 1024;
        final long GIGA = MEGA * 1024;
        float result;
        String formated;

        if( bytes >= GIGA) {
            result = bytes / GIGA;
            formated = formater.format(result) + " GB";
        } else if( bytes >= MEGA ) {
            result = bytes / MEGA;
            formated = formater.format(result) + " MB";
        } else if( bytes >= KILO ) {
            result = bytes / KILO;
            formated = formater.format(result) + " KB";
        } else {
            result = bytes;
            formated = formater.format(result) + " bytes";
        }
        return formated;
    }

    private Element setNode(String name, String content, Document doc){
        Element e = doc.createElement(name);
        e.appendChild(doc.createTextNode(content));
        return e;
    }


}
