package com.megabras.bluelogg.tm25r;

import android.app.Activity;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.os.Handler;
import android.util.Log;

import com.megabras.bluelogg.MUtils;
import com.megabras.bluelogg.R;
import com.megabras.bluelogg.extended.ReportProvider;
import com.megabras.modbus.TM25R.Objects;
import com.megabras.modbus.TM25R.TestData;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

public class Report extends ReportProvider {
	private final String TAG = "Report";
	private Locale local = Locale.getDefault();
    private double snapScale  = 0;
    private double snapWidth  = 7.25;
    private double snapHeight = 8.999;
    private double picScale   = 0;
    private double picWidth   = 6.75;
    private double picHeight  = 9.00;

	private TestData
			testData;

	private ArrayList<Objects>
			objList;

	private Database
			database;

	public Report(Activity activity, Handler handler, Database database, TestData testData, ArrayList<Objects> objList){
		super(activity);
		this.activity 	= activity;
		this.handler 	= handler;
		this.testData	= testData;
		this.objList 	= objList;
		this.database   = database;
	}

	private String getFileName(File folder, String site){
		site = MUtils.removeSpecialChars(site);
		File file = new File(folder, site + ".odt");
		return file.toString();
	}

	public void saveReport() {
		try {
			String fileName = getFileName(testData.getFolder(), testData.getFolder().getName());
			File temp = new File(fileName);			

			if(temp.exists()){
				temp.delete();
			}

			handler.obtainMessage(Controller.mHandler.REPORT_UPDATE, 1, -1, text(R.string.saving)).sendToTarget();
			
			//Decompress
			InputStream fin    = activity.getAssets().open("template.odt");
			ZipInputStream zin = new ZipInputStream(fin); 
			ZipEntry ze;
			
			//Compress
			FileOutputStream dest =  new FileOutputStream(fileName);
			ZipOutputStream  out  =  new ZipOutputStream( new BufferedOutputStream(dest) );
			
			while( (ze = zin.getNextEntry()) != null ) {
				if(ze.getName().equals("content.xml")){
					out.putNextEntry(ze);
					InputStream con = getContentXml();

					for (int c = con.read(); c != -1; c = con.read()) {
						out.write(c);
					}
					zin.closeEntry();
				}
				else if(ze.getName().equals("styles.xml")){
					out.putNextEntry(ze);
					InputStream con = getStylesXml();
					for (int c = con.read(); c != -1; c = con.read()) {
						out.write(c);
					}
					zin.closeEntry();
				}
				else if(ze.getName().equals("META-INF/manifest.xml")){
					out.putNextEntry(ze);
					InputStream con = getManifestXml();
					for (int c = con.read(); c != -1; c = con.read()) {
						out.write(c);
					}
					zin.closeEntry();
				}
				else {
					out.putNextEntry(ze); 				
					for (int c = zin.read(); c != -1; c = zin.read()) { 
						out.write(c); 
					}				
					zin.closeEntry();
				}
			}

			/** IMAGES */
			for (Objects o : objList){
				File img = database.getPictureFile(testData.getFolder(), o.getObjectName());
				if(img.exists()){
					handler.obtainMessage(Controller.mHandler.REPORT_UPDATE, 1, -1, text(R.string.inserting_pictures)).sendToTarget();
					ZipEntry zeImg = new ZipEntry("Pictures/"+img.getName());
					out.putNextEntry(zeImg);
					InputStream inImg = new FileInputStream(img);
					byte[] bit = new byte[(int) img.length()];
					inImg.read(bit);
					out.write(bit);
					inImg.close();
				}
			}

			File snap = database.getMapSnapshotFile(testData.getFolder());

			if(snap.exists()){
				handler.obtainMessage(Controller.mHandler.REPORT_UPDATE, 1, -1, text(R.string.inserting_pictures)).sendToTarget();
				ZipEntry zeImg = new ZipEntry("Pictures/"+snap.getName());
				out.putNextEntry(zeImg);
				InputStream inImg = new FileInputStream(snap);
				byte[] bit = new byte[(int) snap.length()];
				inImg.read(bit);
				out.write(bit);
				inImg.close();
			}

			out.close();
			zin.close();

			handler.obtainMessage(Controller.mHandler.REPORT_FINISHED, 1, -1, temp).sendToTarget();
		} catch (Exception e) {
			Log.e("Report",e.getMessage() + "");
			for (StackTraceElement ste : e.getStackTrace()){
				Log.e("Report",ste.toString());
			}
		}
	}

	
	private String text(int id){
		return activity.getResources().getString(id).toUpperCase(local);
	}

	
	private void createNewTables(Document doc){		
		Node body = doc.getElementsByTagName("office:text").item(0);		

		List<String> objects = database.getObjectsList(testData.getFile());

		for (String obj : objects){			
			Element table = doc.createElement("table:table");				
			table.setAttributeNode( setAttribute( doc, "table:name", "table-results" ) );
			table.setAttributeNode( setAttribute( doc, "table:style-name", "table-results" ) );

			Element column;
			column = doc.createElement("table:table-column");
			column.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.A" ) );
			column.setAttributeNode( setAttribute( doc, "table:number-columns-repeated", "3" ) );			
			table.appendChild(column);

			column = doc.createElement("table:table-column");
			column.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.D" ) );
			table.appendChild(column);
		
			Element row;
			Element cell;
			Element line;

			//ROW 1		
			row = doc.createElement("table:table-row");		
			row.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.1" ) );
			
			cell = doc.createElement("table:table-cell");
			cell.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.A1" ) );
			cell.setAttributeNode( setAttribute( doc, "table:number-columns-spanned", "4" ) );
			cell.setAttributeNode( setAttribute( doc, "office:value-type", "string" ) );
						
			line = doc.createElement("text:p");
			line.setAttribute("text:style-name", "P5");
			line.setTextContent( text(R.string.object_name) );			
			cell.appendChild(line);
			row.appendChild(cell);
			
			cell = doc.createElement("table:covered-table-cell");
			row.appendChild(cell);
			cell = doc.createElement("table:covered-table-cell");
			row.appendChild(cell);
			cell = doc.createElement("table:covered-table-cell");
			row.appendChild(cell);
			
			table.appendChild(row);
			
			//ROW 2
			row = doc.createElement("table:table-row");			
			
			cell = doc.createElement("table:table-cell");
			cell.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.A2" ) );
			cell.setAttributeNode( setAttribute( doc, "table:number-columns-spanned", "4" ) );
			cell.setAttributeNode( setAttribute( doc, "office:value-type", "string" ) );
						
			line = doc.createElement("text:p");
			line.setAttribute("text:style-name", "P11");
			line.setTextContent( obj.toUpperCase(Locale.getDefault()) );			
			cell.appendChild(line);
			row.appendChild(cell);
			
			cell = doc.createElement("table:covered-table-cell");
			row.appendChild(cell);
			cell = doc.createElement("table:covered-table-cell");
			row.appendChild(cell);
			cell = doc.createElement("table:covered-table-cell");
			row.appendChild(cell);
			
			table.appendChild(row);
			
			//ROW 3
			row = doc.createElement("table:table-row");	
			row.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.1" ) );			
			cell = doc.createElement("table:table-cell");
			cell.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.A3" ) );
			cell.setAttributeNode( setAttribute( doc, "office:value-type", "string" ) );						
			line = doc.createElement("text:p");
			line.setAttribute("text:style-name", "P5");
			line.setTextContent( text(R.string.latitude) );			
			cell.appendChild(line);			
			row.appendChild(cell);
			
			cell = doc.createElement("table:table-cell");
			cell.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.A3" ) );
			cell.setAttributeNode( setAttribute( doc, "office:value-type", "string" ) );						
			line = doc.createElement("text:p");
			line.setAttribute("text:style-name", "P5");
			line.setTextContent( text(R.string.longitude) );			
			cell.appendChild(line);			
			row.appendChild(cell);
			
			cell = doc.createElement("table:table-cell");
			cell.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.A3" ) );
			cell.setAttributeNode( setAttribute( doc, "office:value-type", "string" ) );						
			line = doc.createElement("text:p");
			line.setAttribute("text:style-name", "P5");
			line.setTextContent( text(R.string.altitude) );			
			cell.appendChild(line);			
			row.appendChild(cell);
			
			cell = doc.createElement("table:table-cell");
			cell.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.A3" ) );
			cell.setAttributeNode( setAttribute( doc, "office:value-type", "string" ) );						
			line = doc.createElement("text:p");
			line.setAttribute("text:style-name", "P5");
			line.setTextContent( text(R.string.accuracy) );			
			cell.appendChild(line);			
			row.appendChild(cell);
			
			table.appendChild(row);

			//ROW 4
			String[] gps = database.getGpsData(testData.getFile(), obj);

			row = doc.createElement("table:table-row");	
			row.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.4" ) );			
			cell = doc.createElement("table:table-cell");
			cell.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.A4" ) );
			cell.setAttributeNode( setAttribute( doc, "office:value-type", "string" ) );						
			line = doc.createElement("text:p");
			line.setAttribute("text:style-name", "P12");
			line.setTextContent( gps[1] );			
			cell.appendChild(line);			
			row.appendChild(cell);
			
			cell = doc.createElement("table:table-cell");
			cell.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.A4" ) );
			cell.setAttributeNode( setAttribute( doc, "office:value-type", "string" ) );						
			line = doc.createElement("text:p");
			line.setAttribute("text:style-name", "P12");
			line.setTextContent( gps[0] );			
			cell.appendChild(line);			
			row.appendChild(cell);
			
			cell = doc.createElement("table:table-cell");
			cell.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.A4" ) );
			cell.setAttributeNode( setAttribute( doc, "office:value-type", "string" ) );						
			line = doc.createElement("text:p");
			line.setAttribute("text:style-name", "P12");
			line.setTextContent( gps[2] );			
			cell.appendChild(line);			
			row.appendChild(cell);
			
			cell = doc.createElement("table:table-cell");
			cell.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.A4" ) );
			cell.setAttributeNode( setAttribute( doc, "office:value-type", "string" ) );						
			line = doc.createElement("text:p");
			line.setAttribute("text:style-name", "P12");
			line.setTextContent( gps[3] );			
			cell.appendChild(line);			
			row.appendChild(cell);
			
			table.appendChild(row);
			
			//ROW 5			
			row = doc.createElement("table:table-row");			
			row.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.1" ) );
			
			cell = doc.createElement("table:table-cell");
			cell.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.A3" ) );
			cell.setAttributeNode( setAttribute( doc, "table:number-columns-spanned", "4" ) );
			cell.setAttributeNode( setAttribute( doc, "office:value-type", "string" ) );
						
			line = doc.createElement("text:p");
			line.setAttribute("text:style-name", "P5");
			line.setTextContent( text(R.string.results) );			
			cell.appendChild(line);
			row.appendChild(cell);
			
			cell = doc.createElement("table:covered-table-cell");
			row.appendChild(cell);
			cell = doc.createElement("table:covered-table-cell");
			row.appendChild(cell);
			cell = doc.createElement("table:covered-table-cell");
			row.appendChild(cell);
			
			table.appendChild(row);
			
			//ROW 6
			row = doc.createElement("table:table-row");
			row.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.6" ) );

			cell = doc.createElement("table:table-cell");
			cell.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.A6" ) );
			cell.setAttributeNode( setAttribute( doc, "table:number-columns-spanned", "4" ) );
			cell.setAttributeNode( setAttribute( doc, "office:value-type", "string" ) );

			//HERE
			ReportTable tabla;
			Objects mObj = database.getObject(testData, obj);
			tabla = new ReportTable(activity, doc);
			tabla.createTable("table-result");
			tabla.setColumns(6);
			tabla.setRowsNew(mObj.getMeasurements());

			cell.appendChild(tabla.getTable());
			row.appendChild(cell);

			cell = doc.createElement("table:covered-table-cell");
			row.appendChild(cell);
			cell = doc.createElement("table:covered-table-cell");
			row.appendChild(cell);
			cell = doc.createElement("table:covered-table-cell");
			row.appendChild(cell);

			table.appendChild(row);

			File img = database.getPictureFile(testData.getFolder(), obj);

			if (img.exists()){
				//ROW 7			
				row = doc.createElement("table:table-row");			
				row.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.1" ) );			
				cell = doc.createElement("table:table-cell");
				cell.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.A3" ) );
				cell.setAttributeNode( setAttribute( doc, "table:number-columns-spanned", "4" ) );
				cell.setAttributeNode( setAttribute( doc, "office:value-type", "string" ) );						
				line = doc.createElement("text:p");
				line.setAttribute("text:style-name", "P5");
				line.setTextContent( text(R.string.picture) );			
				cell.appendChild(line);
				row.appendChild(cell);			
				cell = doc.createElement("table:covered-table-cell");
				row.appendChild(cell);
				cell = doc.createElement("table:covered-table-cell");
				row.appendChild(cell);
				cell = doc.createElement("table:covered-table-cell");
				row.appendChild(cell);			
				table.appendChild(row);
				
				if (picScale <= 0){
					BitmapFactory.Options options = new BitmapFactory.Options();
					options.inPreferredConfig = Bitmap.Config.ARGB_8888;
					Bitmap bitmap = BitmapFactory.decodeFile(img.getAbsolutePath(), options);
					picScale = (double) bitmap.getWidth() / (double) bitmap.getHeight();					
					if (picScale < 1){
						picScale = 1 / picScale;
					}					
					picWidth = picHeight / picScale;
				}
				
				//ROW 8
				row = doc.createElement("table:table-row");
				row.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.5" ) );	
				
				cell = doc.createElement("table:table-cell");
				cell.setAttributeNode( setAttribute( doc, "table:style-name", "table-results.A5" ) );
				cell.setAttributeNode( setAttribute( doc, "table:number-columns-spanned", "4" ) );
				cell.setAttributeNode( setAttribute( doc, "office:value-type", "string" ) );				
				line = doc.createElement("text:p");
				line.setAttribute("text:style-name", "P10");
				Element drawFrame = doc.createElement("draw:frame");				
				drawFrame.setAttribute("draw:name", "figura1");
				drawFrame.setAttribute("text:anchor-type", "as-char");
				drawFrame.setAttribute("svg:width", picWidth+"cm");
				drawFrame.setAttribute("svg:height", picHeight+"cm");
				
				Element drawImage = doc.createElement("draw:image");
				drawImage.setAttribute("xlink:href", "Pictures/"+img.getName());
				drawImage.setAttribute("xlink:type", "simple");
				drawImage.setAttribute("xlink:show", "embed");
				drawImage.setAttribute("xlink:actuate", "onLoad");				
				drawFrame.appendChild(drawImage);
				line.appendChild(drawFrame);	
				cell.appendChild(line);
				row.appendChild(cell);
				
				cell = doc.createElement("table:covered-table-cell");
				row.appendChild(cell);
				cell = doc.createElement("table:covered-table-cell");
				row.appendChild(cell);
				cell = doc.createElement("table:covered-table-cell");
				row.appendChild(cell);
				table.appendChild(row);
			}
			
			body.appendChild(table);
		}
	}

	private Attr setAttribute(Document doc, String attributeName, String attributeValue){
		Attr attr = doc.createAttribute(attributeName);
		attr.setValue(attributeValue);
		return attr;
	}

	public InputStream getContentXml(){		
		try{
			DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();			
			DocumentBuilder	documentBuilder = documentBuilderFactory.newDocumentBuilder();			 
			Document doc = documentBuilder.parse( activity.getAssets().open("content.xml") );

			handler.obtainMessage(Controller.mHandler.REPORT_UPDATE, 1, -1, text(R.string.creating_tables)).sendToTarget();

			createNewTables(doc);

			doc.getElementById("L-GENERAL-INFO").setTextContent( text(R.string.general_information) );

			SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm", Locale.US);
			doc.getElementById("L-TEST-DATE").setTextContent( text(R.string.date).toUpperCase() );
			doc.getElementById("F-TEST-DATE").setTextContent( dateFormat.format(new Date()) );
			doc.getElementById("L-EQUIPMENT").setTextContent( text(R.string.equipment) );

			doc.getElementById("F-EQUIPMENT").setTextContent( testData.getModel() + " - " + testData.getSerial() );

			doc.getElementById("L-OPERATOR").setTextContent( text(R.string.operator) );
			doc.getElementById("F-OPERATOR").setTextContent( testData.getOperatorName() + " / " + testData.getOperatorExtension() + " / " + testData.getOperatorEmail() );
			doc.getElementById("L-COMMENTS").setTextContent( text(R.string.comments) );
			doc.getElementById("L-COMP").setTextContent( text(R.string.firm) );
			doc.getElementById("L-MAP").setTextContent( text(R.string.map) );

			String[] company = database.getCompany();

			if (company!=null){
				doc.getElementById("F-COMP-NAME").setTextContent( company[0] );
				doc.getElementById("F-COMP-ADDRESS").setTextContent( company[1] );
				doc.getElementById("F-COMP-ADDRESS-2").setTextContent( company[2] + " - " + company[3] + " - " + company[4] );
				doc.getElementById("F-COMP-PHONE").setTextContent( text(R.string.phone) + " " + company[5] + " / " + text(R.string.fax) + " " + company[6] );
			}

			doc.getElementById("L-SITE").setTextContent( text(R.string.test_site) );

			doc.getElementById("F-SITE-NAME").setTextContent( testData.getSiteName() );
			doc.getElementById("F-SITE-CONTACT").setTextContent( testData.getSiteContact() );
			doc.getElementById("F-SITE-ADDRESS").setTextContent( testData.getSiteAddress() );
			doc.getElementById("F-SITE-ADDRESS-2").setTextContent( testData.getSiteState() + " - " + testData.getSiteCity() + " - " + testData.getSiteZipCode() );
			doc.getElementById("F-SITE-PHONE").setTextContent( text(R.string.phone) + " " + testData.getSitePhone() + " / " + text(R.string.fax) + " " + testData.getSiteFax() );

			Node com = doc.getElementById("F-COMMENTS").getParentNode();

			com.removeChild( doc.getElementById("F-COMMENTS") );

			String[] comments = database.getComments(testData.getFile()).split("\\n");

			for (String li : comments){
				Element line = doc.createElement("text:p");
				line.setAttribute("text:style-name", "P8");
				line.setTextContent(li);
				com.appendChild(line);
			}

			File snap = database.getMapSnapshotFile(testData.getFolder());

			if (snap.exists()){
				if (snapScale <= 0){
					BitmapFactory.Options options = new BitmapFactory.Options();
					options.inPreferredConfig = Bitmap.Config.ARGB_8888;
					Bitmap bitmap = BitmapFactory.decodeFile(snap.getAbsolutePath(), options);
					snapScale = (double) bitmap.getWidth() / (double) bitmap.getHeight();
					if (snapScale < 1){
						snapScale = 1 / snapScale;
					}
					snapWidth = snapHeight / snapScale;
				}
				Element drawFrame = doc.createElement("draw:frame");
				drawFrame.setAttribute("draw:name", "figura1");
				drawFrame.setAttribute("text:anchor-type", "as-char");
				drawFrame.setAttribute("svg:width", snapWidth+"cm");
				drawFrame.setAttribute("svg:height", snapHeight+"cm");

				Element drawImage = doc.createElement("draw:image");
				drawImage.setAttribute("xlink:href", "Pictures/"+snap.getName());
				drawImage.setAttribute("xlink:type", "simple");
				drawImage.setAttribute("xlink:show", "embed");
				drawImage.setAttribute("xlink:actuate", "onLoad");
				drawFrame.appendChild(drawImage);
				doc.getElementById("snapshot").appendChild(drawFrame);
			} else {
				Element mainTable = doc.getElementById("tablegeneral");
				Node map1 = doc.getElementById("map-1");
				Node map2 = doc.getElementById("map-2");
				mainTable.removeChild(map1);
				mainTable.removeChild(map2);
			}
			
			ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
			Source xmlSource = new DOMSource(doc);
			Result outputTarget = new StreamResult(outputStream);
			TransformerFactory.newInstance().newTransformer().transform(xmlSource, outputTarget);

			return new ByteArrayInputStream(outputStream.toByteArray());
		}
		catch (Exception e) {
			Log.e(TAG, e.getMessage());
			for (StackTraceElement ste : e.getStackTrace())
				Log.e(TAG, ste.toString());
			return null;
		}		
	}

	public InputStream getStylesXml(){		
		try{			
			DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();			
			DocumentBuilder	documentBuilder = documentBuilderFactory.newDocumentBuilder();			 
			Document doc = documentBuilder.parse( activity.getAssets().open("styles.xml") );

			doc.getElementById("L-TITLE").setTextContent( text(R.string.ground_tester_report) );
			
			ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
			Source xmlSource = new DOMSource(doc);
			Result outputTarget = new StreamResult(outputStream);
			TransformerFactory.newInstance().newTransformer().transform(xmlSource, outputTarget);
			return new ByteArrayInputStream(outputStream.toByteArray());
		} catch (Exception e) {
			return null;
		}		
	}

	public InputStream getManifestXml(){		
		try{			
			DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();			
			DocumentBuilder	documentBuilder = documentBuilderFactory.newDocumentBuilder();			 
			Document doc = documentBuilder.parse( activity.getAssets().open("manifest.xml") );
			Element manifest = (Element) doc.getElementsByTagName("manifest:manifest").item(0);	
			
			List<String> objects = database.getObjectsList(testData.getFile());

			for (String obj : objects){
				File img = database.getPictureFile(testData.getFolder(), obj);
				if(img.exists()){
					Element el = doc.createElement("manifest:file-entry");
					el.setAttribute("manifest:media-type", "image/jpeg");
					el.setAttribute("manifest:full-path", "Pictures/"+img.getName());					
					manifest.appendChild(el);					
				}
			}
			
			File snap = database.getMapSnapshotFile(testData.getFolder());

			if (snap.exists()){
				Element el = doc.createElement("manifest:file-entry");
				el.setAttribute("manifest:media-type", "image/jpeg");
				el.setAttribute("manifest:full-path", "Pictures/"+snap.getName());
				manifest.appendChild(el);
			}
			
			ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
			Source xmlSource = new DOMSource(doc);
			Result outputTarget = new StreamResult(outputStream);
			TransformerFactory.newInstance().newTransformer().transform(xmlSource, outputTarget);
			return new ByteArrayInputStream(outputStream.toByteArray());
		} catch (Exception e) {
			return null;
		}		
	}
}
