package com.megabras.bluelogg.DT70KV;

import android.annotation.SuppressLint;
import android.app.DatePickerDialog;
import android.app.Dialog;
import android.app.TimePickerDialog;
import android.content.Context;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Handler;
import android.os.Message;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.view.inputmethod.InputMethodManager;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.DatePicker;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.TimePicker;

import com.megabras.bluelogg.extended.CommonUI;
import com.megabras.bluelogg.MUtils;
import com.megabras.bluelogg.Main;
import com.megabras.bluelogg.R;
import com.megabras.bluelogg.bluetooth.BluetoothClassic;
import com.megabras.bluelogg.extended.Audio;
import com.megabras.bluelogg.extended.GPS;
import com.megabras.bluelogg.extended.GoogleMaps;
import com.megabras.bluelogg.extended.LogCat;
import com.megabras.bluelogg.extended.MapData;
import com.megabras.bluelogg.extended.Picture;
import com.megabras.bluelogg.extended.SlidingTabLayout;
import com.megabras.modbus.DT70KV.Device;
import com.megabras.modbus.DT70KV.Device.Changed;
import com.megabras.modbus.DT70KV.Measurement;
import com.megabras.modbus.DT70KV.Protocol;
import com.megabras.modbus.DT70KV.TestData;
import com.megabras.modbus.Modbus;

import java.io.File;
import java.lang.ref.WeakReference;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import androidx.fragment.app.FragmentActivity;
import androidx.viewpager.widget.ViewPager;

public class Controller extends com.megabras.bluelogg.Controller implements GPS.GpsInterface, Observer, Picture.PictureInterface
{
    /***********************************************************************************************
     * REGION Fields
     ***********************************************************************************************/

    protected String
            TAG = "DT70KV.Controller";

    public static String
            model,
            serial;

    protected FragmentActivity
            activity;

    private Menu
            ui_menu;

    private MenuItem
            ui_btn_device_config;

    private final Helper
            helper;

    private ViewPager
            viewPager;

    private PagerAdapter
            pagerAdapter;

    private View
            pageTestSetup,
            pageGeneral,
            pageFilesManager;

    private int
            controlButtonsID    = R.layout.dt70kv_control_buttons,
            optionsID,
            timeFormat          = 0,
            selectedIndex       = -1;

    @SuppressWarnings("WeakerAccess")
    protected String
            DEVICE_CURRENT_UNIT = Protocol.DEVICE_CURRENT_UNIT;

    @SuppressWarnings("WeakerAccess")
    protected int
            DEVICE_MIN_AMPS     = Protocol.DEVICE_MIN_AMPS,
            DEVICE_STEP_AMPS    = Protocol.DEVICE_STEP_AMPS,
            DEVICE_MAX_AMP      = Protocol.DEVICE_MAX_AMPS;

    private LinearLayout
            ui_toolbar;

    private Protocol
            modbus = new Protocol();

    private Button
            ui_btn_start,
            ui_btn_stop;

    private ImageButton
            ui_btn_setup,
            ui_btn_save,
            ui_btn_manage,
            ui_btn_gps,
            ui_btn_picture,
            ui_btn_voice,
            ui_btn_map;

    private TextView
            ui_txw_free_memory;

    private CommonUI
            mCommonUI;

    private TestData
            testData;

    private Picture
            picture;

    private GPS
            mGPS;

    private Audio
            audio;

    private GoogleMaps
            map;

    private mHandler
            handler;

    private Dialog
            dlgProgress,
            dialogFinished;

    protected Main
            parent;

    private Device
            device = new Device();

    private final String[]
            GLOVE_CLASSES;

    private Database
            database;

    protected com.megabras.modbus.DT70KV.FormatData
            formatData;

    private ReportPDF
            report;

    private FrameLayout
            overlay;

    /***********************************************************************************************
     * REGION Constructor
     ***********************************************************************************************/

    @SuppressLint("CommitPrefEdits")
    public Controller(FragmentActivity fragmentActivity, CommonUI ui, String model, String serial)
    {
        helper              = new Helper( fragmentActivity );
        activity            = fragmentActivity;
        parent              = (Main)activity;
        pageGeneral         = View.inflate( activity, R.layout.hp60kv_fgr_general_setup, null );
        ui_txw_free_memory  = pageGeneral.findViewById( R.id.device_memory );
        pageTestSetup       = View.inflate( activity, R.layout.dt70kv_fgr_test_setup   , null );
        Controller.model    = model;
        Controller.serial   = serial.trim();
        handler             = new mHandler(this);
        mCommonUI           = ui;
        picture             = Picture.getInstance();
        picture             .initialize( fragmentActivity, this );
        mGPS                = new GPS(this, fragmentActivity);
        audio               = new Audio(fragmentActivity);
        map                 = new GoogleMaps(fragmentActivity);
        device              .addObserver(this);
        String DB_NAME      = model + "_" + serial + "_REGISTERS.db";
        database            = new Database(activity, Controller.model, Controller.serial, DB_NAME);
        report              = new ReportPDF(activity, database.getFolder(), database.getTemporaryFolder(), handler);
        overlay             = MUtils.getOverlay(activity, OVERLAY_ID);
        GLOVE_CLASSES       = activity.getResources().getStringArray( R.array.glove_class );

        formatData          = new com.megabras.modbus.DT70KV.FormatData();

        LogCat.LEVEL = LogCat.LEVEL_NONE;
    }

    /***********************************************************************************************
     * REGION Implements Controller
     ***********************************************************************************************/

    public void createSwipe()
    {
        pagerAdapter = new PagerAdapter( activity.getSupportFragmentManager(), activity.getBaseContext() );
        viewPager    = activity.findViewById( R.id.pager );

        if ( viewPager == null )
            return;

        viewPager.setAdapter( pagerAdapter );
        viewPager.setOffscreenPageLimit( pagerAdapter.getCount() );

        //Create customs Tabs
        SlidingTabLayout tabLayout = activity.findViewById(R.id.sliding_tabs);
        tabLayout.setDistributeEvenly(true);
        tabLayout.setCustomTabView(R.layout.custom_tab, 0);
        tabLayout.setBackground(activity.getResources().getDrawable(R.drawable.chamfer_black_top));
        tabLayout.setCustomTabColorizer(new SlidingTabLayout.TabColorizer()
        {
            public int getIndicatorColor(int position) {
                return Color.GRAY;
            }

            public int getDividerColor(int position) {
                return Color.WHITE;
            }
        });
        tabLayout.setViewPager(viewPager);
    }

    public void createControlButtons(LinearLayout container)
    {
        final View commandButtons = View.inflate(activity, controlButtonsID, null);

        commandButtons.setLayoutParams(
                new LinearLayout.LayoutParams(
                        LinearLayout.LayoutParams.MATCH_PARENT,
                        (int)activity.getResources().getDimension(R.dimen.md15kvr_controls_height)));
        container.removeAllViews();
        container.addView( commandButtons );

        ui_btn_start = commandButtons.findViewById( R.id.bt_start );
        ui_btn_start.setOnClickListener(v ->
                device.buttonStart()
        );

        ui_btn_stop = commandButtons.findViewById( R.id.bt_stop );
        ui_btn_stop.setOnClickListener(v ->
            device.buttonStop()
        );

        ui_btn_setup = commandButtons.findViewById( R.id.bt_setup );
        ui_btn_setup.setOnClickListener(v ->
                device.buttonSetup()
        );
    }

    private void dialogReplaceData()
    {
        Dialog dialog = helper.getDialog( R.layout.dialog_yes_no_white, R.drawable.ic_warning, R.string.warning );
        ((TextView) dialog.findViewById(R.id.message)).setText( activity.getString(R.string.glove_data_already, testData.getGloveSerial()) );

        final Button ui_btn_dlg_no = dialog.findViewById(R.id.btn_no);
        ui_btn_dlg_no.setOnClickListener(v ->
                dialog.dismiss()
        );

        final Button ui_btn_dlg_yes = dialog.findViewById(R.id.btn_yes);
        ui_btn_dlg_yes.setOnClickListener(v ->
        {
            long id = testData.getID();
            String gloveSerial = testData.getGloveSerial();

            database.deleteTable(serial, id);

            testData = new TestData(this, model, serial);
            testData.setGloveSerial( gloveSerial );
            testData.isIncomplete(true);

            database.addNewTable( testData );

            dialogStartTest();
            dialog.dismiss();
        });

        dialog.show();
    }

    private void dialogStartTest()
    {
        Dialog dialog = helper.getDialog( R.layout.dialog_yes_no_white, R.drawable.ic_warning, R.string.md_warning_test_start );
        ((TextView) dialog.findViewById(R.id.message)).setText( activity.getResources().getString(R.string.hipot_is_grounded).toUpperCase() );

        final Button ui_btn_dlg_no = dialog.findViewById(R.id.btn_no);
        ui_btn_dlg_no.setOnClickListener(v ->
                dialog.dismiss()
        );

        final Button ui_btn_dlg_yes = dialog.findViewById(R.id.btn_yes);
        ui_btn_dlg_yes.setOnClickListener(v ->
        {
            device.requestStart();
            dialog.dismiss();
        });

        dialog.show();
    }

    private void dialogGloveSerial(boolean isNew)
    {
        Dialog dlg = helper.getDialog( R.layout.dialog_input_white, R.drawable.ic_new, R.string.serial );
        ((TextView) dlg.findViewById(R.id.ui_txw_message)).setText( R.string.glove_serial );

        final EditText ui_edt_serial = dlg.findViewById( R.id.ui_edt_field );
        ui_edt_serial.setOnFocusChangeListener((edt, hasFocus) ->
        {
            try
            {
                InputMethodManager imm = (InputMethodManager) activity.getSystemService( Context.INPUT_METHOD_SERVICE );
                if ( edt instanceof EditText && hasFocus )
                {
                    imm.toggleSoftInput( InputMethodManager.SHOW_FORCED, InputMethodManager.HIDE_IMPLICIT_ONLY );
                }
                else
                {
                    imm.hideSoftInputFromWindow(ui_edt_serial.getWindowToken(), 0);
                }
            }
            catch (Exception ignored)
            {
            }
        });
        ui_edt_serial.requestFocus();

        View.OnClickListener onClose = view ->
        {
            ui_edt_serial.clearFocus();
            dlg.dismiss();
        };

        Button ui_btn_dlg_no = dlg.findViewById( R.id.ui_btn_no );
        ui_btn_dlg_no.setText( activity.getString(R.string.cancel).toUpperCase() );
        ui_btn_dlg_no.setOnClickListener( onClose );
        dlg.findViewById( R.id.btn_close_dlg ).setOnClickListener( onClose );

        Button ui_btn_dlg_yes = dlg.findViewById( R.id.ui_btn_yes );
        ui_btn_dlg_yes.setText( activity.getString(R.string.save).toUpperCase() );
        ui_btn_dlg_yes.findViewById( R.id.ui_btn_yes ).setOnClickListener(v1 ->
        {
            String gloveSerial = ui_edt_serial.getText().toString();
            if ( serial.length() > 0 )
            {
                if ( isNew )
                {
                    testData = new TestData(this, model, serial);
                    testData.setGloveSerial(gloveSerial);
                    testData.isIncomplete(true);
                    database.addNewTable(testData);
                    ui_edt_serial.clearFocus();
                    dlg.dismiss();
                    helper.closeOverlay(overlay);
                    pageFiles();
                }
                else
                {
                    testData.setGloveSerial(gloveSerial);
                    database.save(testData);
                    testData.setFileSaveOrOpened();
                    ui_edt_serial.clearFocus();
                    dlg.dismiss();
                }
            }
        });

        dlg.show();
    }

    public void setToolBarId(int toolBarId)
    {
        ui_toolbar     = activity.findViewById( toolBarId );
        ui_toolbar     .findViewById( R.id.bt_export  ).setVisibility(View.GONE);
        ui_toolbar     .findViewById( R.id.bt_details ).setVisibility(View.GONE);
        ui_btn_map     = ui_toolbar.findViewById( R.id.btn_map    );
        ui_btn_save    = ui_toolbar.findViewById( R.id.bt_save    );
        ui_btn_manage  = ui_toolbar.findViewById( R.id.bt_manager );
        ui_btn_gps     = ui_toolbar.findViewById( R.id.bt_gps     );
        ui_btn_picture = ui_toolbar.findViewById( R.id.bt_picture );
        ui_btn_voice   = ui_toolbar.findViewById( R.id.bt_record  );

        ui_btn_save.setEnabled( false );
        ui_btn_voice.setEnabled( false );
        ui_btn_picture.setEnabled( false );
        ui_btn_gps.setEnabled( false );
        ui_btn_map.setEnabled( false );

        ui_btn_save.setOnClickListener(v ->
        {
            if ( testData != null )
            {
                if ( testData.getGloveSerial().length() > 0 )
                {
                    database.save(testData); // SAVE AND SET NOT INCOMPLETE
                    testData.setFileSaveOrOpened();
                }
                else
                {
                    dialogGloveSerial( false );
                }
            }
        });

        ui_btn_manage.setOnClickListener(v -> pageFiles());

        ui_btn_picture.findViewById(R.id.bt_picture).setOnClickListener(v ->
        {
            if (Main.permissionToPictureAccepted)
                takePicture();
            else
                ((Main)activity).requestPermission(Main.REQUEST_PICTURE_PERMISSION);
        });

        ui_btn_gps.setOnClickListener(v ->
        {
            if (Main.permissionToGPSAccepted)
                getGPS();
            else
                ((Main)activity).requestPermission(Main.REQUEST_GPS_PERMISSION);
        });

        ui_btn_voice.setOnClickListener(v ->
        {
            if (Main.permissionToRecordAccepted)
                recordAudio();
            else
                ((Main)activity).requestPermission(Main.REQUEST_RECORD_AUDIO_PERMISSION);
        });

        ui_btn_map.setOnClickListener(v ->
        {
            if(Main.permissionToMemoryAccepted)
                map();
            else
                ((Main)activity).requestPermission(Main.REQUEST_MEMORY_MAP_PERMISSION);
        });

        ui_btn_manage.setEnabled(true);

        device.setState( Changed.DISCONNECTED ); // SETS THE INITIAL STATE HERE TO ENSURE THAT ALL VIEWS ARE INFLATED
        helper.closeOverlay(overlay);
    }

    private void map()
    {
        if ( testData == null || testData.getGpsAcuraccy() == 0 )
            return;

        database.createFolder(activity, model);

        MapData mapData = new MapData
        (
                testData.getTestNumber()+"",
                testData.getGpsLongitude(),
                testData.getGpsLatitude(),
                testData.getGpsAltitude()+"",
                testData.getGpsAcuraccy()+""
        );
        map.showMap( database.getMapSnapshotFile(testData.getSerial(),testData.getID()+""), mapData );
    }

    public void onClose()
    {
        if ( mGPS != null    ) mGPS.cancel();
        if ( picture != null ) picture.close();
    }

    public void setDateTimeFormat(int dateFormat, int timeFormat)
    {
        device.setDateFormat(dateFormat);
        device.setTimeFormat(timeFormat);
    }

    public void setAutoConn(boolean enable)
    {
        device.setAutoConnection(enable);
    }

    public boolean getAutoConn()
    {
        return device.getAutoConnection();
    }

    public void sendMessage(com.megabras.modbus.Message message)
    {
        parent.addMessage(message);
    }

    public com.megabras.modbus.Message statusMessage()
    {
        return modbus.read.status(2);
    }

    public com.megabras.modbus.Message stopMessage()
    {
        return modbus.write.stop();
    }

    public void readGeneralConfig()
    {
        userMessage( R.string.read_general_params, CommonUI.COLOR_YELLOW, true );

        device.setState( Changed.BUSY );

        if ( ui_btn_device_config == null )
            ui_btn_device_config = ui_menu.findItem(optionsID);

        sendMessage( modbus.read.date() );
        sendMessage( modbus.read.memoryPointer() );
        sendMessage( modbus.read.setup() );
    }

    public void setMenuItemOptions(Menu menu, int optionsID)
    {
        this.ui_menu   = menu;
        this.optionsID = optionsID;

        if ( ui_btn_device_config == null )
            ui_btn_device_config = menu.findItem(optionsID);

        if ( pagerAdapter == null )
            return;
        pagerAdapter.resetMainScreen();
        pagerAdapter.resetCharts();
    }

    private void userMessage(int stringID, int color, boolean busy)
    {
        String message = activity.getString(stringID);
        parent.message(message, color, busy);
    }

    /***********************************************************************************************
     * REGION Handler
     ***********************************************************************************************/

    public static class mHandler extends Handler
    {
        private final WeakReference<Controller>
                mClass;

        static final int
                REPORT_UPDATE   = 0,
                REPORT_FINISHED = 1,
                REPORT_ERROR    = 2;

        public mHandler(Controller c)
        {
            mClass = new WeakReference<>(c);
        }

        @Override
        public void handleMessage(Message msg)
        {
            final Controller c = mClass.get();

            switch( msg.what )
            {
                case mHandler.REPORT_UPDATE:
                    c.updateProgressBar( (String) msg.obj );
                    break;
                case mHandler.REPORT_FINISHED:
                    c.dlgProgress.dismiss();
                    c.pagerAdapter.resetCharts();
                    if ( c.testData!=null )
                    {
                        c.pagerAdapter.fillChartsAndTable( c.testData, c.device );
                    }
                    break;
                case mHandler.REPORT_ERROR:
                    c.dlgProgress.dismiss();
                    c.pagerAdapter.resetCharts();
                    if ( c.testData!=null )
                    {
                        c.pagerAdapter.fillChartsAndTable( c.testData, c.device );
                    }
                    break;
            }
        }
    }

    /***********************************************************************************************
     * REGION Picture
     ***********************************************************************************************/

    public void takePicture()
    {
        if( database!=null && testData!=null )
        {
            database.createFolder(activity, model);
            File file = database.getPictureFile( testData.getSerial(), testData.getID()+"" );
            picture.showDialog( file );
        }
    }

    public void pictureStatus(final int status)
    {
        activity.runOnUiThread(() ->
        {
            switch ( status )
            {
                case Picture.WAIT:
                    mCommonUI.changeButtonImage(R.id.bt_picture, R.drawable.ic_wait, false);
                    break;
                case Picture.FINISHED:
                    mCommonUI.changeButtonImage(R.id.bt_picture, R.drawable.ic_picture_states, true);
                    break;
            }
        });
    }

    /***********************************************************************************************
     * REGION Permission
     ***********************************************************************************************/

    public void permissionCallback(int permission)
    {
        switch (permission)
        {
            case Main.REQUEST_RECORD_AUDIO_PERMISSION:
                recordAudio();
                break;
            case Main.REQUEST_PICTURE_PERMISSION:
                takePicture();
                break;
            case Main.REQUEST_GPS_PERMISSION:
                getGPS();
                break;
            case Main.REQUEST_MEMORY_REPORT_PERMISSION:
                export(selectedIndex);
                break;
            case Main.REQUEST_MEMORY_MAP_PERMISSION:
                map();
                break;
        }
    }

    /***********************************************************************************************
     * REGION GPS
     ***********************************************************************************************/

    private void getGPS()
    {
        if ( testData == null )
            return;

        if ( testData.getGpsAcuraccy() == 0 )
            mGPS.getDataGPS();
        else
        {
            String[] gps = new String[]{
                    testData.getGpsLongitude()+"",
                    testData.getGpsLatitude()+"",
                    testData.getGpsAltitude()+"",
                    testData.getGpsAcuraccy()+""
            };
            mGPS.dialogConfirmReplace( gps );
        }
    }

    public void GpsData(int which, String[] data)
    {
        switch ( which )
        {
            case GPS.RESULT:
                if ( data!=null && testData!=null)
                {
                    testData.setGpsLongitude( Double.parseDouble(data[0]) );
                    testData.setGpsLatitude ( Double.parseDouble(data[1]) );
                    testData.setGpsAltitude ( Double.parseDouble(data[2]) );
                    testData.setGpsAcuraccy ( Double.parseDouble(data[3]) );
                    database.updateGpsData  ( testData );
                }
                mCommonUI.changeButtonImage(ui_btn_gps, R.drawable.ic_gps_states, true);
                break;
            case GPS.WAIT:
                mCommonUI.changeButtonImage(ui_btn_gps, R.drawable.ic_wait, false);
                break;
            case GPS.CANCELLED:
                mCommonUI.changeButtonImage(ui_btn_gps, R.drawable.ic_gps_states, true);
                break;
        }
    }

    /***********************************************************************************************
     * REGION AUDIO
     ***********************************************************************************************/

    private void recordAudio()
    {
        if ( testData!=null )
        {
            database.createFolder(activity, model);
            File location = database.getAudioFile(testData.getSerial(), testData.getID()+"" );
            audio.setFile( location );
            audio.showDialog(testData.getTestNumber()+"");
        }
    }

    /***********************************************************************************************
     * REGION Report
     ***********************************************************************************************/

    private void creatDialogProgress()
    {
        dlgProgress = new Dialog(activity);
        dlgProgress.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dlgProgress.setCanceledOnTouchOutside(false);
        dlgProgress.setContentView(R.layout.progress_bar);
        ((TextView) dlgProgress.findViewById(R.id.dlg_title)).setText(R.string.please_wait);
        ((TextView) dlgProgress.findViewById(R.id.message)).setText(R.string.saving_data_in_pdf);
        dlgProgress.findViewById(R.id.dlg_icon).setVisibility(View.GONE);
    }

    private void updateProgressBar(String progress)
    {
        ((TextView)dlgProgress.findViewById(R.id.status)).setText(progress);
    }

    /***********************************************************************************************
     * REGION CALLBACKs
     ***********************************************************************************************/

    private TimePickerDialog.OnTimeSetListener timeListener = new TimePickerDialog.OnTimeSetListener()
    {
        @Override
        public void onTimeSet(TimePicker view, int hourOfDay, int minute)
        {
            if(pageGeneral!=null)
            {
                TextView time = pageGeneral.findViewById(R.id.device_time);
                Calendar c = Calendar.getInstance();
                c.setTime(device.getDate());
                c.set(Calendar.HOUR_OF_DAY, hourOfDay);
                c.set(Calendar.MINUTE, minute);
                c.set(Calendar.SECOND, 0);
                device.setDate( c.getTime() );
                time.setText(  formatData.time( device.getTimeFormat(), device.getDate() ) );
            }
        }
    };

    private void pickTime(int format)
    {
        Calendar calendar = new GregorianCalendar();
        calendar.setTime(device.getDate());
        int hour     = calendar.get(Calendar.HOUR_OF_DAY);
        int minute   = calendar.get(Calendar.MINUTE);
        boolean is24 = format==Modbus.TIME_24H;
        new TimePickerDialog(activity, timeListener, hour, minute, is24).show();
    }

    private DatePickerDialog.OnDateSetListener dateListener = new DatePickerDialog.OnDateSetListener()
    {
        public void onDateSet(DatePicker arg0, int year, int month, int day)
        {
            if ( pageGeneral!=null )
            {
                TextView date  = pageGeneral.findViewById(R.id.device_date);
                TextView time  = pageGeneral.findViewById(R.id.device_time);
                String parts[] = time.getText().toString().split(":");
                int hour       = Integer.parseInt(parts[0]);
                int minutes    = Integer.parseInt(parts[1]);

                Calendar c     = Calendar.getInstance();
                c.set(year, month, day, hour, minutes);
                device.setDate( c.getTime() );
                date.setText( formatData.date(device.getDateFormat(), device.getDate()) );
            }
        }
    };

    private void pickDate()
    {
        Calendar calendar = new GregorianCalendar();
        calendar.setTime(device.getDate());
        int year  = calendar.get(Calendar.YEAR);
        int month = calendar.get(Calendar.MONTH);
        int day   = calendar.get(Calendar.DAY_OF_MONTH);
        new DatePickerDialog(activity, dateListener, year, month, day).show();
    }

    /***********************************************************************************************
     * REGION Pages
     ***********************************************************************************************/

    private void pageTestSetup()
    {
        overlay.removeAllViews();
        overlay.addView(pageTestSetup);
        overlay.setVisibility(View.VISIBLE);

        // ### FIELDS ###
        final TextView ui_txw_max_amp = pageTestSetup.findViewById( R.id.ui_txw_max_amp );
        final TextView ui_txw_timer   = pageTestSetup.findViewById( R.id.ui_txw_timer );
        final Spinner ui_spr_standard = helper.getSpinner( R.id.ui_spr_standard, Device.STANDARDS_NAMES, pageTestSetup );
        final Spinner ui_spr_mode     = helper.getSpinner( R.id.ui_spr_mode    , GLOVE_CLASSES         , pageTestSetup );
        final Spinner ui_spr_max_amp  = helper.getSpinner( R.id.ui_spr_max_amp , R.array.enable_disable, pageTestSetup );

        // ### FILL FIELDS ###
        ui_txw_timer.setText        ( formatData.elapsed( device.getTestDuration() ) );
        ui_spr_standard.setSelection( device.getStandard()   );
        ui_spr_mode.setSelection    ( device.getGloveClass() );

        if ( device.getMaxAmp() > 0 )
        {
            ui_spr_max_amp.setSelection( 0 );
            ui_txw_max_amp.setText( device.getMaxAmp() + " " + DEVICE_CURRENT_UNIT );
        }
        else
        {
            ui_spr_max_amp.setSelection( 1 );
            ui_txw_max_amp.setText( DEVICE_MIN_AMPS + " " + DEVICE_CURRENT_UNIT );
        }

        // ### ACTIONS ###
        // MAX CURRENT
        ImageButton ui_btn_max_amp = pageTestSetup.findViewById( R.id.ui_btn_max_amp );
        ui_btn_max_amp.setOnClickListener(v -> helper.pickNumber
        (
                ui_txw_max_amp,
                R.string.max_leakage,
                DEVICE_CURRENT_UNIT,
                DEVICE_MIN_AMPS,
                DEVICE_MAX_AMP,
                DEVICE_STEP_AMPS
        ));

        // TEST DURATION
        ImageButton ui_btn_timer = pageTestSetup.findViewById( R.id.ui_btn_timer );
        ui_btn_timer.setOnClickListener( v ->
        {
            if ( ui_spr_standard.getSelectedItemPosition() == Device.STANDARD_IEC60903 )
                helper.pickTimer( ui_txw_timer, device.getTimerMin(), device.getTimerMax(), device.getTimerStepIEC() );
            else
                helper.pickTimer( ui_txw_timer, device.getTimerMin(), device.getTimerMax(), device.getTimerStep() );
        });

        // STANDARD SELECTION
        ui_spr_standard.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener()
        {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id)
            {
                if ( position == Device.STANDARD_ASTMD120 )
                {
                    ui_btn_timer.setEnabled( false );
                    ui_txw_timer.setEnabled( false );
                }
                else
                {
                    ui_btn_timer.setEnabled( true );
                    ui_txw_timer.setEnabled( true );
                }

                ui_txw_timer.setText( formatData.elapsed( device.getTimerMax() ) );
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {}
        });

        // MODE SELECTION
        ui_spr_max_amp.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener()
        {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id)
            {
                if ( position == 0 )
                    pageTestSetup.findViewById(R.id.lineMaxCurrent).setVisibility(View.VISIBLE);
                else
                    pageTestSetup.findViewById(R.id.lineMaxCurrent).setVisibility(View.GONE);
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {}
        });

        // ### SAVE ###
        Button ui_btn_save = pageTestSetup.findViewById(R.id.buttonSave);
        ui_btn_save.setOnClickListener(v ->
        {
            if ( ui_spr_standard.getSelectedItemPosition() != device.getStandard() )
            {
                sendMessage( modbus.write.standard( ui_spr_standard.getSelectedItemPosition() ) );
            }

            if ( ui_spr_mode.getSelectedItemPosition() != device.getGloveClass() )
            {
                sendMessage( modbus.write.gloveClass( ui_spr_mode.getSelectedItemPosition() ) );
            }

            String[] parts = ui_txw_timer.getText().toString().split(":");
            int minutes    = Integer.parseInt( parts[0] ) * 60;
            int seconds    = Integer.parseInt( parts[1] );
            int time       = minutes + seconds;

            if ( time != device.getTestDuration() )
            {
                sendMessage(modbus.write.gloveTestDuration(time));
            }

            if ( ui_spr_max_amp.getSelectedItemPosition() == 0 )
            {
                try
                {
                    String ampString = ui_txw_max_amp.getText().toString();
                    ampString        = ampString.replace( Protocol.DEVICE_CURRENT_UNIT, "" );
                    ampString        = ampString.trim();
                    int amp          = Integer.parseInt(ampString);
                    sendMessage( modbus.write.maxAmp( amp ) );
                }
                catch (Exception ignored)
                {
                }
            }
            else
            {
                sendMessage( modbus.write.maxAmp( 0 ) );
            }

            device.setState( Changed.BUSY );
            userMessage( R.string.updating, CommonUI.COLOR_YELLOW, true );
            helper.closeOverlay(overlay);
        });

        Button ui_btn_cancel = pageTestSetup.findViewById(R.id.buttonCancel);
        ui_btn_cancel.setOnClickListener(v -> helper.closeOverlay(overlay) );
    }

    private void pageGeneralSetup()
    {
        overlay.removeAllViews();
        overlay.addView(pageGeneral);
        overlay.setVisibility(View.VISIBLE);

        LinearLayout ui_ll_buzzer = pageGeneral.findViewById(R.id.ui_ll_buzzer);
        ui_ll_buzzer.setVisibility(View.GONE);

        final TextView
                date   = pageGeneral.findViewById(R.id.device_date),
                time   = pageGeneral.findViewById(R.id.device_time);

        final Spinner
                spinnerConnect    = helper.getSpinner(R.id.spinnerAutoConnection, R.array.enable_disable,   pageGeneral),
                spinnerDateFormat = helper.getSpinner(R.id.spinnerDateFormat,     R.array.date_format,      pageGeneral),
                spinnerTimeFormat = helper.getSpinner(R.id.spinnerTimeFormat,     R.array.time_format,      pageGeneral),
                spinnerLanguage   = helper.getSpinner(R.id.spinnerLanguage,       R.array.language_options, pageGeneral);

        date.setText( formatData.date(device.getDateFormat(), device.getDate()) );
        time.setText( formatData.time(device.getTimeFormat(), device.getDate()) );

        pageGeneral.findViewById(R.id.line_gauge).setVisibility(View.GONE);

        ImageButton btEraseMem = pageGeneral.findViewById(R.id.button_memory);
        btEraseMem.setOnClickListener(v ->
        {

            final Dialog dialog = helper.getDialog( R.layout.dialog_yes_no_white, R.drawable.ic_warning, R.string.erase_memory );
            final TextView message = dialog.findViewById(R.id.message);
            message.setText(R.string.erase_memory_confirm);

            dialog.findViewById( R.id.btn_no ).setOnClickListener(v1 -> dialog.dismiss());

            dialog.findViewById( R.id.btn_yes ).setOnClickListener(v12 ->
            {
                sendMessage( modbus.write.memoryErase() );
                if ( ui_txw_free_memory != null )
                    ui_txw_free_memory.setText( activity.getString(R.string.wait) );
                dialog.dismiss();
            });

            dialog.show();
        });

        if(device.getAutoConnection())
        {
            spinnerConnect.setSelection(0);
        }
        else
        {
            spinnerConnect.setSelection(1);
        }

        switch (device.getDateFormat())
        {
            case Modbus.DATE_DD_MM_YY:
                spinnerDateFormat.setSelection(0);
                break;
            case Modbus.DATE_MM_DD_YY:
                spinnerDateFormat.setSelection(1);
                break;
            default:
                spinnerDateFormat.setSelection(2);
                break;
        }

        switch (device.getTimeFormat())
        {
            case Modbus.TIME_24H:
                spinnerTimeFormat.setSelection(0);
                break;
            default:
                spinnerTimeFormat.setSelection(1);
                break;
        }

        switch (device.getLanguage())
        {
            case Modbus.LANG_ENGLISH:
                spinnerLanguage.setSelection(0);
                break;
            case Modbus.LANG_SPANISH:
                spinnerLanguage.setSelection(1);
                break;
            default:
                spinnerLanguage.setSelection(2);
                break;
        }

        spinnerDateFormat.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                switch (position){
                    case 0:
                        date.setText( formatData.date(Modbus.DATE_DD_MM_YY, device.getDate()) );
                        break;
                    case 1:
                        date.setText( formatData.date(Modbus.DATE_MM_DD_YY, device.getDate()) );
                        break;
                    default:
                        date.setText( formatData.date(Modbus.DATE_YYYY_MM_DD, device.getDate()) );
                        break;
                }
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {}
        });

        spinnerTimeFormat.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {

                switch (position){
                    case 0:
                        time.setText( formatData.time(Modbus.TIME_24H, device.getDate()) );
                        timeFormat = Modbus.TIME_24H;
                        break;
                    default:
                        time.setText( formatData.time(Modbus.TIME_12H, device.getDate()) );
                        timeFormat = Modbus.TIME_12H;
                        break;
                }
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {}
        });

        ImageButton buttonDate = pageGeneral.findViewById(R.id.button_date);
        buttonDate.setOnClickListener(v -> pickDate());

        ImageButton buttonTime = pageGeneral.findViewById(R.id.button_time);
        buttonTime.setOnClickListener(v -> pickTime(timeFormat));

        Button buttonSave = pageGeneral.findViewById(R.id.buttonSave);
        buttonSave.setOnClickListener(v -> {
            switch (spinnerConnect.getSelectedItemPosition()) {
                case 0:
                    device.setAutoConnection(true);
                    break;
                case 1:
                    device.setAutoConnection(false);
                    break;
            }

            int dateFormat, timeFormat, language;
            // DATE
            switch (spinnerDateFormat.getSelectedItemPosition()) {
                case 0:
                    dateFormat = Modbus.DATE_DD_MM_YY;
                    break;
                case 1:
                    dateFormat = Modbus.DATE_MM_DD_YY;
                    break;
                default:
                    dateFormat = Modbus.DATE_YYYY_MM_DD;
                    break;
            }

            switch (spinnerTimeFormat.getSelectedItemPosition()) {
                case 0:
                    timeFormat = Modbus.TIME_24H;
                    break;
                default:
                    timeFormat = Modbus.TIME_12H;
                    break;
            }

            switch (spinnerLanguage.getSelectedItemPosition()) {
                case 0:
                    language = Modbus.LANG_ENGLISH;
                    break;
                case 1:
                    language = Modbus.LANG_SPANISH;
                    break;
                default:
                    language = Modbus.LANG_PORTUGUESE;
                    break;
            }

            device.setDateFormat(dateFormat);
            device.setTimeFormat(timeFormat);
            device.setLanguage(language);

            device.setState( Changed.BUSY );
            userMessage( R.string.updating, CommonUI.COLOR_YELLOW, true );

            sendMessage( modbus.write.date( device.getDate() ) );
            sendMessage( modbus.write.setup(device) );
            ((Main)activity).updateAutoConn(device.getAutoConnection());
            ((Main)activity).updateDateTimeFormat(dateFormat, timeFormat);
            helper.closeOverlay(overlay);
        });

        Button buttonCancel = pageGeneral.findViewById(R.id.buttonCancel);
        buttonCancel.setOnClickListener(v ->
        {
            ui_btn_device_config.setEnabled(true);
            helper.closeOverlay(overlay);
        });
    }

    private void pageFiles()
    {
        if ( pageFilesManager == null )
        {
            pageFilesManager = View.inflate(activity, R.layout.dt70kv_files_manager, null);
        }

        overlay.removeAllViews();
        overlay.addView(pageFilesManager);
        overlay.setVisibility(View.VISIBLE);

        final ListView list  = pageFilesManager.findViewById(R.id.list_savedfiles);

        final Button
                ui_btn_file_delete= pageFilesManager.findViewById( R.id.bt_delete ),
                ui_btn_file_open  = pageFilesManager.findViewById( R.id.bt_open   ),
                ui_btn_file_new   = pageFilesManager.findViewById( R.id.bt_new    ),
                ui_btn_file_rename= pageFilesManager.findViewById( R.id.bt_rename ),
                ui_btn_file_close = pageFilesManager.findViewById( R.id.bt_close  ),
                ui_btn_file_export= pageFilesManager.findViewById( R.id.bt_export );

        ui_btn_file_delete.setEnabled(false);
        ui_btn_file_open.setEnabled(false);
        ui_btn_file_close.setEnabled(false);
        ui_btn_file_export.setEnabled(false);
        ui_btn_file_rename.setVisibility(View.GONE);

        ui_btn_file_new.setOnClickListener(v ->
            dialogGloveSerial( true )
        );

        final List<FileDetails> fileList = database.getSavedFilesList(device.getDateFormat(), device.getTimeFormat());
        int selectedListIndex = -1;

        TableFilesAdapter adapter = new TableFilesAdapter( activity );
        list.setAdapter    ( adapter );
        list.setChoiceMode ( ListView.CHOICE_MODE_SINGLE );
        list.setItemChecked( selectedListIndex, true );
        list.setSelection  ( selectedListIndex );

        if ( fileList!=null && fileList.size()>0 )
        {
            for(int i=0; i<fileList.size(); i++)
            {
                // Already opened
                if ( testData!=null && testData.getID() == fileList.get(i).FILE_NAME )
                {
                    selectedListIndex = i;
                    ui_btn_file_open.setEnabled( false );
                    adapter.add( "\u25C9  " + fileList.get(i).GLOVE_SERIAL, fileList.get(i).TEST_NUMBER, fileList.get(i).DATE ); // "u25C9 = ◉ | u25EF  u2705 u2611 u25A3";
                    list.setItemChecked( selectedListIndex, true );
                }
                else
                {
                    adapter.add( "\u25EF  " + fileList.get(i).GLOVE_SERIAL, fileList.get(i).TEST_NUMBER, fileList.get(i).DATE ); // "u25EF = ◯ | u25EF  u2705 u2611 u25A3";
                }
            }

            adapter.notifyDataSetChanged();

            if ( selectedListIndex < 0 )
            {
                ui_btn_file_delete.setEnabled(false);
                ui_btn_file_open.setEnabled(false);
                ui_btn_file_close.setEnabled(false);
                ui_btn_file_export.setEnabled(false);
            }
            else
            {
                ui_btn_file_delete.setEnabled(true);
                ui_btn_file_open.setEnabled(false);
                ui_btn_file_close.setEnabled(true);
                ui_btn_file_export.setEnabled(true);
            }

            list.setOnItemClickListener((parent, view1, position, id) ->
            {
                ui_btn_file_delete.setEnabled(true);
                ui_btn_file_export.setEnabled(true);
                ui_btn_file_open.setEnabled(true);
                ui_btn_file_close.setEnabled(false);

                if ( fileList.size() > 0 )
                {
                    // Already opened
                    if ( testData!=null && testData.getID() == fileList.get(position).FILE_NAME )
                    {
                        ui_btn_file_open.setEnabled(false);
                        ui_btn_file_close.setEnabled(true);
                    }
                }
            });
        }
        else
        {
            list.setAdapter(null);
        }

        ui_btn_file_delete.setOnClickListener(v ->
        {
            final Dialog dialog = helper.getDialog( R.layout.dialog_yes_no_white, R.drawable.ic_warning, R.string.erase_memory );
            final TextView message = dialog.findViewById(R.id.message);
            message.setText(R.string.file_delete_all_confirm);

            dialog.findViewById(R.id.btn_no).setOnClickListener(v1 -> dialog.dismiss());

            dialog.findViewById(R.id.btn_yes).setOnClickListener(v12 ->
            {
                int selectedIndex = list.getCheckedItemPosition();
                if(selectedIndex > -1)
                {
                    if ( fileList==null )
                        return;

                    database.deleteTable( serial, fileList.get(selectedIndex).FILE_NAME );
                    if ( testData != null && testData.getID() == fileList.get(selectedIndex).FILE_NAME )
                    {
                        testData.terminate();
                        testData = null;
                    }
                }
                dialog.dismiss();
                helper.closeOverlay(overlay);
                pageFiles();
            });

            dialog.show();
        });

        Observer observer = this;

        ui_btn_file_open.setOnClickListener(v ->
        {
            int selectedIndex = list.getCheckedItemPosition();
            if ( selectedIndex > -1 )
            {
                testData = null;

                if ( fileList == null )
                    return;

                testData = database.getResult(observer, fileList.get(selectedIndex).FILE_NAME, fileList.get(selectedIndex).GLOVE_SERIAL );
                if ( testData != null ) testData.setFileSaveOrOpened();
            }
            helper.closeOverlay(overlay);
        });

        ui_btn_file_close.setOnClickListener(v ->
        {
            list.clearChoices();
            testData.terminate();
            testData = null;
            ui_btn_file_close.setEnabled(false);
            ui_btn_file_open.setEnabled(false);
            ui_btn_file_delete.setEnabled(false);
            helper.closeOverlay(overlay);
            pageFiles();
        });

        ui_btn_file_export.setOnClickListener(v ->
        {
            selectedIndex = list.getCheckedItemPosition();
            if(Main.permissionToMemoryAccepted)
                export(selectedIndex);
            else
                ((Main)activity).requestPermission(Main.REQUEST_MEMORY_REPORT_PERMISSION);

        });

        pageFilesManager.findViewById(R.id.bt_cancel).setOnClickListener(v -> helper.closeOverlay(overlay));
    }

    private void export(int index)
    {
        if( index > -1 && pagerAdapter != null )
        {
            final List<FileDetails> fileList = database.getSavedFilesList( device.getDateFormat(), device.getTimeFormat() );
            if ( fileList == null ) return;

            final TestData exportData = database.getResult( null, fileList.get(index).FILE_NAME, fileList.get(index).GLOVE_SERIAL );
            if ( exportData == null ) return;

            pagerAdapter.fillChartsAndTable( exportData, device );

            if( dlgProgress == null ) creatDialogProgress();
            dlgProgress.show();

            new Thread(() ->
            {
                File voltage = helper.saveChartImage(pagerAdapter.chartVoltage(), database.getTemporaryFolder());
                File current = helper.saveChartImage(pagerAdapter.chartCurrent(), database.getTemporaryFolder());

                report.generate
                (
                        exportData,
                        device,
                        database.getPictureFile    ( exportData.getSerial(), exportData.getID()+"" ),
                        database.getMapSnapshotFile( exportData.getSerial(), exportData.getID()+"" ),
                        new File[]{ voltage, current }
                );
            }).start();
            closeOverlay(activity);
        }
    }

    /***********************************************************************************************
     * REGION State machine
     ***********************************************************************************************/

    public void stateChanged(int state)
    {
        switch (state)
        {
            case BluetoothClassic.STATE_CONNECTED:
                device.setState( Changed.CONNECTED );
                break;
            case BluetoothClassic.STATE_CONNECTING:
                device.setState( Changed.CONNECTING );
                break;
            case BluetoothClassic.STATE_NOT_RESPONDING:
                device.setState( Changed.NOT_RESPONDING );
                break;
            case BluetoothClassic.STATE_FAIL:
                device.setState( Changed.DISCONNECTED );
                break;
            case BluetoothClassic.STATE_LOST:
                device.setState( Changed.DISCONNECTED );
                break;
            case BluetoothClassic.STATE_CLOSED:
                device.setState( Changed.DISCONNECTED );
                break;
        }
    }

    private void updateTestData(TestData.Changed whatChanged)
    {
        if ( testData == null )
            return;

        TestData.Changed what = TestData.Changed.values()[whatChanged.ordinal()];

        switch (what)
        {
            case RESET:
                pagerAdapter.resetCharts();
                ui_btn_save.setEnabled( false );
                ui_btn_voice.setEnabled( false );
                ui_btn_picture.setEnabled( false );
                ui_btn_gps.setEnabled( false );
                ui_btn_map.setEnabled( false );
                break;
            case TEST_NUMBER:
                pagerAdapter.setTestNumber ( testData.getTestNumber()  );
                break;
            case DATE:
                if ( testData.getDate() != null )
                {
                    String date = formatData.dateTime(testData.getDate(), device.getDateFormat(), device.getTimeFormat());
                    pagerAdapter.setTestDate(date);
                }
                break;
            case GLOVE_CLASS:
                pagerAdapter.setTestMode( GLOVE_CLASSES[testData.getGloveClass()] );
                break;
            case MEASUREMENT:
                pagerAdapter.updateChartsAndTable( testData.getMeasurementLast() );
                break;
            case RESULT_PASS_FAIL:
                pagerAdapter.table().setResult( testData.getTestResult() );
                break;
            case FILE_READY_TO_SAVE:
                ui_btn_save.setEnabled( true );
                break;
            case FILE_SAVED_OR_OPENED:
                ui_btn_save.setEnabled( false );
                pagerAdapter.resetCharts();
                pagerAdapter.mainScreen().resetRT();
                if ( testData == null ) return;

                if ( !testData.isIncomplete() )
                {
                    ui_btn_voice.setEnabled( true  );
                    ui_btn_picture.setEnabled( true  );
                    ui_btn_gps.setEnabled( true  );
                    ui_btn_map.setEnabled( true  );
                    pagerAdapter.fillChartsAndTable(testData, device);
                    viewPager.setCurrentItem(pagerAdapter.getCount()-1, true);
                }
                else
                {
                    ui_btn_voice.setEnabled( false );
                    ui_btn_picture.setEnabled( false );
                    ui_btn_gps.setEnabled( false );
                    ui_btn_map.setEnabled( false );
                    pagerAdapter.setGloveSerial( testData.getGloveSerial() );
                }
                break;
        }
    }

    private void updateDevice(Device.Changed whatChanged)
    {
        Device.Changed what = Device.Changed.values()[whatChanged.ordinal()];
        switch ( what )
        {
            case BUTTON_START:
                if ( testData != null ) // HAS FILE
                {
                    if ( testData.getGloveSerial().length() <= 0 ) // NOT SAVED
                    {
                        testData = new TestData(this, model, serial);
                        testData.isIncomplete(true);
                        dialogStartTest();
                    }
                    else // ALREADY SAVED
                    {
                        if (testData.isIncomplete()) // SAVED BUT WITHOUT TEST RESULTS
                        {
                            long id   = testData.getID();
                            String sn = testData.getGloveSerial();
                            testData = new TestData(this, model, serial);
                            testData.isIncomplete( true );
                            testData.setID(id);
                            testData.setGloveSerial(sn);
                            dialogStartTest();
                        }
                        else
                            dialogReplaceData();
                    }
                }
                else // NEW
                {
                    dialogStartTest();
                }
                break;
            case REQUEST_START:
                ui_btn_start.setEnabled( false );
                ui_btn_setup.setEnabled( false );
                userMessage( R.string.md_warning_test_start, CommonUI.COLOR_YELLOW, true);
                sendMessage( modbus.write.start() );
                break;
            case BUTTON_STOP:
                ui_btn_stop.setEnabled( false );
                sendMessage( modbus.write.stop() );
                break;
            case BUTTON_SETUP:
                pageTestSetup();
                break;
            case STANDY_BY:
                userMessage(R.string.ready, CommonUI.COLOR_GREEN, false);
                if ( !device.getStatusOne(Device.ST_ONE_AUTO_PILOT) )
                {
                    ui_btn_start.setEnabled( true );
                    ui_btn_setup.setEnabled( true );
                    ui_btn_device_config.setEnabled( true );
                }
                ui_btn_stop.setEnabled( true );
                ui_btn_start.setVisibility( View.VISIBLE );
                ui_btn_stop.setVisibility( View.GONE    );
                break;
            case BUSY:
                ui_btn_start.setEnabled( false );
                ui_btn_stop.setEnabled( false );
                ui_btn_setup.setEnabled( false );
                ui_btn_device_config.setEnabled( false );
                break;
            case MEMORY_MAP:
                Modbus.setRevisionMap( device.getMemoryMap() );
                userMessage(R.string.memory_map_was_read, CommonUI.COLOR_YELLOW, true);
                break;
            case FIRMWARE:
                userMessage(R.string.firmware_was_read, CommonUI.COLOR_YELLOW, true);
                break;
            case HARDWARE:
                userMessage(R.string.firmware_was_read, CommonUI.COLOR_YELLOW, true);
                break;
            case VERSION:
                parent.setFirmwareVersion( device.getVersion() );
                break;
            case DISCONNECTED:
                if ( overlay.getChildCount() > 0 && overlay.getChildAt(0) == pageGeneral || overlay.getChildAt(0) == pageTestSetup )
                {
                    overlay.removeAllViews();
                    overlay.setVisibility(View.GONE);
                }
                ui_btn_manage.setEnabled( true  );
                ui_btn_start.setEnabled( false );
                ui_btn_stop.setEnabled( false );
                ui_btn_setup.setEnabled( false );
                pagerAdapter.resetMainScreen();
                int dateFormat = device.getDateFormat();
                int timeFormat = device.getTimeFormat();
                device = new Device();
                device.addObserver(this);
                device.setDateFormat( dateFormat );
                device.setTimeFormat( timeFormat );
                break;
            case CONNECTING:
                ui_btn_start.setEnabled( false );
                ui_btn_stop.setEnabled( false );
                ui_btn_setup.setEnabled( false );
                ui_btn_device_config.setEnabled( false );
                break;
            case CONNECTED:
                ui_btn_manage.setEnabled( true  );
                ui_btn_start.setEnabled( true  );
                ui_btn_stop.setEnabled( false );
                ui_btn_setup.setEnabled( true  );
                ui_btn_device_config.setEnabled( false );
                sendMessage( modbus.read.memoryMap() );
                device.setState( Changed.BUSY );
                break;
            case AUTO_PILOT:
                if ( device.getStatusOne(Device.ST_ONE_AUTO_PILOT) )
                {
                    helper.warnAboutLock(device.getStatusOne(Device.ST_ONE_SPARK_DETECTED));
                    ui_btn_start.setEnabled( false );
                    ui_btn_setup.setEnabled( false );
                    ui_btn_device_config.setEnabled( false );
                }
                else
                {
                    ui_btn_start.setEnabled( true );
                    ui_btn_setup.setEnabled( true );
                    ui_btn_device_config.setEnabled( true );
                }
                break;
            case INTERNAL_ERROR:
                if ( device.getStatusTwo(Device.ST_TWO_INTERNAL_ERROR) )
                {
                    helper.warnInternalError();
                    ui_btn_start.setEnabled( false );
                    ui_btn_setup.setEnabled( false );
                    ui_btn_device_config.setEnabled( false );
                }
                else
                {
                    ui_btn_start.setEnabled( true );
                    ui_btn_setup.setEnabled( true );
                    ui_btn_device_config.setEnabled( true );
                }
                break;
            case EMERGENCY_STOP:
                pagerAdapter.mainScreen().isEmergencyStop( device.getStatusOne(Device.ST_ONE_EMERGENCY_STOP) );
                break;
            case GENERATION_LOCK:
                pagerAdapter.mainScreen().isGenerationLock( device.getStatusOne(Device.ST_ONE_GENERATION_LOCK) );
                break;
            case SPARK_DETECTED:
                if ( device.getTestNumber() > 0 )
                    pagerAdapter.mainScreen().isSpark( device.getStatusOne(Device.ST_ONE_SPARK_DETECTED) );
                else
                    pagerAdapter.mainScreen().isSpark( false );
                break;
            case OVERCURRENT:
                if ( device.getTestNumber() > 0 )
                    pagerAdapter.mainScreen().isOvercurrent( device.getStatusOne(Device.ST_ONE_OVERCURRENT) );
                else
                    pagerAdapter.mainScreen().isOvercurrent( false );
                break;
            case INTERNAL_PROTECTION:
                pagerAdapter.mainScreen().isInternalProtection( device.getStatusOne(Device.ST_ONE_INTERNAL_PROTECTION) );
                break;
            case RESULT_PASS:
                if ( device.getTestNumber() > 0 )
                {
                    if ( testData != null ) testData.setTestResult( device.getTestResult() );
                    pagerAdapter.mainScreen().isResultPass(true);
                }
                else
                    pagerAdapter.mainScreen().isResultPass   ( false );
                pagerAdapter.mainScreen().isResultFail   ( false );
                pagerAdapter.mainScreen().isResultAborted( false );
                break;
            case RESULT_FAIL:
                if ( device.getTestNumber() > 0 )
                {
                    if ( testData != null ) testData.setTestResult( device.getTestResult() );
                    pagerAdapter.mainScreen().isResultFail(true);
                }
                else
                    pagerAdapter.mainScreen().isResultFail   ( false );
                pagerAdapter.mainScreen().isResultPass   ( false );
                pagerAdapter.mainScreen().isResultAborted( false );
                break;
            case RESULT_ABORTED:
                if ( device.getTestNumber() > 0 )
                {
                    if ( testData != null ) testData.setTestResult( device.getTestResult() );
                    pagerAdapter.mainScreen().isResultAborted(true);
                }
                else
                    pagerAdapter.mainScreen().isResultAborted( false );
                pagerAdapter.mainScreen().isResultPass   ( false );
                pagerAdapter.mainScreen().isResultFail   ( false );
                break;
            case RESULT_CLEAR:
                pagerAdapter.mainScreen().isResultPass   ( false );
                pagerAdapter.mainScreen().isResultFail   ( false );
                pagerAdapter.mainScreen().isResultAborted( false );
                break;
            case STANDARD:
                pagerAdapter.mainScreen().setStandard  ( device.getStandardAsString() );
                pagerAdapter.mainScreen().setMaxVoltage( formatData.volts( device.getVoltsByGloveClass() ) );
                break;
            case GLOVE_CLASS:
                pagerAdapter.mainScreen().setGloveClass( device.getGloveClass() );
                pagerAdapter.mainScreen().setMaxVoltage( formatData.volts( device.getVoltsByGloveClass() ) );
                break;
            case TEST_DURATION:
                pagerAdapter.mainScreen().setTestDuration( formatData.elapsed( device.getTestDuration() ) );
                break;
            case MAX_AMP:
                if ( device.getMaxAmp() > 0 )
                    pagerAdapter.mainScreen().setMaxAmp( formatData.amps( device.getMaxAmp() ) );
                else
                    pagerAdapter.mainScreen().setMaxAmp( " - - - " );
                break;
            case CONFIG_CHANGED:
                userMessage(R.string.md_configurations_changed, CommonUI.COLOR_YELLOW, true);
                sendMessage( modbus.write.resetConfigChanged() );
                device.setState( Changed.BUSY );
                break;
            case FREE_MEMORY:
                if ( ui_txw_free_memory != null )
                    ui_txw_free_memory.setText( device.getFreeMemory() + " %" );
                break;
            case TEST_STARTING:
                ui_btn_start.setVisibility( View.GONE );
                ui_btn_stop.setVisibility( View.VISIBLE );
                ui_btn_stop.setEnabled( true );
                ui_btn_setup.setEnabled( false );
                ui_btn_manage.setEnabled( false );
                ui_btn_device_config.setEnabled( false );
                if ( device.getStatusOne( Device.ST_ONE_AUTO_PILOT ) && testData != null )
                {
                    testData.terminate();
                    testData = null;
                }
                if ( testData != null ) pagerAdapter.setGloveSerial( testData.getGloveSerial() );
                userMessage( R.string.md_warning_test_start, CommonUI.COLOR_YELLOW, true);
                break;
            case TEST_RUNNING:
                ui_btn_start.setVisibility( View.GONE );
                ui_btn_stop.setVisibility( View.VISIBLE );
                ui_btn_stop.setEnabled( true );
                ui_btn_setup.setEnabled( false );
                ui_btn_manage.setEnabled( false );
                ui_btn_device_config.setEnabled( false );
                if ( device.getStatusOne( Device.ST_ONE_AUTO_PILOT ) && testData != null )
                {
                    testData.terminate();
                    testData = null;
                }
                if ( testData != null ) pagerAdapter.setGloveSerial( testData.getGloveSerial() );
                userMessage( R.string.md_test_running, CommonUI.COLOR_RED, true);
                break;
            case TEST_FINISHED:
                ui_btn_start.setVisibility( View.VISIBLE );
                ui_btn_stop.setVisibility( View.GONE    );
                if ( !device.getStatusOne(Device.ST_ONE_AUTO_PILOT) )
                {
                    ui_btn_start.setEnabled( true );
                    ui_btn_setup.setEnabled( true );
                    ui_btn_device_config.setEnabled( true );
                }
                ui_btn_manage.setEnabled( true );
                pagerAdapter.mainScreen().rampHide();
                testData.setTestResult( device.getTestResult() );
                testData.setFileReadyToSave();
                userMessage( R.string.finished, CommonUI.COLOR_GREEN, false);
                handler.postDelayed(() -> sendMessage( modbus.read.voltageDischarge() ), 4000);
                dialogTestFinished(false);
                break;
            case TEST_CANCELLED:
                ui_btn_start.setVisibility( View.VISIBLE );
                ui_btn_stop.setVisibility( View.GONE    );
                if ( !device.getStatusOne(Device.ST_ONE_AUTO_PILOT) )
                {
                    ui_btn_start.setEnabled( true );
                }
                ui_btn_setup.setEnabled ( true );
                ui_btn_manage.setEnabled( true );
                ui_btn_device_config.setEnabled( true );
                pagerAdapter.mainScreen().rampHide();
                if ( testData != null ) testData.setFileReadyToSave();
                userMessage( R.string.test_cancelled, CommonUI.COLOR_YELLOW, false);
                dialogTestFinished(true);
                break;
            case RAMP_UP:
                pagerAdapter.mainScreen().rampUp();
                break;
            case RAMP_FIX:
                pagerAdapter.mainScreen().rampFix();
                break;
            case RAMP_DOWN:
                pagerAdapter.mainScreen().rampDown();
                break;
            case RAMP_FINISHED:
                pagerAdapter.mainScreen().rampHide();
                break;
            case RT_TEST_NUMBER:
                pagerAdapter.mainScreen().setTestNumber( device.getTestNumber() );
                if ( testData == null )
                    testData = new TestData(this, model, serial);
                testData.setMemoryMap  ( device.getMemoryMap()     );
                testData.setHardware   ( device.getHardware()      );
                testData.setFirmware   ( device.getFirmware()      );
                testData.setTestNumber ( device.getTestNumber()    );
                testData.setTestMode   ( device.getOperationMode() );
                testData.setGloveClass ( device.getGloveClass()    );
                break;
            case RT_DATE:
                testData.setDate( device.getTestDate() );
                break;
            case RT_MEASUREMENT:
                pagerAdapter.mainScreen().setMeasurement( device.getMeasurement(), device.getTestDuration() );
                testData.addMeasument( device.getMeasurement() );
                break;
        }
    }

    @Override
    public void update(Observable observable, Object arg1)
    {
        if ( observable instanceof Device )
        {
            if ( arg1 instanceof Device.Changed )
            {
                updateDevice((Device.Changed) arg1);
            }
        }
        else if ( observable instanceof TestData )
        {
            if ( arg1 instanceof TestData.Changed )
            {
                updateTestData((TestData.Changed) arg1);
            }
        }
    }

    private void dialogTestFinished(boolean aborted)
    {
        String results;

        if ( device.getStatusOne(6) && device.getStatusOne(7) )
            results = activity.getString(R.string.result).toUpperCase() + " : " + activity.getString(R.string.pass);
        else if ( !device.getStatusOne(6) && device.getStatusOne(7) )
            results = activity.getString(R.string.result).toUpperCase() + " : " + activity.getString(R.string.fail);
        else if ( device.getStatusOne(6) )
            results = activity.getString(R.string.result).toUpperCase() + " : " + activity.getString(R.string.aborted);
        else
            results = activity.getString(R.string.result).toUpperCase() + " : " + " - - - ";

        if ( device.getStatusOne(4) )
            results += "\r\n" + activity.getString(R.string.reason).toUpperCase() + " : " + activity.getString(R.string.spark_detection).toUpperCase();
        else if (device.getStatusOne(5))
            results += "\r\n" + activity.getString(R.string.reason).toUpperCase() + " : " + activity.getString(R.string.overcurrent).toUpperCase();

        if (dialogFinished==null)
        {
            dialogFinished = new Dialog(activity);
            dialogFinished.requestWindowFeature(Window.FEATURE_NO_TITLE);
            dialogFinished.setCanceledOnTouchOutside(false);
            dialogFinished.setCancelable(false);
            dialogFinished.setContentView(R.layout.dialog_message_white);
            if (dialogFinished.getWindow() != null)
            {
                dialogFinished.getWindow().setBackgroundDrawable( new ColorDrawable( Color.TRANSPARENT ) );
            }
            ((TextView) dialogFinished.findViewById( R.id.dlg_title )).setText( R.string.md_test_finished );
            ((ImageView) dialogFinished.findViewById( R.id.dlg_icon )).setImageResource( R.drawable.ic_warning );

            dialogFinished.findViewById( R.id.ui_iv_warning ).setVisibility( View.GONE );
            dialogFinished.findViewById( R.id.btn_yes ).setOnClickListener( v -> dialogFinished.dismiss() );
            dialogFinished.findViewById( R.id.btn_close_dlg ).setVisibility( View.GONE );
        }

        if ( aborted )
        {
            results = activity.getString(R.string.result).toUpperCase() + " : " + activity.getString(R.string.aborted);
            ((TextView) dialogFinished.findViewById(R.id.observation)).setText("");
            dialogFinished.findViewById( R.id.btn_yes ).setEnabled(true);
        }
        else
        {
            ((TextView) dialogFinished.findViewById(R.id.observation)).setText(activity.getString(R.string.wait));
            dialogFinished.findViewById(R.id.btn_yes).setEnabled(false);
        }

        ((TextView) dialogFinished.findViewById(R.id.message)).setText(results);
        dialogFinished.show();
    }

    public void processResponse(int id, String rx)
    {
        switch (id)
        {
            case Protocol.ID_REAL_TIME_READ:
                device.setStatusOne( modbus.parse.statusOne( rx ) );
                device.setStatusTwo( modbus.parse.statusTwo( rx ) );

                if ( device.getStatusOne( Device.ST_ONE_RUNNING ) )
                {
                    if ( device.getTestNumber() < 0 )
                        sendMessage( modbus.read.testNumber() );
                    sendMessage( modbus.read.status( Protocol.ST_AMPS ) );
                }

                if ( rx.length() >= (Protocol.ST_AMPS * 4) )
                {
                    if ( device.getTestDate() == null )
                        device.setTestDate( modbus.parse.statusMeasurementDate(rx) );
                    Measurement mea = modbus.parse.statusMeasurement( rx );
                    device.setMeasurement( mea );
                }
                break;
            case Protocol.ID_DISCHARGE_READ:
                double discharge = modbus.parse.voltageDischarge(rx);
                if ( discharge > 100 )
                {
                    sendMessage( modbus.read.voltageDischarge() );
                    if ( dialogFinished != null )
                    {
                        String result = activity.getString(R.string.potential).toUpperCase() + ": " + formatData.volts(discharge);
                        ((TextView) dialogFinished.findViewById(R.id.observation)).setText(result);

                        if (dialogFinished.findViewById(R.id.ui_iv_warning).getVisibility()!=View.VISIBLE)
                            dialogFinished.findViewById(R.id.ui_iv_warning).setVisibility(View.VISIBLE);
                        else
                            dialogFinished.findViewById(R.id.ui_iv_warning).setVisibility(View.INVISIBLE);
                    }
                }
                else
                {
                    if (dialogFinished!=null)
                    {
                        String result = activity.getString(R.string.potential).toUpperCase() + ": " + "< 100 V" ;
                        ((TextView) dialogFinished.findViewById(R.id.observation)).setText(result);
                        dialogFinished.findViewById(R.id.ui_iv_warning).setVisibility(View.INVISIBLE);
                        ((TextView) dialogFinished.findViewById(R.id.dlg_title)).setText(R.string.md_test_finished);
                        ((Button)dialogFinished.findViewById(R.id.btn_yes)).setText(R.string.ok);
                        dialogFinished.findViewById(R.id.btn_yes).setEnabled(true);
                    }
                }
                break;
            case Protocol.ID_TEST_NUMBER_READ:
                device.setTestNumber( modbus.parse.testNumber( rx ) );
                break;
            case Protocol.ID_MEMORY_MAP_READ:
                device.setMemoryMap( modbus.parse.memoryMap( rx ) );
                sendMessage( modbus.read.firmware()          );
                sendMessage( modbus.read.hardware()          );
                sendMessage( modbus.read.standard()          );
                sendMessage( modbus.read.gloveClass()        );
                sendMessage( modbus.read.gloveTestDuration() );
                sendMessage( modbus.read.maxAmp()            );
                break;
            case Protocol.ID_HARDWARE_READ:
                device.setHardware( modbus.parse.hardware( rx ) );
                break;
            case Protocol.ID_FIRMWARE_READ:
                device.setFirmware( modbus.parse.firmware( rx ) );
                break;
            case Protocol.ID_STANDARD_WRITE:
                sendMessage( modbus.read.standard() );
                break;
            case Protocol.ID_STANDARD_READ:
                device.setStandard( modbus.parse.standard( rx ) );
                break;
            case Protocol.ID_GLOVE_CLASS_WRITE:
                sendMessage( modbus.read.gloveClass() );
                break;
            case Protocol.ID_GLOVE_CLASS_READ:
                device.setGloveClass( modbus.parse.gloveClass( rx ) );
                break;
            case Protocol.ID_GLOVE_TEST_TIME_WRITE:
                sendMessage( modbus.read.gloveTestDuration() );
                break;
            case Protocol.ID_GLOVE_TEST_TIME_READ:
                device.setTestDuration( modbus.parse.gloveTestDuration( rx ) );
                break;
            case Protocol.ID_MAX_AMP_WRITE:
                sendMessage( modbus.read.maxAmp() );
                break;
            case Protocol.ID_MAX_AMP_READ:
                device.setMaxAmp( modbus.parse.maxAmp( rx ) );
                device.setState( Changed.STANDY_BY );
                break;
            case Protocol.ID_MEM_POINTER_READ:
                device.setFreeMemory( modbus.parse.memoryFree(rx) );
                break;
            case Protocol.ID_MEM_POINTER_WRITE:
                sendMessage( modbus.read.memoryPointer() );
                break;
            case Protocol.ID_DATE_READ:
                device.setDate( modbus.parse.date( rx ) );
                break;
            case Protocol.ID_GENERAL_SETUP_READ:
                modbus.parse.setup(device, rx);
                pageGeneralSetup();
                break;
            case Protocol.ID_CONFIG_CHANGE_WRITE:
                sendMessage( modbus.read.standard()          );
                sendMessage( modbus.read.gloveClass()        );
                sendMessage( modbus.read.gloveTestDuration() );
                sendMessage( modbus.read.maxAmp()            );
                break;
        }
    }
}
